/**
 * @file arcfour.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Генератор ПСЧ.
 * @version 0.1
 * @date 2024-05-01
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#ifndef ARCFOUR_H
#define ARCFOUR_H

#include <cdefs.h>
#include <stdint.h>
#include <stddef.h>

__BEGIN_DECLS

/**
 * @brief Инициализация ПСЧ
 * @param state Текущее состояние ПСЧ
 * @param key Ключ инициализации
 * @param len Длина ключа (1..256)
 */
void arcfour_key_setup(uint8_t state[256], const uint8_t key[], int len);

/**
 * @brief Генератор ПСЧ последовательности
 * @param state Текущее состояние ПСЧ
 * @param out Буфер для ПСЧ (размером len)
 * @param len Длина буфера out
 */
void arcfour_generate_stream(uint8_t state[256], uint8_t out[], size_t len);

__END_DECLS

#endif