/**
 * @file base64.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Кодирование/декодирование по алгоритму BASE64.
 * @version 0.1
 * @date 2024-04-25
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#ifndef BASE64_H
#define BASE64_H

#include <cdefs.h>
#include <stdint.h>
#include <stddef.h>

__BEGIN_DECLS

// #define TEST_BASE64

/**
 * @brief Кодирование BASE64
 * Возвращает размер выходных данных. Если вызывается c out = NULL, то просто возвращается размер,
 * который был бы у выходных данных (без завершающего 0).
 * @param in Входные данные.
 * @param out Выходные данные.
 * @param len Размер входных данных.
 * @param newline_flag 
 * @return size_t 
 */
size_t base64_encode(const uint8_t in[], uint8_t out[], size_t len, int newline_flag);

/**
 * @brief Декодирование BASE64
 * Возвращает размер выходных данных. Если вызывается c out = NULL, то просто возвращается размер,
 * который был бы у выходных данных (без завершающего 0).
 * @param in Входные данные.
 * @param out Выходные данные.
 * @param len Размер входных данных.
 * @return size_t 
 */
size_t base64_decode(const uint8_t in[], uint8_t out[], size_t len);

#ifdef TEST_BASE64

int test_base64();

#endif

__END_DECLS

#endif
