/**
 * @file blowfish.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Шифрование по алгоритму BLOWFISH.
 * @version 0.1
 * @date 2024-04-27
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#ifndef BLOWFISH_H
#define BLOWFISH_H

#include <cdefs.h>
#include <stddef.h>
#include <stdint.h>

// #define TEST_BLOWFISH

#define BLOWFISH_BLOCK_SIZE 8           // Blowfish operates on 8 bytes at a time

typedef struct {
    uint32_t p[18];
    uint32_t s[4][256];
} BLOWFISH_KEY;

__BEGIN_DECLS

/**
 * @brief Инициализация контекста.
 * @param user_key Ключ шифрования.
 * @param keystruct Контекст.
 * @param len 
 */
void blowfish_key_setup(const uint8_t user_key[], BLOWFISH_KEY *keystruct, size_t len);

/**
 * @brief Шифрование блока по алгоритму BLOWFISH.
 * @param in Входные данные.
 * @param out Выходные данные.
 * @param keystruct Контекст.
 */
void blowfish_encrypt(const uint8_t in[], uint8_t out[], const BLOWFISH_KEY *keystruct);

/**
 * @brief Расшифровка блока по алгоритму BLOWFISH.
 * @param in Входные данные.
 * @param out Выходные данные.
 * @param keystruct Контекст.
 */
void blowfish_decrypt(const uint8_t in[], uint8_t out[], const BLOWFISH_KEY *keystruct);

#ifdef TEST_BLOWFISH

int test_blowfish(void);

#endif

__END_DECLS

#endif
