/**
 * @file ctype.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Объявления функций для классификации и преобразования отдельных символов.
 * @version 0.1
 * @date 2024-03-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef CTYPE_H
#define CTYPE_H

#include <cdefs.h>

__BEGIN_DECLS

/**
 * @brief Возвращает ненулевое значение, если её аргумент является десятичной цифрой, в противном случае возвращается нуль.
 * 
 * @param c Символ
 * @return int 
 */
int isdigit(int c);

/**
 * @brief Возвращает ненулевое значение, если её аргумент является ASCII буквой, в противном случае возвращается нуль.
 * 
 * @param c Символ
 * @return int 
 */
int isalpha(int c);

/**
 * @brief Символ является заглавным символом ASCII.
 * 
 * @param c 
 * @return int 
 */
int isupper(int c);

/**
 * @brief Символ является пробелом, табуляцией или CR/LF.
 * 
 * @param c 
 * @return int 
 */
int isspace(int c);

/**
 * @brief Переводит буквы нижнего регистра в верхний регистр (только ASCII)
 * 
 * @param c 
 * @return int 
 */
int toupper(int c);

/**
 * @brief Переводит буквы верхнего регистра в нижний регистр (только ASCII)
 * 
 * @param c 
 * @return int 
 */
int tolower(int c);

__END_DECLS

#endif
