/**
 * @file fastmem.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Реализация быстрых вариантов функций работы с памятью.
 * @version 0.1
 * @date 2024-04-05
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef FASTMEM_H
#define FASTMEM_H

#include <cdefs.h>
#include <stddef.h>

__BEGIN_DECLS

/**
 * @brief Заполняет буфер указанным значением.
 * В отличии от memset, для выполнения функция использует системный вызов, что является более быстрым.
 * При некорректном значении параметров возможен крах ПЛК!
 * @param dest Указатель на буфер назначения
 * @param val Значение
 * @param len Размер буфера
 */
void fast_memset(void *dest, int val, size_t len);

/**
 * @brief Копирование области памяти.
 * Копирует len байт из области src в область dest. 
 * В отличии от memmove, для выполнения функция использует системный вызов, что является более быстрым.
 * При некорректном значении параметров возможен крах ПЛК!
 * @param dest Буфер назначения
 * @param src Исходный буфер
 * @param len Количество копируемых байт
 */
void fast_memmove(void *dest, const void *src, size_t len);

__END_DECLS

#endif
