/**
 * @file float.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * @version 0.1
 * @date 2024-03-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef FLOAT_H
#define FLOAT_H

/* Number of decimal digits, q, such that any floating-point number with q
   decimal digits can be rounded into a floating-point number with p radix b
   digits and back again without change to the q decimal digits,

	p * log10(b)			if b is a power of 10
	floor((p - 1) * log10(b))	otherwise
*/
#undef FLT_DIG
#undef DBL_DIG
#undef LDBL_DIG
#define FLT_DIG         __FLT_DIG__
#define DBL_DIG         __DBL_DIG__
#define LDBL_DIG        __LDBL_DIG__

/* Minimum int x such that FLT_RADIX**(x-1) is a normalized float, emin */
#undef FLT_MIN_EXP
#undef DBL_MIN_EXP
#undef LDBL_MIN_EXP
#define FLT_MIN_EXP     __FLT_MIN_EXP__
#define DBL_MIN_EXP     __DBL_MIN_EXP__
#define LDBL_MIN_EXP    __LDBL_MIN_EXP__

/* Minimum negative integer such that 10 raised to that power is in the
   range of normalized floating-point numbers,

	ceil(log10(b) * (emin - 1))
*/
#undef FLT_MIN_10_EXP
#undef DBL_MIN_10_EXP
#undef LDBL_MIN_10_EXP
#define FLT_MIN_10_EXP  __FLT_MIN_10_EXP__
#define DBL_MIN_10_EXP  __DBL_MIN_10_EXP__
#define LDBL_MIN_10_EXP __LDBL_MIN_10_EXP__

/* Maximum int x such that FLT_RADIX**(x-1) is a representable float, emax.  */
#undef FLT_MAX_EXP
#undef DBL_MAX_EXP
#undef LDBL_MAX_EXP
#define FLT_MAX_EXP     __FLT_MAX_EXP__
#define DBL_MAX_EXP     __DBL_MAX_EXP__
#define LDBL_MAX_EXP    __LDBL_MAX_EXP__

/* Maximum integer such that 10 raised to that power is in the range of
   representable finite floating-point numbers,

	floor(log10((1 - b**-p) * b**emax))
*/
#undef FLT_MAX_10_EXP
#undef DBL_MAX_10_EXP
#undef LDBL_MAX_10_EXP
#define FLT_MAX_10_EXP  __FLT_MAX_10_EXP__
#define DBL_MAX_10_EXP  __DBL_MAX_10_EXP__
#define LDBL_MAX_10_EXP __LDBL_MAX_10_EXP__

/* Maximum representable finite floating-point number,

	(1 - b**-p) * b**emax
*/
#undef FLT_MAX
#undef DBL_MAX
#undef LDBL_MAX
#define FLT_MAX         __FLT_MAX__
#define DBL_MAX         __DBL_MAX__
#define LDBL_MAX        __LDBL_MAX__

/* The difference between 1 and the least value greater than 1 that is
   representable in the given floating point type, b**1-p.  */
#undef FLT_EPSILON
#undef DBL_EPSILON
#undef LDBL_EPSILON
#define FLT_EPSILON     __FLT_EPSILON__
#define DBL_EPSILON     __DBL_EPSILON__
#define LDBL_EPSILON    __LDBL_EPSILON__

/* Radix of exponent representation, b. */
#undef FLT_RADIX
#define FLT_RADIX       __FLT_RADIX__

/* Number of base-FLT_RADIX digits in the significand, p.  */
#undef FLT_MANT_DIG
#undef DBL_MANT_DIG
#undef LDBL_MANT_DIG
#define FLT_MANT_DIG    __FLT_MANT_DIG__
#define DBL_MANT_DIG    __DBL_MANT_DIG__
#define LDBL_MANT_DIG   __LDBL_MANT_DIG__

/* Versions of DECIMAL_DIG for each floating-point type.  */
#undef FLT_DECIMAL_DIG
#undef DBL_DECIMAL_DIG
#undef LDBL_DECIMAL_DIG
#define FLT_DECIMAL_DIG     __FLT_DECIMAL_DIG__
#define DBL_DECIMAL_DIG     __DBL_DECIMAL_DIG__
#define LDBL_DECIMAL_DIG    __LDBL_DECIMAL_DIG__

/* Whether types support subnormal numbers.  */
#undef FLT_HAS_SUBNORM
#undef DBL_HAS_SUBNORM
#undef LDBL_HAS_SUBNORM
#define FLT_HAS_SUBNORM     __FLT_HAS_DENORM__
#define DBL_HAS_SUBNORM     __DBL_HAS_DENORM__
#define LDBL_HAS_SUBNORM    __LDBL_HAS_DENORM__

/* Minimum positive values, including subnormals.  */
#undef FLT_TRUE_MIN
#undef DBL_TRUE_MIN
#undef LDBL_TRUE_MIN
#define FLT_TRUE_MIN        __FLT_DENORM_MIN__
#define DBL_TRUE_MIN        __DBL_DENORM_MIN__
#define LDBL_TRUE_MIN       __LDBL_DENORM_MIN__




#if defined __STDC_VERSION__ && __STDC_VERSION__ > 201710L
/* Maximum finite positive value with MANT_DIG digits in the
   significand taking their maximum value.  */
#undef FLT_NORM_MAX
#undef DBL_NORM_MAX
#undef LDBL_NORM_MAX
#define FLT_NORM_MAX        __FLT_NORM_MAX__
#define DBL_NORM_MAX        __DBL_NORM_MAX__
#define LDBL_NORM_MAX       __LDBL_NORM_MAX__

/* Whether each type matches an IEC 60559 format.  */
#undef FLT_IS_IEC_60559
#undef DBL_IS_IEC_60559
#undef LDBL_IS_IEC_60559
#define FLT_IS_IEC_60559    __FLT_IS_IEC_60559__
#define DBL_IS_IEC_60559    __DBL_IS_IEC_60559__
#define LDBL_IS_IEC_60559   __LDBL_IS_IEC_60559__

/* Infinity in type float; not defined if infinity not supported.  */
#if __FLT_HAS_INFINITY__
#undef INFINITY
#define INFINITY            (__builtin_inff ())
#endif

/* Quiet NaN, if supported for float.  */
#if __FLT_HAS_QUIET_NAN__
#undef NAN
#define NAN                 (__builtin_nanf (""))
#endif

/* Signaling NaN, if supported for each type.  All formats supported
   by GCC support either both quiet and signaling NaNs, or neither
   kind of NaN.  */
#if __FLT_HAS_QUIET_NAN__
#undef FLT_SNAN
#define FLT_SNAN            (__builtin_nansf (""))
#endif
#if __DBL_HAS_QUIET_NAN__
#undef DBL_SNAN
#define DBL_SNAN            (__builtin_nans (""))
#endif
#if __LDBL_HAS_QUIET_NAN__
#undef LDBL_SNAN
#define LDBL_SNAN           (__builtin_nansl (""))
#endif
#endif /* C23 */

#ifdef __FLT32_MANT_DIG__
#undef FLT32_MANT_DIG
#define FLT32_MANT_DIG      __FLT32_MANT_DIG__
#undef FLT32_DIG
#define FLT32_DIG           __FLT32_DIG__
#undef FLT32_MIN_EXP
#define FLT32_MIN_EXP       __FLT32_MIN_EXP__
#undef FLT32_MIN_10_EXP
#define FLT32_MIN_10_EXP    __FLT32_MIN_10_EXP__
#undef FLT32_MAX_EXP
#define FLT32_MAX_EXP       __FLT32_MAX_EXP__
#undef FLT32_MAX_10_EXP
#define FLT32_MAX_10_EXP    __FLT32_MAX_10_EXP__
#undef FLT32_MAX
#define FLT32_MAX           __FLT32_MAX__
#undef FLT32_EPSILON
#define FLT32_EPSILON       __FLT32_EPSILON__
#undef FLT32_MIN
#define FLT32_MIN           __FLT32_MIN__
#undef FLT32_DECIMAL_DIG
#define FLT32_DECIMAL_DIG   __FLT32_DECIMAL_DIG__
#undef FLT32_TRUE_MIN
#define FLT32_TRUE_MIN      __FLT32_DENORM_MIN__
#if defined __STDC_VERSION__ && __STDC_VERSION__ > 201710L
#undef FLT32_SNAN
#define FLT32_SNAN          (__builtin_nansf32 (""))
#endif /* C23 */
#endif /* __FLT32_MANT_DIG__.  */

#ifdef __FLT64_MANT_DIG__
#undef FLT64_MANT_DIG
#define FLT64_MANT_DIG      __FLT64_MANT_DIG__
#undef FLT64_DIG
#define FLT64_DIG           __FLT64_DIG__
#undef FLT64_MIN_EXP
#define FLT64_MIN_EXP       __FLT64_MIN_EXP__
#undef FLT64_MIN_10_EXP
#define FLT64_MIN_10_EXP    __FLT64_MIN_10_EXP__
#undef FLT64_MAX_EXP
#define FLT64_MAX_EXP       __FLT64_MAX_EXP__
#undef FLT64_MAX_10_EXP
#define FLT64_MAX_10_EXP    __FLT64_MAX_10_EXP__
#undef FLT64_MAX
#define FLT64_MAX           __FLT64_MAX__
#undef FLT64_EPSILON
#define FLT64_EPSILON       __FLT64_EPSILON__
#undef FLT64_MIN
#define FLT64_MIN           __FLT64_MIN__
#undef FLT64_DECIMAL_DIG
#define FLT64_DECIMAL_DIG   __FLT64_DECIMAL_DIG__
#undef FLT64_TRUE_MIN
#define FLT64_TRUE_MIN      __FLT64_DENORM_MIN__
#if defined __STDC_VERSION__ && __STDC_VERSION__ > 201710L
#undef FLT64_SNAN
#define FLT64_SNAN          (__builtin_nansf64 (""))
#endif /* C23 */
#endif /* __FLT64_MANT_DIG__.  */

#ifdef __FLT128_MANT_DIG__
#undef FLT128_MANT_DIG
#define FLT128_MANT_DIG     __FLT128_MANT_DIG__
#undef FLT128_DIG
#define FLT128_DIG          __FLT128_DIG__
#undef FLT128_MIN_EXP
#define FLT128_MIN_EXP      __FLT128_MIN_EXP__
#undef FLT128_MIN_10_EXP
#define FLT128_MIN_10_EXP   __FLT128_MIN_10_EXP__
#undef FLT128_MAX_EXP
#define FLT128_MAX_EXP      __FLT128_MAX_EXP__
#undef FLT128_MAX_10_EXP
#define FLT128_MAX_10_EXP   __FLT128_MAX_10_EXP__
#undef FLT128_MAX
#define FLT128_MAX          __FLT128_MAX__
#undef FLT128_EPSILON
#define FLT128_EPSILON      __FLT128_EPSILON__
#undef FLT128_MIN
#define FLT128_MIN          __FLT128_MIN__
#undef FLT128_DECIMAL_DIG
#define FLT128_DECIMAL_DIG  __FLT128_DECIMAL_DIG__
#undef FLT128_TRUE_MIN
#define FLT128_TRUE_MIN     __FLT128_DENORM_MIN__
#if defined __STDC_VERSION__ && __STDC_VERSION__ > 201710L
#undef FLT128_SNAN
#define FLT128_SNAN         (__builtin_nansf128 (""))
#endif /* C23 */
#endif /* __FLT128_MANT_DIG__.  */

#endif
