/**
 * @file satmodel.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * @version 0.2
 * @date 2024-04-22
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef SATMODEL_H
#define SATMODEL_H

#define MODEL_UNDEFINED             0
#define MODEL_SATURNPLC             1
#define MODEL_MP_AI7_AO2_DI6_DO4    2

#if __has_include("model.h")
    #include "model.h"
#else
    #define MODEL MODEL_SATURNPLC
#endif

#if MODEL==MODEL_SATURNPLC
    #define HW_SIZE_RAM         458752
    #define HW_SIZE_ROM         1048576
    #define HW_SIZE_EPROM       1024
    #define HW_SIZE_BACKUPRAM   4096
    #define HW_COUNT_DI         10
    #define HW_COUNT_DO         11
    #define HW_COUNT_AI         2
    #define HW_COUNT_AO         2
    #define HW_COUNT_TEMP       5
    #define HW_COUNT_SER        1
    #define HW_COUNT_ETH        1
    #define HW_KEYBOARD         1
    #define HW_SCREEN_WIDTH     320u
    #define HW_SCREEN_HEIGHT    240u
#elif MODEL==MODEL_MP_AI7_AO2_DI6_DO4
    #define HW_SIZE_RAM         458752
    #define HW_SIZE_ROM         1048576
    #define HW_SIZE_EPROM       1024
    #define HW_SIZE_BACKUPRAM   4096
    #define HW_COUNT_DI         6
    #define HW_COUNT_DO         4
    #define HW_COUNT_AI         7
    #define HW_COUNT_AO         2
    #define HW_COUNT_TEMP       0
    #define HW_COUNT_SER        2
    #define HW_COUNT_ETH        1
    #define HW_KEYBOARD         1
    #define HW_SCREEN_WIDTH     320u
    #define HW_SCREEN_HEIGHT    240u
#else
    #error "Unknown PLC model"
#endif

#endif
