/**
 * @file satkbd.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с клавиатурой.
 * @version 0.1
 * @date 2024-03-17
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef SATKBD_H
#define SATKBD_H

#include <cdefs.h>
#include <plcmodel.h>

__BEGIN_DECLS

#if HW_KEYBOARD==1

/**
 * @brief Нажата стрелка вверх
 */
#define KEY_UP           1

/**
 * @brief Нажата стрелка вправо
 */
#define KEY_RIGHT        2

/**
 * @brief Нажата стрелка вниз
 */
#define KEY_DOWN         4

/**
 * @brief Нажата стрелка влево
 */
#define KEY_LEFT         8

/**
 * @brief Включение работы с клавиатурой
 * Функция разрешает или запрещает работу с клавиатурой в программе.
 * @param EnableKeyboard Тип обработчика нажатия клавиш: 0 - использовать системный обработчик, 1 - использовать программный обработчик.
 */
void SetEnableKeyboard(int ek);

/**
 * @brief Получение кода нажатой кнопки клавиатуры.
 * Для использования предварительно должен быть установлен режим програмного обработчика нажатия клавиш.
 * Если пользователь нажал 2 кнопки, то возвращается сумма кодов нажатых кнопок.
 * Если нажатых кнопок нет, то функция возвращает значение 0.
 * @example
 * SetEnableKeyboard(1);
 * if(GetKey()==KEY_UP) ...
 * @return int  Код нажатых кнопок
 */
int GetKey(void);

/**
 * @brief Программная имитация нажатия кнопки клавиатуры.
 * 
 * @param key Код кнопки
 */
void PressKey(int key);

#endif          // #if HW_KEYBOARD==1

__END_DECLS

#endif
