/**
 * @file satmio.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с входами и выходами контроллера.
 * @version 0.1
 * @date 2024-03-14
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef SATMIO_H
#define SATMIO_H

#include <cdefs.h>
#include <plcmodel.h>

#if MODEL==MODEL_SATURNPLC
    #define DI1                 0
    #define DI2                 1
    #define DI3                 2
    #define DI4                 3
    #define DI5                 4
    #define DI6                 5
    #define DI7                 6
    #define DI8                 7
    #define DI9                 8
    #define DI10                9

    #define DO1                 0
    #define DO2                 1
    #define DO3                 2
    #define DO4                 3
    #define DO5                 4
    #define DO6                 5
    #define DO7                 6
    #define DO8                 7
    #define DO9                 8
    #define DO10                9
    #define DO11                10

    #define TEMP1               0
    #define TEMP2               1
    #define TEMP3               2
    #define TEMP4               3
    #define TEMP5               4

    #define AO1                 0
    #define AO2                 1

    #define AI1                 0
    #define AI2                 1
#elif MODEL==MODEL_MP_AI7_AO2_DI6_DO4
    #define DI1                 0
    #define DI2                 1
    #define DI3                 2
    #define DI4                 3
    #define DI5                 4
    #define DI6                 5

    #define DO1                 0
    #define DO2                 1
    #define DO3                 2
    #define DO4                 3

    #define AO1                 0
    #define AO2                 1

    #define AI1                 0
    #define AI2                 1
    #define AI3                 2
    #define AI4                 3
    #define AI5                 4
    #define AI6                 5
    #define AI7                 6
#endif

__BEGIN_DECLS

#if HW_COUNT_DI>0

/**
 * @brief Получение состояния дискретного входа
 * 
 * @param idx Индекс входа: 0..(HW_COUNT_DI-1)
 * @return Состояние входа: 0/1
 */
int GetDI(int idx);

#endif

#if HW_COUNT_DO>0

/**
 * @brief Получение состояния дискретного выхода
 * 
 * @param idx Индекс выхода: 0..(HW_COUNT_DO-1)
 * @return Состояние выхода: 0/1
 */
int GetDO(int idx);

/**
 * @brief Управление дискретным выходом
 * 
 * @param index Индекс выхода: 0..(HW_COUNT_DO-1)
 * @param val Значение: 0-выключить, 1-включить
 */
void SetDO(int index, int val);

#endif

#if HW_COUNT_TEMP>0

/**
 * @brief Получение измеренного значения температуры T
 * 
 * @param index Индекс входа: 0..(HW_COUNT_T-1)
 * @return Значение температуры
 */
float GetT(int index);

#endif

#if HW_COUNT_AI>0

/**
 * @brief Получение состояния аналоговых входов AI
 * 
 * @param index Индекс входа: 0..(HW_COUNT_AI-1)
 * @return Значение напряжения на аналоговом входе
 */
float GetAI(int index);

#endif

#if HW_COUNT_AO>0

/**
 * @brief Получение состояния аналоговых выходов AO
 * 
 * @param index Индекс выхода: 0..(HW_COUNT_AO-1)
 * @return Значение напряжения на аналоговом выходе
 */
float GetAO(int index);

/**
 * @brief Установка значения на аналоговом выходе AO
 * 
 * @param index Индекс выхода: 0..(HW_COUNT_AO-1)
 * @param val Устанавливаемое значение напряжения 0.0..10.0
 */
void SetAO(int index, float val);

#endif

#if HW_COUNT_AI>0 || HW_COUNT_TEMP>0

/**
 * @brief Получение битовой маски ошибок измерения температуры и аналоговых входов
 * 
 * @return Битовая маска
 */
int ADCErrors(void);

#endif

/**
 * @brief Управление звуковым сигналом
 * 
 * @param t Включение/выключение звукового сигнала
 */
void Beep(int t);

__END_DECLS

#endif
