/**
 * @file satparams.h
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Функции установки и получения значений параметров ПЛК.
 * @version 0.1
 * @date 2024-04-12
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#ifndef SATPARAMS_H
#define SATPARAMS_H

#include <cdefs.h>
#include <stdint.h>

__BEGIN_DECLS

#define PARAM_ID_IPADDR             0   ///< IP-адрес контроллера (структура in_addr)
#define PARAM_ID_NETMASK            1   ///< Маска подсети (структура in_addr)
#define PARAM_ID_IPGATE             2   ///< IP-адрес шлюза (структура in_addr)
#define PARAM_ID_SERIAL             3   ///< Серийный номер контроллера (unsigned int)
#define PARAM_ID_INFO               4   ///< Получение информации о контроллере (структура plc_info)
#define PARAM_ID_NVSERIAL           5   ///< Серийный номер контроллера в формате сетевых переменных (unsigned int)

#define PARAM_ID_HWRANDSTATE        200 ///< Статус генератора случайных чисел (unsigned int)

struct plc_info {                                        
    uint8_t     version_Hi;             ///< Версия ПО - Hi
    uint8_t     version_Lo;             ///< Версия ПО - Lo
    uint8_t     year;                   ///< Год
    uint8_t     month;                  ///< Месяц
    uint16_t    number;                 ///< Серийный номер
    uint8_t     groupAddress;           ///< Групповой адрес контроллера
    uint8_t     address;                ///< Адрес контроллера
    uint8_t     name[16];               ///< Название модуля
    uint8_t     comment[48];            ///< Комментарий пользователя
} __attribute__((packed));

typedef struct plc_info PLC_INFO;

/**
 * @brief Установка значения параметра ПЛК.
 * Функция возвращает 1 при успешном выполнении или 0 при ошибке.
 * @param id Идентификатор параметра.
 * @param value Указатель на значение параметра.
 * @return int 
 */
int SetPLCParameter(int id, void *value);

/**
 * @brief Получение значения параметра ПЛК.
 * Функция возвращает 1 при успешном выполнении или 0 при ошибке.
 * При успешном выполнении функция копирует в буфер value значение запрошенного параметра.
 * @param id Идентификатор параметра.
 * @param value Указатель на буфер в который будет помещено значение параметра.
 * @return int 
 */
int GetPLCParameter(int id, void *value);

__END_DECLS

#endif
