/**
 * @file cjson_crt.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с JSON.
 * @version 0.1
 * @date 2024-05-16
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 * Библиотека разработана на основе оригинальных исходных текстов библиотеки cJSON:
 * @copyright Copyright (c) 2009-2017 Dave Gamble and cJSON contributors
 * @copyright Copyright (c) 2015-2016 The Khronos Group Inc.
 * @copyright Copyright (c) 2015-2016 Valve Corporation
 * @copyright Copyright (c) 2015-2016 LunarG, Inc.
 * @license MIT
 */

#include <string.h>

#include <cjson.h>

/**
 * @brief Создание нового элемента JSON с типом cJSON_NULL.
 * @return cJSON* 
 */
cJSON *cJSON_CreateNull(void) {
    cJSON *item = cJSON_New_Item();
    if (item) item->type = cJSON_NULL;
    return item;
}

/**
 * @brief Создание нового элемента JSON с типом cJSON_True или cJSON_False.
 * @param b Значение элемента: 1-true,0-false
 * @return cJSON* 
 */
cJSON *cJSON_CreateBool(int b) {
    cJSON *item = cJSON_New_Item();
    if (item) item->type = b ? cJSON_True : cJSON_False;
    return item;
}

/**
 * @brief Создание нового элемента JSON с числовым типом (cJSON_Number).
 * @param num Значение элемента
 * @return cJSON* 
 */
cJSON *cJSON_CreateNumber(double num) {
    cJSON *item = cJSON_New_Item();
    if (item) {
        item->type = cJSON_Number;
        item->valuedouble = num;
        item->valueint = (int)num;
    }
    return item;
}

/**
 * @brief Создание нового элемента JSON со строковым типом (cJSON_String).
 * @param string Указатель на строку
 * @return cJSON* 
 */
cJSON *cJSON_CreateString(const char *string) {
    cJSON *item = cJSON_New_Item();
    if (item) {
        item->type = cJSON_String;
        item->valuestring = strdup(string);
    }
    return item;
}

/**
 * @brief Создание нового элемента JSON с типом объект (cJSON_Object).
 * @return cJSON* 
 */
cJSON *cJSON_CreateArray(void) {
    cJSON *item = cJSON_New_Item();
    if (item) item->type = cJSON_Array;
    return item;
}

/**
 * @brief Создание нового элемента JSON с типом объект (cJSON_Object).
 * @return cJSON* 
 */
cJSON *cJSON_CreateObject(void) {
    cJSON *item = cJSON_New_Item();
    if (item) item->type = cJSON_Object;
    return item;
}
