/**
 * @file cjson_min.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с JSON.
 * @version 0.1
 * @date 2024-05-16
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 * Библиотека разработана на основе оригинальных исходных текстов библиотеки cJSON:
 * @copyright Copyright (c) 2009-2017 Dave Gamble and cJSON contributors
 * @copyright Copyright (c) 2015-2016 The Khronos Group Inc.
 * @copyright Copyright (c) 2015-2016 Valve Corporation
 * @copyright Copyright (c) 2015-2016 LunarG, Inc.
 * @license MIT
 */

#include <cjson.h>

/**
 * @brief Минификация строки JSON.
 * Функция удаляет ненужные символы: пробелы, табуляцию, коментарии, CR/LF и тп.
 * Результат помещается на место строки, указанной в параметре, в конце новой строки добаляется символ \0.
 * Функция не выполняет выделение или освобождение памяти.
 * @param json Указатель на строку с текстом JSON.
 */
void cJSON_Minify(char *json) {
    char *into = json;
    while (*json) {
        if (*json == ' ')
            json++;
        else if (*json == '\t')
            json++;
        else if (*json == '\r')
            json++;
        else if (*json == '\n')
            json++;
        else if (*json == '/' && json[1] == '/')
            while (*json && *json != '\n') json++;
        else if (*json == '/' && json[1] == '*') {
            while (*json && !(*json == '*' && json[1] == '/')) json++;
            json += 2;
        }
        else if (*json == '\"') {
            *into++ = *json++;
            while (*json && *json != '\"') {
                if (*json == '\\') *into++ = *json++;
                *into++ = *json++;
            }
            *into++ = *json++;
        }
        else
            *into++ = *json++;
    }
    *into = 0;
}
