/**
 * @file arcfour.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Генератор ПСЧ.
 * @version 0.1
 * @date 2024-05-01
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stdint.h>
#include <stddef.h>
#include <arcfour.h>

/**
 * @brief Инициализация ПСЧ
 * @param state Текущее состояние ПСЧ (массив 256 байт)
 * @param key Ключ инициализации
 * @param len Длина ключа (1..256)
 */
void arcfour_key_setup(uint8_t state[256], const uint8_t key[], int len) {
    int i, j;
    uint8_t t;

    for (i = 0; i < 256; ++i) state[i] = i;
    for (i = 0, j = 0; i < 256; ++i) {
        j = (j + state[i] + key[i % len]) % 256;
        t = state[i];
        state[i] = state[j];
        state[j] = t;
    }
}

/**
 * @brief Генератор ПСЧ последовательности
 * @param state Текущее состояние ПСЧ
 * @param out Буфер для ПСЧ (размером len)
 * @param len Длина буфера out
 */
void arcfour_generate_stream(uint8_t state[256], uint8_t out[], size_t len) {
    int i, j;
    size_t idx;
    uint8_t t;

    for (idx = 0, i = 0, j = 0; idx < len; ++idx)  {
        i = (i + 1) % 256;
        j = (j + state[i]) % 256;
        t = state[i];
        state[i] = state[j];
        state[j] = t;
        out[idx] = state[(state[i] + state[j]) % 256];
    }
}
