/**
 * @file httpsrv.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции сервера работы по протоколу HTTP.
 * @version 0.1
 * @date 2024-05-19
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <syscallp.h>
#include <string.h>
#include <stdlib.h>

#include <http.h>

static char *httpsrv_data = NULL;

/**
 * @brief Возвращает количество запросов к HTTP серверу.
 * @return unsigned int Количество запросов.
 */
inline unsigned int HttpSrvDataCount(void) {
    return __internal_syscall_invoke1(SYSCALL_HttpSrvDataCount, 0);
}

/**
 * @brief Функция устанавливает пользовательские данные, которые будет возвращаены ПЛК в ответ на HTTP-запрос по URL "/data".
 * @param data Указатель на строку ответа.
 * @param type Тип данных: элемент перечисления httpsrv_content_type.
 */
inline void StartHttpSrvData(char *data, http_content_type_t type) {
    if(!data) return;
    //
    char *newdata = strdup(data);
    __internal_syscall_invoke2(SYSCALL_StartHttpSrvData, (long)newdata, type);
    //
    if(httpsrv_data) free(httpsrv_data);
    httpsrv_data = newdata;
}

/**
 * @brief Функция отменяет использование данных пользователя, установленных вызовом функции StartHttpSrvData.
 */
inline void StopHttpSrvData(void) {
    __internal_syscall_invoke1(SYSCALL_StopHttpSrvData, 0);
    //
    if(httpsrv_data) {
        free(httpsrv_data);
        httpsrv_data = NULL;
    }
}
