/**
 * @file mathd.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Тригонометрические функции двойной точности.
 * @version 0.1
 * @date 2024-04-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <syscallp.h>
#include <math.h>

/**
 * @brief Вычисление значения синуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double sin(double x) {
    return __internal_dfp_invoke1(SYSCALL_sin, x);
}

/**
 * @brief Вычисление значения косинуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double cos(double x) {
    return __internal_dfp_invoke1(SYSCALL_cos, x);
}

/**
 * @brief Вычисление значения тангенса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double tan(double x) {
    return __internal_dfp_invoke1(SYSCALL_tan, x);
}

/**
 * @brief Вычисление значения арксинуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double asin(double x) {
    return __internal_dfp_invoke1(SYSCALL_asin, x);
}

/**
 * @brief Вычисление значения арккосинуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double acos(double x) {
    return __internal_dfp_invoke1(SYSCALL_acos, x);
}

/**
 * @brief Вычисление значения арктангенса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double atan(double x) {
    return __internal_dfp_invoke1(SYSCALL_atan, x);
}

/**
 * @brief Вычисление значения гиперболического синуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double sinh(double x) {
    return __internal_dfp_invoke1(SYSCALL_sinh, x);
}

/**
 * @brief Вычисление значения гиперболического косинуса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double cosh(double x){
    return __internal_dfp_invoke1(SYSCALL_cosh, x);
}

/**
 * @brief Вычисление значения гиперболического тангенса x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double tanh(double x) {
    return __internal_dfp_invoke1(SYSCALL_tanh, x);
}

/**
 * @brief Вычисление значения экспоненты x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double exp(double x) {
    return __internal_dfp_invoke1(SYSCALL_exp, x);
}

/**
 * @brief Вычисление значения степени по основанию 2.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double exp2(double x) {
    return __internal_dfp_invoke1(SYSCALL_exp2, x);
}

/**
 * @brief Вычисление значения функции "exp (x) - 1"
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double expm1(double x) {
    return __internal_dfp_invoke1(SYSCALL_expm1, x);
}

/**
 * @brief Вычисление значения натурального логарифма x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double log(double x) {
    return __internal_dfp_invoke1(SYSCALL_log, x);
}

/**
 * @brief Вычисление значения десятичного логарифма x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double log10(double x) {
    return __internal_dfp_invoke1(SYSCALL_log10, x);
}

/**
 * @brief Вычисление значения эквивалентное "log (1 + x)".
 * Вычисляется методом, выдающим точные значения, даже если значение x близко к нулю.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double log1p(double x) {
    return __internal_dfp_invoke1(SYSCALL_log1p, x);
}

/**
 * @brief Вычисление значения логарифма по основанию 2 от аргумента x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double log2(double x) {
    return __internal_dfp_invoke1(SYSCALL_log2, x);
}

/**
 * @brief Вычисление абсолютного значения x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double fabs(double x) {
    return __internal_dfp_invoke1(SYSCALL_fabs, x);
}

/**
 * @brief Вычисление квадратного корня x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double sqrt(double x) {
    return __internal_dfp_invoke1(SYSCALL_sqrt, x);
}

/**
 * @brief Округление до наименьшего целого, не меньшего, чем аргумент x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double ceil(double x) {
    return __internal_dfp_invoke1(SYSCALL_ceil, x);
}

/**
 * @brief Округление до наибольшего целого значения, но не большее x.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double floor(double x) {
    return __internal_dfp_invoke1(SYSCALL_floor, x);
}

/**
 * @brief Округление до ближайшего целого в направлении от нуля.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double round(double x) {
    return __internal_dfp_invoke1(SYSCALL_round, x);
}

/**
 * @brief Округление до ближайшего целого числа в направлении к нулю.
 * Аргумент и результат число с плавающей точкой двойной точности.
 * @param x 
 * @return double 
 */
double trunc(double x) {
    return __internal_dfp_invoke1(SYSCALL_trunc, x);
}

/**
 * @brief Функция возвращает значение x в степени y.
 * Аргументы и результат число с плавающей точкой двойной точности.
 * @param x 
 * @param y 
 * @return double 
 */
double pow(double x, double y) {
    return __internal_dfp_invoke2(SYSCALL_pow, x, y);
}

/**
 * @brief Функция вычисления остатка от деления (с плавающей точкой).
 * Возращаемое значение: x - n * y - где n есть частное x / y, округленное к нулю до ближайшего целого числа.
 * Аргументы и результат число с плавающей точкой двойной точности.
 * @param x 
 * @param y 
 * @return double 
 */
double fmod(double x, double y) {
    return __internal_dfp_invoke2(SYSCALL_fmod, x, y);
}

/**
 * @brief Computes the IEEE remainder of the floating point division operation x/y.
 * Аргументы и результат число с плавающей точкой двойной точности.
 * @param x 
 * @param y 
 * @return double 
 */
double remainder(double x, double y) {
    return __internal_dfp_invoke2(SYSCALL_remainder, x, y);
}
