/**
 * @file mathf.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Тригонометрические функции одинарной точности.
 * @version 0.1
 * @date 2024-04-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <syscallp.h>
#include <math.h>

/**
 * @brief Вычисление значения синуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float
 */
float sinf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_sinf, x);
}

/**
 * @brief Вычисление значения косинуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float
 */
float cosf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_cosf, x);
}

/**
 * @brief Вычисление значения тангенса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float tanf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_tanf, x);
}

/**
 * @brief Вычисление значения арксинуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float asinf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_asinf,  x);
}

/**
 * @brief Вычисление значения арккосинуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float acosf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_acosf,  x);
}

/**
 * @brief Вычисление значения арктангенса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float atanf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_atanf,  x);
}

/**
 * @brief Вычисление значения гиперболического синуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float sinhf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_sinhf,  x);
}

/**
 * @brief Вычисление значения гиперболического косинуса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float coshf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_coshf,  x);
}

/**
 * @brief Вычисление значения гиперболического тангенса x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float tanhf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_tanhf,  x);
}

/**
 * @brief Вычисление значения экспоненты x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float expf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_expf,   x);
}

/**
 * @brief Вычисление значения степени по основанию 2.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float exp2f(float x) {
    return __internal_syscallf_invoke1(SYSCALL_exp2f,  x);
}

/**
 * @brief Вычисление значения функции "exp (x) - 1"
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float expm1f(float x) {
    return __internal_syscallf_invoke1(SYSCALL_expm1f, x);
}

/**
 * @brief Вычисление значения натурального логарифма x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float logf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_logf,   x);
}

/**
 * @brief Вычисление значения десятичного логарифма x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float log10f(float x) {
    return __internal_syscallf_invoke1(SYSCALL_log10f, x);
}

/**
 * @brief Вычисление значения эквивалентное "log (1 + x)".
 * Вычисляется методом, выдающим точные значения, даже если значение x близко к нулю.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float log1pf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_log1pf, x);
}

/**
 * @brief Вычисление значения логарифма по основанию 2 от аргумента x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float log2f(float x) {
    return __internal_syscallf_invoke1(SYSCALL_log2f,  x);
}

/**
 * @brief Вычисление абсолютного значения x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float fabsf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_fabsf,  x);
}

/**
 * @brief Вычисление квадратного корня x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float sqrtf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_sqrtf,  x);
}

/**
 * @brief Округление до наименьшего целого, не меньшего, чем аргумент x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float ceilf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_ceilf,  x);
}

/**
 * @brief Округление до наибольшего целого значения, но не большее x.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float floorf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_floorf, x);
}

/**
 * @brief Округление до ближайшего целого в направлении от нуля.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float roundf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_roundf, x);
}

/**
 * @brief Округление до ближайшего целого числа в направлении к нулю.
 * Аргумент и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @return float 
 */
float truncf(float x) {
    return __internal_syscallf_invoke1(SYSCALL_truncf, x);
}

/**
 * @brief Функция возвращает значение x в степени y.
 * Аргументы и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @param y 
 * @return float 
 */
float powf(float x, float y) {
    return __internal_syscallf_invoke2(SYSCALL_powf, x, y);
}

/**
 * @brief Функция вычисления остатка от деления (с плавающей точкой).
 * Возращаемое значение: x - n * y - где n есть частное x / y, округленное к нулю до ближайшего целого числа.
 * Аргументы и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @param y 
 * @return float 
 */
float fmodf(float x, float y) {
    return __internal_syscallf_invoke2(SYSCALL_fmodf, x, y);
}

/**
 * @brief Computes the IEEE remainder of the floating point division operation x/y.
 * Аргументы и результат число с плавающей точкой одинарной точности.
 * @param x 
 * @param y 
 * @return float 
 */
float remainderf(float x, float y) {
    return __internal_syscallf_invoke2(SYSCALL_remainderf, x, y);
}
