/**
 * @file fastmem.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Быстрые (и опасные) варианты функций работы с памятью.
 * @version 0.1
 * @date 2024-04-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <syscallp.h>
#include <fastmem.h>

/**
 * @brief Заполняет буфер указанным значением.
 * 
 * @param dest Указатель на буфер назначения
 * @param val Значение
 * @param len Размер буфера
 */
void fast_memset(void *dest, int val, size_t len) {
    __internal_syscall(SYSCALL_memset,  (long)(dest), val, len, 0, 0, 0);
}

/**
 * @brief Копирование области памяти.
 * Копирует len байт из области src в область dest. 
 * @param dest Буфер назначения
 * @param src Исходный буфер
 * @param len Количество копируемых байт
 */
void fast_memmove(void *dest, const void *src, size_t len) {
    __internal_syscall(SYSCALL_memmove, (long)(dest), (long)(src), len, 0, 0, 0);
}
