/**
 * @file satkbd.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с клавиатурой.
 * @version 0.1
 * @date 2024-03-17
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <syscallp.h>
#include <satkbd.h>

/**
 * @brief Включение работы с клавиатурой
 * Функция разрешает или запрещает работу с клавиатурой в программе.
 * @param EnableKeyboard Тип обработчика нажатия клавиш: 0 - использовать системный обработчик, 1 - использовать программный обработчик.
 */
void SetEnableKeyboard(int ek) {
    __internal_syscall_invoke1(SYSCALL_EnableKeyboard, ek);
}

/**
 * @brief Получение кода нажатой кнопки клавиатуры.
 * Для использования предварительно должен быть установлен режим програмного обработчика нажатия клавиш.
 * Если пользователь нажал 2 кнопки, то возвращается сумма кодов нажатых кнопок.
 * Если нажатых кнопок нет, то функция возвращает значение 0.
 * @example
 * SetEnableKeyboard(1);
 * if(GetKey()==KEY_UP) ...
 * @return int  Код нажатых кнопок
 */
int GetKey(void) {
    return __internal_syscall_invoke1(SYSCALL_GetKey, 0);
}

/**
 * @brief Программная имитация нажатия кнопки клавиатуры.
 * 
 * @param key Код кнопки
 */
void PressKey(int key) {
    __internal_syscall_invoke1(SYSCALL_PressKey, key);
}
