/**
 * @file satmio.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с входами и выходами контроллера.
 * @version 0.1
 * @date 2024-03-14
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <syscallp.h>
#include <satmio.h>


// float __internal_syscall_wrapper_fi(long n, long p1) {
//     FP_UNION_S_T t;
//     t.s32 = __internal_syscall_invoke1(n, p1);
//     return t.f;
// }

// void __internal_syscall_wrapper_vif(long n, long p1, float f1) {
//     FP_UNION_S_T t;
//     t.f = f1;
//     __internal_syscall(n, p1, t.s32, 0, 0, 0, 0);
// }

float   __internal_syscall_wrapper_fi(long n, long p1);

void    __internal_syscall_wrapper_vif(long n, long p1, float f1);

/**
 * @brief Получение состояния дискретного входа
 * 
 * @param idx Индекс входа: 0..9
 * @return Состояние входа: 0/1
 */
int GetDI(int idx) {
    return __internal_syscall_invoke1(SYSCALL_GetDI, idx);
}

/**
 * @brief Получение состояния дискретного выхода
 * 
 * @param idx Индекс выхода: 0..10
 * @return Состояние выхода: 0/1
 */
int GetDO(int idx) {
    return __internal_syscall_invoke1(SYSCALL_GetDO, idx);
}

/**
 * @brief Управление дискретным выходом
 * 
 * @param index Индекс выхода: 0..10
 * @param val Значение: 0-выключить, 1-включить
 */
void SetDO(int index, int val) {
    __internal_syscall_invoke2(SYSCALL_SetDO, index, val);
}

/**
 * @brief Получение измеренного значения температуры T
 * 
 * @param index Индекс входа: 0..4
 * @return Значение температуры
 */
float GetT(int index) {
    return __internal_syscall_wrapper_fi(SYSCALL_GetT, index);
}

/**
 * @brief Получение состояния аналоговых входов AI
 * 
 * @param index Индекс входа: 0..1
 * @return Значение напряжения на аналоговом входе
 */
float GetAI(int index) {
    return __internal_syscall_wrapper_fi(SYSCALL_GetAI, index);
}

/**
 * @brief Получение состояния аналоговых выходов AO
 * 
 * @param index Индекс выхода: 0..1
 * @return Значение напряжения на аналоговом выходе
 */
float GetAO(int index) {
    return __internal_syscall_wrapper_fi(SYSCALL_GetAO, index);
}

/**
 * @brief Установка значения на аналоговом выходе AO
 * 
 * @param index Индекс выхода: 0..1
 * @param val Устанавливаемое значение напряжения 0.0..10.0
 */
void SetAO(int index, float val) {
    __internal_syscall_wrapper_vif(SYSCALL_SetAO, index, val);
}

/**
 * @brief Получение битовой маски ошибок измерения температуры и аналоговых входов
 * 
 * @return Битовая маска
 */
int ADCErrors(void) {
    return __internal_syscall_invoke1(SYSCALL_GetADCErrors, 0);
}

/**
 * @brief Управление звуковым сигналом
 * 
 * @param t Включение/выключение звукового сигнала
 */
void Beep(int t) {
    __internal_syscall_invoke1(SYSCALL_Beep, t);
}
