/**
 * @file satparams.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * Функции установки и получения значений параметров ПЛК.
 * @version 0.1
 * @date 2024-04-12
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <syscallp.h>

/**
 * @brief Установка значения параметра ПЛК.
 * Функция возвращает 1 при успешном выполнении или 0 при ошибке.
 * @param id Идентификатор параметра.
 * @param value Указатель на значение параметра.
 * @return int 
 */
int SetPLCParameter(int id, void *value) {
    return __internal_syscall_invoke2(SYSCALL_SetPLCParameter, id, (long)value);
}

/**
 * @brief Получение значения параметра ПЛК.
 * Функция возвращает 1 при успешном выполнении или 0 при ошибке.
 * При успешном выполнении функция копирует в буфер value значение запрошенного параметра.
 * @param id Идентификатор параметра.
 * @param value Указатель на буфер в который будет помещено значение параметра.
 * @return int 
 */
int GetPLCParameter(int id, void *value) {
    return __internal_syscall_invoke2(SYSCALL_GetPLCParameter, id, (long)value);
}
