/*
Распределение областей памяти:

 0x00000000 - 0x3FFFFFFF  - память программ (.text + .rodata + .sdata)
 0x40000000 - 0x7FFFFFFF  - RAM (.bss + .data + .heap + .stack)
 0x80000000 - 0xBFFFFFFF  - резерв 1
 0xC0000000 - 0xFFFFFFFF  - резерв 2

0x00000000                                           0x3FFFFFFF
+-------------------------------------------------------------+
|                              ROM                            |
+--------+----------+-------+---------------------------------+
|  text  |  rodata  | sdata |               ...               |
+--------+----------+-------+---------------------------------+
| <------- rom_size ------> |

0x40000000                                           0x7FFFFFFF
+-------------------------------------------------------------+
|                           RAM                               |
+---------+---------+-------------------------------+---------+
|   bss   |  data   | heap ->              <- stack |   ...   |
+---------+---------+-------------------------------+---------+
| <------------------ ram_size -------------------> |
*/

MEMORY {
  PLC_ROM (rx)  : ORIGIN = 0x00000000, LENGTH = 0x40000000
  PLC_RAM (rw)  : ORIGIN = 0x40000000, LENGTH = 0x40000000
}

ENTRY(_start)

SECTIONS 
{
  /* text: код в FLASH */
  .text : {
    crt0.o(.text)
    /* *(EXCLUDE_FILE(crt0.o) .text) */
    *(.text)
    *(.text*)
    *(.rodata)
    *(.rodata*)

    *(.srodata.cst16)
    *(.srodata.cst8)
    *(.srodata.cst4)
    *(.srodata.cst2)
    *(.srodata .srodata.*)

  } > PLC_ROM

  /* bss: неинициализированные переменные в RAM */
  .bss : {
    *(.bss)
    *(.bss*)
  } > PLC_RAM

  /* sdata: инициализированные переменные в FLASH */
  /* data:  инициализированные переменные в RAM */
  .data : AT ( ADDR(.text) + SIZEOF(.text)) {
    _data_start = .;
    *(.sdata)
    *(.sdata*)

    _heap_start = ALIGN(., 4);
  } > PLC_RAM

  _bss_start    = ADDR(.bss);
  _bss_size     = SIZEOF(.bss);
  _sdata_start  = LOADADDR(.data);
  _sdata_size   = SIZEOF(.data);

}
