/**
 * @file sscanf.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Простая реализация функции sscanf(): поддерживается только тип %d.
 * @version 0.1
 * @date 2024-04-03.
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <ctype.h>
#include <stdarg.h>

int sscanf(const char *str, const char *format, ...) {
    const char *start = str;
    va_list args;
    
    va_start(args, format);
    for ( ; *format != '\0'; format++) {
        if (*format == '%' && format[1] == 'd') {
            int positive;
            int value;
            int *valp;
            
            if (*str == '-') {
                positive = 0;
                str++;
            } else {
                positive = 1;
            }
            if (!isdigit(*str)) break;
            value = 0;
            do {
                value = (value * 10) - (*str - '0');
                str++;
            } while (isdigit(*str));
            if (positive) value = -value;
            valp = va_arg(args, int *);
            *valp = value;
            format++;
        } else if (*format == *str) {
            str++;
        } else break;
    }
    va_end(args);
    return str - start;
}
