/**
 * @file stdio.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * @version 0.1
 * @date 2024-03-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <ctype.h>
#include <syscallp.h>
#include <satscr.h>

// ---------------------------------------------------------------------------------------------------------

typedef enum {
    wait_esc        = 0,
    wait_bracket    = 1,
    wait_m          = 2
} esc_state_t;

int putchar(int c) {
    static esc_state_t esc_state = wait_esc;
    static int esc_num;
    //
    switch (esc_state) {
        case wait_esc:
            if(c == '\e') {
                // начало ESC-последовательности
                esc_state = wait_bracket;
                return 0;
            }
            break;
        case wait_bracket:
            if(c == '[') {
                esc_state = wait_m;
                esc_num = 0;
                return 0;
            } else {
                // неправильная ESC-последовательность
                esc_state = wait_esc;
                return 0;
            }
        case wait_m:
            if(c == 'm') {
                // установка цвета символов
                esc_state = wait_esc;
                //
                switch (esc_num) {
                    case 0:   FontColor = 0xFFFF; BrushColor = 0; return 0;
                    case 2:   ClrScr();                       return 0;
                    //
                    case 30:  FontColor  = RGB(0,0,0);        return 0;
                    case 31:  FontColor  = RGB(170,0,0);      return 0;
                    case 32:  FontColor  = RGB(0,170,0);      return 0;
                    case 33:  FontColor  = RGB(170,85,0);     return 0;
                    case 34:  FontColor  = RGB(0,0,170);      return 0;
                    case 35:  FontColor  = RGB(170,0,170);    return 0;
                    case 36:  FontColor  = RGB(0,170,170);    return 0;
                    case 37:  FontColor  = RGB(170,170,170);  return 0;
                    //
                    case 40:  BrushColor = RGB(0,0,0);        return 0;
                    case 41:  BrushColor = RGB(170,0,0);      return 0;
                    case 42:  BrushColor = RGB(0,170,0);      return 0;
                    case 43:  BrushColor = RGB(170,85,0);     return 0;
                    case 44:  BrushColor = RGB(0,0,170);      return 0;
                    case 45:  BrushColor = RGB(170,0,170);    return 0;
                    case 46:  BrushColor = RGB(0,170,170);    return 0;
                    case 47:  BrushColor = RGB(170,170,170);  return 0;
                    //
                    case 90:  FontColor  = RGB(85,85,85);     return 0;
                    case 91:  FontColor  = RGB(255,85,85);    return 0;
                    case 92:  FontColor  = RGB(85,255,85);    return 0;
                    case 93:  FontColor  = RGB(255,255,85);   return 0;
                    case 94:  FontColor  = RGB(85,85,255);    return 0;
                    case 95:  FontColor  = RGB(255,85,255);   return 0;
                    case 96:  FontColor  = RGB(85,255,255);   return 0;
                    case 97:  FontColor  = RGB(255,255,255);  return 0;
                    //
                    case 100: BrushColor = RGB(85,85,85);     return 0;
                    case 101: BrushColor = RGB(255,85,85);    return 0;
                    case 102: BrushColor = RGB(85,255,85);    return 0;
                    case 103: BrushColor = RGB(255,255,85);   return 0;
                    case 104: BrushColor = RGB(85,85,255);    return 0;
                    case 105: BrushColor = RGB(255,85,255);   return 0;
                    case 106: BrushColor = RGB(85,255,255);   return 0;
                    case 107: BrushColor = RGB(255,255,255);  return 0;
                }
                return 0;
            } else {
                if(isdigit(c)) {
                    esc_num = esc_num * 10 + c - '0';
                    return 0;
                } else {
                    // неправильная ESC-последовательность
                    esc_state = wait_esc;
                    return 0;
                }
            }
    }
    //
    return __internal_syscall_invoke1(SYSCALL_PUTCHAR, c);
}

inline void charset(int char_set) {
    __internal_syscall_invoke1(SYSCALL_SETCHARSET, char_set);
}

int puts(const char *s) {
    int i=0;
    while(s[i]) putchar(s[i++]);
    return 0;
}
