/**
 * @file bsearch.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * @version 0.1
 * @date 2024-04-19
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <stdlib.h>

/**
 * @brief Двоичный поиск в отсортированном массиве 
 * Функция выполняет поиск в массиве из nmemb объектов (на начальный элемент его, размером size, указывает аргумент base) элемент, совпадающий с объектом, обозначенным key.
 * Содержимое массива должно быть расположено по возрастанию в соответствии с функцией compar, производящей сравнение.
 * Функция compar имеет два параметра (в первый передается указатель на объект key, во второй - элемент массива) и возвращает целое число больше нуля,
 * если key больше элемента массива; меньше нуля, если меньше, и ноль, если они равны.
 * @param key Искомый элемент
 * @param base0 Указатель на начальный элемент массива
 * @param nmemb Количество элементов в массиве
 * @param size Размер элемента массива
 * @param compar Функция сравнения двух элементов
 * @return void* 
 */
void *bsearch (const void *key, const void *base0, size_t nmemb, size_t size, int (*compar)(const void *, const void *)) {
    const char *base = (const char *) base0;
    int lim, cmp;
    const void *p;
    //
    for (lim = nmemb; lim != 0; lim >>= 1) {
        p = base + (lim >> 1) * size;
        cmp = (*compar)(key, p);
        if (cmp == 0) return (void *)p;
        if (cmp > 0) {
            base = (const char *)p + size;
            lim--;
        }
    }
    return NULL;
}
