/**
 * @file ctype.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Объявления функций для классификации и преобразования отдельных символов.
 * @version 0.1
 * @date 2024-03-09
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <ctype.h>

/**
 * @brief Возвращает ненулевое значение, если её аргумент является десятичной цифрой, в противном случае возвращается нуль.
 * 
 * @param c Символ
 * @return int 
 */
inline int isdigit(int c) {
    return (c >= '0' && c <= '9')?1:0;
}

/**
 * @brief Возвращает ненулевое значение, если её аргумент является ASCII буквой, в противном случае возвращается нуль.
 * 
 * @param c Символ
 * @return int 
 */
inline int isalpha(int c) {
    return (((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z')))?1:0;
}

/**
 * @brief Символ является заглавным символом ASCII.
 * 
 * @param c 
 * @return int 
 */
inline int isupper(int c) {
    return ((c >= 'A') && (c <= 'Z'))?1:0;
}

/**
 * @brief Символ является пробелом, табуляцией или CR/LF.
 * 
 * @param c 
 * @return int 
 */
inline int isspace(int c) {
    return ((c == ' ') || (c == '\n') || (c == '\r') || (c == '\t'))?1:0;
}

/**
 * @brief Переводит буквы нижнего регистра в верхний регистр (только ASCII)
 * 
 * @param c 
 * @return int 
 */
inline int toupper(int c) {
    if(isalpha(c)) return c & 0x5f;
    return c;
}

/**
 * @brief Переводит буквы верхнего регистра в нижний регистр (только ASCII)
 * 
 * @param c 
 * @return int 
 */
inline int tolower(int c) {
    if(isalpha(c)) return c | 32;
    return c;
}
