/**
 * @file rand.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC"
 * @version 0.1
 * @date 2024-04-19
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include <stdlib.h>

/**
 * @brief Состояние генератора ПСЧ
 */
static unsigned long rand_next = 1;

/**
 * @brief Генератор целых псевдослучайных чисел в диапазоне 0..RAND_MAX
 * 
 * @return int 
 */
int rand() {
#ifdef  USE_WEAK_SEEDING
    // класическая (историческая) реализация генератора
    return ((rand_next = rand_next * 1103515245 + 12345) % ((unsigned long)RAND_MAX + 1));
#else
    // улучшенный алгоритм
    long hi, lo, x;
    hi = rand_next / 127773;
    lo = rand_next % 127773;
    x = 16807 * lo - 2836 * hi;
    if (x < 0) x += 0x7fffffff;
    rand_next = x;
    //
    return (x - 1);
#endif
}

/**
 * @brief Инициализация генератора псевдослучайных чисел.
 * По умолчанию генератор ПСЧ использует начальное значение 1.
 * @param seed 
 */
void srand(unsigned int seed) {
    rand_next = seed;
}
