/**
 * @file string.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками и памятью.
 * @version 0.1
 * @date 2024-04-18
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include "string.h"

#define USE_ASM_MEMMOVE

#ifdef USE_ASM_MEMMOVE
void *__memmove (void *dest, const void *src, size_t len);

void *memmove (void *dest, const void *src, size_t len) {
    return __memmove(dest, src, len);
}
#else
void *memmove (void *dest, const void *src, size_t len) {
    if (dest < src) {
        const char *firsts = (const char *) src;
        char *firstd = (char *) dest;
        while (len--) *firstd++ = *firsts++;
    } else {
        const char *lasts = (const char *)src + (len-1);
        char *lastd = (char *)dest + (len-1);
        while (len--) *lastd-- = *lasts--;
    }
    return dest;
}
#endif

void *memcpy(void *dest, const void *src, size_t len) __attribute__((alias("memmove")));
