/**
 * @file string.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками и памятью.
 * @version 0.1
 * @date 2024-04-18
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include "string.h"

/**
 * @brief Реентерабельная версия strtok().
 * @param s Указатель на строку, которую нужно разбить на лексемы.
 * @param delim Указатель на строку, содержащую разделители.
 * @param save_ptr Учёта контекста между последующими вызовами при анализе одной и той же строки.
 * @return char* 
 */
char *strtok_r(char *s, const char *delim, char **save_ptr) {
    char *end;
    //
    if(s == NULL) s = *save_ptr;
    if(*s == '\0') {
        *save_ptr = s;
        return NULL;
    }
    // Scan leading delimiters
    s += strspn(s, delim);
    if (*s == '\0') {
        *save_ptr = s;
        return NULL;
    }
    // Find the end of the token
    end = s + strcspn (s, delim);
    if (*end == '\0') {
        *save_ptr = end;
        return s;
    }
    // Terminate the token and make *SAVE_PTR point past it
    *end = '\0';
    *save_ptr = end + 1;
    //
    return s;
}

/**
 * @brief Поиск лексем в строке.
 * Последовательность вызовов функции разбивает строку на лексемы, разделенные символами разделителями.
 * Функция возвращает указатель на первую найденную лексему в строке. Если лексем не найдено, то возвращается пустой указатель. 
 * @param s Указатель на строку, которую нужно разбить на лексемы.
 * @param delim Указатель на строку, содержащую разделители.
 * @return char* 
 */
char *strtok(char *s, const char *delim) {
    static char *olds;
    return strtok_r(s, delim, &olds);
}
