/**
 * @file tzset.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы с временем.
 * @version 0.1
 * @date 2024-03-10
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 * 
 */

#include "stdio.h"
#include "syscallp.h"
#include "time.h"

/**
 * @brief Наименования текущей TZ.
 * Пример: "RTZ 2"
 */
char *tzname[2];

/**
 * @brief Смещение в секундах текущей TZ относительно GMT.
 * Положительные значения - западнее GMT, отрицательные значения - восточнее GMT.
 * Пример: -10800 = GMT+3
 */
long timezone;

/**
 * @brief Признак перехода на летнее время.
 * 0 - не используется, !=0 - используется
 */
int daylight;

/**
 * @brief Инициализирует информацию о преобразованиях времени.
 * Функция tzset() инициализирует переменную tzname из переменной окружения TZ и автоматически вызывается другими функциями,
 * зависящими от временной зоны. Она также устанавливает переменные timezone (секунды западнее GMT) и daylight
 * (если временная зона не использует переход на "летнее" время, то 0, в противном случае ненулевое значение).
 */
void tzset(void) {
    static char _tzname[12];
    //
    long t = __internal_syscall_invoke1(SYSCALL_TZSET, 0);
    if(t != timezone) {
        timezone = t;
        //
        // если изменилось значение формируем новое имя для зоны
        sprintf(_tzname, "GMT%+d", -timezone / 3600);
        tzname[0] = _tzname;
        tzname[1] = _tzname;
    }
    // перехода на летнее/зимнее время у нас нет
    // daylight = 0;
}
