/**
 * @file wcscasestr.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stddef.h>

#include <wchar.h>

/**
 * @brief Поиск без учёта регистра символов UTF32 подстроки wcs2 в UTF32 строке wcs1.
 * Если подстрока найдена, то функция возвращает указатель на начало найденного вхождения в строке wcs1.
 * Если подстрока не найдена, то функция возвращает NULL.
 * @param wcs1 Указатель на строку 1.
 * @param wcs2 Указатель на строку 2.
 * @return wchar_t* 
 */
wchar_t* wcscasestr(const wchar_t* wcs1, const wchar_t* wcs2) {
    wchar_t* p = (wchar_t*)wcs1;
    size_t len = 0;

    if (wcs1 && *wcs1 && wcs2 && *wcs2) {
        len = wcslen(wcs2);
        while (*p) {
            if (wcsncasecmp(p, wcs2, len) == 0) return (wchar_t*)p;
            p++;
        }
    }
    return NULL;
}
