/**
 * @file mbscasestr.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stdlib.h>

#include <wchar.h>

/**
 * @brief Возвращает указатель на на начало символа с индексом ztForwardUtf8Chars в строке UTF8.
 * @param pmbc Указатель на строку 1.
 * @param ztForwardUtf8Chars Индекс символа.
 * @return char* 
 */
static char* ForwardUtf8Chars(const char* pmbc, size_t ztForwardUtf8Chars) {
    size_t ztUtf8 = 0;
    size_t ztNumUtf8Chars = 0;
    //
    while ((ztNumUtf8Chars < ztForwardUtf8Chars) && (pmbc[ztUtf8] != 0)) {
        ztNumUtf8Chars++;
        //
        if ((pmbc[ztUtf8] & 0b10000000) == 0) ztUtf8 += 1;
        else 
        if ((pmbc[ztUtf8] & 0b11100000) == 0b11000000) ztUtf8 += 2;
        else 
        if ((pmbc[ztUtf8] & 0b11110000) == 0b11100000) ztUtf8 += 3;
        else ztUtf8 += 4;
    }
    return (char*)&pmbc[ztUtf8];
}


/**
 * @brief Поиск без учёта регистра символов UTF8 подстроки pmbs2 в UTF32 строке pmbs1.
 * Если подстрока найдена, то функция возвращает указатель на начало найденного вхождения в строке pmbs1.
 * Если подстрока не найдена, то функция возвращает NULL.
 * @param pmbs1 Указатель на строку 1.
 * @param pmbs2 Указатель на строку 2.
 * @return char* 
 */
char *mbscasestr(const char* pmbs1, const char* pmbs2) {
    wchar_t* pUtf32s1 = mbstowcs(pmbs1);
    if (pUtf32s1) {
        wchar_t* pUtf32s2 = mbstowcs(pmbs2);
        wchar_t* pUtf32Found = wcscasestr(pUtf32s1, pUtf32s2);
        if (pUtf32s2) {
            char* pUtf8Found = 0;
            size_t ztSteps = pUtf32Found - pUtf32s1;
            if (pUtf32Found) pUtf8Found = ForwardUtf8Chars(pmbs1, ztSteps);
            free(pUtf32s1);
            free(pUtf32s2);
            //
            return pUtf8Found;
        }
        free(pUtf32s1);
        return NULL;
    }
    return NULL;
}
