/**
 * @file wcsncasecmp.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stdlib.h>
#include <string.h>

#include <wchar.h>

/**
 * @brief Сравнение строк UTF32 с ограничением длины и без учёта регистра.
 * Возвращает значение 0 (строки одинаковые), <0 - строка 1 меньше или 1 - строка 1 больше.
 * @param wcs1 Указатель на строку 1.
 * @param wcs2 Указатель на строку 2.
 * @param ztCount Максимальное количество сравниваемых символов.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int wcsncasecmp(const wchar_t* wcs1, const wchar_t* wcs2, size_t ztCount) {
    wchar_t* pStr1Low = 0;
    wchar_t* pStr2Low = 0;
    int iResult = (-1);

    if (wcs1 && *wcs1 && wcs2 && *wcs2) {
        pStr1Low = (wchar_t*)calloc(wcslen(wcs1) + 1, sizeof(wchar_t));
        if (pStr1Low) {
            pStr2Low = (wchar_t*)calloc(wcslen(wcs2) + 1, sizeof(wchar_t));
            if (pStr2Low) {
                memcpy((wchar_t*)pStr1Low, wcs1, wcslen(wcs1) * sizeof(wchar_t));
                memcpy((wchar_t*)pStr2Low, wcs2, wcslen(wcs2) * sizeof(wchar_t));
                wcslwr(pStr1Low);
                wcslwr(pStr2Low);
                iResult = wcsncmp(pStr1Low, pStr2Low, ztCount);
                free(pStr1Low);
                free(pStr2Low);
            } else free(pStr1Low);
        }
    }
    return iResult;
}

/**
 * @brief Сравнение строк UTF32 без учёта регистра.
 * Возвращает значение 0 (строки одинаковые), <0 - строка 1 меньше или 1 - строка 1 больше.
 * @param wcs1 Указатель на строку 1.
 * @param wcs2 Указатель на строку 2.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int wcscasecmp(const wchar_t* wcs1, const wchar_t* wcs2) {
    return wcsncasecmp(wcs1, wcs2, (size_t)(-1));
}
