/**
 * @file mbsncasecmp.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stdlib.h>

#include <wchar.h>

/**
 * @brief Регистронезависимвое сравнение двух строк UTF8 с ограничением длины.
 * При ошибке функция возвращает значение -1.
 * @param pmbs1 Указатель на строку 1.
 * @param pmbs2 Указатель на строку 2.
 * @param ztCount Максимальная длина строк.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int mbsncasecmp(const char* pmbs1, const char* pmbs2, size_t ztCount) {
    wchar_t* pUtf32s1 = mbstowcs(pmbs1);
    if (pUtf32s1) {
        wchar_t* pUtf32s2 = mbstowcs(pmbs2);
        if (pUtf32s2) {
            int iDiff = wcsncasecmp(pUtf32s1, pUtf32s2, ztCount);
            free(pUtf32s1);
            free(pUtf32s2);
            return iDiff;
        }
        free(pUtf32s1);
        return -1;
    }
    return -1;
}

/**
 * @brief Регистронезависимвое сравнение двух строк UTF8.
 * При ошибке функция возвращает значение -1.
 * @param pmbs1 Указатель на строку 1.
 * @param pmbs2 Указатель на строку 2.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int mbscasecmp(const char* pmbs1, const char* pmbs2) {
    return mbsncasecmp(pmbs1, pmbs2, (size_t)(-1));
}
