/**
 * @file wcsncmp.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <wchar.h>

/**
 * @brief Сравнение строк UTF32 с учётом регистра.
 * Возвращает значение 0 (строки одинаковые), <0 - строка 1 меньше или 1 - строка 1 больше.
 * @param wcs1 Указатель на строку 1.
 * @param wcs2 Указатель на строку 2.
 * @param ztCount Максимальное количество сравниваемых символов.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int wcsncmp(const wchar_t* wcs1, const wchar_t* wcs2, size_t ztCount) {
    if (wcs1 && *wcs1 && wcs2 && *wcs2) {
        for (; ztCount--; wcs1++, wcs2++) {
            int iDiff = *wcs1 - *wcs2;
            if (iDiff != 0 || !*wcs1 || !*wcs2) return iDiff;
        }
        return 0;
    }
    return (-1);
}

/**
 * @brief Сравнение строк UTF32 с учётом регистра.
 * Возвращает значение 0 (строки одинаковые), <0 - строка 1 меньше или 1 - строка 1 больше.
 * @param wcs1 Указатель на строку 1.
 * @param wcs2 Указатель на строку 2.
 * @return int 0-строки одинаковые, >0-строка 1 больше, <0-строка 1 меньше.
 */
int wcscmp(const wchar_t* wcs1, const wchar_t* wcs2) {
    return wcsncmp(wcs1, wcs2, (size_t)(-1));
}
