/**
 * @file mbsmakelwr.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-11
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <stdlib.h>

#include <wchar.h>

/**
 * @brief Преобразование символов строки UTF8 в нижний регистр.
 * Функция выделяет память для новой строки UTF8, выполняет преобразование и возвращает указатель на строку UTF8.
 * Выделенная память должна быть в последствии освобождена функцией free.
 * @param pmbc Указатель на строку UTF8.
 * @return char* Указатель на новую строку - результат преобразования регистра символов.
 */
char* mbsmakelwr(const char* pmbc) {
    wchar_t* pwcs = mbstowcs(pmbc);
    if (pwcs && *pwcs) {
        wcslwr(pwcs);
        {
            char* pUtf8Lwr = wcstombs(pwcs);
            free(pwcs);
            return pUtf8Lwr;
        }
    }
    return NULL;
}
