#define TEST_PG         0
#define TEST_SUN        0
#define TEST_NETVAR     0
#define TEST_MODBUS     0
#define TEST_MERC230    0
#define TEST_HASH       0
#define TEST_AES        0
#define TEST_BASE64     0
#define TEST_BLOWFISH   0
#define TEST_RC4        0
#define TEST_ICONV      0
#define TEST_LANMON     0
#define TEST_PARAMS     0
#define TEST_WCHAR      0
#define TEST_CJSON      0
#define TEST_SOCK       0
#define TEST_URL        0
#define TEST_HTTP       0
#define TEST_WEATHER    0
#define TEST_MQTT       1

#include "satplc.h"
#include "satscr.h"
#include "satmio.h"
#include "satmem.h"
#include "satkbd.h"

#include "satser.h"

#if TEST_SUN == 1
#include "satsun.h"
#endif

#if TEST_MERC230 == 1
#include "merc230.h"
#endif

#if TEST_MODBUS == 1
#include "modbus.h"
#endif

#if TEST_SOCK == 1
#include "satsock.h"
#endif

#if TEST_HASH == 1
#include "hash.h"
#endif

#if TEST_PARAMS == 1
#include "satparams.h"
#endif

#if TEST_NETVAR == 1
#include "netvars.h"
#endif

#if TEST_PG == 1
#include "pgsql.h"
#endif

#if TEST_AES == 1
#include "aes.h"
#endif

#if TEST_BASE64 == 1
#include "base64.h"
#endif

#if TEST_BLOWFISH == 1
#include "blowfish.h"
#endif

#if TEST_RC4 == 1
#include "rc4.h"
#endif

#if TEST_ICONV == 1
#include "iconv.h"
#endif

#if TEST_LANMON == 1
#include "lanmon.h"
#endif

#if TEST_WCHAR == 1
#include "wchar.h"
#endif

#if TEST_CJSON == 1
#include "cjson.h"
#endif

#if TEST_HTTP == 1 || TEST_URL == 1
#include "cjson.h"
#include "http.h"
#endif

#if TEST_WEATHER == 1
#include "weather.h"
#endif

#if TEST_MQTT == 1
#include "mqtt.h"
#endif


#if TEST_MERC230 == 1
/**
 * @brief Тест чтения данных счётчика электрической энергии Меркурий-230
 */
void TestMercury(void) {
    merc230_current_t   current_data;
    merc230_total_t     total_data;
    //
    int handle = SerialOpen(1, B9600, PARNONE, 1);
    //
    while(1) {
        uint32_t start = GetTickCount();
        //
        int res1 = Merc230_ReadCurrentData(handle, 0, &current_data);
        int res2 = Merc230_ReadTotalData(handle, 0, 2, &total_data);
        //
        printf(CLRSCR HYEL "Меркурий 230 " WHT "(%u мс)" HYEL "\n", GetTickCount()-start);
        printf("~~~~~~~~~~~~\n" CRESET);
        if(res1 > 0) {
            printf("Ua=" HGRN "%.1f" CRESET " Ub=" HGRN "%.1f" CRESET " Uc=" HGRN "%.1f" CRESET "\n", current_data.U[FASE_A], current_data.U[FASE_B], current_data.U[FASE_C]);
            printf("Ia=" HGRN "%.1f" CRESET " Ib=" HGRN "%.1f" CRESET " Ic=" HGRN "%.1f" CRESET "\n", current_data.I[FASE_A], current_data.I[FASE_B], current_data.I[FASE_C]);
            printf("Pa=" HGRN "%.1f" CRESET " Pb=" HGRN "%.1f" CRESET " Pc=" HGRN "%.1f" CRESET "\n", current_data.P[FASE_A], current_data.P[FASE_B], current_data.P[FASE_C]);
            printf("P=" HGRN "%.1f" CRESET " Вт\nQ=" HGRN "%.1f" CRESET " вар\nS=" HGRN "%.1f" CRESET " ВА\n", current_data.P[FASE_SUMMA], current_data.Q[FASE_SUMMA], current_data.S[FASE_SUMMA]);
        } else {
            printf(HRED "Ошибка чтения текущих\n" CRESET);
        }
        //
        if(res2 > 0) {
            printf("Esa=" HGRN "%.3f" CRESET " кВтч\n", total_data.summa[ENERGY_ACTIVE_P]);
            printf("Esr=" HGRN "%.3f" CRESET " кварч\n", total_data.summa[ENERGY_REACTIVE_P]);
            //
            printf("E1a=" HGRN "%.3f" CRESET " кВтч\n", total_data.tarif1[ENERGY_ACTIVE_P]);
            printf("E1r=" HGRN "%.3f" CRESET " кварч\n", total_data.tarif1[ENERGY_REACTIVE_P]);
            //
            printf("E2a=" HGRN "%.3f" CRESET " кВтч\n", total_data.tarif2[ENERGY_ACTIVE_P]);
            printf("E2r=" HGRN "%.3f" CRESET " кварч\n", total_data.tarif2[ENERGY_REACTIVE_P]);
        } else {
            printf(HRED "Ошибка чтения итогов\n" CRESET);
        }
        ShowScr();
        //
        msleep(1000);
    }
}
#endif

#if TEST_MODBUS == 1
/**
 * @brief Тест Modbus
 */
void TestModbus(void) {
    uint16_t registers[32];
    uint16_t coil = 0;
    uint16_t di;
    float spval = 0.0;
    //
    int handle = SerialOpen(1, B115200, PARNONE, 1);

    while (1) {
        mdbs_result_t mdbs_result;
        //
        printf(CLRSCR HGRN "MODBUS\n" CRESET);
        //
        mdbs_result = ModbusReadHoldingRegisters(handle, 48, 16, 4, registers);
        //
        printf("mdbs_result1:" HBLU "%d" CRESET "\n", mdbs_result);
        printf("T1=" HGRN "%f" CRESET "\n", ModbusGetFloatBADC(&registers[0]));
        //
        spval = spval + 1.0;
        ModbusSetFloatBADC(spval, registers);
        mdbs_result = ModbusWriteRegisters(handle, 48, 0x2800, 2, registers);
        //
        printf("mdbs_result2:" HBLU "%d" CRESET "\n", mdbs_result);
        printf("spval:" HGRN "%f" CRESET "\n", spval);
        //
        mdbs_result = ModbusReadHoldingRegisters(handle, 48, 0x2800, 4, registers);
        printf("mdbs_result3:" HBLU "%d" CRESET "\n", mdbs_result);
        printf("sp1=" HGRN "%f" CRESET "\n", ModbusGetFloatBADC(&registers[0]));
        printf("sp2=" HGRN "%f" CRESET "\n", ModbusGetFloatBADC(&registers[2]));
        //
        if(coil) coil = 0; else coil = 0xffff;
        mdbs_result = ModbusWriteCoils(handle, 48, 0x101, 9, (uint8_t *)&coil);
        printf("mdbs_result4:" HBLU "%d" CRESET "\n", mdbs_result);
        //
        mdbs_result = ModbusReadDiscreteInputs(handle, 48, 0x100, 10, (uint8_t *)&di);
        printf("mdbs_result5:" HBLU "%d" CRESET "\n", mdbs_result);
        printf("DI:" HRED "%0.10b" CRESET "\n", di);
        //
        ShowScr();
        //
        msleep(1000);
    }
}
/**
 * @brief Тест Modbus
 */
void TestModbusTCP(void) {
    int handle = CreateSocket(SOCK_STREAM);
    ClrScr();
    while (1) {
        printf("handle=%d\n", handle);
        if(handle) {

            if(!SocketConnected(handle)) {
                int res = SocketConnectBlock(handle, "192.168.1.86", 502, 1000);
                printf("SocketConnectBlock=%d\n", res);
            } else {
                printf("Socket is connected\n");

            }
            //
        }
        ShowScr();
        msleep(1000);
    }
}
#endif

#if TEST_SOCK == 1
void TestSock(void) {
    struct in_addr addr;
    char ip[] = "192.168.1.2";
    char ipbuf[32];
    //
    ClrScr();
    //
    int res = inet_atoh(ip, &addr);
    //
    printf("res=%d, val=%.8x\n", res, addr.S_un.S_addr);
    //
    inet_htop(&addr, ipbuf);
    printf("ip=%s\n", ipbuf);
    //
    ShowScr();
    while (1);
}
#endif

#if TEST_SUN == 1
void TestSun() {
    while (1) {
        ClrScr();
        printf("Угол солнца:" HGRN "%.2f" CRESET "\n", SunPosition(55.75222f, 37.61556f, time(NULL)));
        ShowScr();
        //
        msleep(1000);
    }
}
#endif

#if TEST_NETVAR == 1
void TestSendNetVars(void) {
    int nums[4];
    int vals[4];
    int i;
    //
    nums[0] = 1;
    nums[1] = 5;
    nums[2] = 7;
    nums[3] = 12;
    //
    i = 1;
    //
    while (1) {
        vals[0] = i;
        vals[1] = i*2;
        vals[2] = i*4;
        vals[3] = i*8;
        //
        int res = NetVarsSend(nums, vals, 4);
        //
        ClrScr();
        printf("NetVarsSend=%d\n", res);
        ShowScr();
        msleep(1000);
        //
        i++;
    }
}
/**
 * @brief Асинхронный приём сетевых переменных
 */
void nvcb(int32_t sender, uint16_t group, uint16_t cnt, net_var_t *vars) {
    printf("S:%u G:%u C:%u\n", sender, group, cnt);
    for(int i=0; i<cnt; i++) {
        printf(" I:%u V:%u\n", vars[i].id, vars[i].value);
    }
    ShowScr();
}
void TestRecvNetVars1() {
    ClrScr();
    //
    int res = NetVarsStartEx(nvcb);
    printf("res=%d\n", res);
    ShowScr();
    //
    sleep(120);
    //
    printf("stop\n");
    NetVarsStop();
    printf("finished\n");
    //
    while(1);
}

void TestRecvNetVars2() {
    int nums[4];
    int vals[4];
    //
    nums[0] = 1;
    nums[1] = 5;
    nums[2] = 7;
    nums[3] = 12;
    //
    ClrScr();
    //
    int res= NetVarsStart(nums, vals, NULL, 4);
    printf("res=%d\n", res);
    //
    while(1) {
        ClrScr();
        for(int i=0; i<4; i++) {
            printf("ID:%d V:%d\n", nums[i], vals[i]);
        }
        ShowScr();
        msleep(1000);
    }
    //
    while(1);
}
#endif

#if TEST_PARAMS == 1
void TestParams(void) {
    struct plc_info info;
    struct in_addr  addr;
    uint32_t ser;
    char ips[32];
    //
    GetPLCParameter(PARAM_ID_INFO, &info);
    //
    ClrScr();
    printf("ver_Hi:%u\n", info.version_Hi);
    printf("ver_Lo:%u\n", info.version_Lo);
    printf("Year:%u\n",   info.year);
    printf("Month:%u\n",  info.month);
    printf("Number:%u\n", info.number);
    printf("Addr:%u\n",   info.address);
    //
    GetPLCParameter(PARAM_ID_SERIAL, &ser);
    printf("Ser:%u\n",   ser);
    //
    GetPLCParameter(PARAM_ID_IPADDR, &addr);
    inet_htop(&addr, ips);
    printf("IP:%s\n",   ips);
    //
    GetPLCParameter(PARAM_ID_NETMASK, &addr);
    inet_htop(&addr, ips);
    printf("MASK:%s\n",   ips);
    //
    GetPLCParameter(PARAM_ID_IPGATE, &addr);
    inet_htop(&addr, ips);
    printf("GATE:%s\n",   ips);
    //
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_PG == 1
void TestPG() {
    char errcode[16];
    char buf[1024];
    //
    PgSetBuffer(buf, 1024);
    PgSetTimeout(2000);
    //
    while(1) {
        ClrScr();
        printf("waiting...");
        ShowScr();
        sleep(5);
        ClrScr();

        int res = PgOpen("192.168.1.86", 5432, "test", "postgres", "postgres");
        if(res) {
            while(1) {
                GotoXY(4,32);
                printf("PgState=%u\nPgDataLen=%u\n", PgState(), PgDataLen());
                PgErrorCode(errcode);
                printf("EC=%s\n", errcode);
                ShowScr();
                // if(PgState==pgsOK || PgState==pgsDISCONNECTED)break;
                msleep(100);
            }
            while(1);
        } else {
            printf("Error: %d\n", PgState());
            ShowScr();
        }
        sleep(2);
    }
}
#endif

#if TEST_RC4 == 1
void rc4_test() {
    ClrScr();
    printf("RC4...\n");
    ShowScr();
    //
#ifdef TEST_RC4
    if(test_rc4() == 7) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_AES == 1
void aes_test() {
    ClrScr();
    printf("AES...\n");
    ShowScr();
    //
#ifdef TEST_AES
    if(test_aes() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_BLOWFISH == 1
void blowfish_test() {
    ClrScr();
    printf("Blowfish...\n");
    ShowScr();
    //
#ifdef TEST_BLOWFISH
    if(test_blowfish() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_HASH == 1
void hash_test() {
    ClrScr();
    //
    printf("SHA256...\n");
    ShowScr();
#ifdef TEST_HASH
    if(test_sha256() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    printf("SHA1...\n");
    ShowScr();
#ifdef TEST_HASH
    if(test_sha1() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    printf("MD5...\n");
    ShowScr();
#ifdef TEST_HASH
    if(test_md5() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_BASE64 == 1
void base64_test() {
    ClrScr();
    printf("base64...\n");
    ShowScr();
    //
#ifdef TEST_BASE64
    if(test_base64() == 1) printf("Ok\n"); else printf("Error\n");
#else
    printf("Not executed\n");
#endif
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_ICONV == 1
void iconv_test() {
    char utf8[128];
    char cp1251[128];
    const char str1[] = "abcdefghijklmnopqrstuvwxyz_абвгдеёжзийклмнопрстуфхцчшщъыьэюя";
    const char str2[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ";
    //
    int res;
    ClrScr();
    res = utf8_to_cp1251(str1, cp1251, sizeof(cp1251));
    printf("res=%d\n", res);
    //
    res = cp1251_to_utf8(cp1251, utf8, sizeof(utf8));
    printf("res2=%d\n", res);
    printf("%s\n", utf8);
    //
    res = utf8_to_cp1251(str2, cp1251, sizeof(cp1251));
    printf("res=%d\n", res);
    //
    res = cp1251_to_utf8(cp1251, utf8, sizeof(utf8));
    printf("res2=%d\n", res);
    printf("%s\n", utf8);
    //
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_LANMON == 1
void size_test() {
    ClrScr();
    //
    lm_channel_t lc;
    UINT adr=(UINT)&lc;
    printf("\r\nLM:name 0");
    UINT disp;
    disp=(UINT)&lc.unit-adr;    printf("\r\nLM:unit %u",disp);
    disp=(UINT)&lc.cmnts-adr;   printf("\r\nLM:cmnts %u",disp);
    disp=(UINT)&lc.num-adr;     printf("\r\nLM:num %u",disp);
    disp=(UINT)&lc.type-adr;    printf("\r\nLM:type %u",disp);
    disp=(UINT)&lc.quality-adr; printf("\r\nLM:quality %u",disp);
    disp=(UINT)&lc.value-adr;   printf("\r\nLM:value %u",disp);
    disp=(UINT)&lc.filter-adr;  printf("\r\nLM:filter %u",disp);
    disp=(UINT)&lc.state-adr;   printf("\r\nLM:state %u",disp);
    //
    printf("\nsof(1):%d", sizeof(lm_channel_t));
    printf("\nsof(2):%d", sizeof(lc.state));
    //
    ShowScr();
    while(1);
}
void lm_cb(int client_id, lm_channel_t *channel) {
    switch (client_id) {
        case 10:
            SetAO(AO1, channel->value.i4);
            channel->state.changed = 1;
            break;
        case 11:
            SetAO(AO2, channel->value.r8);
            channel->state.changed = 1;
            break;
    }
}
void lanmon_test() {
    int res;
    //
    ClrScr();
    res = LMStart("192.168.1.86", 3000, 10000, "TTT", "123", lm_cb);
    printf("res=%d\n", res);
    //
    ShowScr();
    while(!LMConnected());
    printf("Connected!\n");
    ShowScr();
    //
    int client_id[12];
    lm_channel_t *channels[12];
    //
    client_id[0] = LMAddChannel("Канал_test_I2", lmtI2,     NULL, "Это тестовый канал 1", 0);
    client_id[1] = LMAddChannel("ch_test_I4",    lmtI4,     "А",  NULL, 0);
    client_id[2] = LMAddChannel("ch_test_R4",    lmtR4,     NULL, NULL, 0);
    client_id[3] = LMAddChannel("ch_test_R8",    lmtR8,     "м3", "Это тестовый канал 4", 0);
    client_id[4] = LMAddChannel("ch_test_DATE",  lmtDATE,   "м3", "Это тестовый канал 5", 1);
    client_id[5] = LMAddChannel("ch_test_BOOL",  lmtBOOL,   "м3", "Это тестовый канал 6", 0);
    client_id[6] = LMAddChannel("ch_test_I1",    lmtI1,     "м3", "Это тестовый канал 7", 0);
    client_id[7] = LMAddChannel("ch_test_UI1",   lmtUI1,    "м3", "Это тестовый канал 8", 0);
    client_id[8] = LMAddChannel("ch_test_UI2",   lmtUI2,    "м3", "Это тестовый канал 9", 0);
    client_id[9] = LMAddChannel("ch_test_UI4",   lmtUI4,    "м3", "Это тестовый канал 10", 0);
    //
    client_id[10] = LMAddChannel("ch_test_I4_ctrl", lmtUI4, "м3", "Это тестовый канал 11", 1);
    client_id[11] = LMAddChannel("ch_test_R8_ctrl", lmtR8,  "м3", "Это тестовый канал 12", 1);
    //
    // Проверка дубликатов
    res = LMAddChannel("Канал_test_I2", lmtI2,     "кг", "Это тестовый канал 1", 0);
    printf("check dup1:%d %s\n", res, (res==-1)?"Ok":"Err");
    res = LMAddChannel("ch_test_UI4",   lmtUI4,    "м3", "Это тестовый канал 10", 0);
    printf("check dup2:%d %s\n", res, (res==-1)?"Ok":"Err");
    //
    for(int i=0; i<12; i++) {
        channels[i] = LMGetChannelInfo(client_id[i]);
    }
    lm_value_t value;
    int     ival = 0;
    float   fval = 0.0;
    double  dval = 0.0;
    //
    while(1) {
        sleep(2);
        ClrScr();
        //
        for(int i=0; i<12; i++) {
            if(i == 4) {
                time_t ct = DateTimeToUnix(channels[4]->value.datetime);
                struct tm *tm =gmtime(&ct);
                printf("%02d-%02d-%04d %02d:%02d:%02d\n", tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_hour, tm->tm_min, tm->tm_sec);
            } else {
                printf("num%02d=%d a%u f=%u %3.2f\n", i, channels[i]->num, channels[i]->state.active?1:0, channels[i]->state.filter & 3, channels[i]->filter);
            }
        }
        //
        ival += 1;
        fval += 1.0;
        dval += 2.0;
        //
        if((ival % 5) == 0) {
            LMSetQuality(client_id[0], lmqFailure);
            LMSetQuality(client_id[1], lmqFailure);
            LMSetQuality(client_id[6], lmqFailure);
            LMSetQuality(client_id[7], lmqFailure);
            LMSetQuality(client_id[8], lmqFailure);
            LMSetQuality(client_id[9], lmqFailure);
        } else {
            value.i4 = ival;
            LMSendValue(client_id[0], &value);
            LMSendValue(client_id[1], &value);
            LMSendValue(client_id[6], &value);
            LMSendValue(client_id[7], &value);
            LMSendValue(client_id[8], &value);
            LMSendValue(client_id[9], &value);
        }
        //
        value.r4 = fval;
        LMSendValue(client_id[2], &value);
        //
        value.r8 = dval;
        LMSendValue(client_id[3], &value);
        //
        // value.datetime = UnixToDateTime(time(NULL));
        // LMSendValue(client_id[4], &value);
        //
        if((ival % 2) == 0) value.boolean = false; else value.boolean = true;
        LMSendValue(client_id[5], &value);
        //
        printf("I4_ctrl:%d\n", channels[10]->value.ui4);
        printf("R8_ctrl:%.2f  ", channels[11]->value.r8);
        //
        printf("Conn: %d", LMConnected());
        //
        ShowScr();
    }
    //
    while(1);
}
#endif

#if TEST_WCHAR == 1
void utf_test() {
    char str[] = "широкая электрификация южных губерний даст мощный толчок подъёму сельского хозяйства\nthe quick brown fox jumps over the lazy dog";
    char *uprstr;
    char *lowstr;
    //
    ClrScr();
    printf("len=%d (%d)\n", mbslen(str), strlen(str));
    //
    uprstr = mbsmakeupr(str);
    // printf("up_len=%d (%d)\n", mbslen(uprstr), strlen(uprstr));
    printf("%s\n", uprstr);
    ShowScr();
    //
    lowstr = mbsmakelwr(uprstr);
    // printf("lw_len=%d\n", mbslen(lowstr));
    printf("%s\n", lowstr);
    ShowScr();
    //
    free(uprstr);
    free(lowstr);
    //
    while(1);
}
#endif

#if TEST_CJSON == 1
void json_test1() {
    //
    // Разбор строки JSON
    //
    char json_str[] = "{ \
        \"Str\": \"str1\",      \
        \"Num\":  1.2345,       \
        \"Arr\":  [1,2,3,4],    \
        \"Bool\": true,         \
        \"N\": null,            \
        \"Obj1\":{ \
            \"Str\": \"str2\",  \
            \"Num\":  6.789,    \
            \"Arr\":  [1.1,2.1,3.1,4.1], \
            \"Bool\": false, \
            \"N\": null \
        }, \
        \"Obj2\":{ \
            \"Str\": \"str3\",  \
            \"Num\":  3.14,     \
            \"Arr\":  [7,8,9,0], \
            \"Bool\": false, \
            \"N\": null \
        } \
    }";
    ClrScr();
    //
    // разбор строки с текстовым представлением JSON
    cJSON *json = cJSON_Parse(json_str);
    if(json) {
        char empty[] = "not found";
        //
        printf("JSON parse OK!\n");
        ShowScr();
        sleep(1);
        ClrScr();
        //
        // Доступ к полям JSON
        printf("Str: %s\n", cJSON_GetStringByPath(json, "Str", empty));             // "str1"
        printf("Num: %f\n", cJSON_GetDoubleByPath(json, "Num", -1));                // 1.2345
        printf("Arr: %d,%d,%d,%d\n",                                                // 1,2,3,4
            cJSON_GetIntByPath(json, "Arr/0", -1),
            cJSON_GetIntByPath(json, "Arr/1", -1),
            cJSON_GetIntByPath(json, "Arr/2", -1),
            cJSON_GetIntByPath(json, "Arr/3", -1));
        printf("Bool: %d\n", cJSON_GetBoolByPath(json, "Bool", -1));                // 1
        //
        printf("Obj1/Str: %s\n", cJSON_GetStringByPath(json, "Obj1/Str", empty));   // "str2"
        printf("Obj1/Num: %f\n", cJSON_GetDoubleByPath(json, "Obj1/Num", -1));      // 6.789
        printf("Obj1/Arr: %.1f,%.1f,%.1f,%.1f\n",                                   // 1.1,1.2,1.3,1.4
            cJSON_GetDoubleByPath(json, "Obj1/Arr/0", -1),
            cJSON_GetDoubleByPath(json, "Obj1/Arr/1", -1),
            cJSON_GetDoubleByPath(json, "Obj1/Arr/2", -1),
            cJSON_GetDoubleByPath(json, "Obj1/Arr/3", -1));
        printf("Obj1/Bool: %d\n", cJSON_GetBoolByPath(json, "Obj1/Bool", -1));      // 0
        //
        printf("Obj2/Str: %s\n", cJSON_GetStringByPath(json, "Obj2/Str", empty));   // "str3"
        printf("Obj2/Num: %f\n", cJSON_GetDoubleByPath(json, "Obj2/Num", -1));      // 3.14
        printf("Obj2/Arr: %d,%d,%d,%d\n",                                           // 7,8,9,0
            cJSON_GetIntByPath(json, "Obj2/Arr/0", -1),
            cJSON_GetIntByPath(json, "Obj2/Arr/1", -1),
            cJSON_GetIntByPath(json, "Obj2/Arr/2", -1),
            cJSON_GetIntByPath(json, "Obj2/Arr/3", -1));
        printf("Obj2/Bool: %d\n", cJSON_GetBoolByPath(json, "Obj2/Bool", -1));      // 0
        //
        // освобождаем память 
        cJSON_Delete(json);
    } else {
        //
        // при разборе строки возникла ошибка
        printf("Error parse JSON:\n");
        printf(HRED "%s" CRESET, cJSON_GetErrorPtr());
    }
    ShowScr();
    //
    while (1);
}
void json_test2() {
    //
    // конструирование JSON
    int intarr[4]   = {1, 2, 3, 4};
    float fltarr[4] = {1.1, 2.2, 3.3, 4.4};
    //
    ClrScr();
    // создаём новый объект
    cJSON *json = cJSON_CreateObject();
    // добавляем в него поля
    cJSON_AddStringToObject(json, "name", "Иванов");
    cJSON_AddNumberToObject(json, "age", 30);
    cJSON_AddBoolToObject(json, "f", 0);
    cJSON_AddNullToObject(json, "n");
    cJSON_AddItemToObject(json, "arri", cJSON_CreateIntArray(intarr, 4));
    // создаём дочерний объект
    cJSON *json_child = cJSON_CreateObject();
    cJSON_AddStringToObject(json_child, "name", "Петров");
    cJSON_AddNumberToObject(json_child, "age", 31);
    cJSON_AddBoolToObject(json_child, "t", 1);
    cJSON_AddNullToObject(json_child, "n");
    cJSON_AddItemToObject(json_child, "arrf", cJSON_CreateFloatArray(fltarr, 4));
    // добавляем дочерний объект к основному
    cJSON_AddItemToObject(json, "obj", json_child);
    //
    // преобразуем в строку (память строки потом необходимо освободить)
    char *str = cJSON_PrintUnformatted(json);
    printf("%s\n", str);
    //
    // освобождаем память
    free(str);
    cJSON_Delete(json);
    //
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_HTTP == 1
void http_test1() {
    int inta[] = {1,23,43,55,66,77,22,34};
    //
    cJSON *json = cJSON_CreateObject();
    cJSON_AddStringToObject(json, "str", "Петров");
    cJSON_AddNumberToObject(json, "rnd", 31);
    cJSON_AddItemToObject(json, "arr", cJSON_CreateIntArray(inta, 8));
    //
    while(1) {
        cJSON_SetNumberValue(cJSON_ByPath(json, "rnd"), rand());
        //
        char *json_str = cJSON_PrintUnformatted(json);
        SetHttpSrvData(json_str, httpdJSON);
        free(json_str);
        //
        ClrScr();
        printf("HttpSrvDataCount=%d\n", HttpSrvDataCount());
        ShowScr();
        //
        sleep(2);
    }
    //
    while(1);
}

void http_test2() {
    int reqn = 1;
    //
    http_response_t resp;
    //
    while(1) {
        ClrScr();
        printf("Make HTTP request: %d\n", reqn++);
        ShowScr();
        //
        // Москва: 524901
        // Минск:  625144
        // Мурманск: 524305
        //
        http_result_t res = HttpGET("http://www.mnppsaturn.ru/weather/?city=524901", NULL, &resp);
        // http_result_t res = HttpGET("http://www.mnppsaturn.ru/metz/resultd.json", NULL, &resp);
        // http_result_t res = HttpGET("http://192.168.1.203/metz/resultd.json", NULL, &resp);
        // http_result_t res = HttpGET("http://192.168.1.86/busy.html", NULL, &resp);
        if(res == HTTP_RES_OK) {
            printf("Success!!!\n");
            printf("code=%d\n", resp.status_code);
            printf("len=%d\n", resp.response_len);
            if(resp.body) printf(HGRN "%s" CRESET, resp.body);
            //
            HttpFreeResponse(&resp);
        } else {
            printf("Error: \n");
            switch (res) {
                case HTTP_RES_BAD_URL:          printf("HTTP_RES_BAD_URL\n"); break;
                case HTTP_RES_BAD_SOCKET:       printf("HTTP_RES_BAD_SOCKET\n"); break;
                case HTTP_RES_NO_CONNECTION:    printf("HTTP_RES_NO_CONNECTION\n"); break;
                case HTTP_RES_RESP_TOO_LARGE:   printf("HTTP_RES_RESP_TOO_LARGE\n"); break;
                case HTTP_RES_EMPTY_RESP:       printf("HTTP_RES_EMPTY_RESP\n"); break;
                case HTTP_RES_BAD_RESP_PROTO:   printf("HTTP_RES_BAD_RESP_PROTO\n"); break;
                case HTTP_RES_BAD_RESP_FORMAT:  printf("HTTP_RES_BAD_RESP_FORMAT\n"); break;
                case HTTP_RES_AUTH_TOO_LARGE:   printf("HTTP_RES_AUTH_TOO_LARGE\n"); break;
                case HTTP_RES_HEADERS_TOO_LARGE:printf("HTTP_RES_HEADERS_TOO_LARGE\n"); break;
                case HTTP_RES_EMPTY_POST_DATA:  printf("HTTP_RES_EMPTY_POST_DATA\n"); break;
                default: printf("%d\n", res);
            }
        }
        ShowScr();
        sleep(4);
    }
    //
    while(1);
}

void http_test3() {
    int reqn = 1;
    //
    http_response_t resp;
    //
    while(1) {
        ClrScr();
        printf("Make HTTP request: %d\n", reqn++);
        ShowScr();
        //
        http_result_t res = HttpPOST("http://httpbin.org/post", NULL, "{\"test\":1}", httpdJSON, &resp);
        if(res == HTTP_RES_OK) {
            printf("Success!!!\n");
            printf("code=%d\n", resp.status_code);
            printf("len=%d\n", resp.response_len);
            // if(resp.body) printf(HGRN "%s" CRESET, resp.body);
            //
            HttpFreeResponse(&resp);
        } else {
            printf("Error: \n");
            switch (res) {
                case HTTP_RES_BAD_URL:          printf("HTTP_RES_BAD_URL\n"); break;
                case HTTP_RES_BAD_SOCKET:       printf("HTTP_RES_BAD_SOCKET\n"); break;
                case HTTP_RES_NO_CONNECTION:    printf("HTTP_RES_NO_CONNECTION\n"); break;
                case HTTP_RES_RESP_TOO_LARGE:   printf("HTTP_RES_RESP_TOO_LARGE\n"); break;
                case HTTP_RES_EMPTY_RESP:       printf("HTTP_RES_EMPTY_RESP\n"); break;
                case HTTP_RES_BAD_RESP_PROTO:   printf("HTTP_RES_BAD_RESP_PROTO\n"); break;
                case HTTP_RES_BAD_RESP_FORMAT:  printf("HTTP_RES_BAD_RESP_FORMAT\n"); break;
                case HTTP_RES_AUTH_TOO_LARGE:   printf("HTTP_RES_AUTH_TOO_LARGE\n"); break;
                case HTTP_RES_HEADERS_TOO_LARGE:printf("HTTP_RES_HEADERS_TOO_LARGE\n"); break;
                case HTTP_RES_EMPTY_POST_DATA:  printf("HTTP_RES_EMPTY_POST_DATA\n"); break;
                default: printf("%d\n", res);
            }
        }
        ShowScr();
        sleep(4);
    }
    //
    while(1);
}
#endif

#if TEST_URL == 1
void url_test() {
    char url[] = "http://login:password@www.mnppsaturn.ru/path1/path2?a=1";
    char empty[] = HRED"empty"CRESET;
    //
    ClrScr();
    //
    url_t *purl = HttpParseURL(url);
    printf("uri :"HGRN"%s"CRESET"\n",  purl->uri?purl->uri:empty);
    printf("schm:"HGRN"%s"CRESET"\n", purl->scheme?purl->scheme:empty);
    printf("host:"HGRN"%s"CRESET"\n", purl->host?purl->host:empty);
    printf("port:%u\n", purl->port);
    printf("path:"HGRN"%s"CRESET"\n", purl->path?purl->path:empty);
    printf("quer:"HGRN"%s"CRESET"\n", purl->query?purl->query:empty);
    printf("frag:"HGRN"%s"CRESET"\n", purl->fragment?purl->fragment:empty);
    printf("user:"HGRN"%s"CRESET"\n", purl->username?purl->username:empty);
    printf("pass:"HGRN"%s"CRESET"\n", purl->password?purl->password:empty);
    //
    HttpFreeURL(purl);
    //
    ShowScr();
    //
    while(1);
}
#endif

#if TEST_WEATHER == 1
void weather_test() {
    int totalr = 0;
    int errors = 0;
    //
    weather_data_t weather_data;
    //
    while(1) {
        totalr++;
        int res = weather(WID_MURMANSK_RU, &weather_data);
        //
        ClrScr();
        printf(HGRN"Погода в Мурманске\n\n"CRESET);
        if(res) {
            const char *wstr = weather_text(weather_data.weather);
            if(wstr) {
                printf(HBLU"%s"CRESET"\n\n", wstr);
            } else {
                printf(HRED"Неопределённая"CRESET"\n\n");
            }
            //
            printf("Температура:"HYEL"%.1f"CRESET"°C\n", weather_data.temperature);
            printf("Давление:"HYEL"%.0f"CRESET"мм рт.ст.\n", weather_data.pressure*0.750062f);
            printf("Отн.влажность:"HYEL"%.0f"CRESET"%%\n", weather_data.humidity);
            printf("Ветер:"HYEL"%.0f"CRESET"м/с ("HYEL"%.0f"CRESET"°)\n", weather_data.wind_speed, weather_data.wind_dir);
            printf("Облачность:"HYEL"%.0f"CRESET"%%\n", weather_data.cloud);
            printf("Видимость:"HYEL"%.0f"CRESET"м\n", weather_data.visibility);
        } else {
            errors++;
            printf("Ошибка получения данных");
        }
        printf(WHT"\n\n\nВсего %d, ошибок %d"CRESET, totalr, errors);
        ShowScr();
        //
        sleep(5);
    }
}
#endif

#if TEST_MQTT == 1

int mqtt_rec_counter = 0;

char last_topic_name[128];
char last_topic_val[128];

void receive_message(void** state, struct mqtt_response_publish *publish) {
    int len;
    //
    len = publish->topic_name_size;
    if(len > 127) len = 127;
    memcpy(last_topic_name, publish->topic_name, len);
    last_topic_name[len] = 0;
    //
    len = publish->application_message_size;
    if(len > 127) len = 127;
    memcpy(last_topic_val, publish->application_message, len);
    last_topic_val[len] = 0;
    //
    mqtt_rec_counter++;
}


void mqtt_test() {
    struct mqtt_client mqtt;
    //
    mqtt_client_init(&mqtt);
    mqtt_client_set_keep_alive(&mqtt, 120);
    
    mqtt_client_add_subscription(&mqtt, "lbs/282c02fffe800009/em/41177483/PS", 0);
    mqtt_client_start(&mqtt, "mqtt://lbs:bZk6lhSfU3qH@192.168.1.220", 2048, 1024, receive_message);
    //
    // mqtt_client_start(&mqtt, "mqtt://test.mosquitto.org", 2048, 1024, receive_message);
    //
    while(1) {
        ClrScr();
        printf("%s\n\n", mqtt_client_error_str(mqtt.error));
        printf("ping:%d/%d\n", mqtt.number_of_keep_alives, mqtt.number_of_ping_resp);
        printf("trt:%.3f\n", mqtt.typical_response_time);
        printf("rec:%d\n", mqtt_rec_counter);
        if(mqtt_rec_counter > 0) {
            printf("\nName:\n");
            printf(HYEL"%s"CRESET, last_topic_name);
            printf("\nValue:\n");
            printf(HBLU"%s"CRESET, last_topic_val);
        }
        ShowScr();
        //
        sleep(1);
    }
}


#endif

int main() {
#if TEST_MERC230 == 1
    TestMercury();
#endif
#if TEST_MODBUS == 1
    TestModbus();
#endif
    // TestSock();
#if TEST_SUN == 1
    TestSun();
#endif
#if TEST_MODBUS == 1
    TestModbusTCP();
#endif
#if TEST_NETVAR == 1
    TestSendNetVars();
    TestRecvNetVars1();
    TestRecvNetVars2();
#endif
    // TestParams();
    // TestPG();
    // base64_test();
    // rc4_test();
    // aes_test();
    // blowfish_test();
    // hash_test();
    // base64_test();
    // hwrand_test();
    // iconv_test();
#if TEST_LANMON == 1
    lanmon_test();
#endif
    // utf_test();
    // size_test();
#if TEST_CJSON == 1
    json_test1();
    json_test2();
#endif
#if TEST_HTTP == 1
    // http_test1();
    // http_test2();
    http_test3();
#endif
#if TEST_URL == 1
    url_test();
#endif
#if TEST_WEATHER == 1
    weather_test();
#endif
#if TEST_MQTT == 1
    mqtt_test();
#endif



    return 1;
}
