/**
 * @file towupper.c
 * @brief Стандартная библиотека контроллера "Сатурн-PLC".
 * Функции работы со строками в кодировках UTF8/16/32.
 * @version 0.1
 * @date 2024-05-14
 * 
 * @copyright Copyright (C) 2024, ООО "МНПП Сатурн"
 */

#include <wctype.h>

/**
 * @brief Перевод "широкого" символа в верхний регистр.
 * @param wc 
 * @return wchar_t 
 */
wchar_t towupper(wchar_t wc) {
    // ASCII
    if ((wc >= 0x61) && (wc <= 0x7a)) return (wc-0x20);
    //
#if UTF_SUPPORT_CYRILLICR == 1
    if((wc >= 0x0430) && (wc <= 0x044F)) return (wc-0x20);  // а..я -> А..Я
    if(wc == 0x0451) return 0x0401;                         // ё -> Ё
#endif
    switch(wc) {
#if UTF_SUPPORT_LATIN == 1
        case 0x00DF:		//	ß	0xc3 0x9f	LATIN SMALL LETTER SHARP S
            return 0x1E9E;	//	ẞ	0xe1 0xba 0x9e	LATIN CAPITAL LETTER SHARP S

        case 0x00E0:		//	à	0xc3 0xa0	LATIN SMALL LETTER A WITH GRAVE
            return 0x00C0;	//	À	0xc3 0x80	LATIN CAPITAL LETTER A WITH GRAVE

        case 0x00E1:		//	á	0xc3 0xa1	LATIN SMALL LETTER A WITH ACUTE
            return 0x00C1;	//	Á	0xc3 0x81	LATIN CAPITAL LETTER A WITH ACUTE

        case 0x00E2:		//	â	0xc3 0xa2	LATIN SMALL LETTER A WITH CIRCUMFLEX
            return 0x00C2;	//	Â	0xc3 0x82	LATIN CAPITAL LETTER A WITH CIRCUMFLEX

        case 0x00E3:		//	ã	0xc3 0xa3	LATIN SMALL LETTER A WITH TILDE
            return 0x00C3;	//	Ã	0xc3 0x83	LATIN CAPITAL LETTER A WITH TILDE

        case 0x00E4:		//	ä	0xc3 0xa4	LATIN SMALL LETTER A WITH DIAERESIS
            return 0x00C4;	//	Ä	0xc3 0x84	LATIN CAPITAL LETTER A WITH DIAERESIS

        case 0x00E5:		//	å	0xc3 0xa5	LATIN SMALL LETTER A WITH RING ABOVE
            return 0x00C5;	//	Å	0xc3 0x85	LATIN CAPITAL LETTER A WITH RING ABOVE

        case 0x00E6:		//	æ	0xc3 0xa6	LATIN SMALL LETTER AE
            return 0x00C6;	//	Æ	0xc3 0x86	LATIN CAPITAL LETTER AE

        case 0x00E7:		//	ç	0xc3 0xa7	LATIN SMALL LETTER C WITH CEDILLA
            return 0x00C7;	//	Ç	0xc3 0x87	LATIN CAPITAL LETTER C WITH CEDILLA

        case 0x00E8:		//	è	0xc3 0xa8	LATIN SMALL LETTER E WITH GRAVE
            return 0x00C8;	//	È	0xc3 0x88	LATIN CAPITAL LETTER E WITH GRAVE

        case 0x00E9:		//	é	0xc3 0xa9	LATIN SMALL LETTER E WITH ACUTE
            return 0x00C9;	//	É	0xc3 0x89	LATIN CAPITAL LETTER E WITH ACUTE

        case 0x00EA:		//	ê	0xc3 0xaa	LATIN SMALL LETTER E WITH CIRCUMFLEX
            return 0x00CA;	//	Ê	0xc3 0x8a	LATIN CAPITAL LETTER E WITH CIRCUMFLEX

        case 0x00EB:		//	ë	0xc3 0xab	LATIN SMALL LETTER E WITH DIAERESIS
            return 0x00CB;	//	Ë	0xc3 0x8b	LATIN CAPITAL LETTER E WITH DIAERESIS

        case 0x00EC:		//	ì	0xc3 0xac	LATIN SMALL LETTER I WITH GRAVE
            return 0x00CC;	//	Ì	0xc3 0x8c	LATIN CAPITAL LETTER I WITH GRAVE

        case 0x00ED:		//	í	0xc3 0xad	LATIN SMALL LETTER I WITH ACUTE
            return 0x00CD;	//	Í	0xc3 0x8d	LATIN CAPITAL LETTER I WITH ACUTE

        case 0x00EE:		//	î	0xc3 0xae	LATIN SMALL LETTER I WITH CIRCUMFLEX
            return 0x00CE;	//	Î	0xc3 0x8e	LATIN CAPITAL LETTER I WITH CIRCUMFLEX

        case 0x00EF:		//	ï	0xc3 0xaf	LATIN SMALL LETTER I WITH DIAERESIS
            return 0x00CF;	//	Ï	0xc3 0x8f	LATIN CAPITAL LETTER I WITH DIAERESIS

        case 0x00F0:		//	ð	0xc3 0xb0	LATIN SMALL LETTER ETH
            return 0x00D0;	//	Ð	0xc3 0x90	LATIN CAPITAL LETTER ETH

        case 0x00F1:		//	ñ	0xc3 0xb1	LATIN SMALL LETTER N WITH TILDE
            return 0x00D1;	//	Ñ	0xc3 0x91	LATIN CAPITAL LETTER N WITH TILDE

        case 0x00F2:		//	ò	0xc3 0xb2	LATIN SMALL LETTER O WITH GRAVE
            return 0x00D2;	//	Ò	0xc3 0x92	LATIN CAPITAL LETTER O WITH GRAVE

        case 0x00F3:		//	ó	0xc3 0xb3	LATIN SMALL LETTER O WITH ACUTE
            return 0x00D3;	//	Ó	0xc3 0x93	LATIN CAPITAL LETTER O WITH ACUTE

        case 0x00F4:		//	ô	0xc3 0xb4	LATIN SMALL LETTER O WITH CIRCUMFLEX
            return 0x00D4;	//	Ô	0xc3 0x94	LATIN CAPITAL LETTER O WITH CIRCUMFLEX

        case 0x00F5:		//	õ	0xc3 0xb5	LATIN SMALL LETTER O WITH TILDE
            return 0x00D5;	//	Õ	0xc3 0x95	LATIN CAPITAL LETTER O WITH TILDE

        case 0x00F6:		//	ö	0xc3 0xb6	LATIN SMALL LETTER O WITH DIAERESIS
            return 0x00D6;	//	Ö	0xc3 0x96	LATIN CAPITAL LETTER O WITH DIAERESIS

        case 0x00F8:		//	ø	0xc3 0xb8	LATIN SMALL LETTER O WITH STROKE
            return 0x00D8;	//	Ø	0xc3 0x98	LATIN CAPITAL LETTER O WITH STROKE

        case 0x00F9:		//	ù	0xc3 0xb9	LATIN SMALL LETTER U WITH GRAVE
            return 0x00D9;	//	Ù	0xc3 0x99	LATIN CAPITAL LETTER U WITH GRAVE

        case 0x00FA:		//	ú	0xc3 0xba	LATIN SMALL LETTER U WITH ACUTE
            return 0x00DA;	//	Ú	0xc3 0x9a	LATIN CAPITAL LETTER U WITH ACUTE

        case 0x00FB:		//	û	0xc3 0xbb	LATIN SMALL LETTER U WITH CIRCUMFLEX
            return 0x00DB;	//	Û	0xc3 0x9b	LATIN CAPITAL LETTER U WITH CIRCUMFLEX

        case 0x00FC:		//	ü	0xc3 0xbc	LATIN SMALL LETTER U WITH DIAERESIS
            return 0x00DC;	//	Ü	0xc3 0x9c	LATIN CAPITAL LETTER U WITH DIAERESIS

        case 0x00FD:		//	ý	0xc3 0xbd	LATIN SMALL LETTER Y WITH ACUTE
            return 0x00DD;	//	Ý	0xc3 0x9d	LATIN CAPITAL LETTER Y WITH ACUTE

        case 0x00FE:		//	þ	0xc3 0xbe	LATIN SMALL LETTER THORN
            return 0x00DE;	//	Þ	0xc3 0x9e	LATIN CAPITAL LETTER THORN

        case 0x00FF:		//	ÿ	0xc3 0xbf	LATIN SMALL LETTER Y WITH DIAERESIS
            return 0x0178;	//	Ÿ	0xc5 0xb8	LATIN CAPITAL LETTER Y WITH DIAERESIS

        case 0x0101:		//	ā	0xc4 0x81	LATIN SMALL LETTER A WITH MACRON
            return 0x0100;	//	Ā	0xc4 0x80	LATIN CAPITAL LETTER A WITH MACRON

        case 0x0103:		//	ă	0xc4 0x83	LATIN SMALL LETTER A WITH BREVE
            return 0x0102;	//	Ă	0xc4 0x82	LATIN CAPITAL LETTER A WITH BREVE

        case 0x0105:		//	ą	0xc4 0x85	LATIN SMALL LETTER A WITH OGONEK
            return 0x0104;	//	Ą	0xc4 0x84	LATIN CAPITAL LETTER A WITH OGONEK

        case 0x0107:		//	ć	0xc4 0x87	LATIN SMALL LETTER C WITH ACUTE
            return 0x0106;	//	Ć	0xc4 0x86	LATIN CAPITAL LETTER C WITH ACUTE

        case 0x0109:		//	ĉ	0xc4 0x89	LATIN SMALL LETTER C WITH CIRCUMFLEX
            return 0x0108;	//	Ĉ	0xc4 0x88	LATIN CAPITAL LETTER C WITH CIRCUMFLEX

        case 0x010B:		//	ċ	0xc4 0x8b	LATIN SMALL LETTER C WITH DOT ABOVE
            return 0x010A;	//	Ċ	0xc4 0x8a	LATIN CAPITAL LETTER C WITH DOT ABOVE

        case 0x010D:		//	č	0xc4 0x8d	LATIN SMALL LETTER C WITH CARON
            return 0x010C;	//	Č	0xc4 0x8c	LATIN CAPITAL LETTER C WITH CARON

        case 0x010F:		//	ď	0xc4 0x8f	LATIN SMALL LETTER D WITH CARON
            return 0x010E;	//	Ď	0xc4 0x8e	LATIN CAPITAL LETTER D WITH CARON

        case 0x0111:		//	đ	0xc4 0x91	LATIN SMALL LETTER D WITH STROKE
            return 0x0110;	//	Đ	0xc4 0x90	LATIN CAPITAL LETTER D WITH STROKE

        case 0x0113:		//	ē	0xc4 0x93	LATIN SMALL LETTER E WITH MACRON
            return 0x0112;	//	Ē	0xc4 0x92	LATIN CAPITAL LETTER E WITH MACRON

        case 0x0115:		//	ĕ	0xc4 0x95	LATIN SMALL LETTER E WITH BREVE
            return 0x0114;	//	Ĕ	0xc4 0x94	LATIN CAPITAL LETTER E WITH BREVE

        case 0x0117:		//	ė	0xc4 0x97	LATIN SMALL LETTER E WITH DOT ABOVE
            return 0x0116;	//	Ė	0xc4 0x96	LATIN CAPITAL LETTER E WITH DOT ABOVE

        case 0x0119:		//	ę	0xc4 0x99	LATIN SMALL LETTER E WITH OGONEK
            return 0x0118;	//	Ę	0xc4 0x98	LATIN CAPITAL LETTER E WITH OGONEK

        case 0x011B:		//	ě	0xc4 0x9b	LATIN SMALL LETTER E WITH CARON
            return 0x011A;	//	Ě	0xc4 0x9a	LATIN CAPITAL LETTER E WITH CARON

        case 0x011D:		//	ĝ	0xc4 0x9d	LATIN SMALL LETTER G WITH CIRCUMFLEX
            return 0x011C;	//	Ĝ	0xc4 0x9c	LATIN CAPITAL LETTER G WITH CIRCUMFLEX

        case 0x011F:		//	ğ	0xc4 0x9f	LATIN SMALL LETTER G WITH BREVE
            return 0x011E;	//	Ğ	0xc4 0x9e	LATIN CAPITAL LETTER G WITH BREVE

        case 0x0121:		//	ġ	0xc4 0xa1	LATIN SMALL LETTER G WITH DOT ABOVE
            return 0x0120;	//	Ġ	0xc4 0xa0	LATIN CAPITAL LETTER G WITH DOT ABOVE

        case 0x0123:		//	ģ	0xc4 0xa3	LATIN SMALL LETTER G WITH CEDILLA
            return 0x0122;	//	Ģ	0xc4 0xa2	LATIN CAPITAL LETTER G WITH CEDILLA

        case 0x0125:		//	ĥ	0xc4 0xa5	LATIN SMALL LETTER H WITH CIRCUMFLEX
            return 0x0124;	//	Ĥ	0xc4 0xa4	LATIN CAPITAL LETTER H WITH CIRCUMFLEX

        case 0x0127:		//	ħ	0xc4 0xa7	LATIN SMALL LETTER H WITH STROKE
            return 0x0126;	//	Ħ	0xc4 0xa6	LATIN CAPITAL LETTER H WITH STROKE

        case 0x0129:		//	ĩ	0xc4 0xa9	LATIN SMALL LETTER I WITH TILDE
            return 0x0128;	//	Ĩ	0xc4 0xa8	LATIN CAPITAL LETTER I WITH TILDE

        case 0x012B:		//	ī	0xc4 0xab	LATIN SMALL LETTER I WITH MACRON
            return 0x012A;	//	Ī	0xc4 0xaa	LATIN CAPITAL LETTER I WITH MACRON

        case 0x012D:		//	ĭ	0xc4 0xad	LATIN SMALL LETTER I WITH BREVE
            return 0x012C;	//	Ĭ	0xc4 0xac	LATIN CAPITAL LETTER I WITH BREVE

        case 0x012F:		//	į	0xc4 0xaf	LATIN SMALL LETTER I WITH OGONEK
            return 0x012E;	//	Į	0xc4 0xae	LATIN CAPITAL LETTER I WITH OGONEK

        case 0x0131:		//	ı	0xc4 0xb1	LATIN SMALL LETTER DOTLESS I
            return 0x0049;	//	I	0x49 LATIN CAPITAL LETTER I

        case 0x0133:		//	ĳ	0xc4 0xb3	LATIN SMALL LETTER LIGATURE IJ
            return 0x0132;	//	Ĳ	0xc4 0xb2	LATIN CAPITAL LETTER LIGATURE IJ

        case 0x0135:		//	ĵ	0xc4 0xb5	LATIN SMALL LETTER J WITH CIRCUMFLEX
            return 0x0134;	//	Ĵ	0xc4 0xb4	LATIN CAPITAL LETTER J WITH CIRCUMFLEX

        case 0x0137:		//	ķ	0xc4 0xb7	LATIN SMALL LETTER K WITH CEDILLA
            return 0x0136;	//	Ķ	0xc4 0xb6	LATIN CAPITAL LETTER K WITH CEDILLA

        case 0x013A:		//	ĺ	0xc4 0xba	LATIN SMALL LETTER L WITH ACUTE
            return 0x0139;	//	Ĺ	0xc4 0xb9	LATIN CAPITAL LETTER L WITH ACUTE

        case 0x013C:		//	ļ	0xc4 0xbc	LATIN SMALL LETTER L WITH CEDILLA
            return 0x013B;	//	Ļ	0xc4 0xbb	LATIN CAPITAL LETTER L WITH CEDILLA

        case 0x013E:		//	ľ	0xc4 0xbe	LATIN SMALL LETTER L WITH CARON
            return 0x013D;	//	Ľ	0xc4 0xbd	LATIN CAPITAL LETTER L WITH CARON

        case 0x0140:		//	ŀ	0xc5 0x80	LATIN SMALL LETTER L WITH MIDDLE DOT
            return 0x013F;	//	Ŀ	0xc4 0xbf	LATIN CAPITAL LETTER L WITH MIDDLE DOT

        case 0x0142:		//	ł	0xc5 0x82	LATIN SMALL LETTER L WITH STROKE
            return 0x0141;	//	Ł	0xc5 0x81	LATIN CAPITAL LETTER L WITH STROKE

        case 0x0144:		//	ń	0xc5 0x84	LATIN SMALL LETTER N WITH ACUTE
            return 0x0143;	//	Ń	0xc5 0x83	LATIN CAPITAL LETTER N WITH ACUTE

        case 0x0146:		//	ņ	0xc5 0x86	LATIN SMALL LETTER N WITH CEDILLA
            return 0x0145;	//	Ņ	0xc5 0x85	LATIN CAPITAL LETTER N WITH CEDILLA

        case 0x0148:		//	ň	0xc5 0x88	LATIN SMALL LETTER N WITH CARON
            return 0x0147;	//	Ň	0xc5 0x87	LATIN CAPITAL LETTER N WITH CARON

        case 0x014B:		//	ŋ	0xc5 0x8b	LATIN SMALL LETTER ENG
            return 0x014A;	//	Ŋ	0xc5 0x8a	LATIN CAPITAL LETTER ENG

        case 0x014D:		//	ō	0xc5 0x8d	LATIN SMALL LETTER O WITH MACRON
            return 0x014C;	//	Ō	0xc5 0x8c	LATIN CAPITAL LETTER O WITH MACRON

        case 0x014F:		//	ŏ	0xc5 0x8f	LATIN SMALL LETTER O WITH BREVE
            return 0x014E;	//	Ŏ	0xc5 0x8e	LATIN CAPITAL LETTER O WITH BREVE

        case 0x0151:		//	ő	0xc5 0x91	LATIN SMALL LETTER O WITH DOUBLE ACUTE
            return 0x0150;	//	Ő	0xc5 0x90	LATIN CAPITAL LETTER O WITH DOUBLE ACUTE

        case 0x0153:		//	œ	0xc5 0x93	LATIN SMALL LETTER LIGATURE OE
            return 0x0152;	//	Œ	0xc5 0x92	LATIN CAPITAL LETTER LIGATURE OE

        case 0x0155:		//	ŕ	0xc5 0x95	LATIN SMALL LETTER R WITH ACUTE
            return 0x0154;	//	Ŕ	0xc5 0x94	LATIN CAPITAL LETTER R WITH ACUTE

        case 0x0157:		//	ŗ	0xc5 0x97	LATIN SMALL LETTER R WITH CEDILLA
            return 0x0156;	//	Ŗ	0xc5 0x96	LATIN CAPITAL LETTER R WITH CEDILLA

        case 0x0159:		//	ř	0xc5 0x99	LATIN SMALL LETTER R WITH CARON
            return 0x0158;	//	Ř	0xc5 0x98	LATIN CAPITAL LETTER R WITH CARON

        case 0x015B:		//	ś	0xc5 0x9b	LATIN SMALL LETTER S WITH ACUTE
            return 0x015A;	//	Ś	0xc5 0x9a	LATIN CAPITAL LETTER S WITH ACUTE

        case 0x015D:		//	ŝ	0xc5 0x9d	LATIN SMALL LETTER S WITH CIRCUMFLEX
            return 0x015C;	//	Ŝ	0xc5 0x9c	LATIN CAPITAL LETTER S WITH CIRCUMFLEX

        case 0x015F:		//	ş	0xc5 0x9f	LATIN SMALL LETTER S WITH CEDILLA
            return 0x015E;	//	Ş	0xc5 0x9e	LATIN CAPITAL LETTER S WITH CEDILLA

        case 0x0161:		//	š	0xc5 0xa1	LATIN SMALL LETTER S WITH CARON
            return 0x0160;	//	Š	0xc5 0xa0	LATIN CAPITAL LETTER S WITH CARON

        case 0x0163:		//	ţ	0xc5 0xa3	LATIN SMALL LETTER T WITH CEDILLA
            return 0x0162;	//	Ţ	0xc5 0xa2	LATIN CAPITAL LETTER T WITH CEDILLA

        case 0x0165:		//	ť	0xc5 0xa5	LATIN SMALL LETTER T WITH CARON
            return 0x0164;	//	Ť	0xc5 0xa4	LATIN CAPITAL LETTER T WITH CARON

        case 0x0167:		//	ŧ	0xc5 0xa7	LATIN SMALL LETTER T WITH STROKE
            return 0x0166;	//	Ŧ	0xc5 0xa6	LATIN CAPITAL LETTER T WITH STROKE

        case 0x0169:		//	ũ	0xc5 0xa9	LATIN SMALL LETTER U WITH TILDE
            return 0x0168;	//	Ũ	0xc5 0xa8	LATIN CAPITAL LETTER U WITH TILDE

        case 0x016B:		//	ū	0xc5 0xab	LATIN SMALL LETTER U WITH MACRON
            return 0x016A;	//	Ū	0xc5 0xaa	LATIN CAPITAL LETTER U WITH MACRON

        case 0x016D:		//	ŭ	0xc5 0xad	LATIN SMALL LETTER U WITH BREVE
            return 0x016C;	//	Ŭ	0xc5 0xac	LATIN CAPITAL LETTER U WITH BREVE

        case 0x016F:		//	ů	0xc5 0xaf	LATIN SMALL LETTER U WITH RING ABOVE
            return 0x016E;	//	Ů	0xc5 0xae	LATIN CAPITAL LETTER U WITH RING ABOVE

        case 0x0171:		//	ű	0xc5 0xb1	LATIN SMALL LETTER U WITH DOUBLE ACUTE
            return 0x0170;	//	Ű	0xc5 0xb0	LATIN CAPITAL LETTER U WITH DOUBLE ACUTE

        case 0x0173:		//	ų	0xc5 0xb3	LATIN SMALL LETTER U WITH OGONEK
            return 0x0172;	//	Ų	0xc5 0xb2	LATIN CAPITAL LETTER U WITH OGONEK

        case 0x0175:		//	ŵ	0xc5 0xb5	LATIN SMALL LETTER W WITH CIRCUMFLEX
            return 0x0174;	//	Ŵ	0xc5 0xb4	LATIN CAPITAL LETTER W WITH CIRCUMFLEX

        case 0x0177:		//	ŷ	0xc5 0xb7	LATIN SMALL LETTER Y WITH CIRCUMFLEX
            return 0x0176;	//	Ŷ	0xc5 0xb6	LATIN CAPITAL LETTER Y WITH CIRCUMFLEX

        case 0x017A:		//	ź	0xc5 0xba	LATIN SMALL LETTER Z WITH ACUTE
            return 0x0179;	//	Ź	0xc5 0xb9	LATIN CAPITAL LETTER Z WITH ACUTE

        case 0x017C:		//	ż	0xc5 0xbc	LATIN SMALL LETTER Z WITH DOT ABOVE
            return 0x017B;	//	Ż	0xc5 0xbb	LATIN CAPITAL LETTER Z WITH DOT ABOVE

        case 0x017E:		//	ž	0xc5 0xbe	LATIN SMALL LETTER Z WITH CARON
            return 0x017D;	//	Ž	0xc5 0xbd	LATIN CAPITAL LETTER Z WITH CARON

        case 0x0180:		//	ƀ	0xc6 0x80	LATIN SMALL LETTER B WITH STROKE
            return 0x0243;	//	Ƀ	0xc9 0x83	LATIN CAPITAL LETTER B WITH STROKE

        case 0x0183:		//	ƃ	0xc6 0x83	LATIN SMALL LETTER B WITH TOPBAR
            return 0x0182;	//	Ƃ	0xc6 0x82	LATIN CAPITAL LETTER B WITH TOPBAR

        case 0x0185:		//	ƅ	0xc6 0x85	LATIN SMALL LETTER TONE SIX
            return 0x0184;	//	Ƅ	0xc6 0x84	LATIN CAPITAL LETTER TONE SIX

        case 0x0188:		//	ƈ	0xc6 0x88	LATIN SMALL LETTER C WITH HOOK
            return 0x0187;	//	Ƈ	0xc6 0x87	LATIN CAPITAL LETTER C WITH HOOK

        case 0x018C:		//	ƌ	0xc6 0x8c	LATIN SMALL LETTER D WITH TOPBAR
            return 0x018B;	//	Ƌ	0xc6 0x8b	LATIN CAPITAL LETTER D WITH TOPBAR

        case 0x0192:		//	ƒ	0xc6 0x92	LATIN SMALL LETTER F WITH HOOK
            return 0x0191;	//	Ƒ	0xc6 0x91	LATIN CAPITAL LETTER F WITH HOOK

        case 0x0195:		//	ƕ	0xc6 0x95	LATIN SMALL LETTER HWAIR
            return 0x01F6;	//	Ƕ	0xc7 0xb6	LATIN CAPITAL LETTER HWAIR

        case 0x0199:		//	ƙ	0xc6 0x99	LATIN SMALL LETTER K WITH HOOK
            return 0x0198;	//	Ƙ	0xc6 0x98	LATIN CAPITAL LETTER K WITH HOOK

        case 0x019A:		//	ƚ	0xc6 0x9a	LATIN SMALL LETTER L WITH BAR
            return 0x023D;	//	Ƚ	0xc8 0xbd	LATIN CAPITAL LETTER L WITH BAR

        case 0x019E:		//	ƞ	0xc6 0x9e	LATIN SMALL LETTER N WITH LONG RIGHT LEG
            return 0x0220;	//	Ƞ	0xc8 0xa0	LATIN CAPITAL LETTER N WITH LONG RIGHT LEG

        case 0x01A1:		//	ơ	0xc6 0xa1	LATIN SMALL LETTER O WITH HORN
            return 0x01A0;	//	Ơ	0xc6 0xa0	LATIN CAPITAL LETTER O WITH HORN

        case 0x01A3:		//	ƣ	0xc6 0xa3	LATIN SMALL LETTER OI
            return 0x01A2;	//	Ƣ	0xc6 0xa2	LATIN CAPITAL LETTER OI

        case 0x01A5:		//	ƥ	0xc6 0xa5	LATIN SMALL LETTER P WITH HOOK
            return 0x01A4;	//	Ƥ	0xc6 0xa4	LATIN CAPITAL LETTER P WITH HOOK

        case 0x01A8:		//	ƨ	0xc6 0xa8	LATIN SMALL LETTER TONE TWO
            return 0x01A7;	//	Ƨ	0xc6 0xa7	LATIN CAPITAL LETTER TONE TWO

        case 0x01AD:		//	ƭ	0xc6 0xad	LATIN SMALL LETTER T WITH HOOK
            return 0x01AC;	//	Ƭ	0xc6 0xac	LATIN CAPITAL LETTER T WITH HOOK

        case 0x01B0:		//	ư	0xc6 0xb0	LATIN SMALL LETTER U WITH HORN
            return 0x01AF;	//	Ư	0xc6 0xaf	LATIN CAPITAL LETTER U WITH HORN

        case 0x01B4:		//	ƴ	0xc6 0xb4	LATIN SMALL LETTER Y WITH HOOK
            return 0x01B3;	//	Ƴ	0xc6 0xb3	LATIN CAPITAL LETTER Y WITH HOOK

        case 0x01B6:		//	ƶ	0xc6 0xb6	LATIN SMALL LETTER Z WITH STROKE
            return 0x01B5;	//	Ƶ	0xc6 0xb5	LATIN CAPITAL LETTER Z WITH STROKE

        case 0x01B9:		//	ƹ	0xc6 0xb9	LATIN SMALL LETTER EZH REVERSED
            return 0x01B8;	//	Ƹ	0xc6 0xb8	LATIN CAPITAL LETTER EZH REVERSED

        case 0x01BD:		//	ƽ	0xc6 0xbd	LATIN SMALL LETTER TONE FIVE
            return 0x01BC;	//	Ƽ	0xc6 0xbc	LATIN CAPITAL LETTER TONE FIVE

        case 0x01BF:		//	ƿ	0xc6 0xbf	LATIN SMALL LETTER WYNN
            return 0x01F7;	//	Ƿ	0xc7 0xb7	LATIN CAPITAL LETTER WYNN

        case 0x01C5:		//	ǅ	0xc7 0x85	LATIN CAPITAL LETTER D WITH SMALL Z WITH CARON
        case 0x01C6:		//	ǆ	0xc7 0x86	LATIN SMALL LETTER DZ WITH CARON
            return 0x01C4;	//	Ǆ	0xc7 0x84	LATIN CAPITAL LETTER DZ WITH CARON

        case 0x01C8:		//	ǈ	0xc7 0x88	LATIN CAPITAL LETTER L WITH SMALL J
        case 0x01C9:		//	ǉ	0xc7 0x89	LATIN SMALL LETTER LJ
            return 0x01C7;	//	Ǉ	0xc7 0x87	LATIN CAPITAL LETTER LJ

        case 0x01CB:		//	ǋ	0xc7 0x8b	LATIN CAPITAL LETTER N WITH SMALL J
        case 0x01CC:		//	ǌ	0xc7 0x8c	LATIN SMALL LETTER NJ
            return 0x01CA;	//	Ǌ	0xc7 0x8a	LATIN CAPITAL LETTER NJ

        case 0x01CE:		//	ǎ	0xc7 0x8e	LATIN SMALL LETTER A WITH CARON
            return 0x01CD;	//	Ǎ	0xc7 0x8d	LATIN CAPITAL LETTER A WITH CARON

        case 0x01D0:		//	ǐ	0xc7 0x90	LATIN SMALL LETTER I WITH CARON
            return 0x01CF;	//	Ǐ	0xc7 0x8f	LATIN CAPITAL LETTER I WITH CARON

        case 0x01D2:		//	ǒ	0xc7 0x92	LATIN SMALL LETTER O WITH CARON
            return 0x01D1;	//	Ǒ	0xc7 0x91	LATIN CAPITAL LETTER O WITH CARON

        case 0x01D4:		//	ǔ	0xc7 0x94	LATIN SMALL LETTER U WITH CARON
            return 0x01D3;	//	Ǔ	0xc7 0x93	LATIN CAPITAL LETTER U WITH CARON

        case 0x01D6:		//	ǖ	0xc7 0x96	LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
            return 0x01D5;	//	Ǖ	0xc7 0x95	LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON

        case 0x01D8:		//	ǘ	0xc7 0x98	LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
            return 0x01D7;	//	Ǘ	0xc7 0x97	LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE

        case 0x01DA:		//	ǚ	0xc7 0x9a	LATIN SMALL LETTER U WITH DIAERESIS AND CARON
            return 0x01D9;	//	Ǚ	0xc7 0x99	LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON

        case 0x01DC:		//	ǜ	0xc7 0x9c	LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
            return 0x01DB;	//	Ǜ	0xc7 0x9b	LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE

        case 0x01DD:		//	ǝ	0xC7 0x9D	LATIN SMALL LETTER TURNED E (look mid page 297 https://www.unicode.org/versions/Unicode14.0.0/ch07.pdf)
            return 0x018E;	//	Ǝ	0xc6 0x8e	LATIN CAPITAL LETTER REVERSED E

        case 0x01DF:		//	ǟ	0xc7 0x9f	LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
            return 0x01DE;	//	Ǟ	0xc7 0x9e	LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON

        case 0x01E1:		//	ǡ	0xc7 0xa1	LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON
            return 0x01E0;	//	Ǡ	0xc7 0xa0	LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON

        case 0x01E3:		//	ǣ	0xc7 0xa3	LATIN SMALL LETTER AE WITH MACRON
            return 0x01E2;	//	Ǣ	0xc7 0xa2	LATIN CAPITAL LETTER AE WITH MACRON

        case 0x01E5:		//	ǥ	0xc7 0xa5	LATIN SMALL LETTER G WITH STROKE
            return 0x01E4;	//	Ǥ	0xc7 0xa4	LATIN CAPITAL LETTER G WITH STROKE

        case 0x01E7:		//	ǧ	0xc7 0xa7	LATIN SMALL LETTER G WITH CARON
            return 0x01E6;	//	Ǧ	0xc7 0xa6	LATIN CAPITAL LETTER G WITH CARON

        case 0x01E9:		//	ǩ	0xc7 0xa9	LATIN SMALL LETTER K WITH CARON
            return 0x01E8;	//	Ǩ	0xc7 0xa8	LATIN CAPITAL LETTER K WITH CARON

        case 0x01EB:		//	ǫ	0xc7 0xab	LATIN SMALL LETTER O WITH OGONEK
            return 0x01EA;	//	Ǫ	0xc7 0xaa	LATIN CAPITAL LETTER O WITH OGONEK

        case 0x01ED:		//	ǭ	0xc7 0xad	LATIN SMALL LETTER O WITH OGONEK AND MACRON
            return 0x01EC;	//	Ǭ	0xc7 0xac	LATIN CAPITAL LETTER O WITH OGONEK AND MACRON

        case 0x01EF:		//	ǯ	0xc7 0xaf	LATIN SMALL LETTER EZH WITH CARON
            return 0x01EE;	//	Ǯ	0xc7 0xae	LATIN CAPITAL LETTER EZH WITH CARON

        case 0x01F2:		//	ǲ	0xc7 0xb2	LATIN CAPITAL LETTER D WITH SMALL Z
        case 0x01F3:		//	ǳ	0xc7 0xb3	LATIN SMALL LETTER DZ
            return 0x01F1;	//	Ǳ	0xc7 0xb1	LATIN CAPITAL LETTER DZ

        case 0x01F5:		//	ǵ	0xc7 0xb5	LATIN SMALL LETTER G WITH ACUTE
            return 0x01F4;	//	Ǵ	0xc7 0xb4	LATIN CAPITAL LETTER G WITH ACUTE

        case 0x01F9:		//	ǹ	0xc7 0xb9	LATIN SMALL LETTER N WITH GRAVE
            return 0x01F8;	//	Ǹ	0xc7 0xb8	LATIN CAPITAL LETTER N WITH GRAVE

        case 0x01FB:		//	ǻ	0xc7 0xbb	LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
            return 0x01FA;	//	Ǻ	0xc7 0xba	LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE

        case 0x01FD:		//	ǽ	0xc7 0xbd	LATIN SMALL LETTER AE WITH ACUTE
            return 0x01FC;	//	Ǽ	0xc7 0xbc	LATIN CAPITAL LETTER AE WITH ACUTE

        case 0x01FF:		//	ǿ	0xc7 0xbf	LATIN SMALL LETTER O WITH STROKE AND ACUTE
            return 0x01FE;	//	Ǿ	0xc7 0xbe	LATIN CAPITAL LETTER O WITH STROKE AND ACUTE

        case 0x0201:		//	ȁ	0xc8 0x81	LATIN SMALL LETTER A WITH DOUBLE GRAVE
            return 0x0200;	//	Ȁ	0xc8 0x80	LATIN CAPITAL LETTER A WITH DOUBLE GRAVE

        case 0x0203:		//	ȃ	0xc8 0x83	LATIN SMALL LETTER A WITH INVERTED BREVE
            return 0x0202;	//	Ȃ	0xc8 0x82	LATIN CAPITAL LETTER A WITH INVERTED BREVE

        case 0x0205:		//	ȅ	0xc8 0x85	LATIN SMALL LETTER E WITH DOUBLE GRAVE
            return 0x0204;	//	Ȅ	0xc8 0x84	LATIN CAPITAL LETTER E WITH DOUBLE GRAVE

        case 0x0207:		//	ȇ	0xc8 0x87	LATIN SMALL LETTER E WITH INVERTED BREVE
            return 0x0206;	//	Ȇ	0xc8 0x86	LATIN CAPITAL LETTER E WITH INVERTED BREVE

        case 0x0209:		//	ȉ	0xc8 0x89	LATIN SMALL LETTER I WITH DOUBLE GRAVE
            return 0x0208;	//	Ȉ	0xc8 0x88	LATIN CAPITAL LETTER I WITH DOUBLE GRAVE

        case 0x020B:		//	ȋ	0xc8 0x8b	LATIN SMALL LETTER I WITH INVERTED BREVE
            return 0x020A;	//	Ȋ	0xc8 0x8a	LATIN CAPITAL LETTER I WITH INVERTED BREVE

        case 0x020D:		//	ȍ	0xc8 0x8d	LATIN SMALL LETTER O WITH DOUBLE GRAVE
            return 0x020C;	//	Ȍ	0xc8 0x8c	LATIN CAPITAL LETTER O WITH DOUBLE GRAVE

        case 0x020F:		//	ȏ	0xc8 0x8f	LATIN SMALL LETTER O WITH INVERTED BREVE
            return 0x020E;	//	Ȏ	0xc8 0x8e	LATIN CAPITAL LETTER O WITH INVERTED BREVE

        case 0x0211:		//	ȑ	0xc8 0x91	LATIN SMALL LETTER R WITH DOUBLE GRAVE
            return 0x0210;	//	Ȑ	0xc8 0x90	LATIN CAPITAL LETTER R WITH DOUBLE GRAVE

        case 0x0213:		//	ȓ	0xc8 0x93	LATIN SMALL LETTER R WITH INVERTED BREVE
            return 0x0212;	//	Ȓ	0xc8 0x92	LATIN CAPITAL LETTER R WITH INVERTED BREVE

        case 0x0215:		//	ȕ	0xc8 0x95	LATIN SMALL LETTER U WITH DOUBLE GRAVE
            return 0x0214;	//	Ȕ	0xc8 0x94	LATIN CAPITAL LETTER U WITH DOUBLE GRAVE

        case 0x0217:		//	ȗ	0xc8 0x97	LATIN SMALL LETTER U WITH INVERTED BREVE
            return 0x0216;	//	Ȗ	0xc8 0x96	LATIN CAPITAL LETTER U WITH INVERTED BREVE

        case 0x0219:		//	ș	0xc8 0x99	LATIN SMALL LETTER S WITH COMMA BELOW
            return 0x0218;	//	Ș	0xc8 0x98	LATIN CAPITAL LETTER S WITH COMMA BELOW

        case 0x021B:		//	ț	0xc8 0x9b	LATIN SMALL LETTER T WITH COMMA BELOW
            return 0x021A;	//	Ț	0xc8 0x9a	LATIN CAPITAL LETTER T WITH COMMA BELOW

        case 0x021D:		//	ȝ	0xc8 0x9d	LATIN SMALL LETTER YOGH
            return 0x021C;	//	Ȝ	0xc8 0x9c	LATIN CAPITAL LETTER YOGH

        case 0x021F:		//	ȟ	0xc8 0x9f	LATIN SMALL LETTER H WITH CARON
            return 0x021E;	//	Ȟ	0xc8 0x9e	LATIN CAPITAL LETTER H WITH CARON

        case 0x0223:		//	ȣ	0xc8 0xa3	LATIN SMALL LETTER OU
            return 0x0222;	//	Ȣ	0xc8 0xa2	LATIN CAPITAL LETTER OU

        case 0x0225:		//	ȥ	0xc8 0xa5	LATIN SMALL LETTER Z WITH HOOK
            return 0x0224;	//	Ȥ	0xc8 0xa4	LATIN CAPITAL LETTER Z WITH HOOK

        case 0x0227:		//	ȧ	0xc8 0xa7	LATIN SMALL LETTER A WITH DOT ABOVE
            return 0x0226;	//	Ȧ	0xc8 0xa6	LATIN CAPITAL LETTER A WITH DOT ABOVE

        case 0x0229:		//	ȩ	0xc8 0xa9	LATIN SMALL LETTER E WITH CEDILLA
            return 0x0228;	//	Ȩ	0xc8 0xa8	LATIN CAPITAL LETTER E WITH CEDILLA

        case 0x022B:		//	ȫ	0xc8 0xab	LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
            return 0x022A;	//	Ȫ	0xc8 0xaa	LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON

        case 0x022D:		//	ȭ	0xc8 0xad	LATIN SMALL LETTER O WITH TILDE AND MACRON
            return 0x022C;	//	Ȭ	0xc8 0xac	LATIN CAPITAL LETTER O WITH TILDE AND MACRON

        case 0x022F:		//	ȯ	0xc8 0xaf	LATIN SMALL LETTER O WITH DOT ABOVE
            return 0x022E;	//	Ȯ	0xc8 0xae	LATIN CAPITAL LETTER O WITH DOT ABOVE

        case 0x0231:		//	ȱ	0xc8 0xb1	LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON
            return 0x0230;	//	Ȱ	0xc8 0xb0	LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON

        case 0x0233:		//	ȳ	0xc8 0xb3	LATIN SMALL LETTER Y WITH MACRON
            return 0x0232;	//	Ȳ	0xc8 0xb2	LATIN CAPITAL LETTER Y WITH MACRON

        case 0x023C:		//	ȼ	0xc8 0xbc	LATIN SMALL LETTER C WITH STROKE
            return 0x023B;	//	Ȼ	0xc8 0xbb	LATIN CAPITAL LETTER C WITH STROKE

        case 0x023F:		//	ȿ	0xc8 0xbf	LATIN SMALL LETTER S WITH SWASH TAIL
            return 0x2C7E;	//	Ȿ	0xe2 0xb1 0xbe	LATIN CAPITAL LETTER S WITH SWASH TAIL

        case 0x0240:		//	ɀ	0xc9 0x80	LATIN SMALL LETTER Z WITH SWASH TAIL
            return 0x2C7F;	//	Ɀ	0xe2 0xb1 0xbf	LATIN CAPITAL LETTER Z WITH SWASH TAIL

        case 0x0242:		//	ɂ	0xc9 0x82	LATIN SMALL LETTER GLOTTAL STOP
            return 0x0241;	//	Ɂ	0xc9 0x81	LATIN CAPITAL LETTER GLOTTAL STOP

        case 0x0247:		//	ɇ	0xc9 0x87	LATIN SMALL LETTER E WITH STROKE
            return 0x0246;	//	Ɇ	0xc9 0x86	LATIN CAPITAL LETTER E WITH STROKE

        case 0x0249:		//	ɉ	0xc9 0x89	LATIN SMALL LETTER J WITH STROKE
            return 0x0248;	//	Ɉ	0xc9 0x88	LATIN CAPITAL LETTER J WITH STROKE

        case 0x024B:		//	ɋ	0xc9 0x8b	LATIN SMALL LETTER SMALL Q WITH HOOK TAIL
            return 0x024A;	//	Ɋ	0xc9 0x8a	LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL

        case 0x024D:		//	ɍ	0xc9 0x8d	LATIN SMALL LETTER R WITH STROKE
            return 0x024C;	//	Ɍ	0xc9 0x8c	LATIN CAPITAL LETTER R WITH STROKE

        case 0x024F:		//	ɏ	0xc9 0x8f	LATIN SMALL LETTER Y WITH STROKE
            return 0x024E;	//	Ɏ	0xc9 0x8e	LATIN CAPITAL LETTER Y WITH STROKE

        case 0x0250:		//	ɐ	0xc9 0x90	LATIN SMALL LETTER TURNED A
            return 0x2C6F;	//	Ɐ	0xe2 0xb1 0xaf	LATIN CAPITAL LETTER TURNED A

        case 0x0251:		//	ɑ	0xc9 0x91	LATIN SMALL LETTER ALPHA
            return 0x2C6D;	//	Ɑ	0xe2 0xb1 0xad	LATIN CAPITAL LETTER ALPHA

        case 0x0252:		//	ɒ	0xc9 0x92	LATIN SMALL LETTER TURNED ALPHA
            return 0x2C70;	//	Ɒ	0xe2 0xb1 0xb0	LATIN CAPITAL LETTER TURNED ALPHA

        case 0x0253:		//	ɓ	0xc9 0x93	LATIN SMALL LETTER B WITH HOOK
            return 0x0181;	//	Ɓ	0xc6 0x81	LATIN CAPITAL LETTER B WITH HOOK

        case 0x0254:		//	ɔ	0xc9 0x94	LATIN SMALL LETTER OPEN O
            return 0x0186;	//	Ɔ	0xc6 0x86	LATIN CAPITAL LETTER OPEN O

        case 0x0256:		//	ɖ	0xc9 0x96	LATIN SMALL LETTER AFRICAN D
            return 0x0189;	//	Ɖ	0xc6 0x89	LATIN CAPITAL LETTER AFRICAN D

        case 0x0257:		//	ɗ	0xc9 0x97	LATIN SMALL LETTER D WITH HOOK
            return 0x018A;	//	Ɗ	0xc6 0x8a	LATIN CAPITAL LETTER D WITH HOOK

        case 0x0259:		//	ə	0xc9 0x99	LATIN SMALL LETTER SCHWA
            return 0x018F;	//	Ə	0xc6 0x8f	LATIN CAPITAL LETTER SCHWA

        case 0x025B:		//	ɛ	0xc9 0x9b	LATIN SMALL LETTER OPEN E
            return 0x0190;	//	Ɛ	0xc6 0x90	LATIN CAPITAL LETTER OPEN E

        case 0x025C:		//	ɜ	0xc9 0x9c	LATIN SMALL LETTER REVERSED OPEN E
            return 0xA7AB;	//	Ɜ	0xea 0x9e 0xab	LATIN CAPITAL LETTER REVERSED OPEN E

        case 0x0260:		//	ɠ	0xc9 0xa0	LATIN SMALL LETTER G WITH HOOK
            return 0x0193;	//	Ɠ	0xc6 0x93	LATIN CAPITAL LETTER G WITH HOOK

        case 0x0261:		//	ɡ	0xc9 0xa1	LATIN SMALL LETTER SCRIPT G
            return 0xA7AC;	//	Ɡ	0xea 0x9e 0xac	LATIN CAPITAL LETTER SCRIPT G

        case 0x0263:		//	ɣ	0xc9 0xa3	LATIN SMALL LETTER GAMMA
            return 0x0194;	//	Ɣ	0xc6 0x94	LATIN CAPITAL LETTER GAMMA

        case 0x0265:		//	ɥ	0xc9 0xa5	LATIN SMALL LETTER TURNED H
            return 0xA78D;	//	Ɥ	0xea 0x9e 0x8d	LATIN CAPITAL LETTER TURNED H

        case 0x0266:		//	ɦ	0xc9 0xa6	LATIN SMALL LETTER H WITH HOOK
            return 0xA7AA;	//	Ɦ	0xea 0x9e 0xaa	LATIN CAPITAL LETTER H WITH HOOK

        case 0x0268:		//	ɨ	0xc9 0xa8	LATIN SMALL LETTER I WITH STROKE
            return 0x0197;	//	Ɨ	0xc6 0x97	LATIN CAPITAL LETTER I WITH STROKE

        case 0x0269:		//	ɩ	0xc9 0xa9	LATIN SMALL LETTER IOTA
            return 0x0196;	//	Ɩ	0xc6 0x96	LATIN CAPITAL LETTER IOTA

        case 0x026A:		//	ɪ	0xc9 0xaa	LATIN SMALL LETTER SMALL CAPITAL I
            return 0xA7AE;	//	Ɪ	0xea 0x9e 0xae	LATIN CAPITAL LETTER SMALL CAPITAL I

        case 0x026B:		//	ɫ	0xc9 0xab	LATIN SMALL LETTER L WITH MIDDLE TILDE
            return 0x2C62;	//	Ɫ	0xe2 0xb1 0xa2	LATIN CAPITAL LETTER L WITH MIDDLE TILDE

        case 0x026C:		//	ɬ	0xc9 0xac	LATIN SMALL LETTER L WITH BELT
            return 0xA7AD;	//	Ɬ	0xea 0x9e 0xad	LATIN CAPITAL LETTER L WITH BELT

        case 0x026F:		//	ɯ	0xc9 0xaf	LATIN SMALL LETTER TURNED M
            return 0x019C;	//	Ɯ	0xc6 0x9c	LATIN CAPITAL LETTER TURNED M

        case 0x0271:		//	ɱ	0xc9 0xb1	LATIN SMALL LETTER M WITH HOOK
            return 0x2C6E;	//	Ɱ	0xe2 0xb1 0xae	LATIN CAPITAL LETTER M WITH HOOK

        case 0x0272:		//	ɲ	0xc9 0xb2	LATIN SMALL LETTER N WITH LEFT HOOK
            return 0x019D;	//	Ɲ	0xc6 0x9d	LATIN CAPITAL LETTER N WITH LEFT HOOK

        case 0x0275:		//	ɵ	0xc9 0xb5	LATIN SMALL LETTER O WITH MIDDLE TILDE
            return 0x019F;	//	Ɵ	0xc6 0x9f	LATIN CAPITAL LETTER O WITH MIDDLE TILDE

        case 0x027D:		//	ɽ	0xc9 0xbd	LATIN SMALL LETTER R WITH TAIL
            return 0x2C64;	//	Ɽ	0xe2 0xb1 0xa4	LATIN CAPITAL LETTER R WITH TAIL

        case 0x0282:		//	ʂ	0xca 0x82	LATIN SMALL LETTER S WITH HOOK
            return 0xA7C5;	//	Ʂ	0xea 0x9f 0x85	LATIN CAPITAL LETTER S WITH HOOK

        case 0x0283:		//	ʃ	0xca 0x83	LATIN SMALL LETTER ESH
            return 0x01A9;	//	Ʃ	0xc6 0xa9	LATIN CAPITAL LETTER ESH

        case 0x0287:		//	ʇ	0xca 0x87	LATIN SMALL LETTER TURNED T
            return 0xA7B1;	//	Ʇ	0xea 0x9e 0xb1	LATIN CAPITAL LETTER TURNED T

        case 0x0288:		//	ʈ	0xca 0x88	LATIN SMALL LETTER T WITH RETROFLEX HOOK
            return 0x01AE;	//	Ʈ	0xc6 0xae	LATIN CAPITAL LETTER T WITH RETROFLEX HOOK

        case 0x0289:		//	ʉ	0xca 0x89	LATIN SMALL LETTER U BAR
            return 0x0244;	//	Ʉ	0xc9 0x84	LATIN CAPITAL LETTER U BAR

        case 0x028A:		//	ʊ	0xca 0x8a	LATIN SMALL LETTER UPSILON
            return 0x01B1;	//	Ʊ	0xc6 0xb1	LATIN CAPITAL LETTER UPSILON

        case 0x028B:		//	ʋ	0xca 0x8b	LATIN SMALL LETTER V WITH HOOK
            return 0x01B2;	//	Ʋ	0xc6 0xb2	LATIN CAPITAL LETTER V WITH HOOK

        case 0x028C:		//	ʌ	0xca 0x8c	LATIN SMALL LETTER TURNED V
            return 0x0245;	//	Ʌ	0xc9 0x85	LATIN CAPITAL LETTER TURNED V

        case 0x0292:		//	ʒ	0xca 0x92	LATIN SMALL LETTER EZH
            return 0x01B7;	//	Ʒ	0xc6 0xb7	LATIN CAPITAL LETTER EZH

        case 0x029D:		//	ʝ	0xca 0x9d	LATIN SMALL LETTER J WITH CROSSED-TAIL
            return 0xA7B2;	//	Ʝ	0xea 0x9e 0xb2	LATIN CAPITAL LETTER J WITH CROSSED-TAIL

        case 0x029E:		//	ʞ	0xca 0x9e	LATIN SMALL LETTER TURNED K
            return 0xA7B0;	//	Ʞ	0xea 0x9e 0xb0	LATIN CAPITAL LETTER TURNED K

#endif
#if UTF_SUPPORT_GREEK == 1
        case 0x0371:		//	ͱ	0xcd 0xb1	GREEK SMALL LETTER HETA
            return 0x0370;	//	Ͱ	0xcd 0xb0	GREEK CAPITAL LETTER HETA

        case 0x0373:		//	ͳ	0xcd 0xb3	GREEK SMALL LETTER ARCHAIC SAMPI
            return 0x0372;	//	Ͳ	0xcd 0xb2	GREEK CAPITAL LETTER ARCHAIC SAMPI

        case 0x0377:		//	ͷ	0xcd 0xb7	GREEK SMALL LETTER PAMPHYLIAN DIGAMMA
            return 0x0376;	//	Ͷ	0xcd 0xb6	GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA

        case 0x037B:		//	ͻ	0xcd 0xbb	GREEK SMALL LETTER REVERSED LUNATE SIGMA SYMBOL
            return 0x03FD;	//	Ͻ	0xcf 0xbd	GREEK CAPITAL LETTER REVERSED LUNATE SIGMA SYMBOL

        case 0x037C:		//	ͼ	0xcd 0xbc	GREEK SMALL LETTER DOTTED LUNATE SIGMA SYMBOL
            return 0x03FE;	//	Ͼ	0xcf 0xbe	GREEK CAPITAL LETTER DOTTED LUNATE SIGMA SYMBOL

        case 0x037D:		//	ͽ	0xcd 0xbd	GREEK SMALL LETTER REVERSED DOTTED LUNATE SIGMA SYMBOL
            return 0x03FF;	//	Ͽ	0xcf 0xbf	GREEK CAPITAL LETTER REVERSED DOTTED LUNATE SIGMA SYMBOL

        case 0x03AC:		//	ά	0xce 0xac	GREEK SMALL LETTER ALPHA WITH TONOS
            return 0x0386;	//	Ά	0xce 0x86	GREEK CAPITAL LETTER ALPHA WITH TONOS

        case 0x03AD:		//	έ	0xce 0xad	GREEK SMALL LETTER EPSILON WITH TONOS
            return 0x0388;	//	Έ	0xce 0x88	GREEK CAPITAL LETTER EPSILON WITH TONOS

        case 0x03AE:		//	ή	0xce 0xae	GREEK SMALL LETTER ETA WITH TONOS
            return 0x0389;	//	Ή	0xce 0x89	GREEK CAPITAL LETTER ETA WITH TONOS

        case 0x03AF:		//	ί	0xce 0xaf	GREEK SMALL LETTER IOTA WITH TONOS
            return 0x038A;	//	Ί	0xce 0x8a	GREEK CAPITAL LETTER IOTA WITH TONOS

        case 0x03B1:		//	α	0xce 0xb1	GREEK SMALL LETTER ALPHA
            return 0x0391;	//	Α	0xce 0x91	GREEK CAPITAL LETTER ALPHA

        case 0x03B2:		//	β	0xce 0xb2	GREEK SMALL LETTER BETA
            return 0x0392;	//	Β	0xce 0x92	GREEK CAPITAL LETTER BETA

        case 0x03B3:		//	γ	0xce 0xb3	GREEK SMALL LETTER GAMMA
            return 0x0393;	//	Γ	0xce 0x93	GREEK CAPITAL LETTER GAMMA

        case 0x03B4:		//	δ	0xce 0xb4	GREEK SMALL LETTER DELTA
            return 0x0394;	//	Δ	0xce 0x94	GREEK CAPITAL LETTER DELTA

        case 0x03B5:		//	ε	0xce 0xb5	GREEK SMALL LETTER EPSILON
            return 0x0395;	//	Ε	0xce 0x95	GREEK CAPITAL LETTER EPSILON

        case 0x03B6:		//	ζ	0xce 0xb6	GREEK SMALL LETTER ZETA
            return 0x0396;	//	Ζ	0xce 0x96	GREEK CAPITAL LETTER ZETA

        case 0x03B7:		//	η	0xce 0xb7	GREEK SMALL LETTER ETA
            return 0x0397;	//	Η	0xce 0x97	GREEK CAPITAL LETTER ETA

        case 0x03B8:		//	θ	0xce 0xb8	GREEK SMALL LETTER THETA
            return 0x0398;	//	Θ	0xce 0x98	GREEK CAPITAL LETTER THETA

        case 0x03B9:		//	ι	0xce 0xb9	GREEK SMALL LETTER IOTA
            return 0x0399;	//	Ι	0xce 0x99	GREEK CAPITAL LETTER IOTA

        case 0x03BA:		//	κ	0xce 0xba	GREEK SMALL LETTER KAPPA
            return 0x039A;	//	Κ	0xce 0x9a	GREEK CAPITAL LETTER KAPPA

        case 0x03BB:		//	λ	0xce 0xbb	GREEK SMALL LETTER LAMDA
            return 0x039B;	//	Λ	0xce 0x9b	GREEK CAPITAL LETTER LAMDA

        case 0x03BC:		//	μ	0xce 0xbc	GREEK SMALL LETTER MU
            return 0x039C;	//	Μ	0xce 0x9c	GREEK CAPITAL LETTER MU

        case 0x03BD:		//	ν	0xce 0xbd	GREEK SMALL LETTER NU
            return 0x039D;	//	Ν	0xce 0x9d	GREEK CAPITAL LETTER NU

        case 0x03BE:		//	ξ	0xce 0xbe	GREEK SMALL LETTER XI
            return 0x039E;	//	Ξ	0xce 0x9e	GREEK CAPITAL LETTER XI

        case 0x03BF:		//	ο	0xce 0xbf	GREEK SMALL LETTER OMICRON
            return 0x039F;	//	Ο	0xce 0x9f	GREEK CAPITAL LETTER OMICRON

        case 0x03C0:		//	π	0xcf 0x80	GREEK SMALL LETTER PI
            return 0x03A0;	//	Π	0xce 0xa0	GREEK CAPITAL LETTER PI

        case 0x03C1:		//	ρ	0xcf 0x81	GREEK SMALL LETTER RHO
            return 0x03A1;	//	Ρ	0xce 0xa1	GREEK CAPITAL LETTER RHO

        case 0x03C2:		//	ς	0xcf 0x82	GREEK SMALL FINAL SIGMA
        case 0x03C3:		//	σ	0xcf 0x83	GREEK SMALL LETTER SIGMA
            return 0x03A3;	//	Σ	0xce 0xa3	GREEK CAPITAL LETTER SIGMA

        case 0x03C4:		//	τ	0xcf 0x84	GREEK SMALL LETTER TAU
            return 0x03A4;	//	Τ	0xce 0xa4	GREEK CAPITAL LETTER TAU

        case 0x03C5:		//	υ	0xcf 0x85	GREEK SMALL LETTER UPSILON
            return 0x03A5;	//	Υ	0xce 0xa5	GREEK CAPITAL LETTER UPSILON

        case 0x03C6:		//	φ	0xcf 0x86	GREEK SMALL LETTER PHI
            return 0x03A6;	//	Φ	0xce 0xa6	GREEK CAPITAL LETTER PHI

        case 0x03C7:		//	χ	0xcf 0x87	GREEK SMALL LETTER CHI
            return 0x03A7;	//	Χ	0xce 0xa7	GREEK CAPITAL LETTER CHI

        case 0x03C8:		//	ψ	0xcf 0x88	GREEK SMALL LETTER PSI
            return 0x03A8;	//	Ψ	0xce 0xa8	GREEK CAPITAL LETTER PSI

        case 0x03C9:		//	ω	0xcf 0x89	GREEK SMALL LETTER OMEGA
            return 0x03A9;	//	Ω	0xce 0xa9	GREEK CAPITAL LETTER OMEGA

        case 0x03CA:		//	ϊ	0xcf 0x8a	GREEK SMALL LETTER IOTA WITH DIALYTIKA
            return 0x03AA;	//	Ϊ	0xce 0xaa	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA

        case 0x03CB:		//	ϋ	0xcf 0x8b	GREEK SMALL LETTER UPSILON WITH DIALYTIKA
            return 0x03AB;	//	Ϋ	0xce 0xab	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA

        case 0x03CC:		//	ό	0xcf 0x8c	GREEK SMALL LETTER OMICRON WITH TONOS
            return 0x038C;	//	Ό	0xce 0x8c	GREEK CAPITAL LETTER OMICRON WITH TONOS

        case 0x03CD:		//	ύ	0xcf 0x8d	GREEK SMALL LETTER UPSILON WITH TONOS
            return 0x038E;	//	Ύ	0xce 0x8e	GREEK CAPITAL LETTER UPSILON WITH TONOS

        case 0x03CE:		//	ώ	0xcf 0x8e	GREEK SMALL LETTER OMEGA WITH TONOS
            return 0x038F;	//	Ώ	0xce 0x8f	GREEK CAPITAL LETTER OMEGA WITH TONOS

        case 0x03D1:		//	ϑ	0xcf 0x91	GREEK SMALL LETTER THETA SYMBOL
            return 0x03F4;	//	ϴ	0xcf 0xb4	GREEK CAPITAL LETTER THETA SYMBOL

        case 0x03D7:		//	ϗ	0xcf 0x97	GREEK SMALL LETTER KAI SYMBOL
            return 0x03CF;	//	Ϗ	0xcf 0x8f	GREEK CAPITAL LETTER KAI SYMBOL

        case 0x03D9:		//	ϙ	0xcf 0x99	GREEK SMALL LETTER ARCHAIC KOPPA
            return 0x03D8;	//	Ϙ	0xcf 0x98	GREEK CAPITAL LETTER ARCHAIC KOPPA

        case 0x03DB:		//	ϛ	0xcf 0x9b	GREEK SMALL LETTER STIGMA
            return 0x03DA;	//	Ϛ	0xcf 0x9a	GREEK CAPITAL LETTER STIGMA

        case 0x03DD:		//	ϝ	0xcf 0x9d	GREEK SMALL LETTER DIGAMMA
            return 0x03DC;	//	Ϝ	0xcf 0x9c	GREEK CAPITAL LETTER DIGAMMA

        case 0x03DF:		//	ϟ	0xcf 0x9f	GREEK SMALL LETTER KOPPA
            return 0x03DE;	//	Ϟ	0xcf 0x9e	GREEK CAPITAL LETTER KOPPA

        case 0x03E1:		//	ϡ	0xcf 0xa1	GREEK SMALL LETTER SAMPI
            return 0x03E0;	//	Ϡ	0xcf 0xa0	GREEK CAPITAL LETTER SAMPI

#endif
#if UTF_SUPPORT_COPTIC == 1
        case 0x03E3:		//	ϣ	0xcf 0xa3	COPTIC SMALL LETTER SHEI
            return 0x03E2;	//	Ϣ	0xcf 0xa2	COPTIC CAPITAL LETTER SHEI

        case 0x03E5:		//	ϥ	0xcf 0xa5	COPTIC SMALL LETTER FEI
            return 0x03E4;	//	Ϥ	0xcf 0xa4	COPTIC CAPITAL LETTER FEI

        case 0x03E7:		//	ϧ	0xcf 0xa7	COPTIC SMALL LETTER KHEI
            return 0x03E6;	//	Ϧ	0xcf 0xa6	COPTIC CAPITAL LETTER KHEI

        case 0x03E9:		//	ϩ	0xcf 0xa9	COPTIC SMALL LETTER HORI
            return 0x03E8;	//	Ϩ	0xcf 0xa8	COPTIC CAPITAL LETTER HORI

        case 0x03EB:		//	ϫ	0xcf 0xab	COPTIC SMALL LETTER GANGIA
            return 0x03EA;	//	Ϫ	0xcf 0xaa	COPTIC CAPITAL LETTER GANGIA

        case 0x03ED:		//	ϭ	0xcf 0xad	COPTIC SMALL LETTER SHIMA
            return 0x03EC;	//	Ϭ	0xcf 0xac	COPTIC CAPITAL LETTER SHIMA

        case 0x03EF:		//	ϯ	0xcf 0xaf	COPTIC SMALL LETTER DEI
            return 0x03EE;	//	Ϯ	0xcf 0xae	COPTIC CAPITAL LETTER DEI

#endif
#if UTF_SUPPORT_GREEK == 1
        case 0x03F2:		//	ϲ	0xcf 0xb2	GREEK SMALL LETTER LUNATE SIGMA SYMBOL
            return 0x03F9;	//	Ϲ	0xcf 0xb9	GREEK CAPITAL LETTER LUNATE SIGMA SYMBOL

        case 0x03F3:		//	ϳ	0xcf 0xb3	GREEK SMALL LETTER YOT
            return 0x037F;	//	Ϳ	0xcd 0xbf	GREEK CAPITAL LETTER YOT

        case 0x03F8:		//	ϸ	0xcf 0xb8	GREEK SMALL LETTER SHO
            return 0x03F7;	//	Ϸ	0xcf 0xb7	GREEK CAPITAL LETTER SHO

        case 0x03FB:		//	ϻ	0xcf 0xbb	GREEK SMALL LETTER SAN
            return 0x03FA;	//	Ϻ	0xcf 0xba	GREEK CAPITAL LETTER SAN

#endif
#if UTF_SUPPORT_CYRILLIC == 1
        case 0x0450:		//	ѐ	0xd1 0x90	CYRILLIC SMALL LETTER IE WITH GRAVE
            return 0x0400;	//	Ѐ	0xd0 0x80	CYRILLIC CAPITAL LETTER IE WITH GRAVE

        case 0x0452:		//	ђ	0xd1 0x92	CYRILLIC SMALL LETTER DJE
            return 0x0402;	//	Ђ	0xd0 0x82	CYRILLIC CAPITAL LETTER DJE

        case 0x0453:		//	ѓ	0xd1 0x93	CYRILLIC SMALL LETTER GJE
            return 0x0403;	//	Ѓ	0xd0 0x83	CYRILLIC CAPITAL LETTER GJE

        case 0x0454:		//	є	0xd1 0x94	CYRILLIC SMALL LETTER UKRAINIAN IE
            return 0x0404;	//	Є	0xd0 0x84	CYRILLIC CAPITAL LETTER UKRAINIAN IE

        case 0x0455:		//	ѕ	0xd1 0x95	CYRILLIC SMALL LETTER DZE
            return 0x0405;	//	Ѕ	0xd0 0x85	CYRILLIC CAPITAL LETTER DZE

        case 0x0456:		//	і	0xd1 0x96	CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
            return 0x0406;	//	І	0xd0 0x86	CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I

        case 0x0457:		//	ї	0xd1 0x97	CYRILLIC SMALL LETTER YI
            return 0x0407;	//	Ї	0xd0 0x87	CYRILLIC CAPITAL LETTER YI

        case 0x0458:		//	ј	0xd1 0x98	CYRILLIC SMALL LETTER JE
            return 0x0408;	//	Ј	0xd0 0x88	CYRILLIC CAPITAL LETTER JE

        case 0x0459:		//	љ	0xd1 0x99	CYRILLIC SMALL LETTER LJE
            return 0x0409;	//	Љ	0xd0 0x89	CYRILLIC CAPITAL LETTER LJE

        case 0x045A:		//	њ	0xd1 0x9a	CYRILLIC SMALL LETTER NJE
            return 0x040A;	//	Њ	0xd0 0x8a	CYRILLIC CAPITAL LETTER NJE

        case 0x045B:		//	ћ	0xd1 0x9b	CYRILLIC SMALL LETTER TSHE
            return 0x040B;	//	Ћ	0xd0 0x8b	CYRILLIC CAPITAL LETTER TSHE

        case 0x045C:		//	ќ	0xd1 0x9c	CYRILLIC SMALL LETTER KJE
            return 0x040C;	//	Ќ	0xd0 0x8c	CYRILLIC CAPITAL LETTER KJE

        case 0x045D:		//	ѝ	0xd1 0x9d	CYRILLIC SMALL LETTER I WITH GRAVE
            return 0x040D;	//	Ѝ	0xd0 0x8d	CYRILLIC CAPITAL LETTER I WITH GRAVE

        case 0x045E:		//	ў	0xd1 0x9e	CYRILLIC SMALL LETTER SHORT U
            return 0x040E;	//	Ў	0xd0 0x8e	CYRILLIC CAPITAL LETTER SHORT U

        case 0x045F:		//	џ	0xd1 0x9f	CYRILLIC SMALL LETTER DZHE
            return 0x040F;	//	Џ	0xd0 0x8f	CYRILLIC CAPITAL LETTER DZHE

        case 0x0461:		//	ѡ	0xd1 0xa1	CYRILLIC SMALL LETTER OMEGA
            return 0x0460;	//	Ѡ	0xd1 0xa0	CYRILLIC CAPITAL LETTER OMEGA

        case 0x0463:		//	ѣ	0xd1 0xa3	CYRILLIC SMALL LETTER YAT
            return 0x0462;	//	Ѣ	0xd1 0xa2	CYRILLIC CAPITAL LETTER YAT

        case 0x0465:		//	ѥ	0xd1 0xa5	CYRILLIC SMALL LETTER IOTIFIED E
            return 0x0464;	//	Ѥ	0xd1 0xa4	CYRILLIC CAPITAL LETTER IOTIFIED E

        case 0x0467:		//	ѧ	0xd1 0xa7	CYRILLIC SMALL LETTER LITTLE YUS
            return 0x0466;	//	Ѧ	0xd1 0xa6	CYRILLIC CAPITAL LETTER LITTLE YUS

        case 0x0469:		//	ѩ	0xd1 0xa9	CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS
            return 0x0468;	//	Ѩ	0xd1 0xa8	CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS

        case 0x046B:		//	ѫ	0xd1 0xab	CYRILLIC SMALL LETTER BIG YUS
            return 0x046A;	//	Ѫ	0xd1 0xaa	CYRILLIC CAPITAL LETTER BIG YUS

        case 0x046D:		//	ѭ	0xd1 0xad	CYRILLIC SMALL LETTER IOTIFIED BIG YUS
            return 0x046C;	//	Ѭ	0xd1 0xac	CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS

        case 0x046F:		//	ѯ	0xd1 0xaf	CYRILLIC SMALL LETTER KSI
            return 0x046E;	//	Ѯ	0xd1 0xae	CYRILLIC CAPITAL LETTER KSI

        case 0x0471:		//	ѱ	0xd1 0xb1	CYRILLIC SMALL LETTER PSI
            return 0x0470;	//	Ѱ	0xd1 0xb0	CYRILLIC CAPITAL LETTER PSI

        case 0x0473:		//	ѳ	0xd1 0xb3	CYRILLIC SMALL LETTER FITA
            return 0x0472;	//	Ѳ	0xd1 0xb2	CYRILLIC CAPITAL LETTER FITA

        case 0x0475:		//	ѵ	0xd1 0xb5	CYRILLIC SMALL LETTER IZHITSA
            return 0x0474;	//	Ѵ	0xd1 0xb4	CYRILLIC CAPITAL LETTER IZHITSA

        case 0x0477:		//	ѷ	0xd1 0xb7	CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT
            return 0x0476;	//	Ѷ	0xd1 0xb6	CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT

        case 0x0479:		//	ѹ	0xd1 0xb9	CYRILLIC SMALL LETTER UK
            return 0x0478;	//	Ѹ	0xd1 0xb8	CYRILLIC CAPITAL LETTER UK

        case 0x047B:		//	ѻ	0xd1 0xbb	CYRILLIC SMALL LETTER ROUND OMEGA
            return 0x047A;	//	Ѻ	0xd1 0xba	CYRILLIC CAPITAL LETTER ROUND OMEGA

        case 0x047D:		//	ѽ	0xd1 0xbd	CYRILLIC SMALL LETTER OMEGA WITH TITLO
            return 0x047C;	//	Ѽ	0xd1 0xbc	CYRILLIC CAPITAL LETTER OMEGA WITH TITLO

        case 0x047F:		//	ѿ	0xd1 0xbf	CYRILLIC SMALL LETTER OT
            return 0x047E;	//	Ѿ	0xd1 0xbe	CYRILLIC CAPITAL LETTER OT

        case 0x0481:		//	ҁ	0xd2 0x81	CYRILLIC SMALL LETTER KOPPA
            return 0x0480;	//	Ҁ	0xd2 0x80	CYRILLIC CAPITAL LETTER KOPPA

        case 0x048B:		//	ҋ	0xd2 0x8b	CYRILLIC SMALL LETTER SHORT I WITH TAIL
            return 0x048A;	//	Ҋ	0xd2 0x8a	CYRILLIC CAPITAL LETTER SHORT I WITH TAIL

        case 0x048D:		//	ҍ	0xd2 0x8d	CYRILLIC SMALL LETTER SEMISOFT SIGN
            return 0x048C;	//	Ҍ	0xd2 0x8c	CYRILLIC CAPITAL LETTER SEMISOFT SIGN

        case 0x048F:		//	ҏ	0xd2 0x8f	CYRILLIC SMALL LETTER ER WITH TICK
            return 0x048E;	//	Ҏ	0xd2 0x8e	CYRILLIC CAPITAL LETTER ER WITH TICK

        case 0x0491:		//	ґ	0xd2 0x91	CYRILLIC SMALL LETTER GHE WITH UPTURN
            return 0x0490;	//	Ґ	0xd2 0x90	CYRILLIC CAPITAL LETTER GHE WITH UPTURN

        case 0x0493:		//	ғ	0xd2 0x93	CYRILLIC SMALL LETTER GHE WITH STROKE
            return 0x0492;	//	Ғ	0xd2 0x92	CYRILLIC CAPITAL LETTER GHE WITH STROKE

        case 0x0495:		//	ҕ	0xd2 0x95	CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK
            return 0x0494;	//	Ҕ	0xd2 0x94	CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK

        case 0x0497:		//	җ	0xd2 0x97	CYRILLIC SMALL LETTER ZHE WITH DESCENDER
            return 0x0496;	//	Җ	0xd2 0x96	CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER

        case 0x0499:		//	ҙ	0xd2 0x99	CYRILLIC SMALL LETTER ZE WITH DESCENDER
            return 0x0498;	//	Ҙ	0xd2 0x98	CYRILLIC CAPITAL LETTER ZE WITH DESCENDER

        case 0x049B:		//	қ	0xd2 0x9b	CYRILLIC SMALL LETTER KA WITH DESCENDER
            return 0x049A;	//	Қ	0xd2 0x9a	CYRILLIC CAPITAL LETTER KA WITH DESCENDER

        case 0x049D:		//	ҝ	0xd2 0x9d	CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE
            return 0x049C;	//	Ҝ	0xd2 0x9c	CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE

        case 0x049F:		//	ҟ	0xd2 0x9f	CYRILLIC SMALL LETTER KA WITH STROKE
            return 0x049E;	//	Ҟ	0xd2 0x9e	CYRILLIC CAPITAL LETTER KA WITH STROKE

        case 0x04A1:		//	ҡ	0xd2 0xa1	CYRILLIC SMALL LETTER BASHKIR KA
            return 0x04A0;	//	Ҡ	0xd2 0xa0	CYRILLIC CAPITAL LETTER BASHKIR KA

        case 0x04A3:		//	ң	0xd2 0xa3	CYRILLIC SMALL LETTER EN WITH DESCENDER
            return 0x04A2;	//	Ң	0xd2 0xa2	CYRILLIC CAPITAL LETTER EN WITH DESCENDER

        case 0x04A5:		//	ҥ	0xd2 0xa5	CYRILLIC SMALL LETTER LIGATURE EN GHE
            return 0x04A4;	//	Ҥ	0xd2 0xa4	CYRILLIC CAPITAL LETTER LIGATURE EN GHE

        case 0x04A7:		//	ҧ	0xd2 0xa7	CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK
            return 0x04A6;	//	Ҧ	0xd2 0xa6	CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK

        case 0x04A9:		//	ҩ	0xd2 0xa9	CYRILLIC SMALL LETTER ABKHASIAN HA
            return 0x04A8;	//	Ҩ	0xd2 0xa8	CYRILLIC CAPITAL LETTER ABKHASIAN HA

        case 0x04AB:		//	ҫ	0xd2 0xab	CYRILLIC SMALL LETTER ES WITH DESCENDER
            return 0x04AA;	//	Ҫ	0xd2 0xaa	CYRILLIC CAPITAL LETTER ES WITH DESCENDER

        case 0x04AD:		//	ҭ	0xd2 0xad	CYRILLIC SMALL LETTER TE WITH DESCENDER
            return 0x04AC;	//	Ҭ	0xd2 0xac	CYRILLIC CAPITAL LETTER TE WITH DESCENDER

        case 0x04AF:		//	ү	0xd2 0xaf	CYRILLIC SMALL LETTER STRAIGHT U
            return 0x04AE;	//	Ү	0xd2 0xae	CYRILLIC CAPITAL LETTER STRAIGHT U

        case 0x04B1:		//	ұ	0xd2 0xb1	CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE
            return 0x04B0;	//	Ұ	0xd2 0xb0	CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE

        case 0x04B3:		//	ҳ	0xd2 0xb3	CYRILLIC SMALL LETTER HA WITH DESCENDER
            return 0x04B2;	//	Ҳ	0xd2 0xb2	CYRILLIC CAPITAL LETTER HA WITH DESCENDER

        case 0x04B5:		//	ҵ	0xd2 0xb5	CYRILLIC SMALL LETTER LIGATURE TE TSE
            return 0x04B4;	//	Ҵ	0xd2 0xb4	CYRILLIC CAPITAL LETTER LIGATURE TE TSE

        case 0x04B7:		//	ҷ	0xd2 0xb7	CYRILLIC SMALL LETTER CHE WITH DESCENDER
            return 0x04B6;	//	Ҷ	0xd2 0xb6	CYRILLIC CAPITAL LETTER CHE WITH DESCENDER

        case 0x04B9:		//	ҹ	0xd2 0xb9	CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE
            return 0x04B8;	//	Ҹ	0xd2 0xb8	CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE

        case 0x04BB:		//	һ	0xd2 0xbb	CYRILLIC SMALL LETTER SHHA
            return 0x04BA;	//	Һ	0xd2 0xba	CYRILLIC CAPITAL LETTER SHHA

        case 0x04BD:		//	ҽ	0xd2 0xbd	CYRILLIC SMALL LETTER ABKHASIAN CHE
            return 0x04BC;	//	Ҽ	0xd2 0xbc	CYRILLIC CAPITAL LETTER ABKHASIAN CHE

        case 0x04BF:		//	ҿ	0xd2 0xbf	CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER
            return 0x04BE;	//	Ҿ	0xd2 0xbe	CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER

        case 0x04C2:		//	ӂ	0xd3 0x82	CYRILLIC SMALL LETTER ZHE WITH BREVE
            return 0x04C1;	//	Ӂ	0xd3 0x81	CYRILLIC CAPITAL LETTER ZHE WITH BREVE

        case 0x04C4:		//	ӄ	0xd3 0x84	CYRILLIC SMALL LETTER KA WITH HOOK
            return 0x04C3;	//	Ӄ	0xd3 0x83	CYRILLIC CAPITAL LETTER KA WITH HOOK

        case 0x04C6:		//	ӆ	0xd3 0x86	CYRILLIC SMALL LETTER EL WITH TAIL
            return 0x04C5;	//	Ӆ	0xd3 0x85	CYRILLIC CAPITAL LETTER EL WITH TAIL

        case 0x04C8:		//	ӈ	0xd3 0x88	CYRILLIC SMALL LETTER EN WITH HOOK
            return 0x04C7;	//	Ӈ	0xd3 0x87	CYRILLIC CAPITAL LETTER EN WITH HOOK

        case 0x04CA:		//	ӊ	0xd3 0x8a	CYRILLIC SMALL LETTER EN WITH TAIL
            return 0x04C9;	//	Ӊ	0xd3 0x89	CYRILLIC CAPITAL LETTER EN WITH TAIL

        case 0x04CC:		//	ӌ	0xd3 0x8c	CYRILLIC SMALL LETTER KHAKASSIAN CHE
            return 0x04CB;	//	Ӌ	0xd3 0x8b	CYRILLIC CAPITAL LETTER KHAKASSIAN CHE

        case 0x04CE:		//	ӎ	0xd3 0x8e	CYRILLIC SMALL LETTER EM WITH TAIL
            return 0x04CD;	//	Ӎ	0xd3 0x8d	CYRILLIC CAPITAL LETTER EM WITH TAIL

        case 0x04CF:		//	ӏ	0xd3 0x8f	CYRILLIC SMALL LETTER PALOCHKA
            return 0x04C0;	//	Ӏ	0xd3 0x80	CYRILLIC CAPITAL LETTER PALOCHKA

        case 0x04D1:		//	ӑ	0xd3 0x91	CYRILLIC SMALL LETTER A WITH BREVE
            return 0x04D0;	//	Ӑ	0xd3 0x90	CYRILLIC CAPITAL LETTER A WITH BREVE

        case 0x04D3:		//	ӓ	0xd3 0x93	CYRILLIC SMALL LETTER A WITH DIAERESIS
            return 0x04D2;	//	Ӓ	0xd3 0x92	CYRILLIC CAPITAL LETTER A WITH DIAERESIS

        case 0x04D5:		//	ӕ	0xd3 0x95	CYRILLIC SMALL LETTER LIGATURE A IE
            return 0x04D4;	//	Ӕ	0xd3 0x94	CYRILLIC CAPITAL LETTER LIGATURE A IE

        case 0x04D7:		//	ӗ	0xd3 0x97	CYRILLIC SMALL LETTER IE WITH BREVE
            return 0x04D6;	//	Ӗ	0xd3 0x96	CYRILLIC CAPITAL LETTER IE WITH BREVE

        case 0x04D9:		//	ә	0xd3 0x99	CYRILLIC SMALL LETTER SCHWA
            return 0x04D8;	//	Ә	0xd3 0x98	CYRILLIC CAPITAL LETTER SCHWA

        case 0x04DB:		//	ӛ	0xd3 0x9b	CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS
            return 0x04DA;	//	Ӛ	0xd3 0x9a	CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS

        case 0x04DD:		//	ӝ	0xd3 0x9d	CYRILLIC SMALL LETTER ZHE WITH DIAERESIS
            return 0x04DC;	//	Ӝ	0xd3 0x9c	CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS

        case 0x04DF:		//	ӟ	0xd3 0x9f	CYRILLIC SMALL LETTER ZE WITH DIAERESIS
            return 0x04DE;	//	Ӟ	0xd3 0x9e	CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS

        case 0x04E1:		//	ӡ	0xd3 0xa1	CYRILLIC SMALL LETTER ABKHASIAN DZE
            return 0x04E0;	//	Ӡ	0xd3 0xa0	CYRILLIC CAPITAL LETTER ABKHASIAN DZE

        case 0x04E3:		//	ӣ	0xd3 0xa3	CYRILLIC SMALL LETTER I WITH MACRON
            return 0x04E2;	//	Ӣ	0xd3 0xa2	CYRILLIC CAPITAL LETTER I WITH MACRON

        case 0x04E5:		//	ӥ	0xd3 0xa5	CYRILLIC SMALL LETTER I WITH DIAERESIS
            return 0x04E4;	//	Ӥ	0xd3 0xa4	CYRILLIC CAPITAL LETTER I WITH DIAERESIS

        case 0x04E7:		//	ӧ	0xd3 0xa7	CYRILLIC SMALL LETTER O WITH DIAERESIS
            return 0x04E6;	//	Ӧ	0xd3 0xa6	CYRILLIC CAPITAL LETTER O WITH DIAERESIS

        case 0x04E9:		//	ө	0xd3 0xa9	CYRILLIC SMALL LETTER BARRED O
            return 0x04E8;	//	Ө	0xd3 0xa8	CYRILLIC CAPITAL LETTER BARRED O

        case 0x04EB:		//	ӫ	0xd3 0xab	CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS
            return 0x04EA;	//	Ӫ	0xd3 0xaa	CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS

        case 0x04ED:		//	ӭ	0xd3 0xad	CYRILLIC SMALL LETTER E WITH DIAERESIS
            return 0x04EC;	//	Ӭ	0xd3 0xac	CYRILLIC CAPITAL LETTER E WITH DIAERESIS

        case 0x04EF:		//	ӯ	0xd3 0xaf	CYRILLIC SMALL LETTER U WITH MACRON
            return 0x04EE;	//	Ӯ	0xd3 0xae	CYRILLIC CAPITAL LETTER U WITH MACRON

        case 0x04F1:		//	ӱ	0xd3 0xb1	CYRILLIC SMALL LETTER U WITH DIAERESIS
            return 0x04F0;	//	Ӱ	0xd3 0xb0	CYRILLIC CAPITAL LETTER U WITH DIAERESIS

        case 0x04F3:		//	ӳ	0xd3 0xb3	CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE
            return 0x04F2;	//	Ӳ	0xd3 0xb2	CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE

        case 0x04F5:		//	ӵ	0xd3 0xb5	CYRILLIC SMALL LETTER CHE WITH DIAERESIS
            return 0x04F4;	//	Ӵ	0xd3 0xb4	CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS

        case 0x04F7:		//	ӷ	0xd3 0xb7	CYRILLIC SMALL LETTER GHE WITH DESCENDER
            return 0x04F6;	//	Ӷ	0xd3 0xb6	CYRILLIC CAPITAL LETTER GHE WITH DESCENDER

        case 0x04F9:		//	ӹ	0xd3 0xb9	CYRILLIC SMALL LETTER YERU WITH DIAERESIS
            return 0x04F8;	//	Ӹ	0xd3 0xb8	CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS

        case 0x04FB:		//	ӻ	0xd3 0xbb	CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK
            return 0x04FA;	//	Ӻ	0xd3 0xba	CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK

        case 0x04FD:		//	ӽ	0xd3 0xbd	CYRILLIC SMALL LETTER HA WITH HOOK
            return 0x04FC;	//	Ӽ	0xd3 0xbc	CYRILLIC CAPITAL LETTER HA WITH HOOK

        case 0x04FF:		//	ӿ	0xd3 0xbf	CYRILLIC SMALL LETTER HA WITH STROKE
            return 0x04FE;	//	Ӿ	0xd3 0xbe	CYRILLIC CAPITAL LETTER HA WITH STROKE

        case 0x0501:		//	ԁ	0xd4 0x81	CYRILLIC SMALL LETTER KOMI DE
            return 0x0500;	//	Ԁ	0xd4 0x80	CYRILLIC CAPITAL LETTER KOMI DE

        case 0x0503:		//	ԃ	0xd4 0x83	CYRILLIC SMALL LETTER KOMI DJE
            return 0x0502;	//	Ԃ	0xd4 0x82	CYRILLIC CAPITAL LETTER KOMI DJE

        case 0x0505:		//	ԅ	0xd4 0x85	CYRILLIC SMALL LETTER KOMI ZJE
            return 0x0504;	//	Ԅ	0xd4 0x84	CYRILLIC CAPITAL LETTER KOMI ZJE

        case 0x0507:		//	ԇ	0xd4 0x87	CYRILLIC SMALL LETTER KOMI DZJE
            return 0x0506;	//	Ԇ	0xd4 0x86	CYRILLIC CAPITAL LETTER KOMI DZJE

        case 0x0509:		//	ԉ	0xd4 0x89	CYRILLIC SMALL LETTER KOMI LJE
            return 0x0508;	//	Ԉ	0xd4 0x88	CYRILLIC CAPITAL LETTER KOMI LJE

        case 0x050B:		//	ԋ	0xd4 0x8b	CYRILLIC SMALL LETTER KOMI NJE
            return 0x050A;	//	Ԋ	0xd4 0x8a	CYRILLIC CAPITAL LETTER KOMI NJE

        case 0x050D:		//	ԍ	0xd4 0x8d	CYRILLIC SMALL LETTER KOMI SJE
            return 0x050C;	//	Ԍ	0xd4 0x8c	CYRILLIC CAPITAL LETTER KOMI SJE

        case 0x050F:		//	ԏ	0xd4 0x8f	CYRILLIC SMALL LETTER KOMI TJE
            return 0x050E;	//	Ԏ	0xd4 0x8e	CYRILLIC CAPITAL LETTER KOMI TJE

        case 0x0511:		//	ԑ	0xd4 0x91	CYRILLIC SMALL LETTER REVERSED ZE
            return 0x0510;	//	Ԑ	0xd4 0x90	CYRILLIC CAPITAL LETTER REVERSED ZE

        case 0x0513:		//	ԓ	0xd4 0x93	CYRILLIC SMALL LETTER EL WITH HOOK
            return 0x0512;	//	Ԓ	0xd4 0x92	CYRILLIC CAPITAL LETTER EL WITH HOOK

        case 0x0515:		//	ԕ	0xd4 0x95	CYRILLIC SMALL LETTER LHA
            return 0x0514;	//	Ԕ	0xd4 0x94	CYRILLIC CAPITAL LETTER LHA

        case 0x0517:		//	ԗ	0xd4 0x97	CYRILLIC SMALL LETTER RHA
            return 0x0516;	//	Ԗ	0xd4 0x96	CYRILLIC CAPITAL LETTER RHA

        case 0x0519:		//	ԙ	0xd4 0x99	CYRILLIC SMALL LETTER YAE
            return 0x0518;	//	Ԙ	0xd4 0x98	CYRILLIC CAPITAL LETTER YAE

        case 0x051B:		//	ԛ	0xd4 0x9b	CYRILLIC SMALL LETTER QA
            return 0x051A;	//	Ԛ	0xd4 0x9a	CYRILLIC CAPITAL LETTER QA

        case 0x051D:		//	ԝ	0xd4 0x9d	CYRILLIC SMALL LETTER WE
            return 0x051C;	//	Ԝ	0xd4 0x9c	CYRILLIC CAPITAL LETTER WE

        case 0x051F:		//	ԟ	0xd4 0x9f	CYRILLIC SMALL LETTER ALEUT KA
            return 0x051E;	//	Ԟ	0xd4 0x9e	CYRILLIC CAPITAL LETTER ALEUT KA

        case 0x0521:		//	ԡ	0xd4 0xa1	CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK
            return 0x0520;	//	Ԡ	0xd4 0xa0	CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK

        case 0x0523:		//	ԣ	0xd4 0xa3	CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK
            return 0x0522;	//	Ԣ	0xd4 0xa2	CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK

        case 0x0525:		//	ԥ	0xd4 0xa5	CYRILLIC SMALL LETTER PE WITH DESCENDER
            return 0x0524;	//	Ԥ	0xd4 0xa4	CYRILLIC CAPITAL LETTER PE WITH DESCENDER

        case 0x0527:		//	ԧ	0xd4 0xa7	CYRILLIC SMALL LETTER SHHA WITH DESCENDER
            return 0x0526;	//	Ԧ	0xd4 0xa6	CYRILLIC CAPITAL LETTER SHHA WITH DESCENDER

        case 0x0529:		//	ԩ	0xd4 0xa9	CYRILLIC SMALL LETTER EN WITH LEFT HOOK
            return 0x0528;	//	Ԩ	0xd4 0xa8	CYRILLIC CAPITAL LETTER EN WITH LEFT HOOK

        case 0x052B:		//	ԫ	0xd4 0xab	CYRILLIC SMALL LETTER DZZHE
            return 0x052A;	//	Ԫ	0xd4 0xaa	CYRILLIC CAPITAL LETTER DZZHE

        case 0x052D:		//	ԭ	0xd4 0xad	CYRILLIC SMALL LETTER DCHE
            return 0x052C;	//	Ԭ	0xd4 0xac	CYRILLIC CAPITAL LETTER DCHE

        case 0x052F:		//	ԯ	0xd4 0xaf	CYRILLIC SMALL LETTER EL WITH DESCENDER
            return 0x052E;	//	Ԯ	0xd4 0xae	CYRILLIC CAPITAL LETTER EL WITH DESCENDER

#endif
#if UTF_SUPPORT_ARMENIAN == 1
        case 0x0561:		//	ա	0xd5 0xa1	ARMENIAN SMALL LETTER AYB
            return 0x0531;	//	Ա	0xd4 0xb1	ARMENIAN CAPITAL LETTER AYB

        case 0x0562:		//	բ	0xd5 0xa2	ARMENIAN SMALL LETTER BEN
            return 0x0532;	//	Բ	0xd4 0xb2	ARMENIAN CAPITAL LETTER BEN

        case 0x0563:		//	գ	0xd5 0xa3	ARMENIAN SMALL LETTER GIM
            return 0x0533;	//	Գ	0xd4 0xb3	ARMENIAN CAPITAL LETTER GIM

        case 0x0564:		//	դ	0xd5 0xa4	ARMENIAN SMALL LETTER DA
            return 0x0534;	//	Դ	0xd4 0xb4	ARMENIAN CAPITAL LETTER DA

        case 0x0565:		//	ե	0xd5 0xa5	ARMENIAN SMALL LETTER ECH
            return 0x0535;	//	Ե	0xd4 0xb5	ARMENIAN CAPITAL LETTER ECH

        case 0x0566:		//	զ	0xd5 0xa6	ARMENIAN SMALL LETTER ZA
            return 0x0536;	//	Զ	0xd4 0xb6	ARMENIAN CAPITAL LETTER ZA

        case 0x0567:		//	է	0xd5 0xa7	ARMENIAN SMALL LETTER EH
            return 0x0537;	//	Է	0xd4 0xb7	ARMENIAN CAPITAL LETTER EH

        case 0x0568:		//	ը	0xd5 0xa8	ARMENIAN SMALL LETTER ET
            return 0x0538;	//	Ը	0xd4 0xb8	ARMENIAN CAPITAL LETTER ET

        case 0x0569:		//	թ	0xd5 0xa9	ARMENIAN SMALL LETTER TO
            return 0x0539;	//	Թ	0xd4 0xb9	ARMENIAN CAPITAL LETTER TO

        case 0x056A:		//	ժ	0xd5 0xaa	ARMENIAN SMALL LETTER ZHE
            return 0x053A;	//	Ժ	0xd4 0xba	ARMENIAN CAPITAL LETTER ZHE

        case 0x056B:		//	ի	0xd5 0xab	ARMENIAN SMALL LETTER INI
            return 0x053B;	//	Ի	0xd4 0xbb	ARMENIAN CAPITAL LETTER INI

        case 0x056C:		//	լ	0xd5 0xac	ARMENIAN SMALL LETTER LIWN
            return 0x053C;	//	Լ	0xd4 0xbc	ARMENIAN CAPITAL LETTER LIWN

        case 0x056D:		//	խ	0xd5 0xad	ARMENIAN SMALL LETTER XEH
            return 0x053D;	//	Խ	0xd4 0xbd	ARMENIAN CAPITAL LETTER XEH

        case 0x056E:		//	ծ	0xd5 0xae	ARMENIAN SMALL LETTER CA
            return 0x053E;	//	Ծ	0xd4 0xbe	ARMENIAN CAPITAL LETTER CA

        case 0x056F:		//	կ	0xd5 0xaf	ARMENIAN SMALL LETTER KEN
            return 0x053F;	//	Կ	0xd4 0xbf	ARMENIAN CAPITAL LETTER KEN

        case 0x0570:		//	հ	0xd5 0xb0	ARMENIAN SMALL LETTER HO
            return 0x0540;	//	Հ	0xd5 0x80	ARMENIAN CAPITAL LETTER HO

        case 0x0571:		//	ձ	0xd5 0xb1	ARMENIAN SMALL LETTER JA
            return 0x0541;	//	Ձ	0xd5 0x81	ARMENIAN CAPITAL LETTER JA

        case 0x0572:		//	ղ	0xd5 0xb2	ARMENIAN SMALL LETTER GHAD
            return 0x0542;	//	Ղ	0xd5 0x82	ARMENIAN CAPITAL LETTER GHAD

        case 0x0573:		//	ճ	0xd5 0xb3	ARMENIAN SMALL LETTER CHEH
            return 0x0543;	//	Ճ	0xd5 0x83	ARMENIAN CAPITAL LETTER CHEH

        case 0x0574:		//	մ	0xd5 0xb4	ARMENIAN SMALL LETTER MEN
            return 0x0544;	//	Մ	0xd5 0x84	ARMENIAN CAPITAL LETTER MEN

        case 0x0575:		//	յ	0xd5 0xb5	ARMENIAN SMALL LETTER YI
            return 0x0545;	//	Յ	0xd5 0x85	ARMENIAN CAPITAL LETTER YI

        case 0x0576:		//	ն	0xd5 0xb6	ARMENIAN SMALL LETTER NOW
            return 0x0546;	//	Ն	0xd5 0x86	ARMENIAN CAPITAL LETTER NOW

        case 0x0577:		//	շ	0xd5 0xb7	ARMENIAN SMALL LETTER SHA
            return 0x0547;	//	Շ	0xd5 0x87	ARMENIAN CAPITAL LETTER SHA

        case 0x0578:		//	ո	0xd5 0xb8	ARMENIAN SMALL LETTER VO
            return 0x0548;	//	Ո	0xd5 0x88	ARMENIAN CAPITAL LETTER VO

        case 0x0579:		//	չ	0xd5 0xb9	ARMENIAN SMALL LETTER CHA
            return 0x0549;	//	Չ	0xd5 0x89	ARMENIAN CAPITAL LETTER CHA

        case 0x057A:		//	պ	0xd5 0xba	ARMENIAN SMALL LETTER PEH
            return 0x054A;	//	Պ	0xd5 0x8a	ARMENIAN CAPITAL LETTER PEH

        case 0x057B:		//	ջ	0xd5 0xbb	ARMENIAN SMALL LETTER JHEH
            return 0x054B;	//	Ջ	0xd5 0x8b	ARMENIAN CAPITAL LETTER JHEH

        case 0x057C:		//	ռ	0xd5 0xbc	ARMENIAN SMALL LETTER RA
            return 0x054C;	//	Ռ	0xd5 0x8c	ARMENIAN CAPITAL LETTER RA

        case 0x057D:		//	ս	0xd5 0xbd	ARMENIAN SMALL LETTER SEH
            return 0x054D;	//	Ս	0xd5 0x8d	ARMENIAN CAPITAL LETTER SEH

        case 0x057E:		//	վ	0xd5 0xbe	ARMENIAN SMALL LETTER VEW
            return 0x054E;	//	Վ	0xd5 0x8e	ARMENIAN CAPITAL LETTER VEW

        case 0x057F:		//	տ	0xd5 0xbf	ARMENIAN SMALL LETTER TIWN
            return 0x054F;	//	Տ	0xd5 0x8f	ARMENIAN CAPITAL LETTER TIWN

        case 0x0580:		//	ր	0xd6 0x80	ARMENIAN SMALL LETTER REH
            return 0x0550;	//	Ր	0xd5 0x90	ARMENIAN CAPITAL LETTER REH

        case 0x0581:		//	ց	0xd6 0x81	ARMENIAN SMALL LETTER CO
            return 0x0551;	//	Ց	0xd5 0x91	ARMENIAN CAPITAL LETTER CO

        case 0x0582:		//	ւ	0xd6 0x82	ARMENIAN SMALL LETTER YIWN
            return 0x0552;	//	Ւ	0xd5 0x92	ARMENIAN CAPITAL LETTER YIWN

        case 0x0583:		//	փ	0xd6 0x83	ARMENIAN SMALL LETTER PIWR
            return 0x0553;	//	Փ	0xd5 0x93	ARMENIAN CAPITAL LETTER PIWR

        case 0x0584:		//	ք	0xd6 0x84	ARMENIAN SMALL LETTER KEH
            return 0x0554;	//	Ք	0xd5 0x94	ARMENIAN CAPITAL LETTER KEH

        case 0x0585:		//	օ	0xd6 0x85	ARMENIAN SMALL LETTER OH
            return 0x0555;	//	Օ	0xd5 0x95	ARMENIAN CAPITAL LETTER OH

        case 0x0586:		//	ֆ	0xd6 0x86	ARMENIAN SMALL LETTER FEH
            return 0x0556;	//	Ֆ	0xd5 0x96	ARMENIAN CAPITAL LETTER FEH

#endif
#if UTF_SUPPORT_GEORGIAN == 1
        case 0x10A0:		//	Ⴀ	0xe1 0x82 0xa0	GEORGIAN LETTER AN
            return 0x1C90;	//	Ა	0xe1 0xb2 0x90	GEORGIAN CAPITAL LETTER AN

        case 0x10A1:		//	Ⴁ	0xe1 0x82 0xa1	GEORGIAN LETTER BAN
            return 0x1C91;	//	Ბ	0xe1 0xb2 0x91	GEORGIAN CAPITAL LETTER BAN

        case 0x10A2:		//	Ⴂ	0xe1 0x82 0xa2	GEORGIAN LETTER GAN
            return 0x1C92;	//	Გ	0xe1 0xb2 0x92	GEORGIAN CAPITAL LETTER GAN

        case 0x10A3:		//	Ⴃ	0xe1 0x82 0xa3	GEORGIAN LETTER DON
            return 0x1C93;	//	Დ	0xe1 0xb2 0x93	GEORGIAN CAPITAL LETTER DON

        case 0x10A4:		//	Ⴄ	0xe1 0x82 0xa4	GEORGIAN LETTER EN
            return 0x1C94;	//	Ე	0xe1 0xb2 0x94	GEORGIAN CAPITAL LETTER EN

        case 0x10A5:		//	Ⴅ	0xe1 0x82 0xa5	GEORGIAN LETTER VIN
            return 0x1C95;	//	Ვ	0xe1 0xb2 0x95	GEORGIAN CAPITAL LETTER VIN

        case 0x10A6:		//	Ⴆ	0xe1 0x82 0xa6	GEORGIAN LETTER ZEN
            return 0x1C96;	//	Ზ	0xe1 0xb2 0x96	GEORGIAN CAPITAL LETTER ZEN

        case 0x10A7:		//	Ⴇ	0xe1 0x82 0xa7	GEORGIAN LETTER TAN
            return 0x1C97;	//	Თ	0xe1 0xb2 0x97	GEORGIAN CAPITAL LETTER TAN

        case 0x10A8:		//	Ⴈ	0xe1 0x82 0xa8	GEORGIAN LETTER IN
            return 0x1C98;	//	Ი	0xe1 0xb2 0x98	GEORGIAN CAPITAL LETTER IN

        case 0x10A9:		//	Ⴉ	0xe1 0x82 0xa9	GEORGIAN LETTER KAN
            return 0x1C99;	//	Კ	0xe1 0xb2 0x99	GEORGIAN CAPITAL LETTER KAN

        case 0x10AA:		//	Ⴊ	0xe1 0x82 0xaa	GEORGIAN LETTER LAS
            return 0x1C9A;	//	Ლ	0xe1 0xb2 0x9a	GEORGIAN CAPITAL LETTER LAS

        case 0x10AB:		//	Ⴋ	0xe1 0x82 0xab	GEORGIAN LETTER MAN
            return 0x1C9B;	//	Მ	0xe1 0xb2 0x9b	GEORGIAN CAPITAL LETTER MAN

        case 0x10AC:		//	Ⴌ	0xe1 0x82 0xac	GEORGIAN LETTER NAR
            return 0x1C9C;	//	Ნ	0xe1 0xb2 0x9c	GEORGIAN CAPITAL LETTER NAR

        case 0x10AD:		//	Ⴍ	0xe1 0x82 0xad	GEORGIAN LETTER ON
            return 0x1C9D;	//	Ო	0xe1 0xb2 0x9d	GEORGIAN CAPITAL LETTER ON

        case 0x10AE:		//	Ⴎ	0xe1 0x82 0xae	GEORGIAN LETTER PAR
            return 0x1C9E;	//	Პ	0xe1 0xb2 0x9e	GEORGIAN CAPITAL LETTER PAR

        case 0x10AF:		//	Ⴏ	0xe1 0x82 0xaf	GEORGIAN LETTER ZHAR
            return 0x1C9F;	//	Ჟ	0xe1 0xb2 0x9f	GEORGIAN CAPITAL LETTER ZHAR

        case 0x10B0:		//	Ⴐ	0xe1 0x82 0xb0	GEORGIAN LETTER RAE
            return 0x1CA0;	//	Რ	0xe1 0xb2 0xa0	GEORGIAN CAPITAL LETTER RAE

        case 0x10B1:		//	Ⴑ	0xe1 0x82 0xb1	GEORGIAN LETTER SAN
            return 0x1CA1;	//	Ს	0xe1 0xb2 0xa1	GEORGIAN CAPITAL LETTER SAN

        case 0x10B2:		//	Ⴒ	0xe1 0x82 0xb2	GEORGIAN LETTER TAR
            return 0x1CA2;	//	Ტ	0xe1 0xb2 0xa2	GEORGIAN CAPITAL LETTER TAR

        case 0x10B3:		//	Ⴓ	0xe1 0x82 0xb3	GEORGIAN LETTER UN
            return 0x1CA3;	//	Უ	0xe1 0xb2 0xa3	GEORGIAN CAPITAL LETTER UN

        case 0x10B4:		//	Ⴔ	0xe1 0x82 0xb4	GEORGIAN LETTER PHAR
            return 0x1CA4;	//	Ფ	0xe1 0xb2 0xa4	GEORGIAN CAPITAL LETTER PHAR

        case 0x10B5:		//	Ⴕ	0xe1 0x82 0xb5	GEORGIAN LETTER KHAR
            return 0x1CA5;	//	Ქ	0xe1 0xb2 0xa5	GEORGIAN CAPITAL LETTER KHAR

        case 0x10B6:		//	Ⴖ	0xe1 0x82 0xb6	GEORGIAN LETTER GHAN
            return 0x1CA6;	//	Ღ	0xe1 0xb2 0xa6	GEORGIAN CAPITAL LETTER GHAN

        case 0x10B7:		//	Ⴗ	0xe1 0x82 0xb7	GEORGIAN LETTER QAR
            return 0x1CA7;	//	Ყ	0xe1 0xb2 0xa7	GEORGIAN CAPITAL LETTER QAR

        case 0x10B8:		//	Ⴘ	0xe1 0x82 0xb8	GEORGIAN LETTER SHIN
            return 0x1CA8;	//	Შ	0xe1 0xb2 0xa8	GEORGIAN CAPITAL LETTER SHIN

        case 0x10B9:		//	Ⴙ	0xe1 0x82 0xb9	GEORGIAN LETTER CHIN
            return 0x1CA9;	//	Ჩ	0xe1 0xb2 0xa9	GEORGIAN CAPITAL LETTER CHIN

        case 0x10BA:		//	Ⴚ	0xe1 0x82 0xba	GEORGIAN LETTER CAN
            return 0x1CAA;	//	Ც	0xe1 0xb2 0xaa	GEORGIAN CAPITAL LETTER CAN

        case 0x10BB:		//	Ⴛ	0xe1 0x82 0xbb	GEORGIAN LETTER JIL
            return 0x1CAB;	//	Ძ	0xe1 0xb2 0xab	GEORGIAN CAPITAL LETTER JIL

        case 0x10BC:		//	Ⴜ	0xe1 0x82 0xbc	GEORGIAN LETTER CIL
            return 0x1CAC;	//	Წ	0xe1 0xb2 0xac	GEORGIAN CAPITAL LETTER CIL

        case 0x10BD:		//	Ⴝ	0xe1 0x82 0xbd	GEORGIAN LETTER CHAR
            return 0x1CAD;	//	Ჭ	0xe1 0xb2 0xad	GEORGIAN CAPITAL LETTER CHAR

        case 0x10BE:		//	Ⴞ	0xe1 0x82 0xbe	GEORGIAN LETTER XAN
            return 0x1CAE;	//	Ხ	0xe1 0xb2 0xae	GEORGIAN CAPITAL LETTER XAN

        case 0x10BF:		//	Ⴟ	0xe1 0x82 0xbf	GEORGIAN LETTER JHAN
            return 0x1CAF;	//	Ჯ	0xe1 0xb2 0xaf	GEORGIAN CAPITAL LETTER JHAN

        case 0x10C0:		//	Ⴠ	0xe1 0x83 0x80	GEORGIAN LETTER HAE
            return 0x1CB0;	//	Ჰ	0xe1 0xb2 0xb0	GEORGIAN CAPITAL LETTER HAE

        case 0x10C1:		//	Ⴡ	0xe1 0x83 0x81	GEORGIAN LETTER HE
            return 0x1CB1;	//	Ჱ	0xe1 0xb2 0xb1	GEORGIAN CAPITAL LETTER HE

        case 0x10C2:		//	Ⴢ	0xe1 0x83 0x82	GEORGIAN LETTER HIE
            return 0x1CB2;	//	Ჲ	0xe1 0xb2 0xb2	GEORGIAN CAPITAL LETTER HIE

        case 0x10C3:		//	Ⴣ	0xe1 0x83 0x83	GEORGIAN LETTER WE
            return 0x1CB3;	//	Ჳ	0xe1 0xb2 0xb3	GEORGIAN CAPITAL LETTER WE

        case 0x10C4:		//	Ⴤ	0xe1 0x83 0x84	GEORGIAN LETTER HAR
            return 0x1CB4;	//	Ჴ	0xe1 0xb2 0xb4	GEORGIAN CAPITAL LETTER HAR

        case 0x10C5:		//	Ⴥ	0xe1 0x83 0x85	GEORGIAN LETTER HOE
            return 0x1CB5;	//	Ჵ	0xe1 0xb2 0xb5	GEORGIAN CAPITAL LETTER HOE

        case 0x10C7:		//	Ⴧ	0xe1 0x83 0x87	GEORGIAN LETTER YN
            return 0x1CB7;	//	Ჷ	0xe1 0xb2 0xb7	GEORGIAN CAPITAL LETTER YN

        case 0x10CD:		//	Ⴭ	0xe1 0x83 0x8d	GEORGIAN LETTER AEN
            return 0x1CBD;	//	Ჽ	0xe1 0xb2 0xbd	GEORGIAN CAPITAL LETTER AEN

        case 0x10D0:		//	ა	0xe1 0x83 0x90	GEORGIAN SMALL LETTER AN
            return 0x1C90;	//	Ა	0xe1 0xb2 0x90	GEORGIAN CAPITAL LETTER AN

        case 0x10D1:		//	ბ	0xe1 0x83 0x91	GEORGIAN SMALL LETTER BAN
            return 0x1C91;	//	Ბ	0xe1 0xb2 0x91	GEORGIAN CAPITAL LETTER BAN

        case 0x10D2:		//	გ	0xe1 0x83 0x92	GEORGIAN SMALL LETTER GAN
            return 0x1C92;	//	Გ	0xe1 0xb2 0x92	GEORGIAN CAPITAL LETTER GAN

        case 0x10D3:		//	დ	0xe1 0x83 0x93	GEORGIAN SMALL LETTER DON
            return 0x1C93;	//	Დ	0xe1 0xb2 0x93	GEORGIAN CAPITAL LETTER DON

        case 0x10D4:		//	ე	0xe1 0x83 0x94	GEORGIAN SMALL LETTER EN
            return 0x1C94;	//	Ე	0xe1 0xb2 0x94	GEORGIAN CAPITAL LETTER EN

        case 0x10D5:		//	ვ	0xe1 0x83 0x95	GEORGIAN SMALL LETTER VIN
            return 0x1C95;	//	Ვ	0xe1 0xb2 0x95	GEORGIAN CAPITAL LETTER VIN

        case 0x10D6:		//	ზ	0xe1 0x83 0x96	GEORGIAN SMALL LETTER ZEN
            return 0x1C96;	//	Ზ	0xe1 0xb2 0x96	GEORGIAN CAPITAL LETTER ZEN

        case 0x10D7:		//	თ	0xe1 0x83 0x97	GEORGIAN SMALL LETTER TAN
            return 0x1C97;	//	Თ	0xe1 0xb2 0x97	GEORGIAN CAPITAL LETTER TAN

        case 0x10D8:		//	ი	0xe1 0x83 0x98	GEORGIAN SMALL LETTER IN
            return 0x1C98;	//	Ი	0xe1 0xb2 0x98	GEORGIAN CAPITAL LETTER IN

        case 0x10D9:		//	კ	0xe1 0x83 0x99	GEORGIAN SMALL LETTER KAN
            return 0x1C99;	//	Კ	0xe1 0xb2 0x99	GEORGIAN CAPITAL LETTER KAN

        case 0x10DA:		//	ლ	0xe1 0x83 0x9a	GEORGIAN SMALL LETTER LAS
            return 0x1C9A;	//	Ლ	0xe1 0xb2 0x9a	GEORGIAN CAPITAL LETTER LAS

        case 0x10DB:		//	მ	0xe1 0x83 0x9b	GEORGIAN SMALL LETTER MAN
            return 0x1C9B;	//	Მ	0xe1 0xb2 0x9b	GEORGIAN CAPITAL LETTER MAN

        case 0x10DC:		//	ნ	0xe1 0x83 0x9c	GEORGIAN SMALL LETTER NAR
            return 0x1C9C;	//	Ნ	0xe1 0xb2 0x9c	GEORGIAN CAPITAL LETTER NAR

        case 0x10DD:		//	ო	0xe1 0x83 0x9d	GEORGIAN SMALL LETTER ON
            return 0x1C9D;	//	Ო	0xe1 0xb2 0x9d	GEORGIAN CAPITAL LETTER ON

        case 0x10DE:		//	პ	0xe1 0x83 0x9e	GEORGIAN SMALL LETTER PAR
            return 0x1C9E;	//	Პ	0xe1 0xb2 0x9e	GEORGIAN CAPITAL LETTER PAR

        case 0x10DF:		//	ჟ	0xe1 0x83 0x9f	GEORGIAN SMALL LETTER ZHAR
            return 0x1C9F;	//	Ჟ	0xe1 0xb2 0x9f	GEORGIAN CAPITAL LETTER ZHAR

        case 0x10E0:		//	რ	0xe1 0x83 0xa0	GEORGIAN SMALL LETTER RAE
            return 0x1CA0;	//	Რ	0xe1 0xb2 0xa0	GEORGIAN CAPITAL LETTER RAE

        case 0x10E1:		//	ს	0xe1 0x83 0xa1	GEORGIAN SMALL LETTER SAN
            return 0x1CA1;	//	Ს	0xe1 0xb2 0xa1	GEORGIAN CAPITAL LETTER SAN

        case 0x10E2:		//	ტ	0xe1 0x83 0xa2	GEORGIAN SMALL LETTER TAR
            return 0x1CA2;	//	Ტ	0xe1 0xb2 0xa2	GEORGIAN CAPITAL LETTER TAR

        case 0x10E3:		//	უ	0xe1 0x83 0xa3	GEORGIAN SMALL LETTER UN
            return 0x1CA3;	//	Უ	0xe1 0xb2 0xa3	GEORGIAN CAPITAL LETTER UN

        case 0x10E4:		//	ფ	0xe1 0x83 0xa4	GEORGIAN SMALL LETTER PHAR
            return 0x1CA4;	//	Ფ	0xe1 0xb2 0xa4	GEORGIAN CAPITAL LETTER PHAR

        case 0x10E5:		//	ქ	0xe1 0x83 0xa5	GEORGIAN SMALL LETTER KHAR
            return 0x1CA5;	//	Ქ	0xe1 0xb2 0xa5	GEORGIAN CAPITAL LETTER KHAR

        case 0x10E6:		//	ღ	0xe1 0x83 0xa6	GEORGIAN SMALL LETTER GHAN
            return 0x1CA6;	//	Ღ	0xe1 0xb2 0xa6	GEORGIAN CAPITAL LETTER GHAN

        case 0x10E7:		//	ყ	0xe1 0x83 0xa7	GEORGIAN SMALL LETTER QAR
            return 0x1CA7;	//	Ყ	0xe1 0xb2 0xa7	GEORGIAN CAPITAL LETTER QAR

        case 0x10E8:		//	შ	0xe1 0x83 0xa8	GEORGIAN SMALL LETTER SHIN
            return 0x1CA8;	//	Შ	0xe1 0xb2 0xa8	GEORGIAN CAPITAL LETTER SHIN

        case 0x10E9:		//	ჩ	0xe1 0x83 0xa9	GEORGIAN SMALL LETTER CHIN
            return 0x1CA9;	//	Ჩ	0xe1 0xb2 0xa9	GEORGIAN CAPITAL LETTER CHIN

        case 0x10EA:		//	ც	0xe1 0x83 0xaa	GEORGIAN SMALL LETTER CAN
            return 0x1CAA;	//	Ც	0xe1 0xb2 0xaa	GEORGIAN CAPITAL LETTER CAN

        case 0x10EB:		//	ძ	0xe1 0x83 0xab	GEORGIAN SMALL LETTER JIL
            return 0x1CAB;	//	Ძ	0xe1 0xb2 0xab	GEORGIAN CAPITAL LETTER JIL

        case 0x10EC:		//	წ	0xe1 0x83 0xac	GEORGIAN SMALL LETTER CIL
            return 0x1CAC;	//	Წ	0xe1 0xb2 0xac	GEORGIAN CAPITAL LETTER CIL

        case 0x10ED:		//	ჭ	0xe1 0x83 0xad	GEORGIAN SMALL LETTER CHAR
            return 0x1CAD;	//	Ჭ	0xe1 0xb2 0xad	GEORGIAN CAPITAL LETTER CHAR

        case 0x10EE:		//	ხ	0xe1 0x83 0xae	GEORGIAN SMALL LETTER XAN
            return 0x1CAE;	//	Ხ	0xe1 0xb2 0xae	GEORGIAN CAPITAL LETTER XAN

        case 0x10EF:		//	ჯ	0xe1 0x83 0xaf	GEORGIAN SMALL LETTER JHAN
            return 0x1CAF;	//	Ჯ	0xe1 0xb2 0xaf	GEORGIAN CAPITAL LETTER JHAN

        case 0x10F0:		//	ჰ	0xe1 0x83 0xb0	GEORGIAN SMALL LETTER HAE
            return 0x1CB0;	//	Ჰ	0xe1 0xb2 0xb0	GEORGIAN CAPITAL LETTER HAE

        case 0x10F1:		//	ჱ	0xe1 0x83 0xb1	GEORGIAN SMALL LETTER HE
            return 0x1CB1;	//	Ჱ	0xe1 0xb2 0xb1	GEORGIAN CAPITAL LETTER HE

        case 0x10F2:		//	ჲ	0xe1 0x83 0xb2	GEORGIAN SMALL LETTER HIE
            return 0x1CB2;	//	Ჲ	0xe1 0xb2 0xb2	GEORGIAN CAPITAL LETTER HIE

        case 0x10F3:		//	ჳ	0xe1 0x83 0xb3	GEORGIAN SMALL LETTER WE
            return 0x1CB3;	//	Ჳ	0xe1 0xb2 0xb3	GEORGIAN CAPITAL LETTER WE

        case 0x10F4:		//	ჴ	0xe1 0x83 0xb4	GEORGIAN SMALL LETTER HAR
            return 0x1CB4;	//	Ჴ	0xe1 0xb2 0xb4	GEORGIAN CAPITAL LETTER HAR

        case 0x10F5:		//	ჵ	0xe1 0x83 0xb5	GEORGIAN SMALL LETTER HOE
            return 0x1CB5;	//	Ჵ	0xe1 0xb2 0xb5	GEORGIAN CAPITAL LETTER HOE

        case 0x10F6:		//	ჶ	0xe1 0x83 0xb6	GEORGIAN SMALL LETTER FI
            return 0x1CB6;	//	Ჶ	0xe1 0xb2 0xb6	GEORGIAN CAPITAL LETTER FI

        case 0x10F7:		//	ჷ	0xe1 0x83 0xb7	GEORGIAN SMALL LETTER YN
            return 0x1CB7;	//	Ჷ	0xe1 0xb2 0xb7	GEORGIAN CAPITAL LETTER YN

        case 0x10F8:		//	ჸ	0xe1 0x83 0xb8	GEORGIAN SMALL LETTER ELIFI
            return 0x1CB8;	//	Ჸ	0xe1 0xb2 0xb8	GEORGIAN CAPITAL LETTER ELIFI

        case 0x10F9:		//	ჹ	0xe1 0x83 0xb9	GEORGIAN SMALL LETTER TURNED GAN
            return 0x1CB9;	//	Ჹ	0xe1 0xb2 0xb9	GEORGIAN CAPITAL LETTER TURNED GAN

        case 0x10FA:		//	ჺ	0xe1 0x83 0xba	GEORGIAN SMALL LETTER AIN
            return 0x1CBA;	//	Ჺ	0xe1 0xb2 0xba	GEORGIAN CAPITAL LETTER AIN

        case 0x10FD:		//	ჽ	0xe1 0x83 0xbd	GEORGIAN SMALL LETTER AEN
            return 0x1CBD;	//	Ჽ	0xe1 0xb2 0xbd	GEORGIAN CAPITAL LETTER AEN

        case 0x10FE:		//	ჾ	0xe1 0x83 0xbe	GEORGIAN SMALL LETTER HARD SIGN
            return 0x1CBE;	//	Ჾ	0xe1 0xb2 0xbe	GEORGIAN CAPITAL LETTER HARD SIGN

        case 0x10FF:		//	ჿ	0xe1 0x83 0xbf	GEORGIAN SMALL LETTER LABIAL SIGN
            return 0x1CBF;	//	Ჿ	0xe1 0xb2 0xbf	GEORGIAN CAPITAL LETTER LABIAL SIGN

#endif
#if UTF_SUPPORT_CHEROKEE == 1
        case 0x13F8:		//	ᏸ	0xe1 0x8f 0xb8	CHEROKEE SMALL LETTER YE
            return 0x13F0;	//	Ᏸ	0xe1 0x8f 0xb0	CHEROKEE CAPITAL LETTER YE

        case 0x13F9:		//	ᏹ	0xe1 0x8f 0xb9	CHEROKEE SMALL LETTER YI
            return 0x13F1;	//	Ᏹ	0xe1 0x8f 0xb1	CHEROKEE CAPITAL LETTER YI

        case 0x13FA:		//	ᏺ	0xe1 0x8f 0xba	CHEROKEE SMALL LETTER YO
            return 0x13F2;	//	Ᏺ	0xe1 0x8f 0xb2	CHEROKEE CAPITAL LETTER YO

        case 0x13FB:		//	ᏻ	0xe1 0x8f 0xbb	CHEROKEE SMALL LETTER YU
            return 0x13F3;	//	Ᏻ	0xe1 0x8f 0xb3	CHEROKEE CAPITAL LETTER YU

        case 0x13FC:		//	ᏼ	0xe1 0x8f 0xbc	CHEROKEE SMALL LETTER YV
            return 0x13F4;	//	Ᏼ	0xe1 0x8f 0xb4	CHEROKEE CAPITAL LETTER YV

        case 0x13FD:		//	ᏽ	0xe1 0x8f 0xbd	CHEROKEE SMALL LETTER MV
            return 0x13F5;	//	Ᏽ	0xe1 0x8f 0xb5	CHEROKEE CAPITAL LETTER MV

#endif
#if UTF_SUPPORT_LATIN == 1
        case 0x1D79:		//	ᵹ	0xe1 0xb5 0xb9	LATIN SMALL LETTER INSULAR G
            return 0xA77D;	//	Ᵹ	0xea 0x9d 0xbd	LATIN CAPITAL LETTER INSULAR G

        case 0x1D7D:		//	ᵽ	0xe1 0xb5 0xbd	LATIN SMALL LETTER P WITH STROKE
            return 0x2C63;	//	Ᵽ	0xe2 0xb1 0xa3	LATIN CAPITAL LETTER P WITH STROKE

        case 0x1D8E:		//	ᶎ	0xe1 0xb6 0x8e	LATIN SMALL LETTER Z WITH PALATAL HOOK
            return 0xA7C6;	//	Ᶎ	0xea 0x9f 0x86	LATIN CAPITAL LETTER Z WITH PALATAL HOOK

        case 0x1E01:		//	ḁ	0xe1 0xb8 0x81	LATIN SMALL LETTER A WITH RING BELOW
            return 0x1E00;	//	Ḁ	0xe1 0xb8 0x80	LATIN CAPITAL LETTER A WITH RING BELOW

        case 0x1E03:		//	ḃ	0xe1 0xb8 0x83	LATIN SMALL LETTER B WITH DOT ABOVE
            return 0x1E02;	//	Ḃ	0xe1 0xb8 0x82	LATIN CAPITAL LETTER B WITH DOT ABOVE

        case 0x1E05:		//	ḅ	0xe1 0xb8 0x85	LATIN SMALL LETTER B WITH DOT BELOW
            return 0x1E04;	//	Ḅ	0xe1 0xb8 0x84	LATIN CAPITAL LETTER B WITH DOT BELOW

        case 0x1E07:		//	ḇ	0xe1 0xb8 0x87	LATIN SMALL LETTER B WITH LINE BELOW
            return 0x1E06;	//	Ḇ	0xe1 0xb8 0x86	LATIN CAPITAL LETTER B WITH LINE BELOW

        case 0x1E09:		//	ḉ	0xe1 0xb8 0x89	LATIN SMALL LETTER C WITH CEDILLA AND ACUTE
            return 0x1E08;	//	Ḉ	0xe1 0xb8 0x88	LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE

        case 0x1E0B:		//	ḋ	0xe1 0xb8 0x8b	LATIN SMALL LETTER D WITH DOT ABOVE
            return 0x1E0A;	//	Ḋ	0xe1 0xb8 0x8a	LATIN CAPITAL LETTER D WITH DOT ABOVE

        case 0x1E0D:		//	ḍ	0xe1 0xb8 0x8d	LATIN SMALL LETTER D WITH DOT BELOW
            return 0x1E0C;	//	Ḍ	0xe1 0xb8 0x8c	LATIN CAPITAL LETTER D WITH DOT BELOW

        case 0x1E0F:		//	ḏ	0xe1 0xb8 0x8f	LATIN SMALL LETTER D WITH LINE BELOW
            return 0x1E0E;	//	Ḏ	0xe1 0xb8 0x8e	LATIN CAPITAL LETTER D WITH LINE BELOW

        case 0x1E11:		//	ḑ	0xe1 0xb8 0x91	LATIN SMALL LETTER D WITH CEDILLA
            return 0x1E10;	//	Ḑ	0xe1 0xb8 0x90	LATIN CAPITAL LETTER D WITH CEDILLA

        case 0x1E13:		//	ḓ	0xe1 0xb8 0x93	LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW
            return 0x1E12;	//	Ḓ	0xe1 0xb8 0x92	LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW

        case 0x1E15:		//	ḕ	0xe1 0xb8 0x95	LATIN SMALL LETTER E WITH MACRON AND GRAVE
            return 0x1E14;	//	Ḕ	0xe1 0xb8 0x94	LATIN CAPITAL LETTER E WITH MACRON AND GRAVE

        case 0x1E17:		//	ḗ	0xe1 0xb8 0x97	LATIN SMALL LETTER E WITH MACRON AND ACUTE
            return 0x1E16;	//	Ḗ	0xe1 0xb8 0x96	LATIN CAPITAL LETTER E WITH MACRON AND ACUTE

        case 0x1E19:		//	ḙ	0xe1 0xb8 0x99	LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW
            return 0x1E18;	//	Ḙ	0xe1 0xb8 0x98	LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW

        case 0x1E1B:		//	ḛ	0xe1 0xb8 0x9b	LATIN SMALL LETTER E WITH TILDE BELOW
            return 0x1E1A;	//	Ḛ	0xe1 0xb8 0x9a	LATIN CAPITAL LETTER E WITH TILDE BELOW

        case 0x1E1D:		//	ḝ	0xe1 0xb8 0x9d	LATIN SMALL LETTER E WITH CEDILLA AND BREVE
            return 0x1E1C;	//	Ḝ	0xe1 0xb8 0x9c	LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE

        case 0x1E1F:		//	ḟ	0xe1 0xb8 0x9f	LATIN SMALL LETTER F WITH DOT ABOVE
            return 0x1E1E;	//	Ḟ	0xe1 0xb8 0x9e	LATIN CAPITAL LETTER F WITH DOT ABOVE

        case 0x1E21:		//	ḡ	0xe1 0xb8 0xa1	LATIN SMALL LETTER G WITH MACRON
            return 0x1E20;	//	Ḡ	0xe1 0xb8 0xa0	LATIN CAPITAL LETTER G WITH MACRON

        case 0x1E23:		//	ḣ	0xe1 0xb8 0xa3	LATIN SMALL LETTER H WITH DOT ABOVE
            return 0x1E22;	//	Ḣ	0xe1 0xb8 0xa2	LATIN CAPITAL LETTER H WITH DOT ABOVE

        case 0x1E25:		//	ḥ	0xe1 0xb8 0xa5	LATIN SMALL LETTER H WITH DOT BELOW
            return 0x1E24;	//	Ḥ	0xe1 0xb8 0xa4	LATIN CAPITAL LETTER H WITH DOT BELOW

        case 0x1E27:		//	ḧ	0xe1 0xb8 0xa7	LATIN SMALL LETTER H WITH DIAERESIS
            return 0x1E26;	//	Ḧ	0xe1 0xb8 0xa6	LATIN CAPITAL LETTER H WITH DIAERESIS

        case 0x1E29:		//	ḩ	0xe1 0xb8 0xa9	LATIN SMALL LETTER H WITH CEDILLA
            return 0x1E28;	//	Ḩ	0xe1 0xb8 0xa8	LATIN CAPITAL LETTER H WITH CEDILLA

        case 0x1E2B:		//	ḫ	0xe1 0xb8 0xab	LATIN SMALL LETTER H WITH BREVE BELOW
            return 0x1E2A;	//	Ḫ	0xe1 0xb8 0xaa	LATIN CAPITAL LETTER H WITH BREVE BELOW

        case 0x1E2D:		//	ḭ	0xe1 0xb8 0xad	LATIN SMALL LETTER I WITH TILDE BELOW
            return 0x1E2C;	//	Ḭ	0xe1 0xb8 0xac	LATIN CAPITAL LETTER I WITH TILDE BELOW

        case 0x1E2F:		//	ḯ	0xe1 0xb8 0xaf	LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE
            return 0x1E2E;	//	Ḯ	0xe1 0xb8 0xae	LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE

        case 0x1E31:		//	ḱ	0xe1 0xb8 0xb1	LATIN SMALL LETTER K WITH ACUTE
            return 0x1E30;	//	Ḱ	0xe1 0xb8 0xb0	LATIN CAPITAL LETTER K WITH ACUTE

        case 0x1E33:		//	ḳ	0xe1 0xb8 0xb3	LATIN SMALL LETTER K WITH DOT BELOW
            return 0x1E32;	//	Ḳ	0xe1 0xb8 0xb2	LATIN CAPITAL LETTER K WITH DOT BELOW

        case 0x1E35:		//	ḵ	0xe1 0xb8 0xb5	LATIN SMALL LETTER K WITH LINE BELOW
            return 0x1E34;	//	Ḵ	0xe1 0xb8 0xb4	LATIN CAPITAL LETTER K WITH LINE BELOW

        case 0x1E37:		//	ḷ	0xe1 0xb8 0xb7	LATIN SMALL LETTER L WITH DOT BELOW
            return 0x1E36;	//	Ḷ	0xe1 0xb8 0xb6	LATIN CAPITAL LETTER L WITH DOT BELOW

        case 0x1E39:		//	ḹ	0xe1 0xb8 0xb9	LATIN SMALL LETTER L WITH DOT BELOW AND MACRON
            return 0x1E38;	//	Ḹ	0xe1 0xb8 0xb8	LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON

        case 0x1E3B:		//	ḻ	0xe1 0xb8 0xbb	LATIN SMALL LETTER L WITH LINE BELOW
            return 0x1E3A;	//	Ḻ	0xe1 0xb8 0xba	LATIN CAPITAL LETTER L WITH LINE BELOW

        case 0x1E3D:		//	ḽ	0xe1 0xb8 0xbd	LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW
            return 0x1E3C;	//	Ḽ	0xe1 0xb8 0xbc	LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW

        case 0x1E3F:		//	ḿ	0xe1 0xb8 0xbf	LATIN SMALL LETTER M WITH ACUTE
            return 0x1E3E;	//	Ḿ	0xe1 0xb8 0xbe	LATIN CAPITAL LETTER M WITH ACUTE

        case 0x1E41:		//	ṁ	0xe1 0xb9 0x81	LATIN SMALL LETTER M WITH DOT ABOVE
            return 0x1E40;	//	Ṁ	0xe1 0xb9 0x80	LATIN CAPITAL LETTER M WITH DOT ABOVE

        case 0x1E43:		//	ṃ	0xe1 0xb9 0x83	LATIN SMALL LETTER M WITH DOT BELOW
            return 0x1E42;	//	Ṃ	0xe1 0xb9 0x82	LATIN CAPITAL LETTER M WITH DOT BELOW

        case 0x1E45:		//	ṅ	0xe1 0xb9 0x85	LATIN SMALL LETTER N WITH DOT ABOVE
            return 0x1E44;	//	Ṅ	0xe1 0xb9 0x84	LATIN CAPITAL LETTER N WITH DOT ABOVE

        case 0x1E47:		//	ṇ	0xe1 0xb9 0x87	LATIN SMALL LETTER N WITH DOT BELOW
            return 0x1E46;	//	Ṇ	0xe1 0xb9 0x86	LATIN CAPITAL LETTER N WITH DOT BELOW

        case 0x1E49:		//	ṉ	0xe1 0xb9 0x89	LATIN SMALL LETTER N WITH LINE BELOW
            return 0x1E48;	//	Ṉ	0xe1 0xb9 0x88	LATIN CAPITAL LETTER N WITH LINE BELOW

        case 0x1E4B:		//	ṋ	0xe1 0xb9 0x8b	LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW
            return 0x1E4A;	//	Ṋ	0xe1 0xb9 0x8a	LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW

        case 0x1E4D:		//	ṍ	0xe1 0xb9 0x8d	LATIN SMALL LETTER O WITH TILDE AND ACUTE
            return 0x1E4C;	//	Ṍ	0xe1 0xb9 0x8c	LATIN CAPITAL LETTER O WITH TILDE AND ACUTE

        case 0x1E4F:		//	ṏ	0xe1 0xb9 0x8f	LATIN SMALL LETTER O WITH TILDE AND DIAERESIS
            return 0x1E4E;	//	Ṏ	0xe1 0xb9 0x8e	LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS

        case 0x1E51:		//	ṑ	0xe1 0xb9 0x91	LATIN SMALL LETTER O WITH MACRON AND GRAVE
            return 0x1E50;	//	Ṑ	0xe1 0xb9 0x90	LATIN CAPITAL LETTER O WITH MACRON AND GRAVE

        case 0x1E53:		//	ṓ	0xe1 0xb9 0x93	LATIN SMALL LETTER O WITH MACRON AND ACUTE
            return 0x1E52;	//	Ṓ	0xe1 0xb9 0x92	LATIN CAPITAL LETTER O WITH MACRON AND ACUTE

        case 0x1E55:		//	ṕ	0xe1 0xb9 0x95	LATIN SMALL LETTER P WITH ACUTE
            return 0x1E54;	//	Ṕ	0xe1 0xb9 0x94	LATIN CAPITAL LETTER P WITH ACUTE

        case 0x1E57:		//	ṗ	0xe1 0xb9 0x97	LATIN SMALL LETTER P WITH DOT ABOVE
            return 0x1E56;	//	Ṗ	0xe1 0xb9 0x96	LATIN CAPITAL LETTER P WITH DOT ABOVE

        case 0x1E59:		//	ṙ	0xe1 0xb9 0x99	LATIN SMALL LETTER R WITH DOT ABOVE
            return 0x1E58;	//	Ṙ	0xe1 0xb9 0x98	LATIN CAPITAL LETTER R WITH DOT ABOVE

        case 0x1E5B:		//	ṛ	0xe1 0xb9 0x9b	LATIN SMALL LETTER R WITH DOT BELOW
            return 0x1E5A;	//	Ṛ	0xe1 0xb9 0x9a	LATIN CAPITAL LETTER R WITH DOT BELOW

        case 0x1E5D:		//	ṝ	0xe1 0xb9 0x9d	LATIN SMALL LETTER R WITH DOT BELOW AND MACRON
            return 0x1E5C;	//	Ṝ	0xe1 0xb9 0x9c	LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON

        case 0x1E5F:		//	ṟ	0xe1 0xb9 0x9f	LATIN SMALL LETTER R WITH LINE BELOW
            return 0x1E5E;	//	Ṟ	0xe1 0xb9 0x9e	LATIN CAPITAL LETTER R WITH LINE BELOW

        case 0x1E61:		//	ṡ	0xe1 0xb9 0xa1	LATIN SMALL LETTER S WITH DOT ABOVE
            return 0x1E60;	//	Ṡ	0xe1 0xb9 0xa0	LATIN CAPITAL LETTER S WITH DOT ABOVE

        case 0x1E63:		//	ṣ	0xe1 0xb9 0xa3	LATIN SMALL LETTER S WITH DOT BELOW
            return 0x1E62;	//	Ṣ	0xe1 0xb9 0xa2	LATIN CAPITAL LETTER S WITH DOT BELOW

        case 0x1E65:		//	ṥ	0xe1 0xb9 0xa5	LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE
            return 0x1E64;	//	Ṥ	0xe1 0xb9 0xa4	LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE

        case 0x1E67:		//	ṧ	0xe1 0xb9 0xa7	LATIN SMALL LETTER S WITH CARON AND DOT ABOVE
            return 0x1E66;	//	Ṧ	0xe1 0xb9 0xa6	LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE

        case 0x1E69:		//	ṩ	0xe1 0xb9 0xa9	LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE
            return 0x1E68;	//	Ṩ	0xe1 0xb9 0xa8	LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE

        case 0x1E6B:		//	ṫ	0xe1 0xb9 0xab	LATIN SMALL LETTER T WITH DOT ABOVE
            return 0x1E6A;	//	Ṫ	0xe1 0xb9 0xaa	LATIN CAPITAL LETTER T WITH DOT ABOVE

        case 0x1E6D:		//	ṭ	0xe1 0xb9 0xad	LATIN SMALL LETTER T WITH DOT BELOW
            return 0x1E6C;	//	Ṭ	0xe1 0xb9 0xac	LATIN CAPITAL LETTER T WITH DOT BELOW

        case 0x1E6F:		//	ṯ	0xe1 0xb9 0xaf	LATIN SMALL LETTER T WITH LINE BELOW
            return 0x1E6E;	//	Ṯ	0xe1 0xb9 0xae	LATIN CAPITAL LETTER T WITH LINE BELOW

        case 0x1E71:		//	ṱ	0xe1 0xb9 0xb1	LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW
            return 0x1E70;	//	Ṱ	0xe1 0xb9 0xb0	LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW

        case 0x1E73:		//	ṳ	0xe1 0xb9 0xb3	LATIN SMALL LETTER U WITH DIAERESIS BELOW
            return 0x1E72;	//	Ṳ	0xe1 0xb9 0xb2	LATIN CAPITAL LETTER U WITH DIAERESIS BELOW

        case 0x1E75:		//	ṵ	0xe1 0xb9 0xb5	LATIN SMALL LETTER U WITH TILDE BELOW
            return 0x1E74;	//	Ṵ	0xe1 0xb9 0xb4	LATIN CAPITAL LETTER U WITH TILDE BELOW

        case 0x1E77:		//	ṷ	0xe1 0xb9 0xb7	LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW
            return 0x1E76;	//	Ṷ	0xe1 0xb9 0xb6	LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW

        case 0x1E79:		//	ṹ	0xe1 0xb9 0xb9	LATIN SMALL LETTER U WITH TILDE AND ACUTE
            return 0x1E78;	//	Ṹ	0xe1 0xb9 0xb8	LATIN CAPITAL LETTER U WITH TILDE AND ACUTE

        case 0x1E7B:		//	ṻ	0xe1 0xb9 0xbb	LATIN SMALL LETTER U WITH MACRON AND DIAERESIS
            return 0x1E7A;	//	Ṻ	0xe1 0xb9 0xba	LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS

        case 0x1E7D:		//	ṽ	0xe1 0xb9 0xbd	LATIN SMALL LETTER V WITH TILDE
            return 0x1E7C;	//	Ṽ	0xe1 0xb9 0xbc	LATIN CAPITAL LETTER V WITH TILDE

        case 0x1E7F:		//	ṿ	0xe1 0xb9 0xbf	LATIN SMALL LETTER V WITH DOT BELOW
            return 0x1E7E;	//	Ṿ	0xe1 0xb9 0xbe	LATIN CAPITAL LETTER V WITH DOT BELOW

        case 0x1E81:		//	ẁ	0xe1 0xba 0x81	LATIN SMALL LETTER W WITH GRAVE
            return 0x1E80;	//	Ẁ	0xe1 0xba 0x80	LATIN CAPITAL LETTER W WITH GRAVE

        case 0x1E83:		//	ẃ	0xe1 0xba 0x83	LATIN SMALL LETTER W WITH ACUTE
            return 0x1E82;	//	Ẃ	0xe1 0xba 0x82	LATIN CAPITAL LETTER W WITH ACUTE

        case 0x1E85:		//	ẅ	0xe1 0xba 0x85	LATIN SMALL LETTER W WITH DIAERESIS
            return 0x1E84;	//	Ẅ	0xe1 0xba 0x84	LATIN CAPITAL LETTER W WITH DIAERESIS

        case 0x1E87:		//	ẇ	0xe1 0xba 0x87	LATIN SMALL LETTER W WITH DOT ABOVE
            return 0x1E86;	//	Ẇ	0xe1 0xba 0x86	LATIN CAPITAL LETTER W WITH DOT ABOVE

        case 0x1E89:		//	ẉ	0xe1 0xba 0x89	LATIN SMALL LETTER W WITH DOT BELOW
            return 0x1E88;	//	Ẉ	0xe1 0xba 0x88	LATIN CAPITAL LETTER W WITH DOT BELOW

        case 0x1E8B:		//	ẋ	0xe1 0xba 0x8b	LATIN SMALL LETTER X WITH DOT ABOVE
            return 0x1E8A;	//	Ẋ	0xe1 0xba 0x8a	LATIN CAPITAL LETTER X WITH DOT ABOVE

        case 0x1E8D:		//	ẍ	0xe1 0xba 0x8d	LATIN SMALL LETTER X WITH DIAERESIS
            return 0x1E8C;	//	Ẍ	0xe1 0xba 0x8c	LATIN CAPITAL LETTER X WITH DIAERESIS

        case 0x1E8F:		//	ẏ	0xe1 0xba 0x8f	LATIN SMALL LETTER Y WITH DOT ABOVE
            return 0x1E8E;	//	Ẏ	0xe1 0xba 0x8e	LATIN CAPITAL LETTER Y WITH DOT ABOVE

        case 0x1E91:		//	ẑ	0xe1 0xba 0x91	LATIN SMALL LETTER Z WITH CIRCUMFLEX
            return 0x1E90;	//	Ẑ	0xe1 0xba 0x90	LATIN CAPITAL LETTER Z WITH CIRCUMFLEX

        case 0x1E93:		//	ẓ	0xe1 0xba 0x93	LATIN SMALL LETTER Z WITH DOT BELOW
            return 0x1E92;	//	Ẓ	0xe1 0xba 0x92	LATIN CAPITAL LETTER Z WITH DOT BELOW

        case 0x1E95:		//	ẕ	0xe1 0xba 0x95	LATIN SMALL LETTER Z WITH LINE BELOW
            return 0x1E94;	//	Ẕ	0xe1 0xba 0x94	LATIN CAPITAL LETTER Z WITH LINE BELOW

        case 0x1EA1:		//	ạ	0xe1 0xba 0xa1	LATIN SMALL LETTER A WITH DOT BELOW
            return 0x1EA0;	//	Ạ	0xe1 0xba 0xa0	LATIN CAPITAL LETTER A WITH DOT BELOW

        case 0x1EA3:		//	ả	0xe1 0xba 0xa3	LATIN SMALL LETTER A WITH HOOK ABOVE
            return 0x1EA2;	//	Ả	0xe1 0xba 0xa2	LATIN CAPITAL LETTER A WITH HOOK ABOVE

        case 0x1EA5:		//	ấ	0xe1 0xba 0xa5	LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE
            return 0x1EA4;	//	Ấ	0xe1 0xba 0xa4	LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE

        case 0x1EA7:		//	ầ	0xe1 0xba 0xa7	LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE
            return 0x1EA6;	//	Ầ	0xe1 0xba 0xa6	LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE

        case 0x1EA9:		//	ẩ	0xe1 0xba 0xa9	LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
            return 0x1EA8;	//	Ẩ	0xe1 0xba 0xa8	LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE

        case 0x1EAB:		//	ẫ	0xe1 0xba 0xab	LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE
            return 0x1EAA;	//	Ẫ	0xe1 0xba 0xaa	LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE

        case 0x1EAD:		//	ậ	0xe1 0xba 0xad	LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW
            return 0x1EAC;	//	Ậ	0xe1 0xba 0xac	LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW

        case 0x1EAF:		//	ắ	0xe1 0xba 0xaf	LATIN SMALL LETTER A WITH BREVE AND ACUTE
            return 0x1EAE;	//	Ắ	0xe1 0xba 0xae	LATIN CAPITAL LETTER A WITH BREVE AND ACUTE

        case 0x1EB1:		//	ằ	0xe1 0xba 0xb1	LATIN SMALL LETTER A WITH BREVE AND GRAVE
            return 0x1EB0;	//	Ằ	0xe1 0xba 0xb0	LATIN CAPITAL LETTER A WITH BREVE AND GRAVE

        case 0x1EB3:		//	ẳ	0xe1 0xba 0xb3	LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE
            return 0x1EB2;	//	Ẳ	0xe1 0xba 0xb2	LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE

        case 0x1EB5:		//	ẵ	0xe1 0xba 0xb5	LATIN SMALL LETTER A WITH BREVE AND TILDE
            return 0x1EB4;	//	Ẵ	0xe1 0xba 0xb4	LATIN CAPITAL LETTER A WITH BREVE AND TILDE

        case 0x1EB7:		//	ặ	0xe1 0xba 0xb7	LATIN SMALL LETTER A WITH BREVE AND DOT BELOW
            return 0x1EB6;	//	Ặ	0xe1 0xba 0xb6	LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW

        case 0x1EB9:		//	ẹ	0xe1 0xba 0xb9	LATIN SMALL LETTER E WITH DOT BELOW
            return 0x1EB8;	//	Ẹ	0xe1 0xba 0xb8	LATIN CAPITAL LETTER E WITH DOT BELOW

        case 0x1EBB:		//	ẻ	0xe1 0xba 0xbb	LATIN SMALL LETTER E WITH HOOK ABOVE
            return 0x1EBA;	//	Ẻ	0xe1 0xba 0xba	LATIN CAPITAL LETTER E WITH HOOK ABOVE

        case 0x1EBD:		//	ẽ	0xe1 0xba 0xbd	LATIN SMALL LETTER E WITH TILDE
            return 0x1EBC;	//	Ẽ	0xe1 0xba 0xbc	LATIN CAPITAL LETTER E WITH TILDE

        case 0x1EBF:		//	ế	0xe1 0xba 0xbf	LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE
            return 0x1EBE;	//	Ế	0xe1 0xba 0xbe	LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE

        case 0x1EC1:		//	ề	0xe1 0xbb 0x81	LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE
            return 0x1EC0;	//	Ề	0xe1 0xbb 0x80	LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE

        case 0x1EC3:		//	ể	0xe1 0xbb 0x83	LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE
            return 0x1EC2;	//	Ể	0xe1 0xbb 0x82	LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE

        case 0x1EC5:		//	ễ	0xe1 0xbb 0x85	LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE
            return 0x1EC4;	//	Ễ	0xe1 0xbb 0x84	LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE

        case 0x1EC7:		//	ệ	0xe1 0xbb 0x87	LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW
            return 0x1EC6;	//	Ệ	0xe1 0xbb 0x86	LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW

        case 0x1EC9:		//	ỉ	0xe1 0xbb 0x89	LATIN SMALL LETTER I WITH HOOK ABOVE
            return 0x1EC8;	//	Ỉ	0xe1 0xbb 0x88	LATIN CAPITAL LETTER I WITH HOOK ABOVE

        case 0x1ECB:		//	ị	0xe1 0xbb 0x8b	LATIN SMALL LETTER I WITH DOT BELOW
            return 0x1ECA;	//	Ị	0xe1 0xbb 0x8a	LATIN CAPITAL LETTER I WITH DOT BELOW

        case 0x1ECD:		//	ọ	0xe1 0xbb 0x8d	LATIN SMALL LETTER O WITH DOT BELOW
            return 0x1ECC;	//	Ọ	0xe1 0xbb 0x8c	LATIN CAPITAL LETTER O WITH DOT BELOW

        case 0x1ECF:		//	ỏ	0xe1 0xbb 0x8f	LATIN SMALL LETTER O WITH HOOK ABOVE
            return 0x1ECE;	//	Ỏ	0xe1 0xbb 0x8e	LATIN CAPITAL LETTER O WITH HOOK ABOVE

        case 0x1ED1:		//	ố	0xe1 0xbb 0x91	LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE
            return 0x1ED0;	//	Ố	0xe1 0xbb 0x90	LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE

        case 0x1ED3:		//	ồ	0xe1 0xbb 0x93	LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE
            return 0x1ED2;	//	Ồ	0xe1 0xbb 0x92	LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE

        case 0x1ED5:		//	ổ	0xe1 0xbb 0x95	LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
            return 0x1ED4;	//	Ổ	0xe1 0xbb 0x94	LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE

        case 0x1ED7:		//	ỗ	0xe1 0xbb 0x97	LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE
            return 0x1ED6;	//	Ỗ	0xe1 0xbb 0x96	LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE

        case 0x1ED9:		//	ộ	0xe1 0xbb 0x99	LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
            return 0x1ED8;	//	Ộ	0xe1 0xbb 0x98	LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW

        case 0x1EDB:		//	ớ	0xe1 0xbb 0x9b	LATIN SMALL LETTER O WITH HORN AND ACUTE
            return 0x1EDA;	//	Ớ	0xe1 0xbb 0x9a	LATIN CAPITAL LETTER O WITH HORN AND ACUTE

        case 0x1EDD:		//	ờ	0xe1 0xbb 0x9d	LATIN SMALL LETTER O WITH HORN AND GRAVE
            return 0x1EDC;	//	Ờ	0xe1 0xbb 0x9c	LATIN CAPITAL LETTER O WITH HORN AND GRAVE

        case 0x1EDF:		//	ở	0xe1 0xbb 0x9f	LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE
            return 0x1EDE;	//	Ở	0xe1 0xbb 0x9e	LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE

        case 0x1EE1:		//	ỡ	0xe1 0xbb 0xa1	LATIN SMALL LETTER O WITH HORN AND TILDE
            return 0x1EE0;	//	Ỡ	0xe1 0xbb 0xa0	LATIN CAPITAL LETTER O WITH HORN AND TILDE

        case 0x1EE3:		//	ợ	0xe1 0xbb 0xa3	LATIN SMALL LETTER O WITH HORN AND DOT BELOW
            return 0x1EE2;	//	Ợ	0xe1 0xbb 0xa2	LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW

        case 0x1EE5:		//	ụ	0xe1 0xbb 0xa5	LATIN SMALL LETTER U WITH DOT BELOW
            return 0x1EE4;	//	Ụ	0xe1 0xbb 0xa4	LATIN CAPITAL LETTER U WITH DOT BELOW

        case 0x1EE7:		//	ủ	0xe1 0xbb 0xa7	LATIN SMALL LETTER U WITH HOOK ABOVE
            return 0x1EE6;	//	Ủ	0xe1 0xbb 0xa6	LATIN CAPITAL LETTER U WITH HOOK ABOVE

        case 0x1EE9:		//	ứ	0xe1 0xbb 0xa9	LATIN SMALL LETTER U WITH HORN AND ACUTE
            return 0x1EE8;	//	Ứ	0xe1 0xbb 0xa8	LATIN CAPITAL LETTER U WITH HORN AND ACUTE

        case 0x1EEB:		//	ừ	0xe1 0xbb 0xab	LATIN SMALL LETTER U WITH HORN AND GRAVE
            return 0x1EEA;	//	Ừ	0xe1 0xbb 0xaa	LATIN CAPITAL LETTER U WITH HORN AND GRAVE

        case 0x1EED:		//	ử	0xe1 0xbb 0xad	LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE
            return 0x1EEC;	//	Ử	0xe1 0xbb 0xac	LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE

        case 0x1EEF:		//	ữ	0xe1 0xbb 0xaf	LATIN SMALL LETTER U WITH HORN AND TILDE
            return 0x1EEE;	//	Ữ	0xe1 0xbb 0xae	LATIN CAPITAL LETTER U WITH HORN AND TILDE

        case 0x1EF1:		//	ự	0xe1 0xbb 0xb1	LATIN SMALL LETTER U WITH HORN AND DOT BELOW
            return 0x1EF0;	//	Ự	0xe1 0xbb 0xb0	LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW

        case 0x1EF3:		//	ỳ	0xe1 0xbb 0xb3	LATIN SMALL LETTER Y WITH GRAVE
            return 0x1EF2;	//	Ỳ	0xe1 0xbb 0xb2	LATIN CAPITAL LETTER Y WITH GRAVE

        case 0x1EF5:		//	ỵ	0xe1 0xbb 0xb5	LATIN SMALL LETTER Y WITH DOT BELOW
            return 0x1EF4;	//	Ỵ	0xe1 0xbb 0xb4	LATIN CAPITAL LETTER Y WITH DOT BELOW

        case 0x1EF7:		//	ỷ	0xe1 0xbb 0xb7	LATIN SMALL LETTER Y WITH HOOK ABOVE
            return 0x1EF6;	//	Ỷ	0xe1 0xbb 0xb6	LATIN CAPITAL LETTER Y WITH HOOK ABOVE

        case 0x1EF9:		//	ỹ	0xe1 0xbb 0xb9	LATIN SMALL LETTER Y WITH TILDE
            return 0x1EF8;	//	Ỹ	0xe1 0xbb 0xb8	LATIN CAPITAL LETTER Y WITH TILDE

        case 0x1EFB:		//	ỻ	0xe1 0xbb 0xbb	LATIN SMALL LETTER MIDDLE-WELSH LL
            return 0x1EFA;	//	Ỻ	0xe1 0xbb 0xba	LATIN CAPITAL LETTER MIDDLE-WELSH LL

        case 0x1EFD:		//	ỽ	0xe1 0xbb 0xbd	LATIN SMALL LETTER MIDDLE-WELSH V
            return 0x1EFC;	//	Ỽ	0xe1 0xbb 0xbc	LATIN CAPITAL LETTER MIDDLE-WELSH V

        case 0x1EFF:		//	ỿ	0xe1 0xbb 0xbf	LATIN SMALL LETTER Y WITH LOOP
            return 0x1EFE;	//	Ỿ	0xe1 0xbb 0xbe	LATIN CAPITAL LETTER Y WITH LOOP

#endif
#if UTF_SUPPORT_GREEK == 1
        case 0x1F00:		//	ἀ	0xe1 0xbc 0x80	GREEK SMALL LETTER ALPHA WITH PSILI
            return 0x1F08;	//	Ἀ	0xe1 0xbc 0x88	GREEK CAPITAL LETTER ALPHA WITH PSILI

        case 0x1F01:		//	ἁ	0xe1 0xbc 0x81	GREEK SMALL LETTER ALPHA WITH DASIA
            return 0x1F09;	//	Ἁ	0xe1 0xbc 0x89	GREEK CAPITAL LETTER ALPHA WITH DASIA

        case 0x1F02:		//	ἂ	0xe1 0xbc 0x82	GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA
            return 0x1F0A;	//	Ἂ	0xe1 0xbc 0x8a	GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA

        case 0x1F03:		//	ἃ	0xe1 0xbc 0x83	GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA
            return 0x1F0B;	//	Ἃ	0xe1 0xbc 0x8b	GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA

        case 0x1F04:		//	ἄ	0xe1 0xbc 0x84	GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA
            return 0x1F0C;	//	Ἄ	0xe1 0xbc 0x8c	GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA

        case 0x1F05:		//	ἅ	0xe1 0xbc 0x85	GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA
            return 0x1F0D;	//	Ἅ	0xe1 0xbc 0x8d	GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA

        case 0x1F06:		//	ἆ	0xe1 0xbc 0x86	GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI
            return 0x1F0E;	//	Ἆ	0xe1 0xbc 0x8e	GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI

        case 0x1F07:		//	ἇ	0xe1 0xbc 0x87	GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI
            return 0x1F0F;	//	Ἇ	0xe1 0xbc 0x8f	GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI

        case 0x1F10:		//	ἐ	0xe1 0xbc 0x90	GREEK SMALL LETTER EPSILON WITH PSILI
            return 0x1F18;	//	Ἐ	0xe1 0xbc 0x98	GREEK CAPITAL LETTER EPSILON WITH PSILI

        case 0x1F11:		//	ἑ	0xe1 0xbc 0x91	GREEK SMALL LETTER EPSILON WITH DASIA
            return 0x1F19;	//	Ἑ	0xe1 0xbc 0x99	GREEK CAPITAL LETTER EPSILON WITH DASIA

        case 0x1F12:		//	ἒ	0xe1 0xbc 0x92	GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA
            return 0x1F1A;	//	Ἒ	0xe1 0xbc 0x9a	GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA

        case 0x1F13:		//	ἓ	0xe1 0xbc 0x93	GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA
            return 0x1F1B;	//	Ἓ	0xe1 0xbc 0x9b	GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA

        case 0x1F14:		//	ἔ	0xe1 0xbc 0x94	GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA
            return 0x1F1C;	//	Ἔ	0xe1 0xbc 0x9c	GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA

        case 0x1F15:		//	ἕ	0xe1 0xbc 0x95	GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA
            return 0x1F1D;	//	Ἕ	0xe1 0xbc 0x9d	GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA

        case 0x1F20:		//	ἠ	0xe1 0xbc 0xa0	GREEK SMALL LETTER ETA WITH PSILI
            return 0x1F28;	//	Ἠ	0xe1 0xbc 0xa8	GREEK CAPITAL LETTER ETA WITH PSILI

        case 0x1F21:		//	ἡ	0xe1 0xbc 0xa1	GREEK SMALL LETTER ETA WITH DASIA
            return 0x1F29;	//	Ἡ	0xe1 0xbc 0xa9	GREEK CAPITAL LETTER ETA WITH DASIA

        case 0x1F22:		//	ἢ	0xe1 0xbc 0xa2	GREEK SMALL LETTER ETA WITH PSILI AND VARIA
            return 0x1F2A;	//	Ἢ	0xe1 0xbc 0xaa	GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA

        case 0x1F23:		//	ἣ	0xe1 0xbc 0xa3	GREEK SMALL LETTER ETA WITH DASIA AND VARIA
            return 0x1F2B;	//	Ἣ	0xe1 0xbc 0xab	GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA

        case 0x1F24:		//	ἤ	0xe1 0xbc 0xa4	GREEK SMALL LETTER ETA WITH PSILI AND OXIA
            return 0x1F2C;	//	Ἤ	0xe1 0xbc 0xac	GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA

        case 0x1F25:		//	ἥ	0xe1 0xbc 0xa5	GREEK SMALL LETTER ETA WITH DASIA AND OXIA
            return 0x1F2D;	//	Ἥ	0xe1 0xbc 0xad	GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA

        case 0x1F26:		//	ἦ	0xe1 0xbc 0xa6	GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI
            return 0x1F2E;	//	Ἦ	0xe1 0xbc 0xae	GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI

        case 0x1F27:		//	ἧ	0xe1 0xbc 0xa7	GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI
            return 0x1F2F;	//	Ἧ	0xe1 0xbc 0xaf	GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI

        case 0x1F30:		//	ἰ	0xe1 0xbc 0xb0	GREEK SMALL LETTER IOTA WITH PSILI
            return 0x1F38;	//	Ἰ	0xe1 0xbc 0xb8	GREEK CAPITAL LETTER IOTA WITH PSILI

        case 0x1F31:		//	ἱ	0xe1 0xbc 0xb1	GREEK SMALL LETTER IOTA WITH DASIA
            return 0x1F39;	//	Ἱ	0xe1 0xbc 0xb9	GREEK CAPITAL LETTER IOTA WITH DASIA

        case 0x1F32:		//	ἲ	0xe1 0xbc 0xb2	GREEK SMALL LETTER IOTA WITH PSILI AND VARIA
            return 0x1F3A;	//	Ἲ	0xe1 0xbc 0xba	GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA

        case 0x1F33:		//	ἳ	0xe1 0xbc 0xb3	GREEK SMALL LETTER IOTA WITH DASIA AND VARIA
            return 0x1F3B;	//	Ἳ	0xe1 0xbc 0xbb	GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA

        case 0x1F34:		//	ἴ	0xe1 0xbc 0xb4	GREEK SMALL LETTER IOTA WITH PSILI AND OXIA
            return 0x1F3C;	//	Ἴ	0xe1 0xbc 0xbc	GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA

        case 0x1F35:		//	ἵ	0xe1 0xbc 0xb5	GREEK SMALL LETTER IOTA WITH DASIA AND OXIA
            return 0x1F3D;	//	Ἵ	0xe1 0xbc 0xbd	GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA

        case 0x1F36:		//	ἶ	0xe1 0xbc 0xb6	GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI
            return 0x1F3E;	//	Ἶ	0xe1 0xbc 0xbe	GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI

        case 0x1F37:		//	ἷ	0xe1 0xbc 0xb7	GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI
            return 0x1F3F;	//	Ἷ	0xe1 0xbc 0xbf	GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI

        case 0x1F40:		//	ὀ	0xe1 0xbd 0x80	GREEK SMALL LETTER OMICRON WITH PSILI
            return 0x1F48;	//	Ὀ	0xe1 0xbd 0x88	GREEK CAPITAL LETTER OMICRON WITH PSILI

        case 0x1F41:		//	ὁ	0xe1 0xbd 0x81	GREEK SMALL LETTER OMICRON WITH DASIA
            return 0x1F49;	//	Ὁ	0xe1 0xbd 0x89	GREEK CAPITAL LETTER OMICRON WITH DASIA

        case 0x1F42:		//	ὂ	0xe1 0xbd 0x82	GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA
            return 0x1F4A;	//	Ὂ	0xe1 0xbd 0x8a	GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA

        case 0x1F43:		//	ὃ	0xe1 0xbd 0x83	GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA
            return 0x1F4B;	//	Ὃ	0xe1 0xbd 0x8b	GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA

        case 0x1F44:		//	ὄ	0xe1 0xbd 0x84	GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA
            return 0x1F4C;	//	Ὄ	0xe1 0xbd 0x8c	GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA

        case 0x1F45:		//	ὅ	0xe1 0xbd 0x85	GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA
            return 0x1F4D;	//	Ὅ	0xe1 0xbd 0x8d	GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA

        case 0x1F51:		//	ὑ	0xe1 0xbd 0x91	GREEK SMALL LETTER UPSILON WITH DASIA
            return 0x1F59;	//	Ὑ	0xe1 0xbd 0x99	GREEK CAPITAL LETTER UPSILON WITH DASIA

        case 0x1F53:		//	ὓ	0xe1 0xbd 0x93	GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA
            return 0x1F5B;	//	Ὓ	0xe1 0xbd 0x9b	GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA

        case 0x1F55:		//	ὕ	0xe1 0xbd 0x95	GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA
            return 0x1F5D;	//	Ὕ	0xe1 0xbd 0x9d	GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA

        case 0x1F57:		//	ὗ	0xe1 0xbd 0x97	GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI
            return 0x1F5F;	//	Ὗ	0xe1 0xbd 0x9f	GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI

        case 0x1F60:		//	ὠ	0xe1 0xbd 0xa0	GREEK SMALL LETTER OMEGA WITH PSILI
            return 0x1F68;	//	Ὠ	0xe1 0xbd 0xa8	GREEK CAPITAL LETTER OMEGA WITH PSILI

        case 0x1F61:		//	ὡ	0xe1 0xbd 0xa1	GREEK SMALL LETTER OMEGA WITH DASIA
            return 0x1F69;	//	Ὡ	0xe1 0xbd 0xa9	GREEK CAPITAL LETTER OMEGA WITH DASIA

        case 0x1F62:		//	ὢ	0xe1 0xbd 0xa2	GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA
            return 0x1F6A;	//	Ὢ	0xe1 0xbd 0xaa	GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA

        case 0x1F63:		//	ὣ	0xe1 0xbd 0xa3	GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA
            return 0x1F6B;	//	Ὣ	0xe1 0xbd 0xab	GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA

        case 0x1F64:		//	ὤ	0xe1 0xbd 0xa4	GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA
            return 0x1F6C;	//	Ὤ	0xe1 0xbd 0xac	GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA

        case 0x1F65:		//	ὥ	0xe1 0xbd 0xa5	GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA
            return 0x1F6D;	//	Ὥ	0xe1 0xbd 0xad	GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA

        case 0x1F66:		//	ὦ	0xe1 0xbd 0xa6	GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI
            return 0x1F6E;	//	Ὦ	0xe1 0xbd 0xae	GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI

        case 0x1F67:		//	ὧ	0xe1 0xbd 0xa7	GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI
            return 0x1F6F;	//	Ὧ	0xe1 0xbd 0xaf	GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI

        case 0x1F70:		//	ὰ	0xe1 0xbd 0xb0	GREEK SMALL LETTER ALPHA WITH VARIA
            return 0x1FBA;	//	Ὰ	0xe1 0xbe 0xba	GREEK CAPITAL LETTER ALPHA WITH VARIA

        case 0x1F71:		//	ά	0xe1 0xbd 0xb1	GREEK SMALL LETTER ALPHA WITH OXIA
            return 0x1FBB;	//	Ά	0xe1 0xbe 0xbb	GREEK CAPITAL LETTER ALPHA WITH OXIA

        case 0x1F72:		//	ὲ	0xe1 0xbd 0xb2	GREEK SMALL LETTER EPSILON WITH VARIA
            return 0x1FC8;	//	Ὲ	0xe1 0xbf 0x88	GREEK CAPITAL LETTER EPSILON WITH VARIA

        case 0x1F73:		//	έ	0xe1 0xbd 0xb3	GREEK SMALL LETTER EPSILON WITH OXIA
            return 0x1FC9;	//	Έ	0xe1 0xbf 0x89	GREEK CAPITAL LETTER EPSILON WITH OXIA

        case 0x1F74:		//	ὴ	0xe1 0xbd 0xb4	GREEK SMALL LETTER ETA WITH VARIA
            return 0x1FCA;	//	Ὴ	0xe1 0xbf 0x8a	GREEK CAPITAL LETTER ETA WITH VARIA

        case 0x1F75:		//	ή	0xe1 0xbd 0xb5	GREEK SMALL LETTER ETA WITH OXIA
            return 0x1FCB;	//	Ή	0xe1 0xbf 0x8b	GREEK CAPITAL LETTER ETA WITH OXIA

        case 0x1F76:		//	ὶ	0xe1 0xbd 0xb6	GREEK SMALL LETTER IOTA WITH VARIA
            return 0x1FDA;	//	Ὶ	0xe1 0xbf 0x9a	GREEK CAPITAL LETTER IOTA WITH VARIA

        case 0x1F77:		//	ί	0xe1 0xbd 0xb7	GREEK SMALL LETTER IOTA WITH OXIA
            return 0x1FDB;	//	Ί	0xe1 0xbf 0x9b	GREEK CAPITAL LETTER IOTA WITH OXIA

        case 0x1F78:		//	ὸ	0xe1 0xbd 0xb8	GREEK SMALL LETTER OMICRON WITH VARIA
            return 0x1FF8;	//	Ὸ	0xe1 0xbf 0xb8	GREEK CAPITAL LETTER OMICRON WITH VARIA

        case 0x1F79:		//	ό	0xe1 0xbd 0xb9	GREEK SMALL LETTER OMICRON WITH OXIA
            return 0x1FF9;	//	Ό	0xe1 0xbf 0xb9	GREEK CAPITAL LETTER OMICRON WITH OXIA

        case 0x1F7A:		//	ὺ	0xe1 0xbd 0xba	GREEK SMALL LETTER UPSILON WITH VARIA
            return 0x1FEA;	//	Ὺ	0xe1 0xbf 0xaa	GREEK CAPITAL LETTER UPSILON WITH VARIA

        case 0x1F7B:		//	ύ	0xe1 0xbd 0xbb	GREEK SMALL LETTER UPSILON WITH OXIA
            return 0x1FEB;	//	Ύ	0xe1 0xbf 0xab	GREEK CAPITAL LETTER UPSILON WITH OXIA

        case 0x1F7C:		//	ὼ	0xe1 0xbd 0xbc	GREEK SMALL LETTER OMEGA WITH VARIA
            return 0x1FFA;	//	Ὼ	0xe1 0xbf 0xba	GREEK CAPITAL LETTER OMEGA WITH VARIA

        case 0x1F7D:		//	ώ	0xe1 0xbd 0xbd	GREEK SMALL LETTER OMEGA WITH OXIA
            return 0x1FFB;	//	Ώ	0xe1 0xbf 0xbb	GREEK CAPITAL LETTER OMEGA WITH OXIA

        case 0x1F80:		//	ᾀ	0xe1 0xbe 0x80	GREEK SMALL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI
            return 0x1F88;	//	ᾈ	0xe1 0xbe 0x88	GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI

        case 0x1F81:		//	ᾁ	0xe1 0xbe 0x81	GREEK SMALL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI
            return 0x1F89;	//	ᾉ	0xe1 0xbe 0x89	GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI

        case 0x1F82:		//	ᾂ	0xe1 0xbe 0x82	GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI
            return 0x1F8A;	//	ᾊ	0xe1 0xbe 0x8a	GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI

        case 0x1F83:		//	ᾃ	0xe1 0xbe 0x83	GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI
            return 0x1F8B;	//	ᾋ	0xe1 0xbe 0x8b	GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI

        case 0x1F84:		//	ᾄ	0xe1 0xbe 0x84	GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI
            return 0x1F8C;	//	ᾌ	0xe1 0xbe 0x8c	GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI

        case 0x1F85:		//	ᾅ	0xe1 0xbe 0x85	GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI
            return 0x1F8D;	//	ᾍ	0xe1 0xbe 0x8d	GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI

        case 0x1F86:		//	ᾆ	0xe1 0xbe 0x86	GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1F8E;	//	ᾎ	0xe1 0xbe 0x8e	GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1F87:		//	ᾇ	0xe1 0xbe 0x87	GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1F8F;	//	ᾏ	0xe1 0xbe 0x8f	GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1F90:		//	ᾐ	0xe1 0xbe 0x90	GREEK SMALL LETTER ETA WITH PSILI AND PROSGEGRAMMENI
            return 0x1F98;	//	ᾘ	0xe1 0xbe 0x98	GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI

        case 0x1F91:		//	ᾑ	0xe1 0xbe 0x91	GREEK SMALL LETTER ETA WITH DASIA AND PROSGEGRAMMENI
            return 0x1F99;	//	ᾙ	0xe1 0xbe 0x99	GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI

        case 0x1F92:		//	ᾒ	0xe1 0xbe 0x92	GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI
            return 0x1F9A;	//	ᾚ	0xe1 0xbe 0x9a	GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI

        case 0x1F93:		//	ᾓ	0xe1 0xbe 0x93	GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI
            return 0x1F9B;	//	ᾛ	0xe1 0xbe 0x9b	GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI

        case 0x1F94:		//	ᾔ	0xe1 0xbe 0x94	GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI
            return 0x1F9C;	//	ᾜ	0xe1 0xbe 0x9c	GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI

        case 0x1F95:		//	ᾕ	0xe1 0xbe 0x95	GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI
            return 0x1F9D;	//	ᾝ	0xe1 0xbe 0x9d	GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI

        case 0x1F96:		//	ᾖ	0xe1 0xbe 0x96	GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1F9E;	//	ᾞ	0xe1 0xbe 0x9e	GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1F97:		//	ᾗ	0xe1 0xbe 0x97	GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1F9F;	//	ᾟ	0xe1 0xbe 0x9f	GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1FA0:		//	ᾠ	0xe1 0xbe 0xa0	GREEK SMALL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI
            return 0x1FA8;	//	ᾨ	0xe1 0xbe 0xa8	GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI

        case 0x1FA1:		//	ᾡ	0xe1 0xbe 0xa1	GREEK SMALL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI
            return 0x1FA9;	//	ᾩ	0xe1 0xbe 0xa9	GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI

        case 0x1FA2:		//	ᾢ	0xe1 0xbe 0xa2	GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI
            return 0x1FAA;	//	ᾪ	0xe1 0xbe 0xaa	GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI

        case 0x1FA3:		//	ᾣ	0xe1 0xbe 0xa3	GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI
            return 0x1FAB;	//	ᾫ	0xe1 0xbe 0xab	GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI

        case 0x1FA4:		//	ᾤ	0xe1 0xbe 0xa4	GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI
            return 0x1FAC;	//	ᾬ	0xe1 0xbe 0xac	GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI

        case 0x1FA5:		//	ᾥ	0xe1 0xbe 0xa5	GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI
            return 0x1FAD;	//	ᾭ	0xe1 0xbe 0xad	GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI

        case 0x1FA6:		//	ᾦ	0xe1 0xbe 0xa6	GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1FAE;	//	ᾮ	0xe1 0xbe 0xae	GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1FA7:		//	ᾧ	0xe1 0xbe 0xa7	GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI
            return 0x1FAF;	//	ᾯ	0xe1 0xbe 0xaf	GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

        case 0x1FB0:		//	ᾰ	0xe1 0xbe 0xb0	GREEK SMALL LETTER ALPHA WITH VRACHY
            return 0x1FB8;	//	Ᾰ	0xe1 0xbe 0xb8	GREEK CAPITAL LETTER ALPHA WITH VRACHY

        case 0x1FB1:		//	ᾱ	0xe1 0xbe 0xb1	GREEK SMALL LETTER ALPHA WITH MACRON
            return 0x1FB9;	//	Ᾱ	0xe1 0xbe 0xb9	GREEK CAPITAL LETTER ALPHA WITH MACRON

        case 0x1FB3:		//	ᾳ	0xe1 0xbe 0xb3	GREEK SMALL LETTER ALPHA WITH PROSGEGRAMMENI
            return 0x1FBC;	//	ᾼ	0xe1 0xbe 0xbc	GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI

        case 0x1FC3:		//	ῃ	0xe1 0xbf 0x83	GREEK SMALL LETTER ETA WITH PROSGEGRAMMENI
            return 0x1FCC;	//	ῌ	0xe1 0xbf 0x8c	GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI

        case 0x1FD0:		//	ῐ	0xe1 0xbf 0x90	GREEK SMALL LETTER IOTA WITH VRACHY
            return 0x1FD8;	//	Ῐ	0xe1 0xbf 0x98	GREEK CAPITAL LETTER IOTA WITH VRACHY

        case 0x1FD1:		//	ῑ	0xe1 0xbf 0x91	GREEK SMALL LETTER IOTA WITH MACRON
            return 0x1FD9;	//	Ῑ	0xe1 0xbf 0x99	GREEK CAPITAL LETTER IOTA WITH MACRON

        case 0x1FE0:		//	ῠ	0xe1 0xbf 0xa0	GREEK SMALL LETTER UPSILON WITH VRACHY
            return 0x1FE8;	//	Ῠ	0xe1 0xbf 0xa8	GREEK CAPITAL LETTER UPSILON WITH VRACHY

        case 0x1FE1:		//	ῡ	0xe1 0xbf 0xa1	GREEK SMALL LETTER UPSILON WITH MACRON
            return 0x1FE9;	//	Ῡ	0xe1 0xbf 0xa9	GREEK CAPITAL LETTER UPSILON WITH MACRON

        case 0x1FE5:		//	ῥ	0xe1 0xbf 0xa5	GREEK SMALL LETTER RHO WITH DASIA
            return 0x1FEC;	//	Ῥ	0xe1 0xbf 0xac	GREEK CAPITAL LETTER RHO WITH DASIA

        case 0x1FF3:		//	ῳ	0xe1 0xbf 0xb3	GREEK SMALL LETTER OMEGA WITH PROSGEGRAMMENI
            return 0x1FFC;	//	ῼ	0xe1 0xbf 0xbc	GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI

#endif
#if UTF_SUPPORT_GLAGOLITIC == 1
        case 0x2C30:		//	ⰰ	0xe2 0xb0 0xb0	GLAGOLITIC SMALL LETTER AZU
            return 0x2C00;	//	Ⰰ	0xe2 0xb0 0x80	GLAGOLITIC CAPITAL LETTER AZU

        case 0x2C31:		//	ⰱ	0xe2 0xb0 0xb1	GLAGOLITIC SMALL LETTER BUKY
            return 0x2C01;	//	Ⰱ	0xe2 0xb0 0x81	GLAGOLITIC CAPITAL LETTER BUKY

        case 0x2C32:		//	ⰲ	0xe2 0xb0 0xb2	GLAGOLITIC SMALL LETTER VEDE
            return 0x2C02;	//	Ⰲ	0xe2 0xb0 0x82	GLAGOLITIC CAPITAL LETTER VEDE

        case 0x2C33:		//	ⰳ	0xe2 0xb0 0xb3	GLAGOLITIC SMALL LETTER GLAGOLI
            return 0x2C03;	//	Ⰳ	0xe2 0xb0 0x83	GLAGOLITIC CAPITAL LETTER GLAGOLI

        case 0x2C34:		//	ⰴ	0xe2 0xb0 0xb4	GLAGOLITIC SMALL LETTER DOBRO
            return 0x2C04;	//	Ⰴ	0xe2 0xb0 0x84	GLAGOLITIC CAPITAL LETTER DOBRO

        case 0x2C35:		//	ⰵ	0xe2 0xb0 0xb5	GLAGOLITIC SMALL LETTER YESTU
            return 0x2C05;	//	Ⰵ	0xe2 0xb0 0x85	GLAGOLITIC CAPITAL LETTER YESTU

        case 0x2C36:		//	ⰶ	0xe2 0xb0 0xb6	GLAGOLITIC SMALL LETTER ZHIVETE
            return 0x2C06;	//	Ⰶ	0xe2 0xb0 0x86	GLAGOLITIC CAPITAL LETTER ZHIVETE

        case 0x2C37:		//	ⰷ	0xe2 0xb0 0xb7	GLAGOLITIC SMALL LETTER DZELO
            return 0x2C07;	//	Ⰷ	0xe2 0xb0 0x87	GLAGOLITIC CAPITAL LETTER DZELO

        case 0x2C38:		//	ⰸ	0xe2 0xb0 0xb8	GLAGOLITIC SMALL LETTER ZEMLJA
            return 0x2C08;	//	Ⰸ	0xe2 0xb0 0x88	GLAGOLITIC CAPITAL LETTER ZEMLJA

        case 0x2C39:		//	ⰹ	0xe2 0xb0 0xb9	GLAGOLITIC SMALL LETTER IZHE
            return 0x2C09;	//	Ⰹ	0xe2 0xb0 0x89	GLAGOLITIC CAPITAL LETTER IZHE

        case 0x2C3A:		//	ⰺ	0xe2 0xb0 0xba	GLAGOLITIC SMALL LETTER INITIAL IZHE
            return 0x2C0A;	//	Ⰺ	0xe2 0xb0 0x8a	GLAGOLITIC CAPITAL LETTER INITIAL IZHE

        case 0x2C3B:		//	ⰻ	0xe2 0xb0 0xbb	GLAGOLITIC SMALL LETTER I
            return 0x2C0B;	//	Ⰻ	0xe2 0xb0 0x8b	GLAGOLITIC CAPITAL LETTER I

        case 0x2C3C:		//	ⰼ	0xe2 0xb0 0xbc	GLAGOLITIC SMALL LETTER DJERVI
            return 0x2C0C;	//	Ⰼ	0xe2 0xb0 0x8c	GLAGOLITIC CAPITAL LETTER DJERVI

        case 0x2C3D:		//	ⰽ	0xe2 0xb0 0xbd	GLAGOLITIC SMALL LETTER KAKO
            return 0x2C0D;	//	Ⰽ	0xe2 0xb0 0x8d	GLAGOLITIC CAPITAL LETTER KAKO

        case 0x2C3E:		//	ⰾ	0xe2 0xb0 0xbe	GLAGOLITIC SMALL LETTER LJUDIJE
            return 0x2C0E;	//	Ⰾ	0xe2 0xb0 0x8e	GLAGOLITIC CAPITAL LETTER LJUDIJE

        case 0x2C3F:		//	ⰿ	0xe2 0xb0 0xbf	GLAGOLITIC SMALL LETTER MYSLITE
            return 0x2C0F;	//	Ⰿ	0xe2 0xb0 0x8f	GLAGOLITIC CAPITAL LETTER MYSLITE

        case 0x2C40:		//	ⱀ	0xe2 0xb1 0x80	GLAGOLITIC SMALL LETTER NASHI
            return 0x2C10;	//	Ⱀ	0xe2 0xb0 0x90	GLAGOLITIC CAPITAL LETTER NASHI

        case 0x2C41:		//	ⱁ	0xe2 0xb1 0x81	GLAGOLITIC SMALL LETTER ONU
            return 0x2C11;	//	Ⱁ	0xe2 0xb0 0x91	GLAGOLITIC CAPITAL LETTER ONU

        case 0x2C42:		//	ⱂ	0xe2 0xb1 0x82	GLAGOLITIC SMALL LETTER POKOJI
            return 0x2C12;	//	Ⱂ	0xe2 0xb0 0x92	GLAGOLITIC CAPITAL LETTER POKOJI

        case 0x2C43:		//	ⱃ	0xe2 0xb1 0x83	GLAGOLITIC SMALL LETTER RITSI
            return 0x2C13;	//	Ⱃ	0xe2 0xb0 0x93	GLAGOLITIC CAPITAL LETTER RITSI

        case 0x2C44:		//	ⱄ	0xe2 0xb1 0x84	GLAGOLITIC SMALL LETTER SLOVO
            return 0x2C14;	//	Ⱄ	0xe2 0xb0 0x94	GLAGOLITIC CAPITAL LETTER SLOVO

        case 0x2C45:		//	ⱅ	0xe2 0xb1 0x85	GLAGOLITIC SMALL LETTER TVRIDO
            return 0x2C15;	//	Ⱅ	0xe2 0xb0 0x95	GLAGOLITIC CAPITAL LETTER TVRIDO

        case 0x2C46:		//	ⱆ	0xe2 0xb1 0x86	GLAGOLITIC SMALL LETTER UKU
            return 0x2C16;	//	Ⱆ	0xe2 0xb0 0x96	GLAGOLITIC CAPITAL LETTER UKU

        case 0x2C47:		//	ⱇ	0xe2 0xb1 0x87	GLAGOLITIC SMALL LETTER FRITU
            return 0x2C17;	//	Ⱇ	0xe2 0xb0 0x97	GLAGOLITIC CAPITAL LETTER FRITU

        case 0x2C48:		//	ⱈ	0xe2 0xb1 0x88	GLAGOLITIC SMALL LETTER HERU
            return 0x2C18;	//	Ⱈ	0xe2 0xb0 0x98	GLAGOLITIC CAPITAL LETTER HERU

        case 0x2C49:		//	ⱉ	0xe2 0xb1 0x89	GLAGOLITIC SMALL LETTER OTU
            return 0x2C19;	//	Ⱉ	0xe2 0xb0 0x99	GLAGOLITIC CAPITAL LETTER OTU

        case 0x2C4A:		//	ⱊ	0xe2 0xb1 0x8a	GLAGOLITIC SMALL LETTER PE
            return 0x2C1A;	//	Ⱊ	0xe2 0xb0 0x9a	GLAGOLITIC CAPITAL LETTER PE

        case 0x2C4B:		//	ⱋ	0xe2 0xb1 0x8b	GLAGOLITIC SMALL LETTER SHTA
            return 0x2C1B;	//	Ⱋ	0xe2 0xb0 0x9b	GLAGOLITIC CAPITAL LETTER SHTA

        case 0x2C4C:		//	ⱌ	0xe2 0xb1 0x8c	GLAGOLITIC SMALL LETTER TSI
            return 0x2C1C;	//	Ⱌ	0xe2 0xb0 0x9c	GLAGOLITIC CAPITAL LETTER TSI

        case 0x2C4D:		//	ⱍ	0xe2 0xb1 0x8d	GLAGOLITIC SMALL LETTER CHRIVI
            return 0x2C1D;	//	Ⱍ	0xe2 0xb0 0x9d	GLAGOLITIC CAPITAL LETTER CHRIVI

        case 0x2C4E:		//	ⱎ	0xe2 0xb1 0x8e	GLAGOLITIC SMALL LETTER SHA
            return 0x2C1E;	//	Ⱎ	0xe2 0xb0 0x9e	GLAGOLITIC CAPITAL LETTER SHA

        case 0x2C4F:		//	ⱏ	0xe2 0xb1 0x8f	GLAGOLITIC SMALL LETTER YERU
            return 0x2C1F;	//	Ⱏ	0xe2 0xb0 0x9f	GLAGOLITIC CAPITAL LETTER YERU

        case 0x2C50:		//	ⱐ	0xe2 0xb1 0x90	GLAGOLITIC SMALL LETTER YERI
            return 0x2C20;	//	Ⱐ	0xe2 0xb0 0xa0	GLAGOLITIC CAPITAL LETTER YERI

        case 0x2C51:		//	ⱑ	0xe2 0xb1 0x91	GLAGOLITIC SMALL LETTER YATI
            return 0x2C21;	//	Ⱑ	0xe2 0xb0 0xa1	GLAGOLITIC CAPITAL LETTER YATI

        case 0x2C52:		//	ⱒ	0xe2 0xb1 0x92	GLAGOLITIC SMALL LETTER SPIDERY HA
            return 0x2C22;	//	Ⱒ	0xe2 0xb0 0xa2	GLAGOLITIC CAPITAL LETTER SPIDERY HA

        case 0x2C53:		//	ⱓ	0xe2 0xb1 0x93	GLAGOLITIC SMALL LETTER YU
            return 0x2C23;	//	Ⱓ	0xe2 0xb0 0xa3	GLAGOLITIC CAPITAL LETTER YU

        case 0x2C54:		//	ⱔ	0xe2 0xb1 0x94	GLAGOLITIC SMALL LETTER SMALL YUS
            return 0x2C24;	//	Ⱔ	0xe2 0xb0 0xa4	GLAGOLITIC CAPITAL LETTER SMALL YUS

        case 0x2C55:		//	ⱕ	0xe2 0xb1 0x95	GLAGOLITIC SMALL LETTER SMALL YUS WITH TAIL
            return 0x2C25;	//	Ⱕ	0xe2 0xb0 0xa5	GLAGOLITIC CAPITAL LETTER SMALL YUS WITH TAIL

        case 0x2C56:		//	ⱖ	0xe2 0xb1 0x96	GLAGOLITIC SMALL LETTER YO
            return 0x2C26;	//	Ⱖ	0xe2 0xb0 0xa6	GLAGOLITIC CAPITAL LETTER YO

        case 0x2C57:		//	ⱗ	0xe2 0xb1 0x97	GLAGOLITIC SMALL LETTER IOTATED SMALL YUS
            return 0x2C27;	//	Ⱗ	0xe2 0xb0 0xa7	GLAGOLITIC CAPITAL LETTER IOTATED SMALL YUS

        case 0x2C58:		//	ⱘ	0xe2 0xb1 0x98	GLAGOLITIC SMALL LETTER BIG YUS
            return 0x2C28;	//	Ⱘ	0xe2 0xb0 0xa8	GLAGOLITIC CAPITAL LETTER BIG YUS

        case 0x2C59:		//	ⱙ	0xe2 0xb1 0x99	GLAGOLITIC SMALL LETTER IOTATED BIG YUS
            return 0x2C29;	//	Ⱙ	0xe2 0xb0 0xa9	GLAGOLITIC CAPITAL LETTER IOTATED BIG YUS

        case 0x2C5A:		//	ⱚ	0xe2 0xb1 0x9a	GLAGOLITIC SMALL LETTER FITA
            return 0x2C2A;	//	Ⱚ	0xe2 0xb0 0xaa	GLAGOLITIC CAPITAL LETTER FITA

        case 0x2C5B:		//	ⱛ	0xe2 0xb1 0x9b	GLAGOLITIC SMALL LETTER IZHITSA
            return 0x2C2B;	//	Ⱛ	0xe2 0xb0 0xab	GLAGOLITIC CAPITAL LETTER IZHITSA

        case 0x2C5C:		//	ⱜ	0xe2 0xb1 0x9c	GLAGOLITIC SMALL LETTER SHTAPIC
            return 0x2C2C;	//	Ⱜ	0xe2 0xb0 0xac	GLAGOLITIC CAPITAL LETTER SHTAPIC

        case 0x2C5D:		//	ⱝ	0xe2 0xb1 0x9d	GLAGOLITIC SMALL LETTER TROKUTASTI A
            return 0x2C2D;	//	Ⱝ	0xe2 0xb0 0xad	GLAGOLITIC CAPITAL LETTER TROKUTASTI A

        case 0x2C5E:		//	ⱞ	0xe2 0xb1 0x9e	GLAGOLITIC SMALL LETTER LATINATE MYSLITE
            return 0x2C2E;	//	Ⱞ	0xe2 0xb0 0xae	GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE

#endif
#if UTF_SUPPORT_LATIN == 1
        case 0x2C61:		//	ⱡ	0xe2 0xb1 0xa1	LATIN SMALL LETTER L WITH DOUBLE BAR
            return 0x2C60;	//	Ⱡ	0xe2 0xb1 0xa0	LATIN CAPITAL LETTER L WITH DOUBLE BAR

        case 0x2C65:		//	ⱥ	0xe2 0xb1 0xa5	LATIN SMALL LETTER A WITH STROKE
            return 0x023A;	//	Ⱥ	0xc8 0xba	LATIN CAPITAL LETTER A WITH STROKE

        case 0x2C66:		//	ⱦ	0xe2 0xb1 0xa6	LATIN SMALL LETTER T WITH DIAGONAL STROKE
            return 0x023E;	//	Ⱦ	0xc8 0xbe	LATIN CAPITAL LETTER T WITH DIAGONAL STROKE

        case 0x2C68:		//	ⱨ	0xe2 0xb1 0xa8	LATIN SMALL LETTER H WITH DESCENDER
            return 0x2C67;	//	Ⱨ	0xe2 0xb1 0xa7	LATIN CAPITAL LETTER H WITH DESCENDER

        case 0x2C6A:		//	ⱪ	0xe2 0xb1 0xaa	LATIN SMALL LETTER K WITH DESCENDER
            return 0x2C69;	//	Ⱪ	0xe2 0xb1 0xa9	LATIN CAPITAL LETTER K WITH DESCENDER

        case 0x2C6C:		//	ⱬ	0xe2 0xb1 0xac	LATIN SMALL LETTER Z WITH DESCENDER
            return 0x2C6B;	//	Ⱬ	0xe2 0xb1 0xab	LATIN CAPITAL LETTER Z WITH DESCENDER

        case 0x2C73:		//	ⱳ	0xe2 0xb1 0xb3	LATIN SMALL LETTER W WITH HOOK
            return 0x2C72;	//	Ⱳ	0xe2 0xb1 0xb2	LATIN CAPITAL LETTER W WITH HOOK

        case 0x2C76:		//	ⱶ	0xe2 0xb1 0xb6	LATIN SMALL LETTER HALF H
            return 0x2C75;	//	Ⱶ	0xe2 0xb1 0xb5	LATIN CAPITAL LETTER HALF H

#endif
#if UTF_SUPPORT_COPTIC == 1
        case 0x2C81:		//	ⲁ	0xe2 0xb2 0x81	COPTIC SMALL LETTER ALFA
            return 0x2C80;	//	Ⲁ	0xe2 0xb2 0x80	COPTIC CAPITAL LETTER ALFA

        case 0x2C83:		//	ⲃ	0xe2 0xb2 0x83	COPTIC SMALL LETTER VIDA
            return 0x2C82;	//	Ⲃ	0xe2 0xb2 0x82	COPTIC CAPITAL LETTER VIDA

        case 0x2C85:		//	ⲅ	0xe2 0xb2 0x85	COPTIC SMALL LETTER GAMMA
            return 0x2C84;	//	Ⲅ	0xe2 0xb2 0x84	COPTIC CAPITAL LETTER GAMMA

        case 0x2C87:		//	ⲇ	0xe2 0xb2 0x87	COPTIC SMALL LETTER DALDA
            return 0x2C86;	//	Ⲇ	0xe2 0xb2 0x86	COPTIC CAPITAL LETTER DALDA

        case 0x2C89:		//	ⲉ	0xe2 0xb2 0x89	COPTIC SMALL LETTER EIE
            return 0x2C88;	//	Ⲉ	0xe2 0xb2 0x88	COPTIC CAPITAL LETTER EIE

        case 0x2C8B:		//	ⲋ	0xe2 0xb2 0x8b	COPTIC SMALL LETTER SOU
            return 0x2C8A;	//	Ⲋ	0xe2 0xb2 0x8a	COPTIC CAPITAL LETTER SOU

        case 0x2C8D:		//	ⲍ	0xe2 0xb2 0x8d	COPTIC SMALL LETTER ZATA
            return 0x2C8C;	//	Ⲍ	0xe2 0xb2 0x8c	COPTIC CAPITAL LETTER ZATA

        case 0x2C8F:		//	ⲏ	0xe2 0xb2 0x8f	COPTIC SMALL LETTER HATE
            return 0x2C8E;	//	Ⲏ	0xe2 0xb2 0x8e	COPTIC CAPITAL LETTER HATE

        case 0x2C91:		//	ⲑ	0xe2 0xb2 0x91	COPTIC SMALL LETTER THETHE
            return 0x2C90;	//	Ⲑ	0xe2 0xb2 0x90	COPTIC CAPITAL LETTER THETHE

        case 0x2C93:		//	ⲓ	0xe2 0xb2 0x93	COPTIC SMALL LETTER IAUDA
            return 0x2C92;	//	Ⲓ	0xe2 0xb2 0x92	COPTIC CAPITAL LETTER IAUDA

        case 0x2C95:		//	ⲕ	0xe2 0xb2 0x95	COPTIC SMALL LETTER KAPA
            return 0x2C94;	//	Ⲕ	0xe2 0xb2 0x94	COPTIC CAPITAL LETTER KAPA

        case 0x2C97:		//	ⲗ	0xe2 0xb2 0x97	COPTIC SMALL LETTER LAULA
            return 0x2C96;	//	Ⲗ	0xe2 0xb2 0x96	COPTIC CAPITAL LETTER LAULA

        case 0x2C99:		//	ⲙ	0xe2 0xb2 0x99	COPTIC SMALL LETTER MI
            return 0x2C98;	//	Ⲙ	0xe2 0xb2 0x98	COPTIC CAPITAL LETTER MI

        case 0x2C9B:		//	ⲛ	0xe2 0xb2 0x9b	COPTIC SMALL LETTER NI
            return 0x2C9A;	//	Ⲛ	0xe2 0xb2 0x9a	COPTIC CAPITAL LETTER NI

        case 0x2C9D:		//	ⲝ	0xe2 0xb2 0x9d	COPTIC SMALL LETTER KSI
            return 0x2C9C;	//	Ⲝ	0xe2 0xb2 0x9c	COPTIC CAPITAL LETTER KSI

        case 0x2C9F:		//	ⲟ	0xe2 0xb2 0x9f	COPTIC SMALL LETTER O
            return 0x2C9E;	//	Ⲟ	0xe2 0xb2 0x9e	COPTIC CAPITAL LETTER O

        case 0x2CA1:		//	ⲡ	0xe2 0xb2 0xa1	COPTIC SMALL LETTER PI
            return 0x2CA0;	//	Ⲡ	0xe2 0xb2 0xa0	COPTIC CAPITAL LETTER PI

        case 0x2CA3:		//	ⲣ	0xe2 0xb2 0xa3	COPTIC SMALL LETTER RO
            return 0x2CA2;	//	Ⲣ	0xe2 0xb2 0xa2	COPTIC CAPITAL LETTER RO

        case 0x2CA5:		//	ⲥ	0xe2 0xb2 0xa5	COPTIC SMALL LETTER SIMA
            return 0x2CA4;	//	Ⲥ	0xe2 0xb2 0xa4	COPTIC CAPITAL LETTER SIMA

        case 0x2CA7:		//	ⲧ	0xe2 0xb2 0xa7	COPTIC SMALL LETTER TAU
            return 0x2CA6;	//	Ⲧ	0xe2 0xb2 0xa6	COPTIC CAPITAL LETTER TAU

        case 0x2CA9:		//	ⲩ	0xe2 0xb2 0xa9	COPTIC SMALL LETTER UA
            return 0x2CA8;	//	Ⲩ	0xe2 0xb2 0xa8	COPTIC CAPITAL LETTER UA

        case 0x2CAB:		//	ⲫ	0xe2 0xb2 0xab	COPTIC SMALL LETTER FI
            return 0x2CAA;	//	Ⲫ	0xe2 0xb2 0xaa	COPTIC CAPITAL LETTER FI

        case 0x2CAD:		//	ⲭ	0xe2 0xb2 0xad	COPTIC SMALL LETTER KHI
            return 0x2CAC;	//	Ⲭ	0xe2 0xb2 0xac	COPTIC CAPITAL LETTER KHI

        case 0x2CAF:		//	ⲯ	0xe2 0xb2 0xaf	COPTIC SMALL LETTER PSI
            return 0x2CAE;	//	Ⲯ	0xe2 0xb2 0xae	COPTIC CAPITAL LETTER PSI

        case 0x2CB1:		//	ⲱ	0xe2 0xb2 0xb1	COPTIC SMALL LETTER OOU
            return 0x2CB0;	//	Ⲱ	0xe2 0xb2 0xb0	COPTIC CAPITAL LETTER OOU

        case 0x2CB3:		//	ⲳ	0xe2 0xb2 0xb3	COPTIC SMALL LETTER DIALECT-P ALEF
            return 0x2CB2;	//	Ⲳ	0xe2 0xb2 0xb2	COPTIC CAPITAL LETTER DIALECT-P ALEF

        case 0x2CB5:		//	ⲵ	0xe2 0xb2 0xb5	COPTIC SMALL LETTER OLD COPTIC AIN
            return 0x2CB4;	//	Ⲵ	0xe2 0xb2 0xb4	COPTIC CAPITAL LETTER OLD COPTIC AIN

        case 0x2CB7:		//	ⲷ	0xe2 0xb2 0xb7	COPTIC SMALL LETTER CRYPTOGRAMMIC EIE
            return 0x2CB6;	//	Ⲷ	0xe2 0xb2 0xb6	COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE

        case 0x2CB9:		//	ⲹ	0xe2 0xb2 0xb9	COPTIC SMALL LETTER DIALECT-P KAPA
            return 0x2CB8;	//	Ⲹ	0xe2 0xb2 0xb8	COPTIC CAPITAL LETTER DIALECT-P KAPA

        case 0x2CBB:		//	ⲻ	0xe2 0xb2 0xbb	COPTIC SMALL LETTER DIALECT-P NI
            return 0x2CBA;	//	Ⲻ	0xe2 0xb2 0xba	COPTIC CAPITAL LETTER DIALECT-P NI

        case 0x2CBD:		//	ⲽ	0xe2 0xb2 0xbd	COPTIC SMALL LETTER CRYPTOGRAMMIC NI
            return 0x2CBC;	//	Ⲽ	0xe2 0xb2 0xbc	COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI

        case 0x2CBF:		//	ⲿ	0xe2 0xb2 0xbf	COPTIC SMALL LETTER OLD COPTIC OOU
            return 0x2CBE;	//	Ⲿ	0xe2 0xb2 0xbe	COPTIC CAPITAL LETTER OLD COPTIC OOU

        case 0x2CC1:		//	ⳁ	0xe2 0xb3 0x81	COPTIC SMALL LETTER SAMPI
            return 0x2CC0;	//	Ⳁ	0xe2 0xb3 0x80	COPTIC CAPITAL LETTER SAMPI

        case 0x2CC3:		//	ⳃ	0xe2 0xb3 0x83	COPTIC SMALL LETTER CROSSED SHEI
            return 0x2CC2;	//	Ⳃ	0xe2 0xb3 0x82	COPTIC CAPITAL LETTER CROSSED SHEI

        case 0x2CC5:		//	ⳅ	0xe2 0xb3 0x85	COPTIC SMALL LETTER OLD COPTIC SHEI
            return 0x2CC4;	//	Ⳅ	0xe2 0xb3 0x84	COPTIC CAPITAL LETTER OLD COPTIC SHEI

        case 0x2CC7:		//	ⳇ	0xe2 0xb3 0x87	COPTIC SMALL LETTER OLD COPTIC ESH
            return 0x2CC6;	//	Ⳇ	0xe2 0xb3 0x86	COPTIC CAPITAL LETTER OLD COPTIC ESH

        case 0x2CC9:		//	ⳉ	0xe2 0xb3 0x89	COPTIC SMALL LETTER AKHMIMIC KHEI
            return 0x2CC8;	//	Ⳉ	0xe2 0xb3 0x88	COPTIC CAPITAL LETTER AKHMIMIC KHEI

        case 0x2CCB:		//	ⳋ	0xe2 0xb3 0x8b	COPTIC SMALL LETTER DIALECT-P HORI
            return 0x2CCA;	//	Ⳋ	0xe2 0xb3 0x8a	COPTIC CAPITAL LETTER DIALECT-P HORI

        case 0x2CCD:		//	ⳍ	0xe2 0xb3 0x8d	COPTIC SMALL LETTER OLD COPTIC HORI
            return 0x2CCC;	//	Ⳍ	0xe2 0xb3 0x8c	COPTIC CAPITAL LETTER OLD COPTIC HORI

        case 0x2CCF:		//	ⳏ	0xe2 0xb3 0x8f	COPTIC SMALL LETTER OLD COPTIC HA
            return 0x2CCE;	//	Ⳏ	0xe2 0xb3 0x8e	COPTIC CAPITAL LETTER OLD COPTIC HA

        case 0x2CD1:		//	ⳑ	0xe2 0xb3 0x91	COPTIC SMALL LETTER L-SHAPED HA
            return 0x2CD0;	//	Ⳑ	0xe2 0xb3 0x90	COPTIC CAPITAL LETTER L-SHAPED HA

        case 0x2CD3:		//	ⳓ	0xe2 0xb3 0x93	COPTIC SMALL LETTER OLD COPTIC HEI
            return 0x2CD2;	//	Ⳓ	0xe2 0xb3 0x92	COPTIC CAPITAL LETTER OLD COPTIC HEI

        case 0x2CD5:		//	ⳕ	0xe2 0xb3 0x95	COPTIC SMALL LETTER OLD COPTIC HAT
            return 0x2CD4;	//	Ⳕ	0xe2 0xb3 0x94	COPTIC CAPITAL LETTER OLD COPTIC HAT

        case 0x2CD7:		//	ⳗ	0xe2 0xb3 0x97	COPTIC SMALL LETTER OLD COPTIC GANGIA
            return 0x2CD6;	//	Ⳗ	0xe2 0xb3 0x96	COPTIC CAPITAL LETTER OLD COPTIC GANGIA

        case 0x2CD9:		//	ⳙ	0xe2 0xb3 0x99	COPTIC SMALL LETTER OLD COPTIC DJA
            return 0x2CD8;	//	Ⳙ	0xe2 0xb3 0x98	COPTIC CAPITAL LETTER OLD COPTIC DJA

        case 0x2CDB:		//	ⳛ	0xe2 0xb3 0x9b	COPTIC SMALL LETTER OLD COPTIC SHIMA
            return 0x2CDA;	//	Ⳛ	0xe2 0xb3 0x9a	COPTIC CAPITAL LETTER OLD COPTIC SHIMA

        case 0x2CDD:		//	ⳝ	0xe2 0xb3 0x9d	COPTIC SMALL LETTER OLD NUBIAN SHIMA
            return 0x2CDC;	//	Ⳝ	0xe2 0xb3 0x9c	COPTIC CAPITAL LETTER OLD NUBIAN SHIMA

        case 0x2CDF:		//	ⳟ	0xe2 0xb3 0x9f	COPTIC SMALL LETTER OLD NUBIAN NGI
            return 0x2CDE;	//	Ⳟ	0xe2 0xb3 0x9e	COPTIC CAPITAL LETTER OLD NUBIAN NGI

        case 0x2CE1:		//	ⳡ	0xe2 0xb3 0xa1	COPTIC SMALL LETTER OLD NUBIAN NYI
            return 0x2CE0;	//	Ⳡ	0xe2 0xb3 0xa0	COPTIC CAPITAL LETTER OLD NUBIAN NYI

        case 0x2CE3:		//	ⳣ	0xe2 0xb3 0xa3	COPTIC SMALL LETTER OLD NUBIAN WAU
            return 0x2CE2;	//	Ⳣ	0xe2 0xb3 0xa2	COPTIC CAPITAL LETTER OLD NUBIAN WAU

        case 0x2CEC:		//	ⳬ	0xe2 0xb3 0xac	COPTIC SMALL LETTER CRYPTOGRAMMIC SHEI
            return 0x2CEB;	//	Ⳬ	0xe2 0xb3 0xab	COPTIC CAPITAL LETTER CRYPTOGRAMMIC SHEI

        case 0x2CEE:		//	ⳮ	0xe2 0xb3 0xae	COPTIC SMALL LETTER CRYPTOGRAMMIC GANGIA
            return 0x2CED;	//	Ⳮ	0xe2 0xb3 0xad	COPTIC CAPITAL LETTER CRYPTOGRAMMIC GANGIA

        case 0x2CF3:		//	ⳳ	0xe2 0xb3 0xb3	COPTIC SMALL LETTER BOHAIRIC KHEI
            return 0x2CF2;	//	Ⳳ	0xe2 0xb3 0xb2	COPTIC CAPITAL LETTER BOHAIRIC KHEI

#endif
#if UTF_SUPPORT_GEORGIAN
        case 0x2D00:		//	ⴀ	0xe2 0xb4 0x80	GEORGIAN LETTER AN
            return 0x1C90;	//	Ა	0xe1 0xb2 0x90	GEORGIAN CAPITAL LETTER AN

        case 0x2D01:		//	ⴁ	0xe2 0xb4 0x81	GEORGIAN LETTER BAN
            return 0x1C91;	//	Ბ	0xe1 0xb2 0x91	GEORGIAN CAPITAL LETTER BAN

        case 0x2D02:		//	ⴂ	0xe2 0xb4 0x82	GEORGIAN LETTER GAN
            return 0x1C92;	//	Გ	0xe1 0xb2 0x92	GEORGIAN CAPITAL LETTER GAN

        case 0x2D03:		//	ⴃ	0xe2 0xb4 0x83	GEORGIAN LETTER DON
            return 0x1C93;	//	Დ	0xe1 0xb2 0x93	GEORGIAN CAPITAL LETTER DON

        case 0x2D04:		//	ⴄ	0xe2 0xb4 0x84	GEORGIAN LETTER EN
            return 0x1C94;	//	Ე	0xe1 0xb2 0x94	GEORGIAN CAPITAL LETTER EN

        case 0x2D05:		//	ⴅ	0xe2 0xb4 0x85	GEORGIAN LETTER VIN
            return 0x1C95;	//	Ვ	0xe1 0xb2 0x95	GEORGIAN CAPITAL LETTER VIN

        case 0x2D06:		//	ⴆ	0xe2 0xb4 0x86	GEORGIAN LETTER ZEN
            return 0x1C96;	//	Ზ	0xe1 0xb2 0x96	GEORGIAN CAPITAL LETTER ZEN

        case 0x2D07:		//	ⴇ	0xe2 0xb4 0x87	GEORGIAN LETTER TAN
            return 0x1C97;	//	Თ	0xe1 0xb2 0x97	GEORGIAN CAPITAL LETTER TAN

        case 0x2D08:		//	ⴈ	0xe2 0xb4 0x88	GEORGIAN LETTER IN
            return 0x1C98;	//	Ი	0xe1 0xb2 0x98	GEORGIAN CAPITAL LETTER IN

        case 0x2D09:		//	ⴉ	0xe2 0xb4 0x89	GEORGIAN LETTER KAN
            return 0x1C99;	//	Კ	0xe1 0xb2 0x99	GEORGIAN CAPITAL LETTER KAN

        case 0x2D0A:		//	ⴊ	0xe2 0xb4 0x8a	GEORGIAN LETTER LAS
            return 0x1C9A;	//	Ლ	0xe1 0xb2 0x9a	GEORGIAN CAPITAL LETTER LAS

        case 0x2D0B:		//	ⴋ	0xe2 0xb4 0x8b	GEORGIAN LETTER MAN
            return 0x1C9B;	//	Მ	0xe1 0xb2 0x9b	GEORGIAN CAPITAL LETTER MAN

        case 0x2D0C:		//	ⴌ	0xe2 0xb4 0x8c	GEORGIAN LETTER NAR
            return 0x1C9C;	//	Ნ	0xe1 0xb2 0x9c	GEORGIAN CAPITAL LETTER NAR

        case 0x2D0D:		//	ⴍ	0xe2 0xb4 0x8d	GEORGIAN LETTER ON
            return 0x1C9D;	//	Ო	0xe1 0xb2 0x9d	GEORGIAN CAPITAL LETTER ON

        case 0x2D0E:		//	ⴎ	0xe2 0xb4 0x8e	GEORGIAN LETTER PAR
            return 0x1C9E;	//	Პ	0xe1 0xb2 0x9e	GEORGIAN CAPITAL LETTER PAR

        case 0x2D0F:		//	ⴏ	0xe2 0xb4 0x8f	GEORGIAN LETTER ZHAR
            return 0x1C9F;	//	Ჟ	0xe1 0xb2 0x9f	GEORGIAN CAPITAL LETTER ZHAR

        case 0x2D10:		//	ⴐ	0xe2 0xb4 0x90	GEORGIAN LETTER RAE
            return 0x1CA0;	//	Რ	0xe1 0xb2 0xa0	GEORGIAN CAPITAL LETTER RAE

        case 0x2D11:		//	ⴑ	0xe2 0xb4 0x91	GEORGIAN LETTER SAN
            return 0x1CA1;	//	Ს	0xe1 0xb2 0xa1	GEORGIAN CAPITAL LETTER SAN

        case 0x2D12:		//	ⴒ	0xe2 0xb4 0x92	GEORGIAN LETTER TAR
            return 0x1CA2;	//	Ტ	0xe1 0xb2 0xa2	GEORGIAN CAPITAL LETTER TAR

        case 0x2D13:		//	ⴓ	0xe2 0xb4 0x93	GEORGIAN LETTER UN
            return 0x1CA3;	//	Უ	0xe1 0xb2 0xa3	GEORGIAN CAPITAL LETTER UN

        case 0x2D14:		//	ⴔ	0xe2 0xb4 0x94	GEORGIAN LETTER PHAR
            return 0x1CA4;	//	Ფ	0xe1 0xb2 0xa4	GEORGIAN CAPITAL LETTER PHAR

        case 0x2D15:		//	ⴕ	0xe2 0xb4 0x95	GEORGIAN LETTER KHAR
            return 0x1CA5;	//	Ქ	0xe1 0xb2 0xa5	GEORGIAN CAPITAL LETTER KHAR

        case 0x2D16:		//	ⴖ	0xe2 0xb4 0x96	GEORGIAN LETTER GHAN
            return 0x1CA6;	//	Ღ	0xe1 0xb2 0xa6	GEORGIAN CAPITAL LETTER GHAN

        case 0x2D17:		//	ⴗ	0xe2 0xb4 0x97	GEORGIAN LETTER QAR
            return 0x1CA7;	//	Ყ	0xe1 0xb2 0xa7	GEORGIAN CAPITAL LETTER QAR

        case 0x2D18:		//	ⴘ	0xe2 0xb4 0x98	GEORGIAN LETTER SHIN
            return 0x1CA8;	//	Შ	0xe1 0xb2 0xa8	GEORGIAN CAPITAL LETTER SHIN

        case 0x2D19:		//	ⴙ	0xe2 0xb4 0x99	GEORGIAN LETTER CHIN
            return 0x1CA9;	//	Ჩ	0xe1 0xb2 0xa9	GEORGIAN CAPITAL LETTER CHIN

        case 0x2D1A:		//	ⴚ	0xe2 0xb4 0x9a	GEORGIAN LETTER CAN
            return 0x1CAA;	//	Ც	0xe1 0xb2 0xaa	GEORGIAN CAPITAL LETTER CAN

        case 0x2D1B:		//	ⴛ	0xe2 0xb4 0x9b	GEORGIAN LETTER JIL
            return 0x1CAB;	//	Ძ	0xe1 0xb2 0xab	GEORGIAN CAPITAL LETTER JIL

        case 0x2D1C:		//	ⴜ	0xe2 0xb4 0x9c	GEORGIAN LETTER CIL
            return 0x1CAC;	//	Წ	0xe1 0xb2 0xac	GEORGIAN CAPITAL LETTER CIL

        case 0x2D1D:		//	ⴝ	0xe2 0xb4 0x9d	GEORGIAN LETTER CHAR
            return 0x1CAD;	//	Ჭ	0xe1 0xb2 0xad	GEORGIAN CAPITAL LETTER CHAR

        case 0x2D1E:		//	ⴞ	0xe2 0xb4 0x9e	GEORGIAN LETTER XAN
            return 0x1CAE;	//	Ხ	0xe1 0xb2 0xae	GEORGIAN CAPITAL LETTER XAN

        case 0x2D1F:		//	ⴟ	0xe2 0xb4 0x9f	GEORGIAN LETTER JHAN
            return 0x1CAF;	//	Ჯ	0xe1 0xb2 0xaf	GEORGIAN CAPITAL LETTER JHAN

        case 0x2D20:		//	ⴠ	0xe2 0xb4 0xa0	GEORGIAN LETTER HAE
            return 0x1CB0;	//	Ჰ	0xe1 0xb2 0xb0	GEORGIAN CAPITAL LETTER HAE

        case 0x2D21:		//	ⴡ	0xe2 0xb4 0xa1	GEORGIAN LETTER HE
            return 0x1CB1;	//	Ჱ	0xe1 0xb2 0xb1	GEORGIAN CAPITAL LETTER HE

        case 0x2D22:		//	ⴢ	0xe2 0xb4 0xa2	GEORGIAN LETTER HIE
            return 0x1CB2;	//	Ჲ	0xe1 0xb2 0xb2	GEORGIAN CAPITAL LETTER HIE

        case 0x2D23:		//	ⴣ	0xe2 0xb4 0xa3	GEORGIAN LETTER WE
            return 0x1CB3;	//	Ჳ	0xe1 0xb2 0xb3	GEORGIAN CAPITAL LETTER WE

        case 0x2D24:		//	ⴤ	0xe2 0xb4 0xa4	GEORGIAN LETTER HAR
            return 0x1CB4;	//	Ჴ	0xe1 0xb2 0xb4	GEORGIAN CAPITAL LETTER HAR

        case 0x2D25:		//	ⴥ	0xe2 0xb4 0xa5	GEORGIAN LETTER HOE
            return 0x1CB5;	//	Ჵ	0xe1 0xb2 0xb5	GEORGIAN CAPITAL LETTER HOE

        case 0x2D27:		//	ⴧ	0xe2 0xb4 0xa7	GEORGIAN LETTER YN
            return 0x1CB7;	//	Ჷ	0xe1 0xb2 0xb7	GEORGIAN CAPITAL LETTER YN

        case 0x2D2D:		//	ⴭ	0xe2 0xb4 0xad	GEORGIAN LETTER AEN
            return 0x1CBD;	//	Ჽ	0xe1 0xb2 0xbd	GEORGIAN CAPITAL LETTER AEN

#endif
#if UTF_SUPPORT_OLDCYRILLIC == 1
        case 0xA641:		//	ꙁ	0xea 0x99 0x81	CYRILLIC SMALL LETTER ZEMLYA
            return 0xA640;	//	Ꙁ	0xea 0x99 0x80	CYRILLIC CAPITAL LETTER ZEMLYA

        case 0xA643:		//	ꙃ	0xea 0x99 0x83	CYRILLIC SMALL LETTER DZELO
            return 0xA642;	//	Ꙃ	0xea 0x99 0x82	CYRILLIC CAPITAL LETTER DZELO

        case 0xA645:		//	ꙅ	0xea 0x99 0x85	CYRILLIC SMALL LETTER REVERSED DZE
            return 0xA644;	//	Ꙅ	0xea 0x99 0x84	CYRILLIC CAPITAL LETTER REVERSED DZE

        case 0xA647:		//	ꙇ	0xea 0x99 0x87	CYRILLIC SMALL LETTER IOTA
            return 0xA646;	//	Ꙇ	0xea 0x99 0x86	CYRILLIC CAPITAL LETTER IOTA

        case 0xA649:		//	ꙉ	0xea 0x99 0x89	CYRILLIC SMALL LETTER DJERV
            return 0xA648;	//	Ꙉ	0xea 0x99 0x88	CYRILLIC CAPITAL LETTER DJERV

        case 0xA64B:		//	ꙋ	0xea 0x99 0x8b	CYRILLIC SMALL LETTER MONOGRAPH UK
            return 0xA64A;	//	Ꙋ	0xea 0x99 0x8a	CYRILLIC CAPITAL LETTER MONOGRAPH UK

        case 0xA64D:		//	ꙍ	0xea 0x99 0x8d	CYRILLIC SMALL LETTER BROAD OMEGA
            return 0xA64C;	//	Ꙍ	0xea 0x99 0x8c	CYRILLIC CAPITAL LETTER BROAD OMEGA

        case 0xA64F:		//	ꙏ	0xea 0x99 0x8f	CYRILLIC SMALL LETTER NEUTRAL YER
            return 0xA64E;	//	Ꙏ	0xea 0x99 0x8e	CYRILLIC CAPITAL LETTER NEUTRAL YER

        case 0xA651:		//	ꙑ	0xea 0x99 0x91	CYRILLIC SMALL LETTER YERU WITH BACK YER
            return 0xA650;	//	Ꙑ	0xea 0x99 0x90	CYRILLIC CAPITAL LETTER YERU WITH BACK YER

        case 0xA653:		//	ꙓ	0xea 0x99 0x93	CYRILLIC SMALL LETTER IOTIFIED YAT
            return 0xA652;	//	Ꙓ	0xea 0x99 0x92	CYRILLIC CAPITAL LETTER IOTIFIED YAT

        case 0xA655:		//	ꙕ	0xea 0x99 0x95	CYRILLIC SMALL LETTER REVERSED YU
            return 0xA654;	//	Ꙕ	0xea 0x99 0x94	CYRILLIC CAPITAL LETTER REVERSED YU

        case 0xA657:		//	ꙗ	0xea 0x99 0x97	CYRILLIC SMALL LETTER IOTIFIED A
            return 0xA656;	//	Ꙗ	0xea 0x99 0x96	CYRILLIC CAPITAL LETTER IOTIFIED A

        case 0xA659:		//	ꙙ	0xea 0x99 0x99	CYRILLIC SMALL LETTER CLOSED LITTLE YUS
            return 0xA658;	//	Ꙙ	0xea 0x99 0x98	CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS

        case 0xA65B:		//	ꙛ	0xea 0x99 0x9b	CYRILLIC SMALL LETTER BLENDED YUS
            return 0xA65A;	//	Ꙛ	0xea 0x99 0x9a	CYRILLIC CAPITAL LETTER BLENDED YUS

        case 0xA65D:		//	ꙝ	0xea 0x99 0x9d	CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS
            return 0xA65C;	//	Ꙝ	0xea 0x99 0x9c	CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS

        case 0xA65F:		//	ꙟ	0xea 0x99 0x9f	CYRILLIC SMALL LETTER YN
            return 0xA65E;	//	Ꙟ	0xea 0x99 0x9e	CYRILLIC CAPITAL LETTER YN

        case 0xA661:		//	ꙡ	0xea 0x99 0xa1	CYRILLIC SMALL LETTER REVERSED TSE
            return 0xA660;	//	Ꙡ	0xea 0x99 0xa0	CYRILLIC CAPITAL LETTER REVERSED TSE

        case 0xA663:		//	ꙣ	0xea 0x99 0xa3	CYRILLIC SMALL LETTER SOFT DE
            return 0xA662;	//	Ꙣ	0xea 0x99 0xa2	CYRILLIC CAPITAL LETTER SOFT DE

        case 0xA665:		//	ꙥ	0xea 0x99 0xa5	CYRILLIC SMALL LETTER SOFT EL
            return 0xA664;	//	Ꙥ	0xea 0x99 0xa4	CYRILLIC CAPITAL LETTER SOFT EL

        case 0xA667:		//	ꙧ	0xea 0x99 0xa7	CYRILLIC SMALL LETTER SOFT EM
            return 0xA666;	//	Ꙧ	0xea 0x99 0xa6	CYRILLIC CAPITAL LETTER SOFT EM

        case 0xA669:		//	ꙩ	0xea 0x99 0xa9	CYRILLIC SMALL LETTER MONOCULAR O
            return 0xA668;	//	Ꙩ	0xea 0x99 0xa8	CYRILLIC CAPITAL LETTER MONOCULAR O

        case 0xA66B:		//	ꙫ	0xea 0x99 0xab	CYRILLIC SMALL LETTER BINOCULAR O
            return 0xA66A;	//	Ꙫ	0xea 0x99 0xaa	CYRILLIC CAPITAL LETTER BINOCULAR O

        case 0xA66D:		//	ꙭ	0xea 0x99 0xad	CYRILLIC SMALL LETTER DOUBLE MONOCULAR O
            return 0xA66C;	//	Ꙭ	0xea 0x99 0xac	CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O

        case 0xA681:		//	ꚁ	0xea 0x9a 0x81	CYRILLIC SMALL LETTER DWE
            return 0xA680;	//	Ꚁ	0xea 0x9a 0x80	CYRILLIC CAPITAL LETTER DWE

        case 0xA683:		//	ꚃ	0xea 0x9a 0x83	CYRILLIC SMALL LETTER DZWE
            return 0xA682;	//	Ꚃ	0xea 0x9a 0x82	CYRILLIC CAPITAL LETTER DZWE

        case 0xA685:		//	ꚅ	0xea 0x9a 0x85	CYRILLIC SMALL LETTER ZHWE
            return 0xA684;	//	Ꚅ	0xea 0x9a 0x84	CYRILLIC CAPITAL LETTER ZHWE

        case 0xA687:		//	ꚇ	0xea 0x9a 0x87	CYRILLIC SMALL LETTER CCHE
            return 0xA686;	//	Ꚇ	0xea 0x9a 0x86	CYRILLIC CAPITAL LETTER CCHE

        case 0xA689:		//	ꚉ	0xea 0x9a 0x89	CYRILLIC SMALL LETTER DZZE
            return 0xA688;	//	Ꚉ	0xea 0x9a 0x88	CYRILLIC CAPITAL LETTER DZZE

        case 0xA68B:		//	ꚋ	0xea 0x9a 0x8b	CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK
            return 0xA68A;	//	Ꚋ	0xea 0x9a 0x8a	CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK

        case 0xA68D:		//	ꚍ	0xea 0x9a 0x8d	CYRILLIC SMALL LETTER TWE
            return 0xA68C;	//	Ꚍ	0xea 0x9a 0x8c	CYRILLIC CAPITAL LETTER TWE

        case 0xA68F:		//	ꚏ	0xea 0x9a 0x8f	CYRILLIC SMALL LETTER TSWE
            return 0xA68E;	//	Ꚏ	0xea 0x9a 0x8e	CYRILLIC CAPITAL LETTER TSWE

        case 0xA691:		//	ꚑ	0xea 0x9a 0x91	CYRILLIC SMALL LETTER TSSE
            return 0xA690;	//	Ꚑ	0xea 0x9a 0x90	CYRILLIC CAPITAL LETTER TSSE

        case 0xA693:		//	ꚓ	0xea 0x9a 0x93	CYRILLIC SMALL LETTER TCHE
            return 0xA692;	//	Ꚓ	0xea 0x9a 0x92	CYRILLIC CAPITAL LETTER TCHE

        case 0xA695:		//	ꚕ	0xea 0x9a 0x95	CYRILLIC SMALL LETTER HWE
            return 0xA694;	//	Ꚕ	0xea 0x9a 0x94	CYRILLIC CAPITAL LETTER HWE

        case 0xA697:		//	ꚗ	0xea 0x9a 0x97	CYRILLIC SMALL LETTER SHWE
            return 0xA696;	//	Ꚗ	0xea 0x9a 0x96	CYRILLIC CAPITAL LETTER SHWE

        case 0xA699:		//	ꚙ	0xea 0x9a 0x99	CYRILLIC SMALL LETTER DOUBLE O
            return 0xA698;	//	Ꚙ	0xea 0x9a 0x98	CYRILLIC CAPITAL LETTER DOUBLE O

        case 0xA69B:		//	ꚛ	0xea 0x9a 0x9b	CYRILLIC SMALL LETTER CROSSED O
            return 0xA69A;	//	Ꚛ	0xea 0x9a 0x9a	CYRILLIC CAPITAL LETTER CROSSED O

#endif
#if UTF_SUPPORT_LATIN == 1
        case 0xA723:		//	ꜣ	0xea 0x9c 0xa3	LATIN SMALL LETTER EGYPTOLOGICAL ALEF
            return 0xA722;	//	Ꜣ	0xea 0x9c 0xa2	LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF

        case 0xA725:		//	ꜥ	0xea 0x9c 0xa5	LATIN SMALL LETTER EGYPTOLOGICAL AIN
            return 0xA724;	//	Ꜥ	0xea 0x9c 0xa4	LATIN CAPITAL LETTER EGYPTOLOGICAL AIN

        case 0xA727:		//	ꜧ	0xea 0x9c 0xa7	LATIN SMALL LETTER HENG
            return 0xA726;	//	Ꜧ	0xea 0x9c 0xa6	LATIN CAPITAL LETTER HENG

        case 0xA729:		//	ꜩ	0xea 0x9c 0xa9	LATIN SMALL LETTER TZ
            return 0xA728;	//	Ꜩ	0xea 0x9c 0xa8	LATIN CAPITAL LETTER TZ

        case 0xA72B:		//	ꜫ	0xea 0x9c 0xab	LATIN SMALL LETTER TRESILLO
            return 0xA72A;	//	Ꜫ	0xea 0x9c 0xaa	LATIN CAPITAL LETTER TRESILLO

        case 0xA72D:		//	ꜭ	0xea 0x9c 0xad	LATIN SMALL LETTER CUATRILLO
            return 0xA72C;	//	Ꜭ	0xea 0x9c 0xac	LATIN CAPITAL LETTER CUATRILLO

        case 0xA72F:		//	ꜯ	0xea 0x9c 0xaf	LATIN SMALL LETTER CUATRILLO WITH COMMA
            return 0xA72E;	//	Ꜯ	0xea 0x9c 0xae	LATIN CAPITAL LETTER CUATRILLO WITH COMMA

        case 0xA733:		//	ꜳ	0xea 0x9c 0xb3	LATIN SMALL LETTER AA
            return 0xA732;	//	Ꜳ	0xea 0x9c 0xb2	LATIN CAPITAL LETTER AA

        case 0xA735:		//	ꜵ	0xea 0x9c 0xb5	LATIN SMALL LETTER AO
            return 0xA734;	//	Ꜵ	0xea 0x9c 0xb4	LATIN CAPITAL LETTER AO

        case 0xA737:		//	ꜷ	0xea 0x9c 0xb7	LATIN SMALL LETTER AU
            return 0xA736;	//	Ꜷ	0xea 0x9c 0xb6	LATIN CAPITAL LETTER AU

        case 0xA739:		//	ꜹ	0xea 0x9c 0xb9	LATIN SMALL LETTER AV
            return 0xA738;	//	Ꜹ	0xea 0x9c 0xb8	LATIN CAPITAL LETTER AV

        case 0xA73B:		//	ꜻ	0xea 0x9c 0xbb	LATIN SMALL LETTER AV WITH HORIZONTAL BAR
            return 0xA73A;	//	Ꜻ	0xea 0x9c 0xba	LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR

        case 0xA73D:		//	ꜽ	0xea 0x9c 0xbd	LATIN SMALL LETTER AY
            return 0xA73C;	//	Ꜽ	0xea 0x9c 0xbc	LATIN CAPITAL LETTER AY

        case 0xA73F:		//	ꜿ	0xea 0x9c 0xbf	LATIN SMALL LETTER REVERSED C WITH DOT
            return 0xA73E;	//	Ꜿ	0xea 0x9c 0xbe	LATIN CAPITAL LETTER REVERSED C WITH DOT

        case 0xA741:		//	ꝁ	0xea 0x9d 0x81	LATIN SMALL LETTER K WITH STROKE
            return 0xA740;	//	Ꝁ	0xea 0x9d 0x80	LATIN CAPITAL LETTER K WITH STROKE

        case 0xA743:		//	ꝃ	0xea 0x9d 0x83	LATIN SMALL LETTER K WITH DIAGONAL STROKE
            return 0xA742;	//	Ꝃ	0xea 0x9d 0x82	LATIN CAPITAL LETTER K WITH DIAGONAL STROKE

        case 0xA745:		//	ꝅ	0xea 0x9d 0x85	LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE
            return 0xA744;	//	Ꝅ	0xea 0x9d 0x84	LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE

        case 0xA747:		//	ꝇ	0xea 0x9d 0x87	LATIN SMALL LETTER BROKEN L
            return 0xA746;	//	Ꝇ	0xea 0x9d 0x86	LATIN CAPITAL LETTER BROKEN L

        case 0xA749:		//	ꝉ	0xea 0x9d 0x89	LATIN SMALL LETTER L WITH HIGH STROKE
            return 0xA748;	//	Ꝉ	0xea 0x9d 0x88	LATIN CAPITAL LETTER L WITH HIGH STROKE

        case 0xA74B:		//	ꝋ	0xea 0x9d 0x8b	LATIN SMALL LETTER O WITH LONG STROKE OVERLAY
            return 0xA74A;	//	Ꝋ	0xea 0x9d 0x8a	LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY

        case 0xA74D:		//	ꝍ	0xea 0x9d 0x8d	LATIN SMALL LETTER O WITH LOOP
            return 0xA74C;	//	Ꝍ	0xea 0x9d 0x8c	LATIN CAPITAL LETTER O WITH LOOP

        case 0xA74F:		//	ꝏ	0xea 0x9d 0x8f	LATIN SMALL LETTER OO
            return 0xA74E;	//	Ꝏ	0xea 0x9d 0x8e	LATIN CAPITAL LETTER OO

        case 0xA751:		//	ꝑ	0xea 0x9d 0x91	LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER
            return 0xA750;	//	Ꝑ	0xea 0x9d 0x90	LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER

        case 0xA753:		//	ꝓ	0xea 0x9d 0x93	LATIN SMALL LETTER P WITH FLOURISH
            return 0xA752;	//	Ꝓ	0xea 0x9d 0x92	LATIN CAPITAL LETTER P WITH FLOURISH

        case 0xA755:		//	ꝕ	0xea 0x9d 0x95	LATIN SMALL LETTER P WITH SQUIRREL TAIL
            return 0xA754;	//	Ꝕ	0xea 0x9d 0x94	LATIN CAPITAL LETTER P WITH SQUIRREL TAIL

        case 0xA757:		//	ꝗ	0xea 0x9d 0x97	LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER
            return 0xA756;	//	Ꝗ	0xea 0x9d 0x96	LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER

        case 0xA759:		//	ꝙ	0xea 0x9d 0x99	LATIN SMALL LETTER Q WITH DIAGONAL STROKE
            return 0xA758;	//	Ꝙ	0xea 0x9d 0x98	LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE

        case 0xA75B:		//	ꝛ	0xea 0x9d 0x9b	LATIN SMALL LETTER R ROTUNDA
            return 0xA75A;	//	Ꝛ	0xea 0x9d 0x9a	LATIN CAPITAL LETTER R ROTUNDA

        case 0xA75D:		//	ꝝ	0xea 0x9d 0x9d	LATIN SMALL LETTER RUM ROTUNDA
            return 0xA75C;	//	Ꝝ	0xea 0x9d 0x9c	LATIN CAPITAL LETTER RUM ROTUNDA

        case 0xA75F:		//	ꝟ	0xea 0x9d 0x9f	LATIN SMALL LETTER V WITH DIAGONAL STROKE
            return 0xA75E;	//	Ꝟ	0xea 0x9d 0x9e	LATIN CAPITAL LETTER V WITH DIAGONAL STROKE

        case 0xA761:		//	ꝡ	0xea 0x9d 0xa1	LATIN SMALL LETTER VY
            return 0xA760;	//	Ꝡ	0xea 0x9d 0xa0	LATIN CAPITAL LETTER VY

        case 0xA763:		//	ꝣ	0xea 0x9d 0xa3	LATIN SMALL LETTER VISIGOTHIC Z
            return 0xA762;	//	Ꝣ	0xea 0x9d 0xa2	LATIN CAPITAL LETTER VISIGOTHIC Z

        case 0xA765:		//	ꝥ	0xea 0x9d 0xa5	LATIN SMALL LETTER THORN WITH STROKE
            return 0xA764;	//	Ꝥ	0xea 0x9d 0xa4	LATIN CAPITAL LETTER THORN WITH STROKE

        case 0xA767:		//	ꝧ	0xea 0x9d 0xa7	LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER
            return 0xA766;	//	Ꝧ	0xea 0x9d 0xa6	LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER

        case 0xA769:		//	ꝩ	0xea 0x9d 0xa9	LATIN SMALL LETTER VEND
            return 0xA768;	//	Ꝩ	0xea 0x9d 0xa8	LATIN CAPITAL LETTER VEND

        case 0xA76B:		//	ꝫ	0xea 0x9d 0xab	LATIN SMALL LETTER ET
            return 0xA76A;	//	Ꝫ	0xea 0x9d 0xaa	LATIN CAPITAL LETTER ET

        case 0xA76D:		//	ꝭ	0xea 0x9d 0xad	LATIN SMALL LETTER IS
            return 0xA76C;	//	Ꝭ	0xea 0x9d 0xac	LATIN CAPITAL LETTER IS

        case 0xA76F:		//	ꝯ	0xea 0x9d 0xaf	LATIN SMALL LETTER CON
            return 0xA76E;	//	Ꝯ	0xea 0x9d 0xae	LATIN CAPITAL LETTER CON

        case 0xA77A:		//	ꝺ	0xea 0x9d 0xba	LATIN SMALL LETTER INSULAR D
            return 0xA779;	//	Ꝺ	0xea 0x9d 0xb9	LATIN CAPITAL LETTER INSULAR D

        case 0xA77C:		//	ꝼ	0xea 0x9d 0xbc	LATIN SMALL LETTER INSULAR F
            return 0xA77B;	//	Ꝼ	0xea 0x9d 0xbb	LATIN CAPITAL LETTER INSULAR F

        case 0xA77F:		//	ꝿ	0xea 0x9d 0xbf	LATIN SMALL LETTER TURNED INSULAR G
            return 0xA77E;	//	Ꝿ	0xea 0x9d 0xbe	LATIN CAPITAL LETTER TURNED INSULAR G

        case 0xA781:		//	ꞁ	0xea 0x9e 0x81	LATIN SMALL LETTER TURNED L
            return 0xA780;	//	Ꞁ	0xea 0x9e 0x80	LATIN CAPITAL LETTER TURNED L

        case 0xA783:		//	ꞃ	0xea 0x9e 0x83	LATIN SMALL LETTER INSULAR R
            return 0xA782;	//	Ꞃ	0xea 0x9e 0x82	LATIN CAPITAL LETTER INSULAR R

        case 0xA785:		//	ꞅ	0xea 0x9e 0x85	LATIN SMALL LETTER INSULAR S
            return 0xA784;	//	Ꞅ	0xea 0x9e 0x84	LATIN CAPITAL LETTER INSULAR S

        case 0xA787:		//	ꞇ	0xea 0x9e 0x87	LATIN SMALL LETTER INSULAR T
            return 0xA786;	//	Ꞇ	0xea 0x9e 0x86	LATIN CAPITAL LETTER INSULAR T

        case 0xA78C:		//	ꞌ	0xea 0x9e 0x8c	LATIN SMALL LETTER SALTILLO
            return 0xA78B;	//	Ꞌ	0xea 0x9e 0x8b	LATIN CAPITAL LETTER SALTILLO

        case 0xA791:		//	ꞑ	0xea 0x9e 0x91	LATIN SMALL LETTER N WITH DESCENDER
            return 0xA790;	//	Ꞑ	0xea 0x9e 0x90	LATIN CAPITAL LETTER N WITH DESCENDER

        case 0xA793:		//	ꞓ	0xea 0x9e 0x93	LATIN SMALL LETTER C WITH BAR
            return 0xA792;	//	Ꞓ	0xea 0x9e 0x92	LATIN CAPITAL LETTER C WITH BAR

        case 0xA794:		//	ꞔ	0xea 0x9e 0x94	LATIN SMALL LETTER C WITH PALATAL HOOK
            return 0xA7C4;	//	Ꞔ	0xea 0x9f 0x84	LATIN CAPITAL LETTER C WITH PALATAL HOOK

        case 0xA797:		//	ꞗ	0xea 0x9e 0x97	LATIN SMALL LETTER B WITH FLOURISH
            return 0xA796;	//	Ꞗ	0xea 0x9e 0x96	LATIN CAPITAL LETTER B WITH FLOURISH

        case 0xA799:		//	ꞙ	0xea 0x9e 0x99	LATIN SMALL LETTER F WITH STROKE
            return 0xA798;	//	Ꞙ	0xea 0x9e 0x98	LATIN CAPITAL LETTER F WITH STROKE

        case 0xA79B:		//	ꞛ	0xea 0x9e 0x9b	LATIN SMALL LETTER VOLAPUK AE
            return 0xA79A;	//	Ꞛ	0xea 0x9e 0x9a	LATIN CAPITAL LETTER VOLAPUK AE

        case 0xA79D:		//	ꞝ	0xea 0x9e 0x9d	LATIN SMALL LETTER VOLAPUK OE
            return 0xA79C;	//	Ꞝ	0xea 0x9e 0x9c	LATIN CAPITAL LETTER VOLAPUK OE

        case 0xA79F:		//	ꞟ	0xea 0x9e 0x9f	LATIN SMALL LETTER VOLAPUK UE
            return 0xA79E;	//	Ꞟ	0xea 0x9e 0x9e	LATIN CAPITAL LETTER VOLAPUK UE

        case 0xA7A1:		//	ꞡ	0xea 0x9e 0xa1	LATIN SMALL LETTER G WITH OBLIQUE STROKE
            return 0xA7A0;	//	Ꞡ	0xea 0x9e 0xa0	LATIN CAPITAL LETTER G WITH OBLIQUE STROKE

        case 0xA7A3:		//	ꞣ	0xea 0x9e 0xa3	LATIN SMALL LETTER K WITH OBLIQUE STROKE
            return 0xA7A2;	//	Ꞣ	0xea 0x9e 0xa2	LATIN CAPITAL LETTER K WITH OBLIQUE STROKE

        case 0xA7A5:		//	ꞥ	0xea 0x9e 0xa5	LATIN SMALL LETTER N WITH OBLIQUE STROKE
            return 0xA7A4;	//	Ꞥ	0xea 0x9e 0xa4	LATIN CAPITAL LETTER N WITH OBLIQUE STROKE

        case 0xA7A7:		//	ꞧ	0xea 0x9e 0xa7	LATIN SMALL LETTER R WITH OBLIQUE STROKE
            return 0xA7A6;	//	Ꞧ	0xea 0x9e 0xa6	LATIN CAPITAL LETTER R WITH OBLIQUE STROKE

        case 0xA7A9:		//	ꞩ	0xea 0x9e 0xa9	LATIN SMALL LETTER S WITH OBLIQUE STROKE
            return 0xA7A8;	//	Ꞩ	0xea 0x9e 0xa8	LATIN CAPITAL LETTER S WITH OBLIQUE STROKE

        case 0xA7B5:		//	ꞵ	0xea 0x9e 0xb5	LATIN SMALL LETTER BETA
            return 0xA7B4;	//	Ꞵ	0xea 0x9e 0xb4	LATIN CAPITAL LETTER BETA

        case 0xA7B7:		//	ꞷ	0xea 0x9e 0xb7	LATIN SMALL LETTER OMEGA
            return 0xA7B6;	//	Ꞷ	0xea 0x9e 0xb6	LATIN CAPITAL LETTER OMEGA

        case 0xA7B9:		//	ꞹ	0xea 0x9e 0xb9	LATIN SMALL LETTER U WITH STROKE
            return 0xA7B8;	//	Ꞹ	0xea 0x9e 0xb8	LATIN CAPITAL LETTER U WITH STROKE

        case 0xA7BB:		//	ꞻ	0xea 0x9e 0xbb	LATIN SMALL LETTER GLOTTAL A
            return 0xA7BA;	//	Ꞻ	0xea 0x9e 0xba	LATIN CAPITAL LETTER GLOTTAL A

        case 0xA7BD:		//	ꞽ	0xea 0x9e 0xbd	LATIN SMALL LETTER GLOTTAL I
            return 0xA7BC;	//	Ꞽ	0xea 0x9e 0xbc	LATIN CAPITAL LETTER GLOTTAL I

        case 0xA7BF:		//	ꞿ	0xea 0x9e 0xbf	LATIN SMALL LETTER GLOTTAL U
            return 0xA7BE;	//	Ꞿ	0xea 0x9e 0xbe	LATIN CAPITAL LETTER GLOTTAL U

        case 0xA7C3:		//	ꟃ	0xea 0x9f 0x83	LATIN SMALL LETTER ANGLICANA W
            return 0xA7C2;	//	Ꟃ	0xea 0x9f 0x82	LATIN CAPITAL LETTER ANGLICANA W

        case 0xA7C8:		//	ꟈ	0xea 0x9f 0x88	LATIN SMALL LETTER D WITH SHORT STROKE OVERLAY
            return 0xA7C7;	//	Ꟈ	0xea 0x9f 0x87	LATIN CAPITAL LETTER D WITH SHORT STROKE OVERLAY

        case 0xA7CA:		//	ꟊ	0xea 0x9f 0x8a	LATIN SMALL LETTER S WITH SHORT STROKE OVERLAY
            return 0xA7C9;	//	Ꟊ	0xea 0x9f 0x89	LATIN CAPITAL LETTER S WITH SHORT STROKE OVERLAY

        case 0xA7F6:		//	ꟶ	0xea 0x9f 0xb6	LATIN SMALL LETTER REVERSED HALF H
            return 0xA7F5;	//	Ꟶ	0xea 0x9f 0xb5	LATIN CAPITAL LETTER REVERSED HALF H

        case 0xAB53:		//	ꭓ	0xea 0xad 0x93	LATIN SMALL LETTER CHI
            return 0xA7B3;	//	Ꭓ	0xea 0x9e 0xb3	LATIN CAPITAL LETTER CHI

#endif
#if UTF_SUPPORT_CHEROKEE == 1
        case 0xAB70:		//	ꭰ	0xea 0xad 0xb0	CHEROKEE SMALL LETTER A
            return 0x13A0;	//	Ꭰ	0xe1 0x8e 0xa0	CHEROKEE CAPITAL LETTER A

        case 0xAB71:		//	ꭱ	0xea 0xad 0xb1	CHEROKEE SMALL LETTER E
            return 0x13A1;	//	Ꭱ	0xe1 0x8e 0xa1	CHEROKEE CAPITAL LETTER E

        case 0xAB72:		//	ꭲ	0xea 0xad 0xb2	CHEROKEE SMALL LETTER I
            return 0x13A2;	//	Ꭲ	0xe1 0x8e 0xa2	CHEROKEE CAPITAL LETTER I

        case 0xAB73:		//	ꭳ	0xea 0xad 0xb3	CHEROKEE SMALL LETTER O
            return 0x13A3;	//	Ꭳ	0xe1 0x8e 0xa3	CHEROKEE CAPITAL LETTER O

        case 0xAB74:		//	ꭴ	0xea 0xad 0xb4	CHEROKEE SMALL LETTER U
            return 0x13A4;	//	Ꭴ	0xe1 0x8e 0xa4	CHEROKEE CAPITAL LETTER U

        case 0xAB75:		//	ꭵ	0xea 0xad 0xb5	CHEROKEE SMALL LETTER V
            return 0x13A5;	//	Ꭵ	0xe1 0x8e 0xa5	CHEROKEE CAPITAL LETTER V

        case 0xAB76:		//	ꭶ	0xea 0xad 0xb6	CHEROKEE SMALL LETTER GA
            return 0x13A6;	//	Ꭶ	0xe1 0x8e 0xa6	CHEROKEE CAPITAL LETTER GA

        case 0xAB77:		//	ꭷ	0xea 0xad 0xb7	CHEROKEE SMALL LETTER KA
            return 0x13A7;	//	Ꭷ	0xe1 0x8e 0xa7	CHEROKEE CAPITAL LETTER KA

        case 0xAB78:		//	ꭸ	0xea 0xad 0xb8	CHEROKEE SMALL LETTER GE
            return 0x13A8;	//	Ꭸ	0xe1 0x8e 0xa8	CHEROKEE CAPITAL LETTER GE

        case 0xAB79:		//	ꭹ	0xea 0xad 0xb9	CHEROKEE SMALL LETTER GI
            return 0x13A9;	//	Ꭹ	0xe1 0x8e 0xa9	CHEROKEE CAPITAL LETTER GI

        case 0xAB7A:		//	ꭺ	0xea 0xad 0xba	CHEROKEE SMALL LETTER GO
            return 0x13AA;	//	Ꭺ	0xe1 0x8e 0xaa	CHEROKEE CAPITAL LETTER GO

        case 0xAB7B:		//	ꭻ	0xea 0xad 0xbb	CHEROKEE SMALL LETTER GU
            return 0x13AB;	//	Ꭻ	0xe1 0x8e 0xab	CHEROKEE CAPITAL LETTER GU

        case 0xAB7C:		//	ꭼ	0xea 0xad 0xbc	CHEROKEE SMALL LETTER GV
            return 0x13AC;	//	Ꭼ	0xe1 0x8e 0xac	CHEROKEE CAPITAL LETTER GV

        case 0xAB7D:		//	ꭽ	0xea 0xad 0xbd	CHEROKEE SMALL LETTER HA
            return 0x13AD;	//	Ꭽ	0xe1 0x8e 0xad	CHEROKEE CAPITAL LETTER HA

        case 0xAB7E:		//	ꭾ	0xea 0xad 0xbe	CHEROKEE SMALL LETTER HE
            return 0x13AE;	//	Ꭾ	0xe1 0x8e 0xae	CHEROKEE CAPITAL LETTER HE

        case 0xAB7F:		//	ꭿ	0xea 0xad 0xbf	CHEROKEE SMALL LETTER HI
            return 0x13AF;	//	Ꭿ	0xe1 0x8e 0xaf	CHEROKEE CAPITAL LETTER HI

        case 0xAB80:		//	ꮀ	0xea 0xae 0x80	CHEROKEE SMALL LETTER HO
            return 0x13B0;	//	Ꮀ	0xe1 0x8e 0xb0	CHEROKEE CAPITAL LETTER HO

        case 0xAB81:		//	ꮁ	0xea 0xae 0x81	CHEROKEE SMALL LETTER HU
            return 0x13B1;	//	Ꮁ	0xe1 0x8e 0xb1	CHEROKEE CAPITAL LETTER HU

        case 0xAB82:		//	ꮂ	0xea 0xae 0x82	CHEROKEE SMALL LETTER HV
            return 0x13B2;	//	Ꮂ	0xe1 0x8e 0xb2	CHEROKEE CAPITAL LETTER HV

        case 0xAB83:		//	ꮃ	0xea 0xae 0x83	CHEROKEE SMALL LETTER LA
            return 0x13B3;	//	Ꮃ	0xe1 0x8e 0xb3	CHEROKEE CAPITAL LETTER LA

        case 0xAB84:		//	ꮄ	0xea 0xae 0x84	CHEROKEE SMALL LETTER LE
            return 0x13B4;	//	Ꮄ	0xe1 0x8e 0xb4	CHEROKEE CAPITAL LETTER LE

        case 0xAB85:		//	ꮅ	0xea 0xae 0x85	CHEROKEE SMALL LETTER LI
            return 0x13B5;	//	Ꮅ	0xe1 0x8e 0xb5	CHEROKEE CAPITAL LETTER LI

        case 0xAB86:		//	ꮆ	0xea 0xae 0x86	CHEROKEE SMALL LETTER LO
            return 0x13B6;	//	Ꮆ	0xe1 0x8e 0xb6	CHEROKEE CAPITAL LETTER LO

        case 0xAB87:		//	ꮇ	0xea 0xae 0x87	CHEROKEE SMALL LETTER LU
            return 0x13B7;	//	Ꮇ	0xe1 0x8e 0xb7	CHEROKEE CAPITAL LETTER LU

        case 0xAB88:		//	ꮈ	0xea 0xae 0x88	CHEROKEE SMALL LETTER LV
            return 0x13B8;	//	Ꮈ	0xe1 0x8e 0xb8	CHEROKEE CAPITAL LETTER LV

        case 0xAB89:		//	ꮉ	0xea 0xae 0x89	CHEROKEE SMALL LETTER MA
            return 0x13B9;	//	Ꮉ	0xe1 0x8e 0xb9	CHEROKEE CAPITAL LETTER MA

        case 0xAB8A:		//	ꮊ	0xea 0xae 0x8a	CHEROKEE SMALL LETTER ME
            return 0x13BA;	//	Ꮊ	0xe1 0x8e 0xba	CHEROKEE CAPITAL LETTER ME

        case 0xAB8B:		//	ꮋ	0xea 0xae 0x8b	CHEROKEE SMALL LETTER MI
            return 0x13BB;	//	Ꮋ	0xe1 0x8e 0xbb	CHEROKEE CAPITAL LETTER MI

        case 0xAB8C:		//	ꮌ	0xea 0xae 0x8c	CHEROKEE SMALL LETTER MO
            return 0x13BC;	//	Ꮌ	0xe1 0x8e 0xbc	CHEROKEE CAPITAL LETTER MO

        case 0xAB8D:		//	ꮍ	0xea 0xae 0x8d	CHEROKEE SMALL LETTER MU
            return 0x13BD;	//	Ꮍ	0xe1 0x8e 0xbd	CHEROKEE CAPITAL LETTER MU

        case 0xAB8E:		//	ꮎ	0xea 0xae 0x8e	CHEROKEE SMALL LETTER NA
            return 0x13BE;	//	Ꮎ	0xe1 0x8e 0xbe	CHEROKEE CAPITAL LETTER NA

        case 0xAB8F:		//	ꮏ	0xea 0xae 0x8f	CHEROKEE SMALL LETTER HNA
            return 0x13BF;	//	Ꮏ	0xe1 0x8e 0xbf	CHEROKEE CAPITAL LETTER HNA

        case 0xAB90:		//	ꮐ	0xea 0xae 0x90	CHEROKEE SMALL LETTER NAH
            return 0x13C0;	//	Ꮐ	0xe1 0x8f 0x80	CHEROKEE CAPITAL LETTER NAH

        case 0xAB91:		//	ꮑ	0xea 0xae 0x91	CHEROKEE SMALL LETTER NE
            return 0x13C1;	//	Ꮑ	0xe1 0x8f 0x81	CHEROKEE CAPITAL LETTER NE

        case 0xAB92:		//	ꮒ	0xea 0xae 0x92	CHEROKEE SMALL LETTER NI
            return 0x13C2;	//	Ꮒ	0xe1 0x8f 0x82	CHEROKEE CAPITAL LETTER NI

        case 0xAB93:		//	ꮓ	0xea 0xae 0x93	CHEROKEE SMALL LETTER NO
            return 0x13C3;	//	Ꮓ	0xe1 0x8f 0x83	CHEROKEE CAPITAL LETTER NO

        case 0xAB94:		//	ꮔ	0xea 0xae 0x94	CHEROKEE SMALL LETTER NU
            return 0x13C4;	//	Ꮔ	0xe1 0x8f 0x84	CHEROKEE CAPITAL LETTER NU

        case 0xAB95:		//	ꮕ	0xea 0xae 0x95	CHEROKEE SMALL LETTER NV
            return 0x13C5;	//	Ꮕ	0xe1 0x8f 0x85	CHEROKEE CAPITAL LETTER NV

        case 0xAB96:		//	ꮖ	0xea 0xae 0x96	CHEROKEE SMALL LETTER QUA
            return 0x13C6;	//	Ꮖ	0xe1 0x8f 0x86	CHEROKEE CAPITAL LETTER QUA

        case 0xAB97:		//	ꮗ	0xea 0xae 0x97	CHEROKEE SMALL LETTER QUE
            return 0x13C7;	//	Ꮗ	0xe1 0x8f 0x87	CHEROKEE CAPITAL LETTER QUE

        case 0xAB98:		//	ꮘ	0xea 0xae 0x98	CHEROKEE SMALL LETTER QUI
            return 0x13C8;	//	Ꮘ	0xe1 0x8f 0x88	CHEROKEE CAPITAL LETTER QUI

        case 0xAB99:		//	ꮙ	0xea 0xae 0x99	CHEROKEE SMALL LETTER QUO
            return 0x13C9;	//	Ꮙ	0xe1 0x8f 0x89	CHEROKEE CAPITAL LETTER QUO

        case 0xAB9A:		//	ꮚ	0xea 0xae 0x9a	CHEROKEE SMALL LETTER QUU
            return 0x13CA;	//	Ꮚ	0xe1 0x8f 0x8a	CHEROKEE CAPITAL LETTER QUU

        case 0xAB9B:		//	ꮛ	0xea 0xae 0x9b	CHEROKEE SMALL LETTER QUV
            return 0x13CB;	//	Ꮛ	0xe1 0x8f 0x8b	CHEROKEE CAPITAL LETTER QUV

        case 0xAB9C:		//	ꮜ	0xea 0xae 0x9c	CHEROKEE SMALL LETTER SA
            return 0x13CC;	//	Ꮜ	0xe1 0x8f 0x8c	CHEROKEE CAPITAL LETTER SA

        case 0xAB9D:		//	ꮝ	0xea 0xae 0x9d	CHEROKEE SMALL LETTER S
            return 0x13CD;	//	Ꮝ	0xe1 0x8f 0x8d	CHEROKEE CAPITAL LETTER S

        case 0xAB9E:		//	ꮞ	0xea 0xae 0x9e	CHEROKEE SMALL LETTER SE
            return 0x13CE;	//	Ꮞ	0xe1 0x8f 0x8e	CHEROKEE CAPITAL LETTER SE

        case 0xAB9F:		//	ꮟ	0xea 0xae 0x9f	CHEROKEE SMALL LETTER SI
            return 0x13CF;	//	Ꮟ	0xe1 0x8f 0x8f	CHEROKEE CAPITAL LETTER SI

        case 0xABA0:		//	ꮠ	0xea 0xae 0xa0	CHEROKEE SMALL LETTER SO
            return 0x13D0;	//	Ꮠ	0xe1 0x8f 0x90	CHEROKEE CAPITAL LETTER SO

        case 0xABA1:		//	ꮡ	0xea 0xae 0xa1	CHEROKEE SMALL LETTER SU
            return 0x13D1;	//	Ꮡ	0xe1 0x8f 0x91	CHEROKEE CAPITAL LETTER SU

        case 0xABA2:		//	ꮢ	0xea 0xae 0xa2	CHEROKEE SMALL LETTER SV
            return 0x13D2;	//	Ꮢ	0xe1 0x8f 0x92	CHEROKEE CAPITAL LETTER SV

        case 0xABA3:		//	ꮣ	0xea 0xae 0xa3	CHEROKEE SMALL LETTER DA
            return 0x13D3;	//	Ꮣ	0xe1 0x8f 0x93	CHEROKEE CAPITAL LETTER DA

        case 0xABA4:		//	ꮤ	0xea 0xae 0xa4	CHEROKEE SMALL LETTER TA
            return 0x13D4;	//	Ꮤ	0xe1 0x8f 0x94	CHEROKEE CAPITAL LETTER TA

        case 0xABA5:		//	ꮥ	0xea 0xae 0xa5	CHEROKEE SMALL LETTER DE
            return 0x13D5;	//	Ꮥ	0xe1 0x8f 0x95	CHEROKEE CAPITAL LETTER DE

        case 0xABA6:		//	ꮦ	0xea 0xae 0xa6	CHEROKEE SMALL LETTER TE
            return 0x13D6;	//	Ꮦ	0xe1 0x8f 0x96	CHEROKEE CAPITAL LETTER TE

        case 0xABA7:		//	ꮧ	0xea 0xae 0xa7	CHEROKEE SMALL LETTER DI
            return 0x13D7;	//	Ꮧ	0xe1 0x8f 0x97	CHEROKEE CAPITAL LETTER DI

        case 0xABA8:		//	ꮨ	0xea 0xae 0xa8	CHEROKEE SMALL LETTER TI
            return 0x13D8;	//	Ꮨ	0xe1 0x8f 0x98	CHEROKEE CAPITAL LETTER TI

        case 0xABA9:		//	ꮩ	0xea 0xae 0xa9	CHEROKEE SMALL LETTER DO
            return 0x13D9;	//	Ꮩ	0xe1 0x8f 0x99	CHEROKEE CAPITAL LETTER DO

        case 0xABAA:		//	ꮪ	0xea 0xae 0xaa	CHEROKEE SMALL LETTER DU
            return 0x13DA;	//	Ꮪ	0xe1 0x8f 0x9a	CHEROKEE CAPITAL LETTER DU

        case 0xABAB:		//	ꮫ	0xea 0xae 0xab	CHEROKEE SMALL LETTER DV
            return 0x13DB;	//	Ꮫ	0xe1 0x8f 0x9b	CHEROKEE CAPITAL LETTER DV

        case 0xABAC:		//	ꮬ	0xea 0xae 0xac	CHEROKEE SMALL LETTER DLA
            return 0x13DC;	//	Ꮬ	0xe1 0x8f 0x9c	CHEROKEE CAPITAL LETTER DLA

        case 0xABAD:		//	ꮭ	0xea 0xae 0xad	CHEROKEE SMALL LETTER TLA
            return 0x13DD;	//	Ꮭ	0xe1 0x8f 0x9d	CHEROKEE CAPITAL LETTER TLA

        case 0xABAE:		//	ꮮ	0xea 0xae 0xae	CHEROKEE SMALL LETTER TLE
            return 0x13DE;	//	Ꮮ	0xe1 0x8f 0x9e	CHEROKEE CAPITAL LETTER TLE

        case 0xABAF:		//	ꮯ	0xea 0xae 0xaf	CHEROKEE SMALL LETTER TLI
            return 0x13DF;	//	Ꮯ	0xe1 0x8f 0x9f	CHEROKEE CAPITAL LETTER TLI

        case 0xABB0:		//	ꮰ	0xea 0xae 0xb0	CHEROKEE SMALL LETTER TLO
            return 0x13E0;	//	Ꮰ	0xe1 0x8f 0xa0	CHEROKEE CAPITAL LETTER TLO

        case 0xABB1:		//	ꮱ	0xea 0xae 0xb1	CHEROKEE SMALL LETTER TLU
            return 0x13E1;	//	Ꮱ	0xe1 0x8f 0xa1	CHEROKEE CAPITAL LETTER TLU

        case 0xABB2:		//	ꮲ	0xea 0xae 0xb2	CHEROKEE SMALL LETTER TLV
            return 0x13E2;	//	Ꮲ	0xe1 0x8f 0xa2	CHEROKEE CAPITAL LETTER TLV

        case 0xABB3:		//	ꮳ	0xea 0xae 0xb3	CHEROKEE SMALL LETTER TSA
            return 0x13E3;	//	Ꮳ	0xe1 0x8f 0xa3	CHEROKEE CAPITAL LETTER TSA

        case 0xABB4:		//	ꮴ	0xea 0xae 0xb4	CHEROKEE SMALL LETTER TSE
            return 0x13E4;	//	Ꮴ	0xe1 0x8f 0xa4	CHEROKEE CAPITAL LETTER TSE

        case 0xABB5:		//	ꮵ	0xea 0xae 0xb5	CHEROKEE SMALL LETTER TSI
            return 0x13E5;	//	Ꮵ	0xe1 0x8f 0xa5	CHEROKEE CAPITAL LETTER TSI

        case 0xABB6:		//	ꮶ	0xea 0xae 0xb6	CHEROKEE SMALL LETTER TSO
            return 0x13E6;	//	Ꮶ	0xe1 0x8f 0xa6	CHEROKEE CAPITAL LETTER TSO

        case 0xABB7:		//	ꮷ	0xea 0xae 0xb7	CHEROKEE SMALL LETTER TSU
            return 0x13E7;	//	Ꮷ	0xe1 0x8f 0xa7	CHEROKEE CAPITAL LETTER TSU

        case 0xABB8:		//	ꮸ	0xea 0xae 0xb8	CHEROKEE SMALL LETTER TSV
            return 0x13E8;	//	Ꮸ	0xe1 0x8f 0xa8	CHEROKEE CAPITAL LETTER TSV

        case 0xABB9:		//	ꮹ	0xea 0xae 0xb9	CHEROKEE SMALL LETTER WA
            return 0x13E9;	//	Ꮹ	0xe1 0x8f 0xa9	CHEROKEE CAPITAL LETTER WA

        case 0xABBA:		//	ꮺ	0xea 0xae 0xba	CHEROKEE SMALL LETTER WE
            return 0x13EA;	//	Ꮺ	0xe1 0x8f 0xaa	CHEROKEE CAPITAL LETTER WE

        case 0xABBB:		//	ꮻ	0xea 0xae 0xbb	CHEROKEE SMALL LETTER WI
            return 0x13EB;	//	Ꮻ	0xe1 0x8f 0xab	CHEROKEE CAPITAL LETTER WI

        case 0xABBC:		//	ꮼ	0xea 0xae 0xbc	CHEROKEE SMALL LETTER WO
            return 0x13EC;	//	Ꮼ	0xe1 0x8f 0xac	CHEROKEE CAPITAL LETTER WO

        case 0xABBD:		//	ꮽ	0xea 0xae 0xbd	CHEROKEE SMALL LETTER WU
            return 0x13ED;	//	Ꮽ	0xe1 0x8f 0xad	CHEROKEE CAPITAL LETTER WU

        case 0xABBE:		//	ꮾ	0xea 0xae 0xbe	CHEROKEE SMALL LETTER WV
            return 0x13EE;	//	Ꮾ	0xe1 0x8f 0xae	CHEROKEE CAPITAL LETTER WV

        case 0xABBF:		//	ꮿ	0xea 0xae 0xbf	CHEROKEE SMALL LETTER YA
            return 0x13EF;	//	Ꮿ	0xe1 0x8f 0xaf	CHEROKEE CAPITAL LETTER YA

#endif
#if UTF_SUPPORT_FWLATIN == 1
        case 0xFF41:		//	ａ	0xef 0xbd 0x81	FULLWIDTH LATIN SMALL LETTER A
            return 0xFF21;	//	Ａ	0xef 0xbc 0xa1	FULLWIDTH LATIN CAPITAL LETTER A

        case 0xFF42:		//	ｂ	0xef 0xbd 0x82	FULLWIDTH LATIN SMALL LETTER B
            return 0xFF22;	//	Ｂ	0xef 0xbc 0xa2	FULLWIDTH LATIN CAPITAL LETTER B

        case 0xFF43:		//	ｃ	0xef 0xbd 0x83	FULLWIDTH LATIN SMALL LETTER C
            return 0xFF23;	//	Ｃ	0xef 0xbc 0xa3	FULLWIDTH LATIN CAPITAL LETTER C

        case 0xFF44:		//	ｄ	0xef 0xbd 0x84	FULLWIDTH LATIN SMALL LETTER D
            return 0xFF24;	//	Ｄ	0xef 0xbc 0xa4	FULLWIDTH LATIN CAPITAL LETTER D

        case 0xFF45:		//	ｅ	0xef 0xbd 0x85	FULLWIDTH LATIN SMALL LETTER E
            return 0xFF25;	//	Ｅ	0xef 0xbc 0xa5	FULLWIDTH LATIN CAPITAL LETTER E

        case 0xFF46:		//	ｆ	0xef 0xbd 0x86	FULLWIDTH LATIN SMALL LETTER F
            return 0xFF26;	//	Ｆ	0xef 0xbc 0xa6	FULLWIDTH LATIN CAPITAL LETTER F

        case 0xFF47:		//	ｇ	0xef 0xbd 0x87	FULLWIDTH LATIN SMALL LETTER G
            return 0xFF27;	//	Ｇ	0xef 0xbc 0xa7	FULLWIDTH LATIN CAPITAL LETTER G

        case 0xFF48:		//	ｈ	0xef 0xbd 0x88	FULLWIDTH LATIN SMALL LETTER H
            return 0xFF28;	//	Ｈ	0xef 0xbc 0xa8	FULLWIDTH LATIN CAPITAL LETTER H

        case 0xFF49:		//	ｉ	0xef 0xbd 0x89	FULLWIDTH LATIN SMALL LETTER I
            return 0xFF29;	//	Ｉ	0xef 0xbc 0xa9	FULLWIDTH LATIN CAPITAL LETTER I

        case 0xFF4A:		//	ｊ	0xef 0xbd 0x8a	FULLWIDTH LATIN SMALL LETTER J
            return 0xFF2A;	//	Ｊ	0xef 0xbc 0xaa	FULLWIDTH LATIN CAPITAL LETTER J

        case 0xFF4B:		//	ｋ	0xef 0xbd 0x8b	FULLWIDTH LATIN SMALL LETTER K
            return 0xFF2B;	//	Ｋ	0xef 0xbc 0xab	FULLWIDTH LATIN CAPITAL LETTER K

        case 0xFF4C:		//	ｌ	0xef 0xbd 0x8c	FULLWIDTH LATIN SMALL LETTER L
            return 0xFF2C;	//	Ｌ	0xef 0xbc 0xac	FULLWIDTH LATIN CAPITAL LETTER L

        case 0xFF4D:		//	ｍ	0xef 0xbd 0x8d	FULLWIDTH LATIN SMALL LETTER M
            return 0xFF2D;	//	Ｍ	0xef 0xbc 0xad	FULLWIDTH LATIN CAPITAL LETTER M

        case 0xFF4E:		//	ｎ	0xef 0xbd 0x8e	FULLWIDTH LATIN SMALL LETTER N
            return 0xFF2E;	//	Ｎ	0xef 0xbc 0xae	FULLWIDTH LATIN CAPITAL LETTER N

        case 0xFF4F:		//	ｏ	0xef 0xbd 0x8f	FULLWIDTH LATIN SMALL LETTER O
            return 0xFF2F;	//	Ｏ	0xef 0xbc 0xaf	FULLWIDTH LATIN CAPITAL LETTER O

        case 0xFF50:		//	ｐ	0xef 0xbd 0x90	FULLWIDTH LATIN SMALL LETTER P
            return 0xFF30;	//	Ｐ	0xef 0xbc 0xb0	FULLWIDTH LATIN CAPITAL LETTER P

        case 0xFF51:		//	ｑ	0xef 0xbd 0x91	FULLWIDTH LATIN SMALL LETTER Q
            return 0xFF31;	//	Ｑ	0xef 0xbc 0xb1	FULLWIDTH LATIN CAPITAL LETTER Q

        case 0xFF52:		//	ｒ	0xef 0xbd 0x92	FULLWIDTH LATIN SMALL LETTER R
            return 0xFF32;	//	Ｒ	0xef 0xbc 0xb2	FULLWIDTH LATIN CAPITAL LETTER R

        case 0xFF53:		//	ｓ	0xef 0xbd 0x93	FULLWIDTH LATIN SMALL LETTER S
            return 0xFF33;	//	Ｓ	0xef 0xbc 0xb3	FULLWIDTH LATIN CAPITAL LETTER S

        case 0xFF54:		//	ｔ	0xef 0xbd 0x94	FULLWIDTH LATIN SMALL LETTER T
            return 0xFF34;	//	Ｔ	0xef 0xbc 0xb4	FULLWIDTH LATIN CAPITAL LETTER T

        case 0xFF55:		//	ｕ	0xef 0xbd 0x95	FULLWIDTH LATIN SMALL LETTER U
            return 0xFF35;	//	Ｕ	0xef 0xbc 0xb5	FULLWIDTH LATIN CAPITAL LETTER U

        case 0xFF56:		//	ｖ	0xef 0xbd 0x96	FULLWIDTH LATIN SMALL LETTER V
            return 0xFF36;	//	Ｖ	0xef 0xbc 0xb6	FULLWIDTH LATIN CAPITAL LETTER V

        case 0xFF57:		//	ｗ	0xef 0xbd 0x97	FULLWIDTH LATIN SMALL LETTER W
            return 0xFF37;	//	Ｗ	0xef 0xbc 0xb7	FULLWIDTH LATIN CAPITAL LETTER W

        case 0xFF58:		//	ｘ	0xef 0xbd 0x98	FULLWIDTH LATIN SMALL LETTER X
            return 0xFF38;	//	Ｘ	0xef 0xbc 0xb8	FULLWIDTH LATIN CAPITAL LETTER X

        case 0xFF59:		//	ｙ	0xef 0xbd 0x99	FULLWIDTH LATIN SMALL LETTER Y
            return 0xFF39;	//	Ｙ	0xef 0xbc 0xb9	FULLWIDTH LATIN CAPITAL LETTER Y

        case 0xFF5A:		//	ｚ	0xef 0xbd 0x9a	FULLWIDTH LATIN SMALL LETTER Z
            return 0xFF3A;	//	Ｚ	0xef 0xbc 0xba	FULLWIDTH LATIN CAPITAL LETTER Z

#endif
#if UTF_SUPPORT_DESERET == 1
        case 0x10428:		//	𐐨	0xf0 0x90 0x90 0xa8	DESERET SMALL LETTER LONG I
            return 0x10400;	//	𐐀	0xf0 0x90 0x90 0x80	DESERET CAPITAL LETTER LONG I

        case 0x10429:		//	𐐩	0xf0 0x90 0x90 0xa9	DESERET SMALL LETTER LONG E
            return 0x10401;	//	𐐁	0xf0 0x90 0x90 0x81	DESERET CAPITAL LETTER LONG E

        case 0x1042A:		//	𐐪	0xf0 0x90 0x90 0xaa	DESERET SMALL LETTER LONG A
            return 0x10402;	//	𐐂	0xf0 0x90 0x90 0x82	DESERET CAPITAL LETTER LONG A

        case 0x1042B:		//	𐐫	0xf0 0x90 0x90 0xab	DESERET SMALL LETTER LONG AH
            return 0x10403;	//	𐐃	0xf0 0x90 0x90 0x83	DESERET CAPITAL LETTER LONG AH

        case 0x1042C:		//	𐐬	0xf0 0x90 0x90 0xac	DESERET SMALL LETTER LONG O
            return 0x10404;	//	𐐄	0xf0 0x90 0x90 0x84	DESERET CAPITAL LETTER LONG O

        case 0x1042D:		//	𐐭	0xf0 0x90 0x90 0xad	DESERET SMALL LETTER LONG OO
            return 0x10405;	//	𐐅	0xf0 0x90 0x90 0x85	DESERET CAPITAL LETTER LONG OO

        case 0x1042E:		//	𐐮	0xf0 0x90 0x90 0xae	DESERET SMALL LETTER SHORT I
            return 0x10406;	//	𐐆	0xf0 0x90 0x90 0x86	DESERET CAPITAL LETTER SHORT I

        case 0x1042F:		//	𐐯	0xf0 0x90 0x90 0xaf	DESERET SMALL LETTER SHORT E
            return 0x10407;	//	𐐇	0xf0 0x90 0x90 0x87	DESERET CAPITAL LETTER SHORT E

        case 0x10430:		//	𐐰	0xf0 0x90 0x90 0xb0	DESERET SMALL LETTER SHORT A
            return 0x10408;	//	𐐈	0xf0 0x90 0x90 0x88	DESERET CAPITAL LETTER SHORT A

        case 0x10431:		//	𐐱	0xf0 0x90 0x90 0xb1	DESERET SMALL LETTER SHORT AH
            return 0x10409;	//	𐐉	0xf0 0x90 0x90 0x89	DESERET CAPITAL LETTER SHORT AH

        case 0x10432:		//	𐐲	0xf0 0x90 0x90 0xb2	DESERET SMALL LETTER SHORT O
            return 0x1040A;	//	𐐊	0xf0 0x90 0x90 0x8a	DESERET CAPITAL LETTER SHORT O

        case 0x10433:		//	𐐳	0xf0 0x90 0x90 0xb3	DESERET SMALL LETTER SHORT OO
            return 0x1040B;	//	𐐋	0xf0 0x90 0x90 0x8b	DESERET CAPITAL LETTER SHORT OO

        case 0x10434:		//	𐐴	0xf0 0x90 0x90 0xb4	DESERET SMALL LETTER AY
            return 0x1040C;	//	𐐌	0xf0 0x90 0x90 0x8c	DESERET CAPITAL LETTER AY

        case 0x10435:		//	𐐵	0xf0 0x90 0x90 0xb5	DESERET SMALL LETTER OW
            return 0x1040D;	//	𐐍	0xf0 0x90 0x90 0x8d	DESERET CAPITAL LETTER OW

        case 0x10436:		//	𐐶	0xf0 0x90 0x90 0xb6	DESERET SMALL LETTER WU
            return 0x1040E;	//	𐐎	0xf0 0x90 0x90 0x8e	DESERET CAPITAL LETTER WU

        case 0x10437:		//	𐐷	0xf0 0x90 0x90 0xb7	DESERET SMALL LETTER YEE
            return 0x1040F;	//	𐐏	0xf0 0x90 0x90 0x8f	DESERET CAPITAL LETTER YEE

        case 0x10438:		//	𐐸	0xf0 0x90 0x90 0xb8	DESERET SMALL LETTER H
            return 0x10410;	//	𐐐	0xf0 0x90 0x90 0x90	DESERET CAPITAL LETTER H

        case 0x10439:		//	𐐹	0xf0 0x90 0x90 0xb9	DESERET SMALL LETTER PEE
            return 0x10411;	//	𐐑	0xf0 0x90 0x90 0x91	DESERET CAPITAL LETTER PEE

        case 0x1043A:		//	𐐺	0xf0 0x90 0x90 0xba	DESERET SMALL LETTER BEE
            return 0x10412;	//	𐐒	0xf0 0x90 0x90 0x92	DESERET CAPITAL LETTER BEE

        case 0x1043B:		//	𐐻	0xf0 0x90 0x90 0xbb	DESERET SMALL LETTER TEE
            return 0x10413;	//	𐐓	0xf0 0x90 0x90 0x93	DESERET CAPITAL LETTER TEE

        case 0x1043C:		//	𐐼	0xf0 0x90 0x90 0xbc	DESERET SMALL LETTER DEE
            return 0x10414;	//	𐐔	0xf0 0x90 0x90 0x94	DESERET CAPITAL LETTER DEE

        case 0x1043D:		//	𐐽	0xf0 0x90 0x90 0xbd	DESERET SMALL LETTER CHEE
            return 0x10415;	//	𐐕	0xf0 0x90 0x90 0x95	DESERET CAPITAL LETTER CHEE

        case 0x1043E:		//	𐐾	0xf0 0x90 0x90 0xbe	DESERET SMALL LETTER JEE
            return 0x10416;	//	𐐖	0xf0 0x90 0x90 0x96	DESERET CAPITAL LETTER JEE

        case 0x1043F:		//	𐐿	0xf0 0x90 0x90 0xbf	DESERET SMALL LETTER KAY
            return 0x10417;	//	𐐗	0xf0 0x90 0x90 0x97	DESERET CAPITAL LETTER KAY

        case 0x10440:		//	𐑀	0xf0 0x90 0x91 0x80	DESERET SMALL LETTER GAY
            return 0x10418;	//	𐐘	0xf0 0x90 0x90 0x98	DESERET CAPITAL LETTER GAY

        case 0x10441:		//	𐑁	0xf0 0x90 0x91 0x81	DESERET SMALL LETTER EF
            return 0x10419;	//	𐐙	0xf0 0x90 0x90 0x99	DESERET CAPITAL LETTER EF

        case 0x10442:		//	𐑂	0xf0 0x90 0x91 0x82	DESERET SMALL LETTER VEE
            return 0x1041A;	//	𐐚	0xf0 0x90 0x90 0x9a	DESERET CAPITAL LETTER VEE

        case 0x10443:		//	𐑃	0xf0 0x90 0x91 0x83	DESERET SMALL LETTER ETH
            return 0x1041B;	//	𐐛	0xf0 0x90 0x90 0x9b	DESERET CAPITAL LETTER ETH

        case 0x10444:		//	𐑄	0xf0 0x90 0x91 0x84	DESERET SMALL LETTER THEE
            return 0x1041C;	//	𐐜	0xf0 0x90 0x90 0x9c	DESERET CAPITAL LETTER THEE

        case 0x10445:		//	𐑅	0xf0 0x90 0x91 0x85	DESERET SMALL LETTER ES
            return 0x1041D;	//	𐐝	0xf0 0x90 0x90 0x9d	DESERET CAPITAL LETTER ES

        case 0x10446:		//	𐑆	0xf0 0x90 0x91 0x86	DESERET SMALL LETTER ZEE
            return 0x1041E;	//	𐐞	0xf0 0x90 0x90 0x9e	DESERET CAPITAL LETTER ZEE

        case 0x10447:		//	𐑇	0xf0 0x90 0x91 0x87	DESERET SMALL LETTER ESH
            return 0x1041F;	//	𐐟	0xf0 0x90 0x90 0x9f	DESERET CAPITAL LETTER ESH

        case 0x10448:		//	𐑈	0xf0 0x90 0x91 0x88	DESERET SMALL LETTER ZHEE
            return 0x10420;	//	𐐠	0xf0 0x90 0x90 0xa0	DESERET CAPITAL LETTER ZHEE

        case 0x10449:		//	𐑉	0xf0 0x90 0x91 0x89	DESERET SMALL LETTER ER
            return 0x10421;	//	𐐡	0xf0 0x90 0x90 0xa1	DESERET CAPITAL LETTER ER

        case 0x1044A:		//	𐑊	0xf0 0x90 0x91 0x8a	DESERET SMALL LETTER EL
            return 0x10422;	//	𐐢	0xf0 0x90 0x90 0xa2	DESERET CAPITAL LETTER EL

        case 0x1044B:		//	𐑋	0xf0 0x90 0x91 0x8b	DESERET SMALL LETTER EM
            return 0x10423;	//	𐐣	0xf0 0x90 0x90 0xa3	DESERET CAPITAL LETTER EM

        case 0x1044C:		//	𐑌	0xf0 0x90 0x91 0x8c	DESERET SMALL LETTER EN
            return 0x10424;	//	𐐤	0xf0 0x90 0x90 0xa4	DESERET CAPITAL LETTER EN

        case 0x1044D:		//	𐑍	0xf0 0x90 0x91 0x8d	DESERET SMALL LETTER ENG
            return 0x10425;	//	𐐥	0xf0 0x90 0x90 0xa5	DESERET CAPITAL LETTER ENG

        case 0x1044E:		//	𐑎	0xf0 0x90 0x91 0x8e	DESERET SMALL LETTER OI
            return 0x10426;	//	𐐦	0xf0 0x90 0x90 0xa6	DESERET CAPITAL LETTER OI

        case 0x1044F:		//	𐑏	0xf0 0x90 0x91 0x8f	DESERET SMALL LETTER EW
            return 0x10427;	//	𐐧	0xf0 0x90 0x90 0xa7	DESERET CAPITAL LETTER EW

#endif
#if UTF_SUPPORT_OSAGE == 1
        case 0x104D8:		//	𐓘	0xf0 0x90 0x93 0x98	OSAGE SMALL LETTER A
            return 0x104B0;	//	𐒰	0xf0 0x90 0x92 0xb0	OSAGE CAPITAL LETTER A

        case 0x104D9:		//	𐓙	0xf0 0x90 0x93 0x99	OSAGE SMALL LETTER AI
            return 0x104B1;	//	𐒱	0xf0 0x90 0x92 0xb1	OSAGE CAPITAL LETTER AI

        case 0x104DA:		//	𐓚	0xf0 0x90 0x93 0x9a	OSAGE SMALL LETTER AIN
            return 0x104B2;	//	𐒲	0xf0 0x90 0x92 0xb2	OSAGE CAPITAL LETTER AIN

        case 0x104DB:		//	𐓛	0xf0 0x90 0x93 0x9b	OSAGE SMALL LETTER AH
            return 0x104B3;	//	𐒳	0xf0 0x90 0x92 0xb3	OSAGE CAPITAL LETTER AH

        case 0x104DC:		//	𐓜	0xf0 0x90 0x93 0x9c	OSAGE SMALL LETTER BRA
            return 0x104B4;	//	𐒴	0xf0 0x90 0x92 0xb4	OSAGE CAPITAL LETTER BRA

        case 0x104DD:		//	𐓝	0xf0 0x90 0x93 0x9d	OSAGE SMALL LETTER CHA
            return 0x104B5;	//	𐒵	0xf0 0x90 0x92 0xb5	OSAGE CAPITAL LETTER CHA

        case 0x104DE:		//	𐓞	0xf0 0x90 0x93 0x9e	OSAGE SMALL LETTER EHCHA
            return 0x104B6;	//	𐒶	0xf0 0x90 0x92 0xb6	OSAGE CAPITAL LETTER EHCHA

        case 0x104DF:		//	𐓟	0xf0 0x90 0x93 0x9f	OSAGE SMALL LETTER E
            return 0x104B7;	//	𐒷	0xf0 0x90 0x92 0xb7	OSAGE CAPITAL LETTER E

        case 0x104E0:		//	𐓠	0xf0 0x90 0x93 0xa0	OSAGE SMALL LETTER EIN
            return 0x104B8;	//	𐒸	0xf0 0x90 0x92 0xb8	OSAGE CAPITAL LETTER EIN

        case 0x104E1:		//	𐓡	0xf0 0x90 0x93 0xa1	OSAGE SMALL LETTER HA
            return 0x104B9;	//	𐒹	0xf0 0x90 0x92 0xb9	OSAGE CAPITAL LETTER HA

        case 0x104E2:		//	𐓢	0xf0 0x90 0x93 0xa2	OSAGE SMALL LETTER HYA
            return 0x104BA;	//	𐒺	0xf0 0x90 0x92 0xba	OSAGE CAPITAL LETTER HYA

        case 0x104E3:		//	𐓣	0xf0 0x90 0x93 0xa3	OSAGE SMALL LETTER I
            return 0x104BB;	//	𐒻	0xf0 0x90 0x92 0xbb	OSAGE CAPITAL LETTER I

        case 0x104E4:		//	𐓤	0xf0 0x90 0x93 0xa4	OSAGE SMALL LETTER KA
            return 0x104BC;	//	𐒼	0xf0 0x90 0x92 0xbc	OSAGE CAPITAL LETTER KA

        case 0x104E5:		//	𐓥	0xf0 0x90 0x93 0xa5	OSAGE SMALL LETTER EHKA
            return 0x104BD;	//	𐒽	0xf0 0x90 0x92 0xbd	OSAGE CAPITAL LETTER EHKA

        case 0x104E6:		//	𐓦	0xf0 0x90 0x93 0xa6	OSAGE SMALL LETTER KYA
            return 0x104BE;	//	𐒾	0xf0 0x90 0x92 0xbe	OSAGE CAPITAL LETTER KYA

        case 0x104E7:		//	𐓧	0xf0 0x90 0x93 0xa7	OSAGE SMALL LETTER LA
            return 0x104BF;	//	𐒿	0xf0 0x90 0x92 0xbf	OSAGE CAPITAL LETTER LA

        case 0x104E8:		//	𐓨	0xf0 0x90 0x93 0xa8	OSAGE SMALL LETTER MA
            return 0x104C0;	//	𐓀	0xf0 0x90 0x93 0x80	OSAGE CAPITAL LETTER MA

        case 0x104E9:		//	𐓩	0xf0 0x90 0x93 0xa9	OSAGE SMALL LETTER NA
            return 0x104C1;	//	𐓁	0xf0 0x90 0x93 0x81	OSAGE CAPITAL LETTER NA

        case 0x104EA:		//	𐓪	0xf0 0x90 0x93 0xaa	OSAGE SMALL LETTER O
            return 0x104C2;	//	𐓂	0xf0 0x90 0x93 0x82	OSAGE CAPITAL LETTER O

        case 0x104EB:		//	𐓫	0xf0 0x90 0x93 0xab	OSAGE SMALL LETTER OIN
            return 0x104C3;	//	𐓃	0xf0 0x90 0x93 0x83	OSAGE CAPITAL LETTER OIN

        case 0x104EC:		//	𐓬	0xf0 0x90 0x93 0xac	OSAGE SMALL LETTER PA
            return 0x104C4;	//	𐓄	0xf0 0x90 0x93 0x84	OSAGE CAPITAL LETTER PA

        case 0x104ED:		//	𐓭	0xf0 0x90 0x93 0xad	OSAGE SMALL LETTER EHPA
            return 0x104C5;	//	𐓅	0xf0 0x90 0x93 0x85	OSAGE CAPITAL LETTER EHPA

        case 0x104EE:		//	𐓮	0xf0 0x90 0x93 0xae	OSAGE SMALL LETTER SA
            return 0x104C6;	//	𐓆	0xf0 0x90 0x93 0x86	OSAGE CAPITAL LETTER SA

        case 0x104EF:		//	𐓯	0xf0 0x90 0x93 0xaf	OSAGE SMALL LETTER SHA
            return 0x104C7;	//	𐓇	0xf0 0x90 0x93 0x87	OSAGE CAPITAL LETTER SHA

        case 0x104F0:		//	𐓰	0xf0 0x90 0x93 0xb0	OSAGE SMALL LETTER TA
            return 0x104C8;	//	𐓈	0xf0 0x90 0x93 0x88	OSAGE CAPITAL LETTER TA

        case 0x104F1:		//	𐓱	0xf0 0x90 0x93 0xb1	OSAGE SMALL LETTER EHTA
            return 0x104C9;	//	𐓉	0xf0 0x90 0x93 0x89	OSAGE CAPITAL LETTER EHTA

        case 0x104F2:		//	𐓲	0xf0 0x90 0x93 0xb2	OSAGE SMALL LETTER TSA
            return 0x104CA;	//	𐓊	0xf0 0x90 0x93 0x8a	OSAGE CAPITAL LETTER TSA

        case 0x104F3:		//	𐓳	0xf0 0x90 0x93 0xb3	OSAGE SMALL LETTER EHTSA
            return 0x104CB;	//	𐓋	0xf0 0x90 0x93 0x8b	OSAGE CAPITAL LETTER EHTSA

        case 0x104F4:		//	𐓴	0xf0 0x90 0x93 0xb4	OSAGE SMALL LETTER TSHA
            return 0x104CC;	//	𐓌	0xf0 0x90 0x93 0x8c	OSAGE CAPITAL LETTER TSHA

        case 0x104F5:		//	𐓵	0xf0 0x90 0x93 0xb5	OSAGE SMALL LETTER DHA
            return 0x104CD;	//	𐓍	0xf0 0x90 0x93 0x8d	OSAGE CAPITAL LETTER DHA

        case 0x104F6:		//	𐓶	0xf0 0x90 0x93 0xb6	OSAGE SMALL LETTER U
            return 0x104CE;	//	𐓎	0xf0 0x90 0x93 0x8e	OSAGE CAPITAL LETTER U

        case 0x104F7:		//	𐓷	0xf0 0x90 0x93 0xb7	OSAGE SMALL LETTER WA
            return 0x104CF;	//	𐓏	0xf0 0x90 0x93 0x8f	OSAGE CAPITAL LETTER WA

        case 0x104F8:		//	𐓸	0xf0 0x90 0x93 0xb8	OSAGE SMALL LETTER KHA
            return 0x104D0;	//	𐓐	0xf0 0x90 0x93 0x90	OSAGE CAPITAL LETTER KHA

        case 0x104F9:		//	𐓹	0xf0 0x90 0x93 0xb9	OSAGE SMALL LETTER GHA
            return 0x104D1;	//	𐓑	0xf0 0x90 0x93 0x91	OSAGE CAPITAL LETTER GHA

        case 0x104FA:		//	𐓺	0xf0 0x90 0x93 0xba	OSAGE SMALL LETTER ZA
            return 0x104D2;	//	𐓒	0xf0 0x90 0x93 0x92	OSAGE CAPITAL LETTER ZA

        case 0x104FB:		//	𐓻	0xf0 0x90 0x93 0xbb	OSAGE SMALL LETTER ZHA
            return 0x104D3;	//	𐓓	0xf0 0x90 0x93 0x93	OSAGE CAPITAL LETTER ZHA

#endif
#if UTF_SUPPORT_HUNGARIAN == 1
        case 0x10CC0:		//	𐳀	0xf0 0x90 0xb3 0x80	OLD HUNGARIAN SMALL LETTER A
            return 0x10C80;	//	𐲀	0xf0 0x90 0xb2 0x80	OLD HUNGARIAN CAPITAL LETTER A

        case 0x10CC1:		//	𐳁	0xf0 0x90 0xb3 0x81	OLD HUNGARIAN SMALL LETTER AA
            return 0x10C81;	//	𐲁	0xf0 0x90 0xb2 0x81	OLD HUNGARIAN CAPITAL LETTER AA

        case 0x10CC2:		//	𐳂	0xf0 0x90 0xb3 0x82	OLD HUNGARIAN SMALL LETTER EB
            return 0x10C82;	//	𐲂	0xf0 0x90 0xb2 0x82	OLD HUNGARIAN CAPITAL LETTER EB

        case 0x10CC3:		//	𐳃	0xf0 0x90 0xb3 0x83	OLD HUNGARIAN SMALL LETTER AMB
            return 0x10C83;	//	𐲃	0xf0 0x90 0xb2 0x83	OLD HUNGARIAN CAPITAL LETTER AMB

        case 0x10CC4:		//	𐳄	0xf0 0x90 0xb3 0x84	OLD HUNGARIAN SMALL LETTER EC
            return 0x10C84;	//	𐲄	0xf0 0x90 0xb2 0x84	OLD HUNGARIAN CAPITAL LETTER EC

        case 0x10CC5:		//	𐳅	0xf0 0x90 0xb3 0x85	OLD HUNGARIAN SMALL LETTER ENC
            return 0x10C85;	//	𐲅	0xf0 0x90 0xb2 0x85	OLD HUNGARIAN CAPITAL LETTER ENC

        case 0x10CC6:		//	𐳆	0xf0 0x90 0xb3 0x86	OLD HUNGARIAN SMALL LETTER ECS
            return 0x10C86;	//	𐲆	0xf0 0x90 0xb2 0x86	OLD HUNGARIAN CAPITAL LETTER ECS

        case 0x10CC7:		//	𐳇	0xf0 0x90 0xb3 0x87	OLD HUNGARIAN SMALL LETTER ED
            return 0x10C87;	//	𐲇	0xf0 0x90 0xb2 0x87	OLD HUNGARIAN CAPITAL LETTER ED

        case 0x10CC8:		//	𐳈	0xf0 0x90 0xb3 0x88	OLD HUNGARIAN SMALL LETTER AND
            return 0x10C88;	//	𐲈	0xf0 0x90 0xb2 0x88	OLD HUNGARIAN CAPITAL LETTER AND

        case 0x10CC9:		//	𐳉	0xf0 0x90 0xb3 0x89	OLD HUNGARIAN SMALL LETTER E
            return 0x10C89;	//	𐲉	0xf0 0x90 0xb2 0x89	OLD HUNGARIAN CAPITAL LETTER E

        case 0x10CCA:		//	𐳊	0xf0 0x90 0xb3 0x8a	OLD HUNGARIAN SMALL LETTER CLOSE E
            return 0x10C8A;	//	𐲊	0xf0 0x90 0xb2 0x8a	OLD HUNGARIAN CAPITAL LETTER CLOSE E

        case 0x10CCB:		//	𐳋	0xf0 0x90 0xb3 0x8b	OLD HUNGARIAN SMALL LETTER EE
            return 0x10C8B;	//	𐲋	0xf0 0x90 0xb2 0x8b	OLD HUNGARIAN CAPITAL LETTER EE

        case 0x10CCC:		//	𐳌	0xf0 0x90 0xb3 0x8c	OLD HUNGARIAN SMALL LETTER EF
            return 0x10C8C;	//	𐲌	0xf0 0x90 0xb2 0x8c	OLD HUNGARIAN CAPITAL LETTER EF

        case 0x10CCD:		//	𐳍	0xf0 0x90 0xb3 0x8d	OLD HUNGARIAN SMALL LETTER EG
            return 0x10C8D;	//	𐲍	0xf0 0x90 0xb2 0x8d	OLD HUNGARIAN CAPITAL LETTER EG

        case 0x10CCE:		//	𐳎	0xf0 0x90 0xb3 0x8e	OLD HUNGARIAN SMALL LETTER EGY
            return 0x10C8E;	//	𐲎	0xf0 0x90 0xb2 0x8e	OLD HUNGARIAN CAPITAL LETTER EGY

        case 0x10CCF:		//	𐳏	0xf0 0x90 0xb3 0x8f	OLD HUNGARIAN SMALL LETTER EH
            return 0x10C8F;	//	𐲏	0xf0 0x90 0xb2 0x8f	OLD HUNGARIAN CAPITAL LETTER EH

        case 0x10CD0:		//	𐳐	0xf0 0x90 0xb3 0x90	OLD HUNGARIAN SMALL LETTER I
            return 0x10C90;	//	𐲐	0xf0 0x90 0xb2 0x90	OLD HUNGARIAN CAPITAL LETTER I

        case 0x10CD1:		//	𐳑	0xf0 0x90 0xb3 0x91	OLD HUNGARIAN SMALL LETTER II
            return 0x10C91;	//	𐲑	0xf0 0x90 0xb2 0x91	OLD HUNGARIAN CAPITAL LETTER II

        case 0x10CD2:		//	𐳒	0xf0 0x90 0xb3 0x92	OLD HUNGARIAN SMALL LETTER EJ
            return 0x10C92;	//	𐲒	0xf0 0x90 0xb2 0x92	OLD HUNGARIAN CAPITAL LETTER EJ

        case 0x10CD3:		//	𐳓	0xf0 0x90 0xb3 0x93	OLD HUNGARIAN SMALL LETTER EK
            return 0x10C93;	//	𐲓	0xf0 0x90 0xb2 0x93	OLD HUNGARIAN CAPITAL LETTER EK

        case 0x10CD4:		//	𐳔	0xf0 0x90 0xb3 0x94	OLD HUNGARIAN SMALL LETTER AK
            return 0x10C94;	//	𐲔	0xf0 0x90 0xb2 0x94	OLD HUNGARIAN CAPITAL LETTER AK

        case 0x10CD5:		//	𐳕	0xf0 0x90 0xb3 0x95	OLD HUNGARIAN SMALL LETTER UNK
            return 0x10C95;	//	𐲕	0xf0 0x90 0xb2 0x95	OLD HUNGARIAN CAPITAL LETTER UNK

        case 0x10CD6:		//	𐳖	0xf0 0x90 0xb3 0x96	OLD HUNGARIAN SMALL LETTER EL
            return 0x10C96;	//	𐲖	0xf0 0x90 0xb2 0x96	OLD HUNGARIAN CAPITAL LETTER EL

        case 0x10CD7:		//	𐳗	0xf0 0x90 0xb3 0x97	OLD HUNGARIAN SMALL LETTER ELY
            return 0x10C97;	//	𐲗	0xf0 0x90 0xb2 0x97	OLD HUNGARIAN CAPITAL LETTER ELY

        case 0x10CD8:		//	𐳘	0xf0 0x90 0xb3 0x98	OLD HUNGARIAN SMALL LETTER EM
            return 0x10C98;	//	𐲘	0xf0 0x90 0xb2 0x98	OLD HUNGARIAN CAPITAL LETTER EM

        case 0x10CD9:		//	𐳙	0xf0 0x90 0xb3 0x99	OLD HUNGARIAN SMALL LETTER EN
            return 0x10C99;	//	𐲙	0xf0 0x90 0xb2 0x99	OLD HUNGARIAN CAPITAL LETTER EN

        case 0x10CDA:		//	𐳚	0xf0 0x90 0xb3 0x9a	OLD HUNGARIAN SMALL LETTER ENY
            return 0x10C9A;	//	𐲚	0xf0 0x90 0xb2 0x9a	OLD HUNGARIAN CAPITAL LETTER ENY

        case 0x10CDB:		//	𐳛	0xf0 0x90 0xb3 0x9b	OLD HUNGARIAN SMALL LETTER O
            return 0x10C9B;	//	𐲛	0xf0 0x90 0xb2 0x9b	OLD HUNGARIAN CAPITAL LETTER O

        case 0x10CDC:		//	𐳜	0xf0 0x90 0xb3 0x9c	OLD HUNGARIAN SMALL LETTER OO
            return 0x10C9C;	//	𐲜	0xf0 0x90 0xb2 0x9c	OLD HUNGARIAN CAPITAL LETTER OO

        case 0x10CDD:		//	𐳝	0xf0 0x90 0xb3 0x9d	OLD HUNGARIAN SMALL LETTER NIKOLSBURG OE
            return 0x10C9D;	//	𐲝	0xf0 0x90 0xb2 0x9d	OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG OE

        case 0x10CDE:		//	𐳞	0xf0 0x90 0xb3 0x9e	OLD HUNGARIAN SMALL LETTER RUDIMENTA OE
            return 0x10C9E;	//	𐲞	0xf0 0x90 0xb2 0x9e	OLD HUNGARIAN CAPITAL LETTER RUDIMENTA OE

        case 0x10CDF:		//	𐳟	0xf0 0x90 0xb3 0x9f	OLD HUNGARIAN SMALL LETTER OEE
            return 0x10C9F;	//	𐲟	0xf0 0x90 0xb2 0x9f	OLD HUNGARIAN CAPITAL LETTER OEE

        case 0x10CE0:		//	𐳠	0xf0 0x90 0xb3 0xa0	OLD HUNGARIAN SMALL LETTER EP
            return 0x10CA0;	//	𐲠	0xf0 0x90 0xb2 0xa0	OLD HUNGARIAN CAPITAL LETTER EP

        case 0x10CE1:		//	𐳡	0xf0 0x90 0xb3 0xa1	OLD HUNGARIAN SMALL LETTER EMP
            return 0x10CA1;	//	𐲡	0xf0 0x90 0xb2 0xa1	OLD HUNGARIAN CAPITAL LETTER EMP

        case 0x10CE2:		//	𐳢	0xf0 0x90 0xb3 0xa2	OLD HUNGARIAN SMALL LETTER ER
            return 0x10CA2;	//	𐲢	0xf0 0x90 0xb2 0xa2	OLD HUNGARIAN CAPITAL LETTER ER

        case 0x10CE3:		//	𐳣	0xf0 0x90 0xb3 0xa3	OLD HUNGARIAN SMALL LETTER SHORT ER
            return 0x10CA3;	//	𐲣	0xf0 0x90 0xb2 0xa3	OLD HUNGARIAN CAPITAL LETTER SHORT ER

        case 0x10CE4:		//	𐳤	0xf0 0x90 0xb3 0xa4	OLD HUNGARIAN SMALL LETTER ES
            return 0x10CA4;	//	𐲤	0xf0 0x90 0xb2 0xa4	OLD HUNGARIAN CAPITAL LETTER ES

        case 0x10CE5:		//	𐳥	0xf0 0x90 0xb3 0xa5	OLD HUNGARIAN SMALL LETTER ESZ
            return 0x10CA5;	//	𐲥	0xf0 0x90 0xb2 0xa5	OLD HUNGARIAN CAPITAL LETTER ESZ

        case 0x10CE6:		//	𐳦	0xf0 0x90 0xb3 0xa6	OLD HUNGARIAN SMALL LETTER ET
            return 0x10CA6;	//	𐲦	0xf0 0x90 0xb2 0xa6	OLD HUNGARIAN CAPITAL LETTER ET

        case 0x10CE7:		//	𐳧	0xf0 0x90 0xb3 0xa7	OLD HUNGARIAN SMALL LETTER ENT
            return 0x10CA7;	//	𐲧	0xf0 0x90 0xb2 0xa7	OLD HUNGARIAN CAPITAL LETTER ENT

        case 0x10CE8:		//	𐳨	0xf0 0x90 0xb3 0xa8	OLD HUNGARIAN SMALL LETTER ETY
            return 0x10CA8;	//	𐲨	0xf0 0x90 0xb2 0xa8	OLD HUNGARIAN CAPITAL LETTER ETY

        case 0x10CE9:		//	𐳩	0xf0 0x90 0xb3 0xa9	OLD HUNGARIAN SMALL LETTER ECH
            return 0x10CA9;	//	𐲩	0xf0 0x90 0xb2 0xa9	OLD HUNGARIAN CAPITAL LETTER ECH

        case 0x10CEA:		//	𐳪	0xf0 0x90 0xb3 0xaa	OLD HUNGARIAN SMALL LETTER U
            return 0x10CAA;	//	𐲪	0xf0 0x90 0xb2 0xaa	OLD HUNGARIAN CAPITAL LETTER U

        case 0x10CEB:		//	𐳫	0xf0 0x90 0xb3 0xab	OLD HUNGARIAN SMALL LETTER UU
            return 0x10CAB;	//	𐲫	0xf0 0x90 0xb2 0xab	OLD HUNGARIAN CAPITAL LETTER UU

        case 0x10CEC:		//	𐳬	0xf0 0x90 0xb3 0xac	OLD HUNGARIAN SMALL LETTER NIKOLSBURG UE
            return 0x10CAC;	//	𐲬	0xf0 0x90 0xb2 0xac	OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG UE

        case 0x10CED:		//	𐳭	0xf0 0x90 0xb3 0xad	OLD HUNGARIAN SMALL LETTER RUDIMENTA UE
            return 0x10CAD;	//	𐲭	0xf0 0x90 0xb2 0xad	OLD HUNGARIAN CAPITAL LETTER RUDIMENTA UE

        case 0x10CEE:		//	𐳮	0xf0 0x90 0xb3 0xae	OLD HUNGARIAN SMALL LETTER EV
            return 0x10CAE;	//	𐲮	0xf0 0x90 0xb2 0xae	OLD HUNGARIAN CAPITAL LETTER EV

        case 0x10CEF:		//	𐳯	0xf0 0x90 0xb3 0xaf	OLD HUNGARIAN SMALL LETTER EZ
            return 0x10CAF;	//	𐲯	0xf0 0x90 0xb2 0xaf	OLD HUNGARIAN CAPITAL LETTER EZ

        case 0x10CF0:		//	𐳰	0xf0 0x90 0xb3 0xb0	OLD HUNGARIAN SMALL LETTER EZS
            return 0x10CB0;	//	𐲰	0xf0 0x90 0xb2 0xb0	OLD HUNGARIAN CAPITAL LETTER EZS

        case 0x10CF1:		//	𐳱	0xf0 0x90 0xb3 0xb1	OLD HUNGARIAN SMALL LETTER ENT-SHAPED SIGN
            return 0x10CB1;	//	𐲱	0xf0 0x90 0xb2 0xb1	OLD HUNGARIAN CAPITAL LETTER ENT-SHAPED SIGN

        case 0x10CF2:		//	𐳲	0xf0 0x90 0xb3 0xb2	OLD HUNGARIAN SMALL LETTER US
            return 0x10CB2;	//	𐲲	0xf0 0x90 0xb2 0xb2	OLD HUNGARIAN CAPITAL LETTER US

#endif
#if UTF_SUPPORT_WARANG_CITI == 1
        case 0x118C0:		//	𑣀	0xf0 0x91 0xa3 0x80	WARANG CITI SMALL LETTER NGAA
            return 0x118A0;	//	𑢠	0xf0 0x91 0xa2 0xa0	WARANG CITI CAPITAL LETTER NGAA

        case 0x118C1:		//	𑣁	0xf0 0x91 0xa3 0x81	WARANG CITI SMALL LETTER A
            return 0x118A1;	//	𑢡	0xf0 0x91 0xa2 0xa1	WARANG CITI CAPITAL LETTER A

        case 0x118C2:		//	𑣂	0xf0 0x91 0xa3 0x82	WARANG CITI SMALL LETTER WI
            return 0x118A2;	//	𑢢	0xf0 0x91 0xa2 0xa2	WARANG CITI CAPITAL LETTER WI

        case 0x118C3:		//	𑣃	0xf0 0x91 0xa3 0x83	WARANG CITI SMALL LETTER YU
            return 0x118A3;	//	𑢣	0xf0 0x91 0xa2 0xa3	WARANG CITI CAPITAL LETTER YU

        case 0x118C4:		//	𑣄	0xf0 0x91 0xa3 0x84	WARANG CITI SMALL LETTER YA
            return 0x118A4;	//	𑢤	0xf0 0x91 0xa2 0xa4	WARANG CITI CAPITAL LETTER YA

        case 0x118C5:		//	𑣅	0xf0 0x91 0xa3 0x85	WARANG CITI SMALL LETTER YO
            return 0x118A5;	//	𑢥	0xf0 0x91 0xa2 0xa5	WARANG CITI CAPITAL LETTER YO

        case 0x118C6:		//	𑣆	0xf0 0x91 0xa3 0x86	WARANG CITI SMALL LETTER II
            return 0x118A6;	//	𑢦	0xf0 0x91 0xa2 0xa6	WARANG CITI CAPITAL LETTER II

        case 0x118C7:		//	𑣇	0xf0 0x91 0xa3 0x87	WARANG CITI SMALL LETTER UU
            return 0x118A7;	//	𑢧	0xf0 0x91 0xa2 0xa7	WARANG CITI CAPITAL LETTER UU

        case 0x118C8:		//	𑣈	0xf0 0x91 0xa3 0x88	WARANG CITI SMALL LETTER E
            return 0x118A8;	//	𑢨	0xf0 0x91 0xa2 0xa8	WARANG CITI CAPITAL LETTER E

        case 0x118C9:		//	𑣉	0xf0 0x91 0xa3 0x89	WARANG CITI SMALL LETTER O
            return 0x118A9;	//	𑢩	0xf0 0x91 0xa2 0xa9	WARANG CITI CAPITAL LETTER O

        case 0x118CA:		//	𑣊	0xf0 0x91 0xa3 0x8a	WARANG CITI SMALL LETTER ANG
            return 0x118AA;	//	𑢪	0xf0 0x91 0xa2 0xaa	WARANG CITI CAPITAL LETTER ANG

        case 0x118CB:		//	𑣋	0xf0 0x91 0xa3 0x8b	WARANG CITI SMALL LETTER GA
            return 0x118AB;	//	𑢫	0xf0 0x91 0xa2 0xab	WARANG CITI CAPITAL LETTER GA

        case 0x118CC:		//	𑣌	0xf0 0x91 0xa3 0x8c	WARANG CITI SMALL LETTER KO
            return 0x118AC;	//	𑢬	0xf0 0x91 0xa2 0xac	WARANG CITI CAPITAL LETTER KO

        case 0x118CD:		//	𑣍	0xf0 0x91 0xa3 0x8d	WARANG CITI SMALL LETTER ENY
            return 0x118AD;	//	𑢭	0xf0 0x91 0xa2 0xad	WARANG CITI CAPITAL LETTER ENY

        case 0x118CE:		//	𑣎	0xf0 0x91 0xa3 0x8e	WARANG CITI SMALL LETTER YUJ
            return 0x118AE;	//	𑢮	0xf0 0x91 0xa2 0xae	WARANG CITI CAPITAL LETTER YUJ

        case 0x118CF:		//	𑣏	0xf0 0x91 0xa3 0x8f	WARANG CITI SMALL LETTER UC
            return 0x118AF;	//	𑢯	0xf0 0x91 0xa2 0xaf	WARANG CITI CAPITAL LETTER UC

        case 0x118D0:		//	𑣐	0xf0 0x91 0xa3 0x90	WARANG CITI SMALL LETTER ENN
            return 0x118B0;	//	𑢰	0xf0 0x91 0xa2 0xb0	WARANG CITI CAPITAL LETTER ENN

        case 0x118D1:		//	𑣑	0xf0 0x91 0xa3 0x91	WARANG CITI SMALL LETTER ODD
            return 0x118B1;	//	𑢱	0xf0 0x91 0xa2 0xb1	WARANG CITI CAPITAL LETTER ODD

        case 0x118D2:		//	𑣒	0xf0 0x91 0xa3 0x92	WARANG CITI SMALL LETTER TTE
            return 0x118B2;	//	𑢲	0xf0 0x91 0xa2 0xb2	WARANG CITI CAPITAL LETTER TTE

        case 0x118D3:		//	𑣓	0xf0 0x91 0xa3 0x93	WARANG CITI SMALL LETTER NUNG
            return 0x118B3;	//	𑢳	0xf0 0x91 0xa2 0xb3	WARANG CITI CAPITAL LETTER NUNG

        case 0x118D4:		//	𑣔	0xf0 0x91 0xa3 0x94	WARANG CITI SMALL LETTER DA
            return 0x118B4;	//	𑢴	0xf0 0x91 0xa2 0xb4	WARANG CITI CAPITAL LETTER DA

        case 0x118D5:		//	𑣕	0xf0 0x91 0xa3 0x95	WARANG CITI SMALL LETTER AT
            return 0x118B5;	//	𑢵	0xf0 0x91 0xa2 0xb5	WARANG CITI CAPITAL LETTER AT

        case 0x118D6:		//	𑣖	0xf0 0x91 0xa3 0x96	WARANG CITI SMALL LETTER AM
            return 0x118B6;	//	𑢶	0xf0 0x91 0xa2 0xb6	WARANG CITI CAPITAL LETTER AM

        case 0x118D7:		//	𑣗	0xf0 0x91 0xa3 0x97	WARANG CITI SMALL LETTER BU
            return 0x118B7;	//	𑢷	0xf0 0x91 0xa2 0xb7	WARANG CITI CAPITAL LETTER BU

        case 0x118D8:		//	𑣘	0xf0 0x91 0xa3 0x98	WARANG CITI SMALL LETTER PU
            return 0x118B8;	//	𑢸	0xf0 0x91 0xa2 0xb8	WARANG CITI CAPITAL LETTER PU

        case 0x118D9:		//	𑣙	0xf0 0x91 0xa3 0x99	WARANG CITI SMALL LETTER HIYO
            return 0x118B9;	//	𑢹	0xf0 0x91 0xa2 0xb9	WARANG CITI CAPITAL LETTER HIYO

        case 0x118DA:		//	𑣚	0xf0 0x91 0xa3 0x9a	WARANG CITI SMALL LETTER HOLO
            return 0x118BA;	//	𑢺	0xf0 0x91 0xa2 0xba	WARANG CITI CAPITAL LETTER HOLO

        case 0x118DB:		//	𑣛	0xf0 0x91 0xa3 0x9b	WARANG CITI SMALL LETTER HORR
            return 0x118BB;	//	𑢻	0xf0 0x91 0xa2 0xbb	WARANG CITI CAPITAL LETTER HORR

        case 0x118DC:		//	𑣜	0xf0 0x91 0xa3 0x9c	WARANG CITI SMALL LETTER HAR
            return 0x118BC;	//	𑢼	0xf0 0x91 0xa2 0xbc	WARANG CITI CAPITAL LETTER HAR

        case 0x118DD:		//	𑣝	0xf0 0x91 0xa3 0x9d	WARANG CITI SMALL LETTER SSUU
            return 0x118BD;	//	𑢽	0xf0 0x91 0xa2 0xbd	WARANG CITI CAPITAL LETTER SSUU

        case 0x118DE:		//	𑣞	0xf0 0x91 0xa3 0x9e	WARANG CITI SMALL LETTER SII
            return 0x118BE;	//	𑢾	0xf0 0x91 0xa2 0xbe	WARANG CITI CAPITAL LETTER SII

        case 0x118DF:		//	𑣟	0xf0 0x91 0xa3 0x9f	WARANG CITI SMALL LETTER VIYO
            return 0x118BF;	//	𑢿	0xf0 0x91 0xa2 0xbf	WARANG CITI CAPITAL LETTER VIYO

#endif
#if UTF_SUPPORT_MEDEFAIDRIN == 1
        case 0x16E60:		//	𖹠	0xf0 0x96 0xb9 0xa0	MEDEFAIDRIN SMALL LETTER M
            return 0x16E40;	//	𖹀	0xf0 0x96 0xb9 0x80	MEDEFAIDRIN CAPITAL LETTER M

        case 0x16E61:		//	𖹡	0xf0 0x96 0xb9 0xa1	MEDEFAIDRIN SMALL LETTER S
            return 0x16E41;	//	𖹁	0xf0 0x96 0xb9 0x81	MEDEFAIDRIN CAPITAL LETTER S

        case 0x16E62:		//	𖹢	0xf0 0x96 0xb9 0xa2	MEDEFAIDRIN SMALL LETTER V
            return 0x16E42;	//	𖹂	0xf0 0x96 0xb9 0x82	MEDEFAIDRIN CAPITAL LETTER V

        case 0x16E63:		//	𖹣	0xf0 0x96 0xb9 0xa3	MEDEFAIDRIN SMALL LETTER W
            return 0x16E43;	//	𖹃	0xf0 0x96 0xb9 0x83	MEDEFAIDRIN CAPITAL LETTER W

        case 0x16E64:		//	𖹤	0xf0 0x96 0xb9 0xa4	MEDEFAIDRIN SMALL LETTER ATIU
            return 0x16E44;	//	𖹄	0xf0 0x96 0xb9 0x84	MEDEFAIDRIN CAPITAL LETTER ATIU

        case 0x16E65:		//	𖹥	0xf0 0x96 0xb9 0xa5	MEDEFAIDRIN SMALL LETTER Z
            return 0x16E45;	//	𖹅	0xf0 0x96 0xb9 0x85	MEDEFAIDRIN CAPITAL LETTER Z

        case 0x16E66:		//	𖹦	0xf0 0x96 0xb9 0xa6	MEDEFAIDRIN SMALL LETTER KP
            return 0x16E46;	//	𖹆	0xf0 0x96 0xb9 0x86	MEDEFAIDRIN CAPITAL LETTER KP

        case 0x16E67:		//	𖹧	0xf0 0x96 0xb9 0xa7	MEDEFAIDRIN SMALL LETTER P
            return 0x16E47;	//	𖹇	0xf0 0x96 0xb9 0x87	MEDEFAIDRIN CAPITAL LETTER P

        case 0x16E68:		//	𖹨	0xf0 0x96 0xb9 0xa8	MEDEFAIDRIN SMALL LETTER T
            return 0x16E48;	//	𖹈	0xf0 0x96 0xb9 0x88	MEDEFAIDRIN CAPITAL LETTER T

        case 0x16E69:		//	𖹩	0xf0 0x96 0xb9 0xa9	MEDEFAIDRIN SMALL LETTER G
            return 0x16E49;	//	𖹉	0xf0 0x96 0xb9 0x89	MEDEFAIDRIN CAPITAL LETTER G

        case 0x16E6A:		//	𖹪	0xf0 0x96 0xb9 0xaa	MEDEFAIDRIN SMALL LETTER F
            return 0x16E4A;	//	𖹊	0xf0 0x96 0xb9 0x8a	MEDEFAIDRIN CAPITAL LETTER F

        case 0x16E6B:		//	𖹫	0xf0 0x96 0xb9 0xab	MEDEFAIDRIN SMALL LETTER I
            return 0x16E4B;	//	𖹋	0xf0 0x96 0xb9 0x8b	MEDEFAIDRIN CAPITAL LETTER I

        case 0x16E6C:		//	𖹬	0xf0 0x96 0xb9 0xac	MEDEFAIDRIN SMALL LETTER K
            return 0x16E4C;	//	𖹌	0xf0 0x96 0xb9 0x8c	MEDEFAIDRIN CAPITAL LETTER K

        case 0x16E6D:		//	𖹭	0xf0 0x96 0xb9 0xad	MEDEFAIDRIN SMALL LETTER A
            return 0x16E4D;	//	𖹍	0xf0 0x96 0xb9 0x8d	MEDEFAIDRIN CAPITAL LETTER A

        case 0x16E6E:		//	𖹮	0xf0 0x96 0xb9 0xae	MEDEFAIDRIN SMALL LETTER J
            return 0x16E4E;	//	𖹎	0xf0 0x96 0xb9 0x8e	MEDEFAIDRIN CAPITAL LETTER J

        case 0x16E6F:		//	𖹯	0xf0 0x96 0xb9 0xaf	MEDEFAIDRIN SMALL LETTER E
            return 0x16E4F;	//	𖹏	0xf0 0x96 0xb9 0x8f	MEDEFAIDRIN CAPITAL LETTER E

        case 0x16E70:		//	𖹰	0xf0 0x96 0xb9 0xb0	MEDEFAIDRIN SMALL LETTER B
            return 0x16E50;	//	𖹐	0xf0 0x96 0xb9 0x90	MEDEFAIDRIN CAPITAL LETTER B

        case 0x16E71:		//	𖹱	0xf0 0x96 0xb9 0xb1	MEDEFAIDRIN SMALL LETTER C
            return 0x16E51;	//	𖹑	0xf0 0x96 0xb9 0x91	MEDEFAIDRIN CAPITAL LETTER C

        case 0x16E72:		//	𖹲	0xf0 0x96 0xb9 0xb2	MEDEFAIDRIN SMALL LETTER U
            return 0x16E52;	//	𖹒	0xf0 0x96 0xb9 0x92	MEDEFAIDRIN CAPITAL LETTER U

        case 0x16E73:		//	𖹳	0xf0 0x96 0xb9 0xb3	MEDEFAIDRIN SMALL LETTER YU
            return 0x16E53;	//	𖹓	0xf0 0x96 0xb9 0x93	MEDEFAIDRIN CAPITAL LETTER YU

        case 0x16E74:		//	𖹴	0xf0 0x96 0xb9 0xb4	MEDEFAIDRIN SMALL LETTER L
            return 0x16E54;	//	𖹔	0xf0 0x96 0xb9 0x94	MEDEFAIDRIN CAPITAL LETTER L

        case 0x16E75:		//	𖹵	0xf0 0x96 0xb9 0xb5	MEDEFAIDRIN SMALL LETTER Q
            return 0x16E55;	//	𖹕	0xf0 0x96 0xb9 0x95	MEDEFAIDRIN CAPITAL LETTER Q

        case 0x16E76:		//	𖹶	0xf0 0x96 0xb9 0xb6	MEDEFAIDRIN SMALL LETTER HP
            return 0x16E56;	//	𖹖	0xf0 0x96 0xb9 0x96	MEDEFAIDRIN CAPITAL LETTER HP

        case 0x16E77:		//	𖹷	0xf0 0x96 0xb9 0xb7	MEDEFAIDRIN SMALL LETTER NY
            return 0x16E57;	//	𖹗	0xf0 0x96 0xb9 0x97	MEDEFAIDRIN CAPITAL LETTER NY

        case 0x16E78:		//	𖹸	0xf0 0x96 0xb9 0xb8	MEDEFAIDRIN SMALL LETTER X
            return 0x16E58;	//	𖹘	0xf0 0x96 0xb9 0x98	MEDEFAIDRIN CAPITAL LETTER X

        case 0x16E79:		//	𖹹	0xf0 0x96 0xb9 0xb9	MEDEFAIDRIN SMALL LETTER D
            return 0x16E59;	//	𖹙	0xf0 0x96 0xb9 0x99	MEDEFAIDRIN CAPITAL LETTER D

        case 0x16E7A:		//	𖹺	0xf0 0x96 0xb9 0xba	MEDEFAIDRIN SMALL LETTER OE
            return 0x16E5A;	//	𖹚	0xf0 0x96 0xb9 0x9a	MEDEFAIDRIN CAPITAL LETTER OE

        case 0x16E7B:		//	𖹻	0xf0 0x96 0xb9 0xbb	MEDEFAIDRIN SMALL LETTER N
            return 0x16E5B;	//	𖹛	0xf0 0x96 0xb9 0x9b	MEDEFAIDRIN CAPITAL LETTER N

        case 0x16E7C:		//	𖹼	0xf0 0x96 0xb9 0xbc	MEDEFAIDRIN SMALL LETTER R
            return 0x16E5C;	//	𖹜	0xf0 0x96 0xb9 0x9c	MEDEFAIDRIN CAPITAL LETTER R

        case 0x16E7D:		//	𖹽	0xf0 0x96 0xb9 0xbd	MEDEFAIDRIN SMALL LETTER O
            return 0x16E5D;	//	𖹝	0xf0 0x96 0xb9 0x9d	MEDEFAIDRIN CAPITAL LETTER O

        case 0x16E7E:		//	𖹾	0xf0 0x96 0xb9 0xbe	MEDEFAIDRIN SMALL LETTER AI
            return 0x16E5E;	//	𖹞	0xf0 0x96 0xb9 0x9e	MEDEFAIDRIN CAPITAL LETTER AI

        case 0x16E7F:		//	𖹿	0xf0 0x96 0xb9 0xbf	MEDEFAIDRIN SMALL LETTER Y
            return 0x16E5F;	//	𖹟	0xf0 0x96 0xb9 0x9f	MEDEFAIDRIN CAPITAL LETTER Y

#endif
#if UTF_SUPPORT_ADLAM == 1
        case 0x1E922:		//	𞤢	0xf0 0x9e 0xa4 0xa2	ADLAM SMALL LETTER ALIF
            return 0x1E900;	//	𞤀	0xf0 0x9e 0xa4 0x80	ADLAM CAPITAL LETTER ALIF

        case 0x1E923:		//	𞤣	0xf0 0x9e 0xa4 0xa3	ADLAM SMALL LETTER DAALI
            return 0x1E901;	//	𞤁	0xf0 0x9e 0xa4 0x81	ADLAM CAPITAL LETTER DAALI

        case 0x1E924:		//	𞤤	0xf0 0x9e 0xa4 0xa4	ADLAM SMALL LETTER LAAM
            return 0x1E902;	//	𞤂	0xf0 0x9e 0xa4 0x82	ADLAM CAPITAL LETTER LAAM

        case 0x1E925:		//	𞤥	0xf0 0x9e 0xa4 0xa5	ADLAM SMALL LETTER MIIM
            return 0x1E903;	//	𞤃	0xf0 0x9e 0xa4 0x83	ADLAM CAPITAL LETTER MIIM

        case 0x1E926:		//	𞤦	0xf0 0x9e 0xa4 0xa6	ADLAM SMALL LETTER BA
            return 0x1E904;	//	𞤄	0xf0 0x9e 0xa4 0x84	ADLAM CAPITAL LETTER BA

        case 0x1E927:		//	𞤧	0xf0 0x9e 0xa4 0xa7	ADLAM SMALL LETTER SINNYIIYHE
            return 0x1E905;	//	𞤅	0xf0 0x9e 0xa4 0x85	ADLAM CAPITAL LETTER SINNYIIYHE

        case 0x1E928:		//	𞤨	0xf0 0x9e 0xa4 0xa8	ADLAM SMALL LETTER PE
            return 0x1E906;	//	𞤆	0xf0 0x9e 0xa4 0x86	ADLAM CAPITAL LETTER PE

        case 0x1E929:		//	𞤩	0xf0 0x9e 0xa4 0xa9	ADLAM SMALL LETTER BHE
            return 0x1E907;	//	𞤇	0xf0 0x9e 0xa4 0x87	ADLAM CAPITAL LETTER BHE

        case 0x1E92A:		//	𞤪	0xf0 0x9e 0xa4 0xaa	ADLAM SMALL LETTER RA
            return 0x1E908;	//	𞤈	0xf0 0x9e 0xa4 0x88	ADLAM CAPITAL LETTER RA

        case 0x1E92B:		//	𞤫	0xf0 0x9e 0xa4 0xab	ADLAM SMALL LETTER E
            return 0x1E909;	//	𞤉	0xf0 0x9e 0xa4 0x89	ADLAM CAPITAL LETTER E

        case 0x1E92C:		//	𞤬	0xf0 0x9e 0xa4 0xac	ADLAM SMALL LETTER FA
            return 0x1E90A;	//	𞤊	0xf0 0x9e 0xa4 0x8a	ADLAM CAPITAL LETTER FA

        case 0x1E92D:		//	𞤭	0xf0 0x9e 0xa4 0xad	ADLAM SMALL LETTER I
            return 0x1E90B;	//	𞤋	0xf0 0x9e 0xa4 0x8b	ADLAM CAPITAL LETTER I

        case 0x1E92E:		//	𞤮	0xf0 0x9e 0xa4 0xae	ADLAM SMALL LETTER O
            return 0x1E90C;	//	𞤌	0xf0 0x9e 0xa4 0x8c	ADLAM CAPITAL LETTER O

        case 0x1E92F:		//	𞤯	0xf0 0x9e 0xa4 0xaf	ADLAM SMALL LETTER DHA
            return 0x1E90D;	//	𞤍	0xf0 0x9e 0xa4 0x8d	ADLAM CAPITAL LETTER DHA

        case 0x1E930:		//	𞤰	0xf0 0x9e 0xa4 0xb0	ADLAM SMALL LETTER YHE
            return 0x1E90E;	//	𞤎	0xf0 0x9e 0xa4 0x8e	ADLAM CAPITAL LETTER YHE

        case 0x1E931:		//	𞤱	0xf0 0x9e 0xa4 0xb1	ADLAM SMALL LETTER WAW
            return 0x1E90F;	//	𞤏	0xf0 0x9e 0xa4 0x8f	ADLAM CAPITAL LETTER WAW

        case 0x1E932:		//	𞤲	0xf0 0x9e 0xa4 0xb2	ADLAM SMALL LETTER NUN
            return 0x1E910;	//	𞤐	0xf0 0x9e 0xa4 0x90	ADLAM CAPITAL LETTER NUN

        case 0x1E933:		//	𞤳	0xf0 0x9e 0xa4 0xb3	ADLAM SMALL LETTER KAF
            return 0x1E911;	//	𞤑	0xf0 0x9e 0xa4 0x91	ADLAM CAPITAL LETTER KAF

        case 0x1E934:		//	𞤴	0xf0 0x9e 0xa4 0xb4	ADLAM SMALL LETTER YA
            return 0x1E912;	//	𞤒	0xf0 0x9e 0xa4 0x92	ADLAM CAPITAL LETTER YA

        case 0x1E935:		//	𞤵	0xf0 0x9e 0xa4 0xb5	ADLAM SMALL LETTER U
            return 0x1E913;	//	𞤓	0xf0 0x9e 0xa4 0x93	ADLAM CAPITAL LETTER U

        case 0x1E936:		//	𞤶	0xf0 0x9e 0xa4 0xb6	ADLAM SMALL LETTER JIIM
            return 0x1E914;	//	𞤔	0xf0 0x9e 0xa4 0x94	ADLAM CAPITAL LETTER JIIM

        case 0x1E937:		//	𞤷	0xf0 0x9e 0xa4 0xb7	ADLAM SMALL LETTER CHI
            return 0x1E915;	//	𞤕	0xf0 0x9e 0xa4 0x95	ADLAM CAPITAL LETTER CHI

        case 0x1E938:		//	𞤸	0xf0 0x9e 0xa4 0xb8	ADLAM SMALL LETTER HA
            return 0x1E916;	//	𞤖	0xf0 0x9e 0xa4 0x96	ADLAM CAPITAL LETTER HA

        case 0x1E939:		//	𞤹	0xf0 0x9e 0xa4 0xb9	ADLAM SMALL LETTER QAAF
            return 0x1E917;	//	𞤗	0xf0 0x9e 0xa4 0x97	ADLAM CAPITAL LETTER QAAF

        case 0x1E93A:		//	𞤺	0xf0 0x9e 0xa4 0xba	ADLAM SMALL LETTER GA
            return 0x1E918;	//	𞤘	0xf0 0x9e 0xa4 0x98	ADLAM CAPITAL LETTER GA

        case 0x1E93B:		//	𞤻	0xf0 0x9e 0xa4 0xbb	ADLAM SMALL LETTER NYA
            return 0x1E919;	//	𞤙	0xf0 0x9e 0xa4 0x99	ADLAM CAPITAL LETTER NYA

        case 0x1E93C:		//	𞤼	0xf0 0x9e 0xa4 0xbc	ADLAM SMALL LETTER TU
            return 0x1E91A;	//	𞤚	0xf0 0x9e 0xa4 0x9a	ADLAM CAPITAL LETTER TU

        case 0x1E93D:		//	𞤽	0xf0 0x9e 0xa4 0xbd	ADLAM SMALL LETTER NHA
            return 0x1E91B;	//	𞤛	0xf0 0x9e 0xa4 0x9b	ADLAM CAPITAL LETTER NHA

        case 0x1E93E:		//	𞤾	0xf0 0x9e 0xa4 0xbe	ADLAM SMALL LETTER VA
            return 0x1E91C;	//	𞤜	0xf0 0x9e 0xa4 0x9c	ADLAM CAPITAL LETTER VA

        case 0x1E93F:		//	𞤿	0xf0 0x9e 0xa4 0xbf	ADLAM SMALL LETTER KHA
            return 0x1E91D;	//	𞤝	0xf0 0x9e 0xa4 0x9d	ADLAM CAPITAL LETTER KHA

        case 0x1E940:		//	𞥀	0xf0 0x9e 0xa5 0x80	ADLAM SMALL LETTER GBE
            return 0x1E91E;	//	𞤞	0xf0 0x9e 0xa4 0x9e	ADLAM CAPITAL LETTER GBE

        case 0x1E941:		//	𞥁	0xf0 0x9e 0xa5 0x81	ADLAM SMALL LETTER ZAL
            return 0x1E91F;	//	𞤟	0xf0 0x9e 0xa4 0x9f	ADLAM CAPITAL LETTER ZAL

        case 0x1E942:		//	𞥂	0xf0 0x9e 0xa5 0x82	ADLAM SMALL LETTER KPO
            return 0x1E920;	//	𞤠	0xf0 0x9e 0xa4 0xa0	ADLAM CAPITAL LETTER KPO

        case 0x1E943:		//	𞥃	0xf0 0x9e 0xa5 0x83	ADLAM SMALL LETTER SHA
            return 0x1E921;	//	𞤡	0xf0 0x9e 0xa4 0xa1	ADLAM CAPITAL LETTER SHA
#endif
        default:
            return wc;
    }
}
